@@ -10,7 +10,7 @@ if [ "$(whoami)" = "root" ]; then
     SUDO_U="runuser -u"
 fi
 
-APT_OPTIONS=""
+APT_OPTIONS="--no-install-recommends"
 if [ "$NON_INTERACTIVE" = "1" ]; then
     APT_OPTIONS="-y"
 fi
@@ -43,7 +43,7 @@ sanity_checks() {
 
 install_packages() {
     echo ">>> Ensuring we have the required packages"
-    packages="$(sed -Ene 's/.*# deb: (.*)/\1,/p' setup.cfg)"
+    packages="$(sed -Ene 's/.*# deb: (.*)/\1,/p' pyproject.toml)"
 
     # shellcheck disable=SC2086
     $SUDO apt-get $APT_OPTIONS satisfy "$packages"
