@@ -12,6 +12,9 @@ function copy_logs ()
 	echo "Copying debusine logs to $DESTINATION"
 	mkdir -p "$DESTINATION"
 	cp -rv /var/log/debusine "$DESTINATION"
+
+	journalctl -u debusine-server-celery.service > "$DESTINATION/debusine-server-celery.log"
+	journalctl -u debusine-server-scheduler.service > "$DESTINATION/debusine-server-scheduler.log"
 }
 
 trap copy_logs EXIT
