@@ -10,7 +10,9 @@ debusine_client_config_directory=~/.config/debusine/client
 mkdir --parents "$debusine_client_config_directory"
 
 user="test-user"
-sudo -u debusine-server debusine-admin create_user "$user" email@example.com > /dev/null
+# shellcheck disable=SC2024
+sudo -u debusine-server debusine-admin create_user "$user" email@example.com \
+  > "$AUTOPKGTEST_TMP"/test-password.txt
 token_client=$(sudo -u debusine-server debusine-admin create_token "$user")
 
 cat << EOF > "$debusine_client_config_directory/config.ini"
@@ -19,9 +21,17 @@ default-server = integration-test
 
 [server:integration-test]
 api-url = https://$(hostname -f)/api
+scope = debusine
 token = $token_client
 EOF
 
+# Give owner permission on the default workspace to the test user
+sudo -u debusine-server debusine-admin group create debusine/Owners
+sudo -u debusine-server debusine-admin \
+  create_workspace --with-owners-group Owners debusine/System
+sudo -u debusine-server debusine-admin \
+  group members debusine/Owners --add "$user"
+
 # Allow test user to add workflow templates
 sudo -u debusine-server debusine-admin shell << EOF
 from django.contrib.auth.models import Permission
