@@ -19,7 +19,6 @@ from debusine.artifacts.local_artifact import (
     LocalArtifact,
     PackageBuildLog,
     SigningInputArtifact,
-    SigningKeyArtifact,
     SigningOutputArtifact,
     SourcePackage,
     Upload,
@@ -37,7 +36,6 @@ __all__ = [
     "LocalArtifact",
     "PackageBuildLog",
     "SigningInputArtifact",
-    "SigningKeyArtifact",
     "SigningOutputArtifact",
     "SourcePackage",
     "Upload",
