@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,7 +13,6 @@ Local artifact's representation.
 See docs/design/ontology.html for the semantics of the Artifacts.
 """
 import abc
-import base64
 import json
 import re
 from collections.abc import Iterable, Sequence
@@ -32,6 +31,7 @@ except ImportError:
 import debusine.artifacts.models as data_models
 from debusine import utils
 from debusine.artifacts.utils import files_in_meta_file_match_files
+from debusine.assets import KeyPurpose
 from debusine.client.models import (
     ArtifactCreateRequest,
     FileRequest,
@@ -846,32 +846,6 @@ class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
         return files
 
 
-class SigningKeyArtifact(LocalArtifact[data_models.DebusineSigningKey]):
-    """Record the existence of a key in the signing service."""
-
-    _category = data_models.ArtifactCategory.SIGNING_KEY
-
-    @classmethod
-    def create(
-        cls,
-        purpose: data_models.KeyPurpose,
-        fingerprint: str,
-        public_key: bytes,
-    ) -> Self:
-        """Return a SigningKeyArtifact with the given properties."""
-        data = data_models.DebusineSigningKey(
-            purpose=purpose,
-            fingerprint=fingerprint,
-            public_key=base64.b64encode(public_key).decode(),
-        )
-        return cls(category=cls._category, data=data)
-
-    @property
-    def public_key_bytes(self) -> bytes:
-        """Return the public key as bytes."""
-        return base64.b64decode(self.data.public_key.encode())
-
-
 class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
     """Input to a Sign task."""
 
@@ -916,15 +890,19 @@ class SigningOutputArtifact(LocalArtifact[data_models.DebusineSigningOutput]):
     @classmethod
     def create(
         cls,
-        purpose: data_models.KeyPurpose,
+        purpose: KeyPurpose,
         fingerprint: str,
         results: Iterable[data_models.SigningResult],
         files: Iterable[Path],
         base_dir: Path,
+        binary_package_name: str | None = None,
     ) -> Self:
         """Return a new SigningOutputArtifact."""
         data = data_models.DebusineSigningOutput(
-            purpose=purpose, fingerprint=fingerprint, results=list(results)
+            purpose=purpose,
+            fingerprint=fingerprint,
+            results=list(results),
+            binary_package_name=binary_package_name,
         )
         artifact = cls(category=cls._category, data=data)
         for file in files:
