@@ -9,7 +9,6 @@
 
 """Unit tests for classes in local_artifacts.py."""
 
-import base64
 import json
 import re
 from datetime import datetime, timedelta
@@ -34,7 +33,6 @@ from debusine.artifacts import (
     LocalArtifact,
     PackageBuildLog,
     SigningInputArtifact,
-    SigningKeyArtifact,
     SigningOutputArtifact,
     SourcePackage,
     Upload,
@@ -46,6 +44,7 @@ from debusine.artifacts.local_artifact import (
     DebianSystemTarballArtifact,
     deb822dict_to_dict,
 )
+from debusine.assets import KeyPurpose
 from debusine.client.models import FileRequest, StrMaxLength255
 from debusine.test import TestCase
 
@@ -1472,26 +1471,6 @@ class DebianSystemImageArtifactTests(TestCase):
             DebianSystemImageArtifact._validate_files(files)
 
 
-class SigningKeyArtifactTests(TestCase):
-    """Tests for SigningKeyArtifact."""
-
-    def test_create(self) -> None:
-        """`create` correctly encodes the given parameters."""
-        artifact = SigningKeyArtifact.create(
-            data_models.KeyPurpose.UEFI, "0" * 64, b"a public key"
-        )
-
-        self.assertEqual(
-            artifact.data,
-            {
-                "purpose": data_models.KeyPurpose.UEFI,
-                "fingerprint": "0" * 64,
-                "public_key": base64.b64encode(b"a public key").decode(),
-            },
-        )
-        self.assertEqual(artifact.public_key_bytes, b"a public key")
-
-
 class SigningInputArtifactTests(TestCase):
     """Tests for SigningInputArtifact."""
 
@@ -1579,7 +1558,7 @@ class SigningOutputArtifactTests(TestCase):
         )
 
         artifact = SigningOutputArtifact.create(
-            data_models.KeyPurpose.UEFI,
+            KeyPurpose.UEFI,
             "0" * 64,
             [result],
             [output_file],
@@ -1590,9 +1569,41 @@ class SigningOutputArtifactTests(TestCase):
         self.assertEqual(
             artifact.data,
             {
-                "purpose": data_models.KeyPurpose.UEFI,
+                "purpose": KeyPurpose.UEFI,
                 "fingerprint": "0" * 64,
                 "results": [result],
+                "binary_package_name": None,
+            },
+        )
+        self.assertEqual(artifact.files, {"package/dir/file.sig": output_file})
+
+    def test_create_binary_package_name(self) -> None:
+        """Create a valid SigningOutputArtifact with a binary package name."""
+        base_dir = self.create_temporary_directory()
+        output_file = base_dir / "package" / "dir" / "file.sig"
+        output_file.parent.mkdir(parents=True)
+        output_file.write_text("signature")
+        result = data_models.SigningResult(
+            file="package/dir/file", output_file="package/dir/file.sig"
+        )
+
+        artifact = SigningOutputArtifact.create(
+            KeyPurpose.UEFI,
+            "0" * 64,
+            [result],
+            [output_file],
+            base_dir,
+            binary_package_name="hello",
+        )
+
+        artifact.validate_model()
+        self.assertEqual(
+            artifact.data,
+            {
+                "purpose": KeyPurpose.UEFI,
+                "fingerprint": "0" * 64,
+                "results": [result],
+                "binary_package_name": "hello",
             },
         )
         self.assertEqual(artifact.files, {"package/dir/file.sig": output_file})
