@@ -11,13 +11,15 @@
 from typing import Any
 from unittest import TestCase
 
-import debusine.artifacts.models as data_models
-
 try:
     import pydantic.v1 as pydantic
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+import debusine.artifacts.models as data_models
+from debusine.assets.models import KeyPurpose
+from debusine.tasks.models import TaskTypes
+
 
 class EnumTests(TestCase):
     """Tests for model enums."""
@@ -34,6 +36,180 @@ class EnumTests(TestCase):
                         self.assertEqual(str(el), el.value)
 
 
+class DebusineHistoricalTaskRunTests(TestCase):
+    """Test the DebusineHistoricalTaskRun model."""
+
+    def test_task_type_validation(self) -> None:
+        """task_type must be a valid task type."""
+        kwargs: dict[str, Any] = {
+            "task_name": "noop",
+            "subject": "subject",
+            "context": "sid/amd64",
+            "timestamp": 0,
+            "work_request_id": 1,
+            "result": "success",
+            "runtime_statistics": data_models.RuntimeStatistics(duration=1),
+        }
+        data_models.DebusineHistoricalTaskRun(
+            task_type=TaskTypes.WORKER, **kwargs
+        )
+        with self.assertRaisesRegex(ValueError, "is not a valid task type"):
+            data_models.DebusineHistoricalTaskRun(
+                task_type="does-not-exist", **kwargs
+            )
+
+    def test_context_valid(self) -> None:
+        """Context fields without colons are allowed."""
+        for context in ("sid/amd64", None):
+            with self.subTest(context=context):
+                data_models.DebusineHistoricalTaskRun(
+                    task_type=TaskTypes.WORKER,
+                    task_name="noop",
+                    subject="subject",
+                    context=context,
+                    timestamp=0,
+                    work_request_id=1,
+                    result="success",
+                    runtime_statistics=data_models.RuntimeStatistics(
+                        duration=1
+                    ),
+                )
+
+    def test_context_invalid(self) -> None:
+        """Context fields with colons are forbidden."""
+        with self.assertRaisesRegex(ValueError, r"may not contain ':'"):
+            data_models.DebusineHistoricalTaskRun(
+                task_type=TaskTypes.WORKER,
+                task_name="noop",
+                subject="subject",
+                context="sid:amd64",
+                timestamp=0,
+                work_request_id=1,
+                result="success",
+                runtime_statistics=data_models.RuntimeStatistics(duration=1),
+            )
+
+
+class DebusineTaskConfigurationTests(TestCase):
+    """Test the DebusineTaskConfiguration model."""
+
+    def test_name(self) -> None:
+        """Test name generation."""
+        data = data_models.DebusineTaskConfiguration(template="test")
+        self.assertEqual(data.name(), "template:test")
+
+        data = data_models.DebusineTaskConfiguration(
+            task_type=TaskTypes.WORKER,
+            task_name="noop",
+            subject="subject",
+            context="sid/amd64",
+        )
+        self.assertEqual(data.name(), "Worker:noop:subject:sid/amd64")
+
+        data = data_models.DebusineTaskConfiguration(
+            task_type=TaskTypes.WORKER,
+            task_name="noop",
+            subject="subject",
+            context="source:debusine",
+        )
+        self.assertEqual(data.name(), "Worker:noop:subject:source%3Adebusine")
+
+    def test_task_type_validation(self) -> None:
+        """task_type must be a valid task type."""
+        kwargs: dict[str, Any] = {
+            "task_name": "noop",
+            "subject": "subject",
+            "context": "sid/amd64",
+        }
+        data_models.DebusineTaskConfiguration(
+            task_type=TaskTypes.WORKER, **kwargs
+        )
+        with self.assertRaisesRegex(ValueError, "is not a valid task type"):
+            data_models.DebusineTaskConfiguration(
+                task_type="does-not-exist", **kwargs
+            )
+        with self.assertRaisesRegex(ValueError, "should be set"):
+            data_models.DebusineTaskConfiguration(task_type=None, **kwargs)
+        with self.assertRaisesRegex(ValueError, "should be set"):
+            data_models.DebusineTaskConfiguration(**kwargs)
+
+    def test_task_name_validation(self) -> None:
+        """Task names must not contain colons."""
+        kwargs: dict[str, Any] = {
+            "task_type": TaskTypes.WORKER,
+            "subject": "subject",
+            "context": "sid/amd64",
+        }
+        data_models.DebusineTaskConfiguration(task_name="noop", **kwargs)
+        with self.assertRaisesRegex(ValueError, "may not contain ':'"):
+            data_models.DebusineTaskConfiguration(task_name="no:op", **kwargs)
+        with self.assertRaisesRegex(ValueError, "should be set"):
+            data_models.DebusineTaskConfiguration(task_name=None, **kwargs)
+        with self.assertRaisesRegex(ValueError, "should be set"):
+            data_models.DebusineTaskConfiguration(**kwargs)
+
+    def test_subject_validation(self) -> None:
+        """Subject must not contain colons."""
+        kwargs: dict[str, Any] = {
+            "task_type": TaskTypes.WORKER,
+            "task_name": "noop",
+            "context": "sid/amd64",
+        }
+        data_models.DebusineTaskConfiguration(subject="subject", **kwargs)
+        data_models.DebusineTaskConfiguration(subject=None, **kwargs)
+        data_models.DebusineTaskConfiguration(**kwargs)
+        with self.assertRaisesRegex(ValueError, "may not contain ':'"):
+            data_models.DebusineTaskConfiguration(subject="sub:ject", **kwargs)
+
+    def test_context_validation(self) -> None:
+        """Context must not contain colons."""
+        kwargs: dict[str, Any] = {
+            "task_type": TaskTypes.WORKER,
+            "task_name": "noop",
+            "subject": "subject",
+        }
+        data_models.DebusineTaskConfiguration(
+            context="source:debusine", **kwargs
+        )
+        data_models.DebusineTaskConfiguration(context="sid/amd64", **kwargs)
+        data_models.DebusineTaskConfiguration(context="a b c", **kwargs)
+        data_models.DebusineTaskConfiguration(context=None, **kwargs)
+        data_models.DebusineTaskConfiguration(**kwargs)
+
+    def test_template_excludes_task_match(self) -> None:
+        """Template excludes the use of task match fields."""
+        for name, value in (
+            ("task_type", TaskTypes.WORKER),
+            ("task_name", "noop"),
+            ("subject", "subject"),
+            ("context", "sid/amd64"),
+        ):
+            with (
+                self.subTest(name=name),
+                self.assertRaisesRegex(
+                    ValueError, "should be empty if template is set"
+                ),
+            ):
+                kwargs: dict[str, Any] = {name: value}
+                data_models.DebusineTaskConfiguration(template="test", **kwargs)
+
+    def test_get_lookup_names(self) -> None:
+        """Test get_lookup_names."""
+        f = data_models.DebusineTaskConfiguration.get_lookup_names
+        for args, expected in (
+            (("t", "n", None, None), ["t:n::"]),
+            (("t", "n", "s", None), ["t:n::", "t:n:s:"]),
+            (("t", "n", None, "c"), ["t:n::", "t:n::c"]),
+            (("t", "n", "s", "c"), ["t:n::", "t:n::c", "t:n:s:", "t:n:s:c"]),
+            (
+                ("t", "n", "s", ":"),
+                ["t:n::", "t:n::%3A", "t:n:s:", "t:n:s:%3A"],
+            ),
+        ):
+            with self.subTest(args=args):
+                self.assertEqual(f(*args), expected)
+
+
 class EmptyArtifactDataTest(TestCase):
     """Test the EmptyArtifactData model."""
 
@@ -368,7 +544,7 @@ class DebianLintianTest(TestCase):
     """Test the DebianLintian model."""
 
     def test_get_label(self) -> None:
-        """get_label returns shortest package name."""
+        """get_label returns package names."""
 
         def make_lintian(**kwargs: str) -> data_models.DebianLintian:
             return data_models.DebianLintian(
@@ -382,14 +558,15 @@ class DebianLintianTest(TestCase):
                 )
             )
 
+        self.assertEqual(make_lintian().get_label(), "lintian (empty)")
         self.assertEqual(
-            make_lintian(hello="hello_1.0-1.dsc").get_label(), "hello"
+            make_lintian(hello="hello_1.0-1.dsc").get_label(), "lintian: hello"
         )
         self.assertEqual(
             make_lintian(
                 libhello="libhello_1.0-1.deb", hello="hello_1.0-1.deb"
             ).get_label(),
-            "hello",
+            "lintian: hello, libhello",
         )
 
 
@@ -414,19 +591,6 @@ class DebianAutopkgtestTest(TestCase):
         self.assertEqual(data.get_label(), "hello")
 
 
-class DebusineSigningKeyTest(TestCase):
-    """Test the DebusineSigningKey model."""
-
-    def test_get_label(self) -> None:
-        """get_label returns fingerprint."""
-        data = data_models.DebusineSigningKey(
-            purpose=data_models.KeyPurpose.UEFI,
-            fingerprint="123456ABCD",
-            public_key="",
-        )
-        self.assertEqual(data.get_label(), "123456ABCD")
-
-
 class DebusineSigningInputTest(TestCase):
     """Test the DebusineSigningInput model."""
 
@@ -513,7 +677,7 @@ class DebusineSigningOutputTest(TestCase):
     def test_get_label(self) -> None:
         """get_label returns None."""
         data = data_models.DebusineSigningOutput(
-            purpose=data_models.KeyPurpose.UEFI,
+            purpose=KeyPurpose.UEFI,
             fingerprint="123456ABCD",
             results=[],
         )
@@ -533,3 +697,101 @@ class DebDiffTest(TestCase):
         self.assertEqual(
             data.get_label(), "debdiff original-package new-package"
         )
+
+
+class GetSourcePackageNameTests(TestCase):
+    """Tests for function get_source_package_name."""
+
+    def test_DebianSourcePackage(self) -> None:
+        """Test artifact_data is a DebianSourcePackage."""
+        artifact_data = data_models.DebianSourcePackage(
+            name="hello", version="1.0", type="dpkg", dsc_fields={}
+        )
+        self.assertEqual(
+            data_models.get_source_package_name(artifact_data), "hello"
+        )
+
+    def test_DebianUpload(self) -> None:
+        """Test artifact_data is a DebianUpload."""
+        artifact_data = data_models.DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Source": "hello",
+                "Architecture": "amd64",
+                "Files": [{"name": "hello_1.0_amd64.deb"}],
+            },
+        )
+        self.assertEqual(
+            data_models.get_source_package_name(artifact_data), "hello"
+        )
+
+    def test_DebianBinaryPackage(self) -> None:
+        """Test artifact_data is a DebianBinaryPackage."""
+        artifact_data = data_models.DebianBinaryPackage(
+            srcpkg_name="hello",
+            srcpkg_version="1.0-1",
+            deb_fields={
+                "Package": "hello-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+        self.assertEqual(
+            data_models.get_source_package_name(artifact_data), "hello"
+        )
+
+    def test_DebianBinaryPackages(self) -> None:
+        """Test artifact_data is a DebianBinaryPackages."""
+        artifact_data = data_models.DebianBinaryPackages(
+            srcpkg_name="hello",
+            srcpkg_version="1.0-1",
+            version="1.0",
+            architecture="amd64",
+            packages=[],
+        )
+        self.assertEqual(
+            data_models.get_source_package_name(artifact_data), "hello"
+        )
+
+    def test_invalid_type(self) -> None:
+        """Test artifact_data is not supported."""
+        artifact_data = data_models.EmptyArtifactData()
+
+        with self.assertRaisesRegex(
+            TypeError, "^Unexpected type: EmptyArtifactData$"
+        ):
+            data_models.get_source_package_name(
+                artifact_data  # type: ignore[arg-type]
+            )
+
+
+class GetBinaryPackageNameTests(TestCase):
+    """Tests for function get_binary_package_name."""
+
+    def test_DebianBinaryPackage(self) -> None:
+        """Test artifact_data is a DebianBinaryPackage."""
+        artifact_data = data_models.DebianBinaryPackage(
+            srcpkg_name="hello",
+            srcpkg_version="1.0-1",
+            deb_fields={
+                "Package": "hello-bin",
+                "Version": "1.0-1+b1",
+                "Architecture": "amd64",
+            },
+            deb_control_files=[],
+        )
+        self.assertEqual(
+            data_models.get_binary_package_name(artifact_data), "hello-bin"
+        )
+
+    def test_invalid_type(self) -> None:
+        """Test artifact_data is not supported."""
+        artifact_data = data_models.EmptyArtifactData()
+
+        with self.assertRaisesRegex(
+            TypeError, "^Unexpected type: EmptyArtifactData$"
+        ):
+            data_models.get_binary_package_name(
+                artifact_data  # type: ignore[arg-type]
+            )
