@@ -42,6 +42,7 @@ from debusine.artifacts import (
     SourcePackage,
     Upload,
 )
+from debusine.assets import AssetCategory, asset_data_model
 from debusine.client import exceptions
 from debusine.client.config import ConfigHandler
 from debusine.client.debusine import Debusine
@@ -54,6 +55,7 @@ from debusine.client.models import (
     WorkRequestResponse,
     WorkflowTemplateRequest,
     model_to_json_serializable_dict,
+    model_to_json_serializable_list,
 )
 from debusine.client.utils import get_debian_package
 
@@ -337,6 +339,48 @@ class Cli:
             help="Show reverse relationships, rather than forward ones",
         )
 
+        create_asset = subparsers.add_parser(
+            "create-asset", help="Create an asset"
+        )
+        create_asset.add_argument(
+            "category",
+            type=AssetCategory,
+            help="Category of asset",
+        )
+        create_asset.add_argument(
+            "--workspace",
+            type=str,
+            required=True,
+            help="Workspace for this asset",
+        )
+        create_asset.add_argument(
+            "--data",
+            type=argparse.FileType("r"),
+            required=True,
+            help="File path (or - for stdin) to read the data "
+            "for the asset. YAML format",
+        )
+
+        list_assets = subparsers.add_parser("list-assets", help="List assets")
+        list_assets.add_argument(
+            "--id",
+            metavar="ID",
+            type=int,
+            help="Show an individual asset by ID.",
+        )
+        list_assets.add_argument(
+            "--workspace",
+            metavar="NAME",
+            type=str,
+            help="List all assets in workspace.",
+        )
+        list_assets.add_argument(
+            "--work-request",
+            metavar="ID",
+            type=int,
+            help="List assets created by work request.",
+        )
+
         on_work_request_completed = subparsers.add_parser(
             "on-work-request-completed",
             help="Execute a command when a work request is completed. "
@@ -492,6 +536,25 @@ class Cli:
                 self._show_artifact_relations(
                     debusine, self.args.id, self.args.reverse
                 )
+            case "create-asset":
+                data = self.args.data.read()
+                self.args.data.close()
+                if self.args.data == sys.stdin:
+                    print("---")  # avoid confusion between input and output
+
+                self._create_asset(
+                    debusine,
+                    self.args.category,
+                    self.args.workspace,
+                    data,
+                )
+            case "list-assets":
+                self._list_assets(
+                    debusine,
+                    asset=self.args.id,
+                    workspace=self.args.workspace,
+                    work_request=self.args.work_request,
+                )
             case "on-work-request-completed":
                 self._on_work_request_completed(
                     debusine,
@@ -1112,6 +1175,62 @@ class Cli:
         relations = [model_to_json_serializable_dict(model) for model in models]
         self._print_yaml(relations)
 
+    def _create_asset(
+        self,
+        debusine: Debusine,
+        category: AssetCategory,
+        workspace: str,
+        data_yaml: str | None,
+    ) -> None:
+        data = self._parse_yaml_data(data_yaml)
+
+        asset_data = asset_data_model(category, data)
+
+        try:
+            with self._api_call_or_fail():
+                asset = debusine.asset_create(
+                    category=category, data=asset_data, workspace=workspace
+                )
+        except DebusineError as err:
+            output = {"result": "failure", "error": err.asdict()}
+            self._print_yaml(output)
+            raise SystemExit(3)
+        else:
+            self._print_yaml(
+                {
+                    "result": "success",
+                    "message": (
+                        f"New asset created in {debusine.base_api_url} "
+                        f"in workspace {asset.workspace} with id {asset.id}."
+                    ),
+                }
+            )
+
+    def _list_assets(
+        self,
+        debusine: Debusine,
+        asset: int | None,
+        workspace: str | None,
+        work_request: int | None,
+    ) -> None:
+        if asset is None and workspace is None and work_request is None:
+            self._fail(
+                "At least one of --id, --workspace, and --work-request must "
+                "be specified."
+            )
+        try:
+            with self._api_call_or_fail():
+                assets = debusine.asset_list(
+                    asset_id=asset,
+                    work_request=work_request,
+                    workspace=workspace,
+                )
+        except DebusineError as err:
+            output = {"result": "failure", "error": err.asdict()}
+            self._print_yaml(output)
+            raise SystemExit(3)
+        self._print_yaml(model_to_json_serializable_list(assets))
+
     def _on_work_request_completed(
         self,
         debusine: Debusine,
