@@ -37,11 +37,16 @@ from requests_toolbelt.downloadutils.stream import stream_response_to_file
 
 from debusine.artifacts import LocalArtifact
 from debusine.artifacts.models import CollectionCategory
+from debusine.assets import AssetCategory, BaseAssetDataModel
 from debusine.client.debusine_http_client import DebusineHttpClient
 from debusine.client.exceptions import TokenDisabledError
 from debusine.client.file_uploader import FileUploader
 from debusine.client.models import (
     ArtifactResponse,
+    AssetCreateRequest,
+    AssetPermissionCheckResponse,
+    AssetResponse,
+    AssetsResponse,
     CreateWorkflowRequest,
     LookupChildType,
     LookupMultipleRequest,
@@ -582,6 +587,76 @@ class Debusine:
 
         return artifact
 
+    def asset_create(
+        self,
+        category: AssetCategory,
+        data: BaseAssetDataModel,
+        *,
+        workspace: str,
+        work_request: int | None = None,
+    ) -> AssetResponse:
+        """Create asset in the debusine server."""
+        return self._debusine_http_client.post(
+            "/asset/",
+            AssetResponse,
+            data=json.loads(
+                AssetCreateRequest(
+                    category=category,
+                    workspace=workspace,
+                    work_request=work_request,
+                    data=data,
+                ).json()
+            ),
+            expected_statuses=[requests.codes.created],
+        )
+
+    def asset_list(
+        self,
+        *,
+        asset_id: int | None = None,
+        work_request: int | None = None,
+        workspace: str | None = None,
+    ) -> AssetsResponse:
+        """Create asset in the debusine server."""
+        params: dict[str, str | int] = {}
+        if asset_id:
+            params["asset"] = asset_id
+        if work_request:
+            params["work_request"] = work_request
+        if workspace:
+            params["workspace"] = workspace
+        if not params:
+            raise ValueError(
+                "At least one of asset, work_request, and workspace must be "
+                "specified"
+            )
+        query = urlencode(params)
+        return self._debusine_http_client.get(
+            f"/asset/?{query}",
+            AssetsResponse,
+        )
+
+    def asset_permission_check(
+        self,
+        *,
+        asset_category: str,
+        asset_slug: str,
+        permission_name: str,
+        artifact_id: int,
+        work_request_id: int,
+        workspace: str,
+    ) -> AssetPermissionCheckResponse:
+        """Check work_request's permission to use an asset on an artifact."""
+        return self._debusine_http_client.post(
+            f"/asset/{asset_category}/{asset_slug}/{permission_name}/",
+            AssetPermissionCheckResponse,
+            data={
+                "artifact_id": artifact_id,
+                "work_request_id": work_request_id,
+                "workspace": workspace,
+            },
+        )
+
     @overload
     def lookup_single(
         self,
