@@ -21,6 +21,7 @@ try:
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+from debusine.assets import AssetCategory, BaseAssetDataModel, asset_data_model
 from debusine.utils import calculate_hash
 
 
@@ -197,6 +198,54 @@ class RemoteArtifact(StrictBaseModel):
     workspace: str
 
 
+class AssetCreateRequest(StrictBaseModel):
+    """Request for the Asset creation API."""
+
+    category: AssetCategory
+    data: BaseAssetDataModel
+    work_request: int | None = None
+    workspace: str
+
+
+class AssetResponse(StrictBaseModel):
+    """Response from an Asset creation / listing API."""
+
+    id: int
+    category: AssetCategory
+    data: BaseAssetDataModel | dict[str, Any]
+    work_request: int | None = None
+    workspace: str
+
+    @pydantic.root_validator
+    def parse_data(cls, values: dict[str, Any]) -> dict[str, Any]:  # noqa: U100
+        """Parse data using the correct data model."""
+        if isinstance(values.get("data"), dict):
+            category = values.get("category")
+            if category:  # pragma: no cover
+                values["data"] = asset_data_model(category, values["data"])
+        return values
+
+
+class AssetsResponse(StrictBaseModel):
+    """A response from the server with multiple AssetResponse objects."""
+
+    # This model parses a list, not an object/dict.
+    __root__: Sequence[AssetResponse]
+
+    def __iter__(self) -> Iterator[AssetResponse]:  # type: ignore[override]
+        """Iterate over individual asset responses."""
+        return iter(self.__root__)
+
+
+class AssetPermissionCheckResponse(StrictBaseModel):
+    """A response from the asset permission check endpoint."""
+
+    has_permission: bool
+    username: str | None
+    user_id: int | None
+    resource: dict[str, Any] | None
+
+
 class RelationType(StrEnum):
     """Possible values for `RelationCreateRequest.type`."""
 
@@ -325,3 +374,12 @@ def model_to_json_serializable_dict(
     serializable = json.loads(model.json(exclude_unset=exclude_unset))
     assert isinstance(serializable, dict)
     return serializable
+
+
+def model_to_json_serializable_list(
+    model: pydantic.BaseModel, exclude_unset: bool = False
+) -> list[dict[Any, Any]]:
+    """Similar to model_to_json_serializable_dict, but for a list response."""
+    serializable = json.loads(model.json(exclude_unset=exclude_unset))
+    assert isinstance(serializable, list)
+    return serializable
