@@ -44,9 +44,13 @@ from debusine.artifacts.models import (
     CollectionCategory,
     EmptyArtifactData,
 )
+from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
+    AssetPermissionCheckResponse,
+    AssetResponse,
+    AssetsResponse,
     CreateWorkflowRequest,
     FileResponse,
     FilesResponseType,
@@ -1106,6 +1110,171 @@ class DebusineTests(TestCase):
             "Artifact file downloaded: %s", destination
         )
 
+    @responses.activate
+    def test_asset_create_success(self) -> None:
+        """Test asset_create succeeds."""
+        workspace = "test"
+        work_request = 5
+
+        asset_data = SigningKeyData(
+            purpose=KeyPurpose.OPENPGP,
+            fingerprint="ABC123",
+            public_key="PUBLIC KEY",
+            description="Description",
+        )
+        asset_request = {
+            "category": AssetCategory.SIGNING_KEY,
+            "workspace": workspace,
+            "data": asset_data.dict(),
+            "work_request": work_request,
+        }
+        asset_response_json = {
+            "id": 99,
+            "category": AssetCategory.SIGNING_KEY,
+            "workspace": workspace,
+            "data": asset_data.dict(),
+            "work_request": work_request,
+        }
+        asset_response = AssetResponse(
+            id=99,
+            category=AssetCategory.SIGNING_KEY,
+            workspace=workspace,
+            data=asset_data,
+            work_request=work_request,
+        )
+
+        responses.add(
+            responses.POST,
+            f"{self.api_url}/1.0/asset/",
+            json=asset_response_json,
+            status=requests.codes.created,
+        )
+
+        actual_asset_response = self.debusine.asset_create(
+            category=AssetCategory.SIGNING_KEY,
+            workspace=workspace,
+            data=asset_data,
+            work_request=work_request,
+        )
+
+        self.assertEqual(actual_asset_response, asset_response)
+
+        self.assert_token_key_included_in_all_requests(self.token)
+
+        assert responses.calls[0].request.body is not None
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body), asset_request
+        )
+
+    def assert_list_assets(
+        self,
+        url: str,
+        asset_id: int | None = None,
+        work_request: int | None = None,
+        workspace: str | None = None,
+    ) -> None:
+        """Call client.asset_list with the specified options, assert results."""
+        asset_data = SigningKeyData(
+            purpose=KeyPurpose.OPENPGP,
+            fingerprint="ABC123",
+            public_key="PUBLIC KEY",
+            description="Description",
+        )
+        assets_response = AssetsResponse.parse_obj(
+            [
+                AssetResponse(
+                    id=99,
+                    category=AssetCategory.SIGNING_KEY,
+                    workspace="workspace",
+                    data=asset_data,
+                    work_request=12,
+                ).dict()
+            ]
+        )
+        assets_response_json = [
+            {
+                "id": 99,
+                "category": AssetCategory.SIGNING_KEY,
+                "workspace": "workspace",
+                "data": asset_data.dict(),
+                "work_request": 12,
+            }
+        ]
+        responses.add(
+            responses.GET,
+            url,
+            json=assets_response_json,
+            status=requests.codes.ok,
+        )
+
+        actual_asset_response = self.debusine.asset_list(
+            asset_id=asset_id, workspace=workspace, work_request=work_request
+        )
+
+        self.assertEqual(actual_asset_response, assets_response)
+
+        self.assert_token_key_included_in_all_requests(self.token)
+
+    @responses.activate
+    def test_asset_list_by_id(self) -> None:
+        """Test asset_list by asset_id."""
+        self.assert_list_assets(
+            url=f"{self.api_url}/1.0/asset/?asset=99", asset_id=99
+        )
+
+    @responses.activate
+    def test_asset_list_by_workspace(self) -> None:
+        """Test asset_list by workspace."""
+        self.assert_list_assets(
+            url=f"{self.api_url}/1.0/asset/?workspace=workspace",
+            workspace="workspace",
+        )
+
+    @responses.activate
+    def test_asset_list_by_work_request(self) -> None:
+        """Test asset_list by work_request_id."""
+        self.assert_list_assets(
+            url=f"{self.api_url}/1.0/asset/?work_request=12",
+            work_request=12,
+        )
+
+    @responses.activate
+    def test_asset_list_without_filter(self) -> None:
+        """Test asset_list without a filter."""
+        with self.assertRaisesRegex(
+            ValueError,
+            r"At least one of asset, work_request, and workspace must be "
+            r"specified",
+        ):
+            self.debusine.asset_list()
+
+    @responses.activate
+    def test_asset_permission_check(self) -> None:
+        permission_check_response = AssetPermissionCheckResponse(
+            has_permission=True,
+            username="testuser",
+            user_id=123,
+            resource={"package": "foo"},
+        )
+        responses.add(
+            responses.POST,
+            (
+                f"{self.debusine.api_url}/asset/debusine:signing-key/"
+                f"openpgp:ABC123/sign_with/"
+            ),
+            json=permission_check_response.dict(),
+        )
+        response = self.debusine.asset_permission_check(
+            asset_category="debusine:signing-key",
+            asset_slug="openpgp:ABC123",
+            permission_name="sign_with",
+            artifact_id=12,
+            work_request_id=13,
+            workspace="workspace",
+        )
+        self.assertEqual(response, permission_check_response)
+        self.assert_token_key_included_in_all_requests(self.token)
+
     @responses.activate
     def test_lookup_single(self) -> None:
         """lookup_single() performs the requested lookup."""
