@@ -17,8 +17,11 @@ try:
 except ImportError:
     import pydantic  # type: ignore
 
+from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.models import (
     ArtifactCreateRequest,
+    AssetResponse,
+    AssetsResponse,
     FileRequest,
     FilesRequestType,
     LookupMultipleResponse,
@@ -318,3 +321,58 @@ class ModelToDictTests(TestCase):
         expected = {"id": obj_id, "created_at": created_at.isoformat()}
 
         self.assertEqual(model_to_json_serializable_dict(model), expected)
+
+
+class AssetResponseTest(TestCase):
+    """Tests for the AssetResponse model."""
+
+    def test_parse(self) -> None:
+        """Test parsing a nested dictionary."""
+        body = {
+            "id": 99,
+            "category": "debusine:signing-key",
+            "workspace": "test",
+            "data": {
+                "purpose": "openpgp",
+                "fingerprint": "ABC123",
+                "public_key": "PUBLIC KEY",
+                "description": "Description",
+            },
+            "work_request": 5,
+        }
+        AssetResponse.parse_obj(body)
+
+    def test_parse_no_category(self) -> None:
+        """Test parsing an asset without a category."""
+        body = {
+            "id": 99,
+            "category": "",
+            "workspace": "test",
+            "data": {},
+            "work_request": 5,
+        }
+        with self.assertRaisesRegex(pydantic.ValidationError, r".*category.*"):
+            AssetResponse.parse_obj(body)
+
+
+class AssetsResponsesTest(TestCase):
+    """Tests for the AssetsResponses model."""
+
+    def test_iter(self) -> None:
+        """Test that AssetsResponses are iterable."""
+        responses = AssetsResponse.parse_obj(
+            [
+                AssetResponse(
+                    id=99,
+                    category=AssetCategory.SIGNING_KEY,
+                    workspace="test",
+                    data=SigningKeyData(
+                        purpose=KeyPurpose.OPENPGP,
+                        fingerprint="ABC123",
+                        public_key="PUBLIC KEY",
+                        description="Description",
+                    ),
+                ).dict()
+            ]
+        )
+        self.assertEqual(len(list(responses)), 1)
