@@ -26,8 +26,8 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
-    KeyPurpose,
 )
+from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import (
     DEFAULT_FILE_STORE_NAME,
@@ -100,7 +100,9 @@ class MigrationTests(TestCase):
     def test_system_workspace_created(self) -> None:
         """Assert System Workspace has been created."""
         workspace = default_workspace()
-        self.assertEqual(workspace.default_file_store, default_file_store())
+        self.assertQuerySetEqual(
+            workspace.scope.file_stores.all(), [default_file_store()]
+        )
 
     def test_artifact_underscore_names(self) -> None:
         """Artifacts have their data migrated to new names."""
@@ -1957,3 +1959,589 @@ class MigrationTests(TestCase):
             ).workflow_runtime_status,
             "Needs Input",
         )
+
+    def test_workrequest_fingerprint_migration(self) -> None:
+        migrate_from = [("db", "0116_assets")]
+        migrate_to = [("db", "0117_signing_work_request_fingerprints")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        default_workspace = old_apps.get_model("db", "Workspace").objects.get(
+            name=DEFAULT_WORKSPACE_NAME
+        )
+        OldArtifact = old_apps.get_model("db", "Artifact")
+        OldWorkRequest = old_apps.get_model("db", "WorkRequest")
+
+        artifacts = [
+            OldArtifact.objects.create(
+                category="debusine:signing-key",
+                workspace=default_workspace,
+                data={
+                    "fingerprint": fingerprint,
+                },
+            )
+            for fingerprint in ["ABC123", "DEF456"]
+        ]
+
+        old_work_requests = [
+            # (type, name, data, dynamic_data, parent_index)
+            (  # 0:
+                TaskTypes.WORKFLOW,
+                "debian_pipeline",
+                {
+                    "upload_key": artifacts[0].id,
+                    "make_signed_source_key": f"{artifacts[1].id}@artifacts",
+                },
+                {},
+                None,
+            ),
+            (  # 1:
+                TaskTypes.WORKFLOW,
+                "make_signed_source",
+                {
+                    "key": "something-complex",
+                },
+                {},
+                None,
+            ),
+            (  # 2:
+                TaskTypes.SIGNING,
+                "sign",
+                {
+                    "key": "something-complex",
+                },
+                {
+                    "key_id": artifacts[0].id,
+                },
+                1,
+            ),
+            (  # 3:
+                TaskTypes.WORKFLOW,
+                "package_upload",
+                {
+                    "key": "something-complex",
+                },
+                {},
+                None,
+            ),
+            (  # 4:
+                TaskTypes.SIGNING,
+                "debsign",
+                {
+                    "key": str(artifacts[1].id),
+                },
+                {
+                    "key_id": artifacts[1].id,
+                },
+                3,
+            ),
+            # error cases:
+            (  # 5: (invalid key)
+                TaskTypes.SIGNING,
+                "sign",
+                {
+                    "key": "something-complex",
+                },
+                {
+                    "key_id": -1,
+                },
+                None,
+            ),
+            (  # 6: (no key)
+                TaskTypes.SIGNING,
+                "sign",
+                {
+                    "key": "something-complex",
+                },
+                {},
+                None,
+            ),
+            (  # 7: (childless workflow)
+                TaskTypes.WORKFLOW,
+                "package_upload",
+                {
+                    "key": "something-complex",
+                },
+                {},
+                None,
+            ),
+            (  # 8: (null key)
+                TaskTypes.WORKFLOW,
+                "package_upload",
+                {
+                    "key": None,
+                },
+                {},
+                None,
+            ),
+            (  # 9: (null dynamic data)
+                TaskTypes.WORKFLOW,
+                "package_upload",
+                {
+                    "key": "something-complex",
+                },
+                None,
+                None,
+            ),
+        ]
+        old_work_request_ids: list[int] = []
+
+        for i, (
+            task_type,
+            task_name,
+            task_data,
+            dynamic_task_data,
+            parent_index,
+        ) in enumerate(old_work_requests):
+            parent_id: int | None = None
+            if parent_index:
+                parent_id = old_work_request_ids[parent_index]
+            work_request = OldWorkRequest.objects.create(
+                created_by_id=self.get_test_user_for_apps(old_apps).id,
+                task_type=task_type,
+                task_name=task_name,
+                workspace=default_workspace,
+                task_data=task_data,
+                dynamic_task_data=dynamic_task_data,
+                parent_id=parent_id,
+            )
+            old_work_request_ids.append(work_request.id)
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        NewWorkRequest = new_apps.get_model("db", "WorkRequest")
+
+        new_work_requests = list(NewWorkRequest.objects.all().order_by("id"))
+        self.assertEqual(
+            new_work_requests[0].task_data,
+            {
+                "upload_key": "ABC123",
+                "make_signed_source_key": "DEF456",
+            },
+        )
+        self.assertEqual(
+            new_work_requests[1].task_data,
+            {"key": "ABC123"},
+        )
+        self.assertEqual(
+            new_work_requests[2].task_data,
+            {"key": "ABC123"},
+        )
+        self.assertNotIn(
+            "key_id",
+            new_work_requests[2].dynamic_task_data,
+        )
+        self.assertEqual(
+            new_work_requests[3].task_data,
+            {"key": "DEF456"},
+        )
+        self.assertEqual(
+            new_work_requests[4].task_data,
+            {"key": "DEF456"},
+        )
+        self.assertNotIn(
+            "key_id",
+            new_work_requests[4].dynamic_task_data,
+        )
+        # error cases:
+        self.assertEqual(
+            new_work_requests[5].task_data,
+            {"key": "something-complex"},
+        )
+        self.assertEqual(
+            new_work_requests[6].task_data,
+            {"key": "something-complex"},
+        )
+        self.assertEqual(
+            new_work_requests[7].task_data,
+            {"key": "something-complex"},
+        )
+        self.assertEqual(
+            new_work_requests[8].task_data,
+            {"key": None},
+        )
+        self.assertEqual(
+            new_work_requests[9].task_data,
+            {"key": "something-complex"},
+        )
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+
+        new_old_work_requests = list(
+            OldWorkRequest.objects.all().order_by("id")
+        )
+
+        self.assertEqual(
+            new_old_work_requests[0].task_data,
+            {
+                "upload_key": f"{artifacts[0].id}@artifacts",
+                "make_signed_source_key": f"{artifacts[1].id}@artifacts",
+            },
+        )
+        self.assertEqual(
+            new_old_work_requests[1].task_data,
+            {"key": f"{artifacts[0].id}@artifacts"},
+        )
+        self.assertEqual(
+            new_old_work_requests[2].task_data,
+            {"key": f"{artifacts[0].id}@artifacts"},
+        )
+        self.assertEqual(
+            new_old_work_requests[2].dynamic_task_data,
+            {"key_id": artifacts[0].id},
+        )
+        self.assertEqual(
+            new_old_work_requests[3].task_data,
+            {"key": f"{artifacts[1].id}@artifacts"},
+        )
+        self.assertEqual(
+            new_old_work_requests[4].task_data,
+            {"key": f"{artifacts[1].id}@artifacts"},
+        )
+        self.assertEqual(
+            new_old_work_requests[4].dynamic_task_data,
+            {"key_id": artifacts[1].id},
+        )
+
+    def test_signing_key_asset_migration(self) -> None:
+        """`debusine:signing-key` artifacts are migrated to assets."""
+        migrate_from = [("db", "0117_signing_work_request_fingerprints")]
+        migrate_to = [("db", "0118_signing_key_assets")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        workspace = old_apps.get_model("db", "Workspace").objects.get(
+            scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+            name=DEFAULT_WORKSPACE_NAME,
+        )
+
+        # list of (category, data)
+        old_artifacts = [
+            (
+                "debusine:signing-key",
+                {
+                    "purpose": KeyPurpose.UEFI,
+                    "fingerprint": "ABC123",
+                    "public_key": "PUBLIC_KEY",
+                },
+            ),
+            (
+                "debusine:signing-key",
+                {
+                    "purpose": KeyPurpose.OPENPGP,
+                    "fingerprint": "ABC123456",
+                    "public_key": "PUBLIC_KEY",
+                },
+            ),
+            # Duplicate fingerprint, will be ignored
+            (
+                "debusine:signing-key",
+                {
+                    "purpose": KeyPurpose.OPENPGP,
+                    "fingerprint": "ABC123",
+                    "public_key": "PUBLIC_KEY",
+                },
+            ),
+            # Random other artifact
+            (
+                "debusine:test",
+                {},
+            ),
+        ]
+        # The duplicate and test remain undeleted
+        # The others keys are re-created
+        new_old_artifacts = old_artifacts[2:] + old_artifacts[:2]
+        new_assets = [
+            (
+                {
+                    "purpose": KeyPurpose.UEFI,
+                    "fingerprint": "ABC123",
+                    "public_key": "PUBLIC_KEY",
+                    "description": None,
+                },
+            ),
+            (
+                {
+                    "purpose": KeyPurpose.OPENPGP,
+                    "fingerprint": "ABC123456",
+                    "public_key": "PUBLIC_KEY",
+                    "description": None,
+                },
+            ),
+        ]
+
+        old_artifact_instances = []
+        self.maxDiff = None
+        for category, data in old_artifacts:
+            artifact = old_apps.get_model("db", "Artifact").objects.create(
+                workspace=workspace,
+                category=category,
+                data=data,
+            )
+            old_artifact_instances.append(artifact)
+
+        old_apps.get_model("db", "ArtifactRelation").objects.create(
+            artifact=old_artifact_instances[0],
+            target=old_artifact_instances[3],  # debusine:test
+            type="relates-to",
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+
+        self.assertQuerySetEqual(
+            new_apps.get_model("db", "Asset")
+            .objects.order_by("id")
+            .values_list("data"),
+            new_assets,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+
+        self.assertQuerySetEqual(
+            old_apps.get_model("db", "Artifact")
+            .objects.order_by("id")
+            .values_list("category", "data"),
+            new_old_artifacts,
+        )
+
+    def test_scope_file_stores_migration(self) -> None:
+        """File stores are migrated from `Workspace` to `Scope`."""
+        migrate_from = [("db", "0119_scope_file_stores_initial")]
+        migrate_to = [("db", "0120_scope_file_stores_data")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        OldFileStore = old_apps.get_model("db", "FileStore")
+        OldScope = old_apps.get_model("db", "Scope")
+        OldWorkspace = old_apps.get_model("db", "Workspace")
+
+        scopes = [
+            OldScope.objects.create(name=f"scope{i}", label=f"Scope {i}")
+            for i in range(4)
+        ]
+        file_stores = [
+            OldFileStore.objects.create(
+                name=f"filestore{i}",
+                backend=FileStore.BackendChoices.MEMORY,
+                configuration={"name": f"filestore{i}"},
+            )
+            for i in range(3)
+        ]
+        for scope, workspace_name, default, others in (
+            (scopes[0], "workspace0", file_stores[0], file_stores[1:]),
+            (scopes[0], "workspace1", file_stores[0], file_stores[1:]),
+            (scopes[0], "workspace2", file_stores[0], file_stores[1:]),
+            (scopes[1], "workspace0", file_stores[1], [file_stores[0]]),
+            (scopes[1], "workspace1", file_stores[1], [file_stores[0]]),
+            (scopes[2], "workspace0", file_stores[2], []),
+        ):
+            workspace = OldWorkspace.objects.create(
+                scope=scope,
+                name=workspace_name,
+                default_file_store=default,
+            )
+            workspace.other_file_stores.set(others)
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        NewFileStore = new_apps.get_model("db", "FileStore")
+        NewScope = new_apps.get_model("db", "Scope")
+        NewWorkspace = new_apps.get_model("db", "Workspace")
+
+        self.assertQuerySetEqual(
+            NewScope.objects.filter(name__startswith="scope").values_list(
+                "name",
+                "file_stores__name",
+                "filestoreinscope__upload_priority",
+                "filestoreinscope__download_priority",
+            ),
+            [
+                ("scope0", "filestore0", 100, 100),
+                ("scope0", "filestore1", None, None),
+                ("scope0", "filestore2", None, None),
+                ("scope1", "filestore1", 100, 100),
+                ("scope1", "filestore0", None, None),
+                ("scope2", "filestore2", 100, 100),
+                ("scope3", None, None, None),
+            ],
+            ordered=False,
+        )
+
+        # Manually overwrite the old workspace file store relations so that
+        # we can check that the reverse migration puts them back.
+        for workspace in NewWorkspace.objects.all():
+            workspace.default_file_store = NewFileStore.objects.get(
+                name=DEFAULT_FILE_STORE_NAME
+            )
+            workspace.other_file_stores.clear()
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+
+        self.assertQuerySetEqual(
+            OldScope.objects.filter(name__startswith="scope").values_list(
+                "name",
+                "workspaces__name",
+                "workspaces__default_file_store__name",
+                "workspaces__other_file_stores__name",
+            ),
+            [
+                ("scope0", "workspace0", "filestore0", "filestore1"),
+                ("scope0", "workspace0", "filestore0", "filestore2"),
+                ("scope0", "workspace1", "filestore0", "filestore1"),
+                ("scope0", "workspace1", "filestore0", "filestore2"),
+                ("scope0", "workspace2", "filestore0", "filestore1"),
+                ("scope0", "workspace2", "filestore0", "filestore2"),
+                ("scope1", "workspace0", "filestore1", "filestore0"),
+                ("scope1", "workspace1", "filestore1", "filestore0"),
+                ("scope2", "workspace0", "filestore2", None),
+                ("scope3", None, None, None),
+            ],
+            ordered=False,
+        )
+
+    def test_scope_file_stores_migration_mismatched_file_stores(self) -> None:
+        """File store migration needs all workspaces in a scope to agree."""
+        migrate_from = [("db", "0119_scope_file_stores_initial")]
+        migrate_to = [("db", "0120_scope_file_stores_data")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        OldFileStore = old_apps.get_model("db", "FileStore")
+        OldScope = old_apps.get_model("db", "Scope")
+        OldWorkspace = old_apps.get_model("db", "Workspace")
+
+        scope = OldScope.objects.create(name="scope0", label="Scope 0")
+        file_stores = [
+            OldFileStore.objects.create(
+                name=f"filestore{i}",
+                backend=FileStore.BackendChoices.MEMORY,
+                configuration={"name": f"filestore{i}"},
+            )
+            for i in range(3)
+        ]
+        for workspace_name, default in (
+            ("workspace0", file_stores[0]),
+            ("workspace1", file_stores[1]),
+        ):
+            OldWorkspace.objects.create(
+                scope=scope, name=workspace_name, default_file_store=default
+            )
+
+        executor.loader.build_graph()
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"^Scope 'scope0' has workspaces with different "
+            r"default file stores; make them agree first$",
+        ):
+            executor.migrate(migrate_to)
+
+        OldWorkspace.objects.filter(scope=scope).delete()
+        for workspace_name, default, others in (
+            ("workspace0", file_stores[0], [file_stores[1]]),
+            ("workspace1", file_stores[0], file_stores[1:]),
+        ):
+            workspace = OldWorkspace.objects.create(
+                scope=scope, name=workspace_name, default_file_store=default
+            )
+            workspace.other_file_stores.set(others)
+
+        executor.loader.build_graph()
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"^Scope 'scope0' has workspaces with different "
+            r"other file stores; make them agree first$",
+        ):
+            executor.migrate(migrate_to)
+
+    def test_file_store_total_size(self) -> None:
+        """A migration initializes `FileStore.total_size`."""
+        migrate_from = [("db", "0128_filestore_add_size_columns")]
+        migrate_to = [("db", "0129_filestore_total_size")]
+
+        executor = MigrationExecutor(
+            connection, progress_callback=_migrate_check_constraints
+        )
+        executor.migrate(migrate_from)
+        old_apps = executor.loader.project_state(migrate_from).apps
+        OldFileStore = old_apps.get_model("db", "FileStore")
+        OldFile = old_apps.get_model("db", "File")
+        OldFileInStore = old_apps.get_model("db", "FileInStore")
+        file_stores = [
+            OldFileStore.objects.create(
+                name=f"filestore{i}",
+                backend=FileStore.BackendChoices.MEMORY,
+                configuration={"name": f"filestore{i}"},
+            )
+            for i in range(2)
+        ]
+        for file_store, length in (
+            (file_stores[0], 0),
+            (file_stores[0], 4),
+            (file_stores[0], 1024),
+            (file_stores[1], 16),
+            (file_stores[1], 2048),
+        ):
+            contents = b"x" * length
+            hash_digest = _calculate_hash_from_data(contents)
+            file = OldFile.objects.create(sha256=hash_digest, size=length)
+            OldFileInStore.objects.create(store=file_store, file=file)
+
+        self.assertQuerySetEqual(
+            OldFileStore.objects.filter(
+                name__startswith="filestore"
+            ).values_list("name", "total_size"),
+            [("filestore0", None), ("filestore1", None)],
+            ordered=False,
+        )
+
+        for file_store in file_stores:
+            file_store.refresh_from_db()
+            self.assertIsNone(file_store.total_size)
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_to)
+        new_apps = executor.loader.project_state(migrate_to).apps
+        NewFileStore = new_apps.get_model("db", "FileStore")
+
+        self.assertQuerySetEqual(
+            NewFileStore.objects.filter(
+                name__startswith="filestore"
+            ).values_list("name", "total_size"),
+            [("filestore0", 1028), ("filestore1", 2064)],
+            ordered=False,
+        )
+
+        executor.loader.build_graph()
+        executor.migrate(migrate_from)
+
+        self.assertQuerySetEqual(
+            OldFileStore.objects.filter(
+                name__startswith="filestore"
+            ).values_list("name", "total_size"),
+            [("filestore0", None), ("filestore1", None)],
+            ordered=False,
+        )
