@@ -1,4 +1,4 @@
-# Copyright 2019, 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -17,7 +17,7 @@ from typing import Any, Generic, Optional, TYPE_CHECKING, TypeVar
 from django.conf import settings
 from django.core.exceptions import PermissionDenied, ValidationError
 from django.db import models, transaction
-from django.db.models import F, Q, QuerySet, UniqueConstraint
+from django.db.models import Exists, F, OuterRef, Q, QuerySet, UniqueConstraint
 from django.db.models.functions import Coalesce
 from django.urls import reverse
 
@@ -51,7 +51,6 @@ ARTIFACT_CATEGORY_ICON_NAMES = {
     ArtifactCategory.LINTIAN: "folder",
     ArtifactCategory.PACKAGE_BUILD_LOG: "journal-text",
     ArtifactCategory.SIGNING_INPUT: "folder",
-    ArtifactCategory.SIGNING_KEY: "folder",
     ArtifactCategory.SIGNING_OUTPUT: "folder",
     ArtifactCategory.SOURCE_PACKAGE: "folder",
     ArtifactCategory.SYSTEM_IMAGE: "folder",
@@ -69,7 +68,6 @@ ARTIFACT_CATEGORY_SHORT_NAMES = {
     ArtifactCategory.LINTIAN: "lintian report",
     ArtifactCategory.PACKAGE_BUILD_LOG: "build log",
     ArtifactCategory.SIGNING_INPUT: "signing input",
-    ArtifactCategory.SIGNING_KEY: "signing key",
     ArtifactCategory.SIGNING_OUTPUT: "signing output",
     ArtifactCategory.SOURCE_PACKAGE: "source package",
     ArtifactCategory.SYSTEM_IMAGE: "system image",
@@ -96,6 +94,19 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
+    # mypy_django_plugin generates an
+    # Artifact@AnnotatedWith[TypedDict({"complete": Any})] model subclass
+    # for this, but we don't have a way to refer to it here.
+    def annotate_complete(self) -> "ArtifactQuerySet[Any]":
+        """Annotate artifacts with whether all their files are complete."""
+        return self.annotate(
+            complete=~Exists(
+                FileInArtifact.objects.filter(
+                    artifact=OuterRef("pk"), complete=False
+                )
+            )
+        )
+
 
 class ArtifactManager(models.Manager["Artifact"]):
     """Manager for the Artifact model."""
@@ -113,8 +124,6 @@ class ArtifactManager(models.Manager["Artifact"]):
         created_by_work_request: Optional["WorkRequest"] = None,
     ) -> "Artifact":
         """Return a new Artifact based on a :class:`LocalArtifact`."""
-        file_store = workspace.default_file_store.get_backend_object()
-
         artifact = Artifact.objects.create(
             category=local_artifact.category,
             workspace=workspace,
@@ -124,7 +133,8 @@ class ArtifactManager(models.Manager["Artifact"]):
 
         for artifact_path, local_path in local_artifact.files.items():
             file = File.from_local_path(local_path)
-            file_store.add_file(local_path, fileobj=file)
+            file_backend = workspace.scope.upload_file_backend(file)
+            file_backend.add_file(local_path, fileobj=file)
             FileInArtifact.objects.create(
                 artifact=artifact, path=artifact_path, file=file, complete=True
             )
