@@ -329,6 +329,29 @@ class GroupManager(models.Manager["Group"]):
         scope_name, group_name = scope_group.split("/", 1)
         return Group.objects.get(scope__name=scope_name, name=group_name)
 
+    def create_ephemeral(self, scope: Scope, name: str, owner: User) -> "Group":
+        """Create an ephemeral group for the given user."""
+        group = self.create(scope=scope, name=name, ephemeral=True)
+        group.users.add(owner)
+        # TODO: sets the user as an admin of the newly created group
+        return group
+
+    def unused_ephemeral(self) -> QuerySet["Group"]:
+        """List ephemeral groups with no roles assigned."""
+        constraints = Q()
+        # Introspect reverse foreign keys to detect which resources have a
+        # *Roles table
+        for field in Group._meta.get_fields():
+            if (
+                field.auto_created
+                and not field.concrete
+                and field.name.endswith("_roles")
+            ):
+                # If this resource is currently assigning roles, then the group
+                # is still in use
+                constraints &= Q(**{f"{field.name}__isnull": True})
+        return Group.objects.filter(ephemeral=True).filter(constraints)
+
 
 class Group(models.Model):
     """
@@ -345,6 +368,9 @@ class Group(models.Model):
         blank=True,
         related_name="debusine_groups",
     )
+    ephemeral = models.BooleanField(
+        default=False, help_text="remove the group if it has no roles assigned"
+    )
 
     objects = GroupManager()
 
