@@ -35,10 +35,12 @@ from django.urls import reverse
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
+    BaseArtifactDataModel,
     CollectionCategory,
     DebusinePromise,
     SINGLETON_COLLECTION_CATEGORIES,
 )
+from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.constraints import JsonDataUniqueConstraint
 from debusine.db.context import context
 from debusine.db.models.permissions import (
@@ -250,11 +252,14 @@ class CollectionItemManager(models.Manager["CollectionItem"]):
         *,
         parent_collection: Collection,
         name: str,
-        data: dict[str, Any],
+        data: BaseArtifactDataModel | dict[str, Any],
         created_by_user: "User",
         created_by_workflow: Optional["WorkRequest"],
     ) -> "CollectionItem":
         """Create a CollectionItem from bare data."""
+        if isinstance(data, BaseArtifactDataModel):
+            data = model_to_json_serializable_dict(data, exclude_unset=True)
+
         match category:
             case BareDataCategory.PROMISE:
                 # Raise ValueError if data is not valid
