@@ -68,14 +68,13 @@ class ArtifactManagerTests(TestCase):
         self.assertEqual(artifact.data, local_artifact.data)
         self.assertEqual(artifact.created_by_work_request, work_request)
         self.assertEqual(artifact.fileinartifact_set.count(), 2)
-        file_store = (
-            self.scenario.workspace.default_file_store.get_backend_object()
-        )
         for file_in_artifact, path in zip(
             artifact.fileinartifact_set.order_by("id"), paths
         ):
             self.assertEqual(file_in_artifact.path, path.name)
-            with file_store.get_stream(file_in_artifact.file) as file:
+            fileobj = file_in_artifact.file
+            file_backend = self.scenario.scope.download_file_backend(fileobj)
+            with file_backend.get_stream(fileobj) as file:
                 self.assertEqual(file.read(), path.read_bytes())
 
     @context.disable_permission_checks()
