@@ -1,4 +1,4 @@
-# Copyright 2019-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -13,6 +13,7 @@ from typing import ClassVar
 
 from asgiref.sync import sync_to_async
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import AnonymousUser
 from django.db import IntegrityError
 from django.utils import timezone
 
@@ -25,6 +26,7 @@ from debusine.db.models import (
     Worker,
     Workspace,
 )
+from debusine.db.models.auth import system_user
 from debusine.db.playground import scenarios
 from debusine.test.django import ChannelsHelpersMixin, TestCase
 
@@ -178,8 +180,69 @@ class TokenManagerTests(TestCase):
         )
 
 
+class UserTests(TestCase):
+    """Tests for the User class."""
+
+    scenario = scenarios.DefaultScopeUser()
+    user: ClassVar[User]
+    system_user: ClassVar[User]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Set up common test data."""
+        super().setUpTestData()
+        cls.user = cls.playground.create_user("test")
+        cls.system_user = system_user()
+
+    def test_can_display_anonymous(self) -> None:
+        """Test can_display with anonymous user."""
+        self.assertPermission(
+            "can_display",
+            AnonymousUser(),
+            denied=[self.scenario.user, self.user, self.system_user],
+        )
+
+    def test_can_display(self) -> None:
+        """Test the can_display predicate."""
+        self.assertPermission(
+            "can_display",
+            [self.scenario.user, self.user, self.system_user],
+            allowed=[self.scenario.user, self.user, self.system_user],
+        )
+
+    def test_can_manage_anonymous(self) -> None:
+        """Test can_manage with anonymous user."""
+        self.assertPermission(
+            "can_manage",
+            AnonymousUser(),
+            denied=[self.scenario.user, self.user, self.system_user],
+        )
+
+    def test_can_manage(self) -> None:
+        """Test can_manage."""
+        self.assertPermission(
+            "can_manage",
+            self.scenario.user,
+            allowed=self.scenario.user,
+            denied=self.user,
+        )
+        self.assertPermission(
+            "can_manage",
+            self.user,
+            allowed=self.user,
+            denied=self.scenario.user,
+        )
+
+    def test_get_absolute_url(self) -> None:
+        """Test the get_absolute_url method."""
+        self.assertEqual(
+            self.scenario.user.get_absolute_url(),
+            f"/-/user/{self.scenario.user.username}/",
+        )
+
+
 class IdentityTests(TestCase):
-    """Test for Identity class."""
+    """Tests for the Identity class."""
 
     def test_str(self) -> None:
         """Stringification should show the unique key."""
@@ -222,6 +285,78 @@ class GroupManagerTests(TestCase):
             with self.subTest(arg=arg), self.assertRaises(exc_class):
                 Group.objects.from_scoped_name(arg)
 
+    def test_create_ephemeral(self) -> None:
+        """Test create_ephemeral."""
+        user = self.playground.get_default_user()
+        group = Group.objects.create_ephemeral(
+            scope=self.scope1, name="test", owner=user
+        )
+        self.assertEqual(group.scope, self.scope1)
+        self.assertEqual(group.name, "test")
+        self.assertQuerySetEqual(group.users.all(), [user])
+
+    def test_unused_ephemeral_unused(self) -> None:
+        """Test unused_ephemeral."""
+        self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
+
+        self.group1.ephemeral = True
+        self.group1.save()
+        self.assertQuerySetEqual(
+            Group.objects.unused_ephemeral(), [self.group1]
+        )
+
+    def test_unused_ephemeral_used_by_scope(self) -> None:
+        """Test unused_ephemeral for a group used by a Scope."""
+        self.group1.ephemeral = True
+        self.group1.save()
+        self.assertQuerySetEqual(
+            Group.objects.unused_ephemeral(), [self.group1]
+        )
+
+        self.group1.assign_role(self.scope1, Scope.Roles.OWNER)
+        self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
+
+    def test_unused_ephemeral_used_by_workspace(self) -> None:
+        """Test unused_ephemeral for a group used by a Workspace."""
+        self.group1.ephemeral = True
+        self.group1.save()
+        self.assertQuerySetEqual(
+            Group.objects.unused_ephemeral(), [self.group1]
+        )
+
+        workspace = self.playground.create_workspace(scope=self.scope1)
+        self.group1.assign_role(workspace, workspace.Roles.OWNER)
+        self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
+
+    def test_unused_ephemeral_used_by_asset(self) -> None:
+        """Test unused_ephemeral for a group used by an Asset."""
+        self.group1.ephemeral = True
+        self.group1.save()
+        self.assertQuerySetEqual(
+            Group.objects.unused_ephemeral(), [self.group1]
+        )
+
+        workspace = self.playground.create_workspace(scope=self.scope1)
+        asset = self.playground.create_signing_key_asset(workspace=workspace)
+        self.group1.assign_role(asset, asset.Roles.OWNER)
+        self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
+
+    def test_unused_ephemeral_used_by_asset_usage(self) -> None:
+        """Test unused_ephemeral for a group used by an AssetUsage."""
+        self.group1.ephemeral = True
+        self.group1.save()
+        self.assertQuerySetEqual(
+            Group.objects.unused_ephemeral(), [self.group1]
+        )
+
+        workspace = self.playground.create_workspace(scope=self.scope1)
+        asset = self.playground.create_signing_key_asset(workspace=workspace)
+        asset_usage = self.playground.create_asset_usage(
+            asset, workspace=workspace
+        )
+        self.group1.assign_role(asset_usage, asset_usage.Roles.SIGNER)
+        self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
+
 
 class GroupTests(TestCase):
     """Test for Group class."""
@@ -240,6 +375,10 @@ class GroupTests(TestCase):
         """Test stringification."""
         self.assertEqual(str(self.group), f"{self.scenario.scope.name}/group")
 
+    def test_defaults(self) -> None:
+        """Test default values."""
+        self.assertFalse(self.group.ephemeral)
+
     def test_unique(self) -> None:
         """Test scoped uniqueness."""
         scope2 = self.playground.get_or_create_scope("scope2")
