@@ -22,6 +22,7 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
+    DebusinePromise,
 )
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import (
@@ -416,11 +417,11 @@ class CollectionItemManagerTests(TestCase):
         """Verify create_from_bare_data with debusine:promise category."""
         category = BareDataCategory.PROMISE
         name = "valid-promise"
-        data = {
-            "promise_work_request_id": 1,
-            "promise_workflow_id": 100,
-            "promise_category": "debian:binary-package",
-        }
+        data = DebusinePromise(
+            promise_work_request_id=1,
+            promise_workflow_id=100,
+            promise_category=ArtifactCategory.BINARY_PACKAGE,
+        )
 
         # Create a CollectionItem with valid DebusinePromiseData
         collection_item = CollectionItem.objects.create_from_bare_data(
@@ -1161,8 +1162,9 @@ class CollectionItemTests(TestCase):
             created_by_user=self.user,
         )
 
+        item_url = item.get_absolute_url()
         self.assertEqual(
-            item.get_absolute_url(),
+            item_url,
             reverse(
                 "workspaces:collections:item_detail",
                 kwargs={
@@ -1174,6 +1176,33 @@ class CollectionItemTests(TestCase):
                 },
             ),
         )
+        self.assertTrue(item_url.endswith("/Name%20of%20the%20item/"))
+
+    def test_get_absolute_url_slash(self) -> None:
+        """Test get_absolute_url results when the item name contains `/`."""
+        item = CollectionItem.objects.create(
+            name="some/hierarchical/name",
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            child_type=CollectionItem.Types.BARE,
+            parent_collection=self.collection,
+            created_by_user=self.user,
+        )
+
+        item_url = item.get_absolute_url()
+        self.assertEqual(
+            item_url,
+            reverse(
+                "workspaces:collections:item_detail",
+                kwargs={
+                    "wname": self.workspace.name,
+                    "ccat": self.collection.category,
+                    "cname": self.collection.name,
+                    "iid": item.pk,
+                    "iname": item.name,
+                },
+            ),
+        )
+        self.assertTrue(item_url.endswith("/some/hierarchical/name/"))
 
     def test_expand_variables(self) -> None:
         """Variables are correctly expanded, with error handling."""
