@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 
 """Unit tests for the workspace models."""
+import datetime
 from functools import partial
 from typing import ClassVar
 from unittest import mock
@@ -16,6 +17,7 @@ from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import PermissionDenied, ValidationError
 from django.db import IntegrityError, transaction
+from django.utils import timezone
 
 from debusine.artifacts.models import CollectionCategory
 from debusine.db.context import ContextConsistencyError, context
@@ -27,12 +29,14 @@ from debusine.db.models import (
     FileStore,
     Group,
     Scope,
+    WorkRequest,
     Workspace,
     default_workspace,
 )
 from debusine.db.models.auth import User
 from debusine.db.models.permissions import PartialCheckResult
 from debusine.db.models.workspaces import (
+    DeleteWorkspaces,
     WORKSPACE_ROLES,
     WorkspaceChain,
     WorkspaceRole,
@@ -60,13 +64,6 @@ class WorkspaceManagerTests(TestCase):
         super().setUpTestData()
         cls.scope1 = cls.playground.get_or_create_scope("Scope1")
 
-    def test_get_absolute_url(self) -> None:
-        """Test the get_absolute_url method."""
-        self.assertEqual(
-            self.scenario.workspace.get_absolute_url(),
-            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/",
-        )
-
     def test_get_roles_model(self) -> None:
         """Test the get_roles_model method."""
         self.assertIs(Workspace.objects.get_roles_model(), WorkspaceRole)
@@ -131,7 +128,9 @@ class WorkspaceManagerTests(TestCase):
 
         # Roles via one group
         self.playground.create_group_role(
-            self.scenario.workspace, Workspace.Roles.OWNER, self.scenario.user
+            self.scenario.workspace,
+            Workspace.Roles.OWNER,
+            users=[self.scenario.user],
         )
         workspace = Workspace.objects.get_for_context(
             self.scenario.workspace.name
@@ -144,7 +143,7 @@ class WorkspaceManagerTests(TestCase):
         self.playground.create_group_role(
             self.scenario.workspace,
             Workspace.Roles.OWNER,
-            self.scenario.user,
+            users=[self.scenario.user],
             name="more owners",
         )
         workspace = Workspace.objects.get_for_context(
@@ -267,6 +266,20 @@ class WorkspaceTests(TestCase):
         """Check that the value is PASS."""
         self.assertEqual(value, PartialCheckResult.PASS)
 
+    def test_get_absolute_url(self) -> None:
+        """Test the get_absolute_url method."""
+        self.assertEqual(
+            self.scenario.workspace.get_absolute_url(),
+            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/",
+        )
+
+    def test_get_absolute_url_configure(self) -> None:
+        """Test the get_absolute_url_configure method."""
+        self.assertEqual(
+            self.scenario.workspace.get_absolute_url_configure(),
+            f"/{settings.DEBUSINE_DEFAULT_SCOPE}/System/configure/",
+        )
+
     def test_is_valid_workspace_name(self) -> None:
         """Test is_valid_workspace_name."""
         valid_names = (
@@ -300,10 +313,7 @@ class WorkspaceTests(TestCase):
 
     def test_workspace_name_validation(self) -> None:
         """Test validation for workspace names."""
-        kwargs = {
-            "scope": self.scenario.scope,
-            "default_file_store": self.playground.get_default_file_store(),
-        }
+        kwargs = {"scope": self.scenario.scope}
         Workspace(name="foo", **kwargs).full_clean()
         Workspace(name="foo_", **kwargs).full_clean()
         with self.assertRaises(ValidationError) as exc:
@@ -317,28 +327,26 @@ class WorkspaceTests(TestCase):
     def test_default_values_fields(self) -> None:
         """Test basic behavior."""
         name = "test"
-        workspace = Workspace(
-            name=name,
-            default_file_store=FileStore.default(),
-            scope=self.scenario.scope,
-        )
+        workspace = Workspace(name=name, scope=self.scenario.scope)
 
         self.assertEqual(workspace.name, name)
         self.assertFalse(workspace.public)
+        self.assertIsNone(workspace.created_at)
+        self.assertIsNone(workspace.expiration_delay)
+
+        pre_save_time = timezone.now()
 
         workspace.clean_fields()
         workspace.save()
+        self.assertGreaterEqual(workspace.created_at, pre_save_time)
 
     @context.disable_permission_checks()
     def test_scoping(self) -> None:
         """Test scoping."""
-        kwargs = {
-            "default_file_store": self.playground.get_default_file_store()
-        }
         scope1 = self.playground.get_or_create_scope("scope1")
         scope2 = self.playground.get_or_create_scope("scope2")
-        Workspace.objects.create(name="test", scope=scope1, **kwargs)
-        Workspace.objects.create(name="test", scope=scope2, **kwargs)
+        Workspace.objects.create(name="test", scope=scope1)
+        Workspace.objects.create(name="test", scope=scope2)
         with (
             transaction.atomic(),
             self.assertRaisesRegex(
@@ -348,7 +356,7 @@ class WorkspaceTests(TestCase):
                 "db_workspace_unique_scope_name",
             ),
         ):
-            Workspace.objects.create(name="test", scope=scope1, **kwargs)
+            Workspace.objects.create(name="test", scope=scope1)
 
     def test_workspace_roles_empty(self) -> None:
         """Test the WORKSPACE_ROLES query filter with no args."""
@@ -361,14 +369,13 @@ class WorkspaceTests(TestCase):
         """Test the WORKSPACE_ROLES query filter."""
         scope_owner = self.playground.create_user("scope_owner")
         self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, scope_owner
+            self.scenario.scope, Scope.Roles.OWNER, users=[scope_owner]
         )
         workspace_owner = self.playground.create_user("workspace_owner")
         self.playground.create_group_role(
             self.scenario.workspace,
             Workspace.Roles.OWNER,
-            scope_owner,
-            workspace_owner,
+            users=[scope_owner, workspace_owner],
         )
 
         owns_scope = dict(scope=[Scope.Roles.OWNER])
@@ -408,14 +415,13 @@ class WorkspaceTests(TestCase):
         """Test the WORKSPACE_ROLES query filter member argument."""
         scope_owner = self.playground.create_user("scope_owner")
         self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, scope_owner
+            self.scenario.scope, Scope.Roles.OWNER, users=[scope_owner]
         )
         workspace_owner = self.playground.create_user("workspace_owner")
         self.playground.create_group_role(
             self.scenario.workspace,
             Workspace.Roles.OWNER,
-            scope_owner,
-            workspace_owner,
+            users=[scope_owner, workspace_owner],
         )
         artifact, _ = self.playground.create_artifact()
 
@@ -477,7 +483,7 @@ class WorkspaceTests(TestCase):
         """Test context_has_role with a different workspace than in context."""
         # User is owner of the scope, not of the workspace
         self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, self.scenario.user
+            self.scenario.scope, Scope.Roles.OWNER, users=[self.scenario.user]
         )
 
         self.scenario.set_current()
@@ -522,7 +528,7 @@ class WorkspaceTests(TestCase):
 
         # User becomes scope owner
         scope_group = self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, self.scenario.user
+            self.scenario.scope, Scope.Roles.OWNER, users=[self.scenario.user]
         )
         with context.local():
             self.scenario.set_current()
@@ -539,7 +545,7 @@ class WorkspaceTests(TestCase):
 
         # User becomes workspace owner
         self.playground.create_group_role(
-            workspace, Workspace.Roles.OWNER, self.scenario.user
+            workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         with context.local():
             self.scenario.set_current()
@@ -568,26 +574,25 @@ class WorkspaceTests(TestCase):
             )
 
     @context.disable_permission_checks()
-    def test_is_file_in_workspace_default_file_store(self) -> None:
+    def test_file_needs_upload_initial_file_store(self) -> None:
         """
-        Test file_stores and is_file_in_workspace return the correct values.
+        Test file_needs_upload returns the correct values.
 
-        The file is added in the default FileStore.
+        The file is added in the scope's initial FileStore.
         """
         fileobj = self.playground.create_file()
         workspace = default_workspace()
-        self.assertQuerySetEqual(workspace.file_stores(fileobj), [])
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        scope = workspace.scope
+        initial_file_store = scope.file_stores.get()
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add file in store
-        FileInStore.objects.create(
-            file=fileobj, store=workspace.default_file_store
-        )
+        FileInStore.objects.create(file=fileobj, store=initial_file_store)
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj), [workspace.default_file_store]
+            scope.download_file_stores(fileobj), [initial_file_store]
         )
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add file to an artifact in another workspace
         other_workspace = self.playground.create_workspace(name="other")
@@ -599,9 +604,9 @@ class WorkspaceTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj), [workspace.default_file_store]
+            scope.download_file_stores(fileobj), [initial_file_store]
         )
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add different but complete file to an artifact in this workspace
         artifact, _ = self.playground.create_artifact(workspace=workspace)
@@ -613,9 +618,9 @@ class WorkspaceTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj), [workspace.default_file_store]
+            scope.download_file_stores(fileobj), [initial_file_store]
         )
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add incomplete file to an artifact in this workspace
         file_in_artifact = FileInArtifact.objects.create(
@@ -623,46 +628,47 @@ class WorkspaceTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj), [workspace.default_file_store]
+            scope.download_file_stores(fileobj), [initial_file_store]
         )
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Mark the previous file as complete
         file_in_artifact.complete = True
         file_in_artifact.save()
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj), [workspace.default_file_store]
+            scope.download_file_stores(fileobj), [initial_file_store]
         )
-        self.assertTrue(workspace.is_file_in_workspace(fileobj))
+        self.assertFalse(workspace.file_needs_upload(fileobj))
 
     @context.disable_permission_checks()
-    def test_is_file_in_workspace_other_file_stores(self) -> None:
+    def test_file_needs_upload_other_file_store(self) -> None:
         """
-        Test file_stores and is_file_in_workspace return the correct values.
+        Test file_stores and file_needs_upload return the correct values.
 
-        The file is added in a FileStore that is the non-default.
+        The file is added in a FileStore other than the scope's initial one.
         """
         fileobj = self.playground.create_file()
         workspace = default_workspace()
-        self.assertQuerySetEqual(workspace.file_stores(fileobj), [])
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        scope = workspace.scope
+        self.assertQuerySetEqual(scope.download_file_stores(fileobj), [])
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
-        # Add file in a store which is not the default one
+        # Add file in a store which is not the initial one
         store = FileStore.objects.create(
             name="nas-01", backend=FileStore.BackendChoices.LOCAL
         )
-        workspace.other_file_stores.add(store)
+        scope.file_stores.add(store)
         workspace.refresh_from_db()
 
-        self.assertQuerySetEqual(workspace.file_stores(fileobj), [])
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertQuerySetEqual(scope.download_file_stores(fileobj), [])
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add file in the store
         FileInStore.objects.create(file=fileobj, store=store)
 
-        self.assertQuerySetEqual(workspace.file_stores(fileobj), [store])
-        self.assertFalse(workspace.is_file_in_workspace(fileobj))
+        self.assertQuerySetEqual(scope.download_file_stores(fileobj), [store])
+        self.assertTrue(workspace.file_needs_upload(fileobj))
 
         # Add file to an artifact in this workspace
         artifact, _ = self.playground.create_artifact(workspace=workspace)
@@ -670,19 +676,19 @@ class WorkspaceTests(TestCase):
             artifact=artifact, path="test", file=fileobj, complete=True
         )
 
-        self.assertQuerySetEqual(workspace.file_stores(fileobj), [store])
-        self.assertTrue(workspace.is_file_in_workspace(fileobj))
+        self.assertQuerySetEqual(scope.download_file_stores(fileobj), [store])
+        self.assertFalse(workspace.file_needs_upload(fileobj))
 
-        # Add file to the default store as well
+        # Add file to the initial store as well
         FileInStore.objects.create(
-            file=fileobj, store=workspace.default_file_store
+            file=fileobj, store=scope.file_stores.earliest("id")
         )
 
         self.assertQuerySetEqual(
-            workspace.file_stores(fileobj),
-            [workspace.default_file_store, store],
+            scope.download_file_stores(fileobj),
+            [scope.file_stores.earliest("id"), store],
         )
-        self.assertTrue(workspace.is_file_in_workspace(fileobj))
+        self.assertFalse(workspace.file_needs_upload(fileobj))
 
     def test_str(self) -> None:
         """Test __str__ method."""
@@ -796,7 +802,7 @@ class WorkspaceTests(TestCase):
             wstart = self.playground.create_workspace(name="start", public=True)
 
             self.playground.create_group_role(
-                wpriv, Workspace.Roles.OWNER, user
+                wpriv, Workspace.Roles.OWNER, users=[user]
             )
 
             other_user = User.objects.create_user(
@@ -977,9 +983,17 @@ class WorkspaceTests(TestCase):
             self.assertFalse(
                 self.scenario.workspace.can_display(self.scenario.user)
             )
+            self.assertFalse(
+                self.scenario.workspace.can_configure(self.scenario.user)
+            )
             self.assertFalse(
                 self.scenario.workspace.can_create_artifacts(self.scenario.user)
             )
+            self.assertFalse(
+                self.scenario.workspace.can_create_experiment_workspace(
+                    self.scenario.user
+                )
+            )
 
     def test_can_display_public(self) -> None:
         """Test the can_display predicate on public workspaces."""
@@ -997,7 +1011,7 @@ class WorkspaceTests(TestCase):
         for role in Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR:
             with self.subTest(role=role), context.local():
                 self.playground.create_group_role(
-                    self.a, role, self.scenario.user, user1
+                    self.a, role, users=[self.scenario.user, user1]
                 )
 
                 context.set_scope(self.scenario.scope)
@@ -1046,22 +1060,81 @@ class WorkspaceTests(TestCase):
             token=self.playground.create_worker_token(),
         )
 
+    def test_can_configure_public(self) -> None:
+        """Test the can_configure predicate on public workspaces."""
+        self.assertPermission(
+            "can_configure",
+            users=(AnonymousUser(), self.scenario.user),
+            denied=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+        )
+
+    def test_can_configure_current(self) -> None:
+        """Test the can_configure predicate on the current workspace."""
+        user1 = self.playground.create_user("test1")
+        for role, expected in (
+            (Workspace.Roles.OWNER, True),
+            (Workspace.Roles.CONTRIBUTOR, False),
+        ):
+            with self.subTest(role=role), context.local():
+                group = self.playground.create_group_role(
+                    self.a, role, users=[self.scenario.user, user1]
+                )
+
+                context.set_scope(self.scenario.scope)
+                context.set_user(self.scenario.user)
+                self.a.set_current()
+
+                # can_configure with current workspace and user is shortcut
+                with self.assertNumQueries(0):
+                    self.assertEqual(
+                        self.a.can_configure(self.scenario.user), expected
+                    )
+
+                with self.assertNumQueries(1):
+                    self.assertEqual(self.a.can_configure(user1), expected)
+
+                group.delete()
+
+    def test_can_configure_by_roles(self) -> None:
+        """Test can_configure workspace behaviour with roles."""
+        self.assertPermissionWhenRole(
+            self.scenario.workspace.can_configure,
+            self.scenario.user,
+            Workspace.Roles.OWNER,
+            scope_roles=Scope.Roles.OWNER,
+        )
+
+    def test_can_configure_with_token(self) -> None:
+        """Test can_configure with a worker token."""
+        # Workspace is not configurable
+        self.assertPermission(
+            "can_configure",
+            users=(AnonymousUser(), self.scenario.user),
+            denied=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+        )
+
+        # Workspace is also not configurable with a worker token
+        self.assertPermission(
+            "can_configure",
+            users=(None, AnonymousUser(), self.scenario.user),
+            denied=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+            token=self.playground.create_worker_token(),
+        )
+
     def test_create_checks(self) -> None:
         """Test can_create_workspace hook in save."""
-        kwargs = {"default_file_store": FileStore.default()}
-
         with mock.patch(
             "debusine.db.models.Scope.can_create_workspace"
         ) as pred:
             test1 = Workspace.objects.create(
-                name="test1", scope=self.scenario.scope, **kwargs
+                name="test1", scope=self.scenario.scope
             )
         pred.assert_called_once_with(None)
         self.assertIsNotNone(test1.pk)
 
         with override_permission(Scope, "can_create_workspace", AllowAll):
             test2 = Workspace.objects.create(
-                name="test2", scope=self.scenario.scope, **kwargs
+                name="test2", scope=self.scenario.scope
             )
         self.assertIsNotNone(test2.pk)
 
@@ -1070,7 +1143,7 @@ class WorkspaceTests(TestCase):
                 PermissionDenied, r"None cannot create workspaces in debusine"
             ):
                 Workspace.objects.create(
-                    name="test3", scope=self.scenario.scope, **kwargs
+                    name="test3", scope=self.scenario.scope
                 )
         self.assertQuerySetEqual(Workspace.objects.filter(name="test3"), [])
 
@@ -1095,7 +1168,7 @@ class WorkspaceTests(TestCase):
         for role in Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR:
             with self.subTest(role=role), context.local():
                 self.playground.create_group_role(
-                    self.a, role, self.scenario.user, user1
+                    self.a, role, users=[self.scenario.user, user1]
                 )
 
                 context.set_scope(self.scenario.scope)
@@ -1165,8 +1238,7 @@ class WorkspaceTests(TestCase):
         self.playground.create_group_role(
             self.scenario.workspace,
             Workspace.Roles.OWNER,
-            self.scenario.user,
-            user1,
+            users=[self.scenario.user, user1],
         )
 
         self.scenario.set_current()
@@ -1190,8 +1262,7 @@ class WorkspaceTests(TestCase):
         self.playground.create_group_role(
             self.scenario.workspace,
             Workspace.Roles.CONTRIBUTOR,
-            self.scenario.user,
-            user1,
+            users=[self.scenario.user, user1],
         )
 
         self.scenario.set_current()
@@ -1224,7 +1295,7 @@ class WorkspaceTests(TestCase):
     def test_can_create_work_requests_with_token(self) -> None:
         """Test can_create_work_requests with a worker token."""
         self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, self.scenario.user
+            self.scenario.scope, Scope.Roles.OWNER, users=[self.scenario.user]
         )
 
         # Anonymous cannot
@@ -1249,6 +1320,134 @@ class WorkspaceTests(TestCase):
             token=self.playground.create_worker_token(),
         )
 
+    def test_can_create_experiment_workspace_public(self) -> None:
+        """Test can_create_experiment_workspace on public workspaces."""
+        self.assertPermission(
+            "can_create_experiment_workspace",
+            users=AnonymousUser(),
+            denied=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+        )
+        self.assertPermission(
+            "can_create_experiment_workspace",
+            users=self.scenario.user,
+            allowed=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+        )
+
+    def test_can_create_experiment_workspace_current(self) -> None:
+        """Test can_create_experiment_workspace on the current workspace."""
+        user1 = self.playground.create_user("test1")
+        self.a.public = False
+        self.a.save()
+        for role in Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR:
+            with self.subTest(role=role), context.local():
+                self.playground.create_group_role(
+                    self.a, role, users=[self.scenario.user, user1]
+                )
+
+                context.set_scope(self.scenario.scope)
+                context.set_user(self.scenario.user)
+                self.a.set_current()
+
+                # can_create_experiment_workspace with current workspace and
+                # user is shortcut
+                with self.assertNumQueries(0):
+                    self.assertTrue(
+                        self.a.can_create_experiment_workspace(
+                            self.scenario.user
+                        )
+                    )
+
+                with self.assertNumQueries(1):
+                    self.assertTrue(
+                        self.a.can_create_experiment_workspace(user1)
+                    )
+
+    def test_can_create_experiment_workspace_by_roles(self) -> None:
+        """Test can_create_experiment_workspace workspace with roles."""
+        # Make workspace private
+        self.scenario.workspace.public = False
+        self.scenario.workspace.save()
+
+        self.assertPermissionWhenRole(
+            self.scenario.workspace.can_create_experiment_workspace,
+            self.scenario.user,
+            (Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR),
+            scope_roles=Scope.Roles.OWNER,
+        )
+
+    def test_can_create_experiment_workspace_with_token(self) -> None:
+        """Test can_create_experiment_workspace with a worker token."""
+        # Make a private
+        self.a.public = False
+        self.a.save()
+
+        # Using a worker token, the permission is always denied
+        self.assertPermission(
+            "can_create_experiment_workspace",
+            users=(None, AnonymousUser(), self.scenario.user),
+            denied=[self.a, self.b, self.c, self.d, self.scenario.workspace],
+            token=self.playground.create_worker_token(),
+        )
+
+    def test_with_expiration_time_no_expiry(self) -> None:
+        """Test with_expiration_time with no expiry."""
+        ws = Workspace.objects.with_expiration_time().get(name="System")
+        self.assertIsNone(ws.expiration_time)
+
+    def test_with_expiration_time_created_at(self) -> None:
+        """Test with_expiration_time with only created_at."""
+        self.a.expiration_delay = datetime.timedelta(days=3)
+        self.a.save()
+        ws = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
+        self.assertEqual(
+            ws.expiration_time, self.a.created_at + self.a.expiration_delay
+        )
+
+    def test_with_expiration_time_work_request_completed(self) -> None:
+        """Test with_expiration_time with only created_at."""
+        self.a.expiration_delay = datetime.timedelta(days=3)
+        self.a.save()
+        wr = self.playground.create_work_request(
+            result=WorkRequest.Results.SUCCESS, workspace=self.a
+        )
+        wr.completed_at = self.a.created_at + datetime.timedelta(days=42)
+        wr.save()
+        ws = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
+        self.assertEqual(
+            ws.expiration_time,
+            self.a.created_at
+            + self.a.expiration_delay
+            + datetime.timedelta(days=42),
+        )
+
+    def test_expire_at_no_expiry(self) -> None:
+        """Test expire_at with no expiry."""
+        self.assertIsNone(self.scenario.workspace.expire_at)
+
+    def test_expire_at(self) -> None:
+        """Test with_expiration_time, computed by expire_at."""
+        self.a.expiration_delay = datetime.timedelta(days=3)
+        self.a.save()
+        self.assertFalse(hasattr(self.a, "expiration_time"))
+        expiration_time = self.a.created_at + self.a.expiration_delay
+        self.assertEqual(self.a.expire_at, expiration_time)
+        self.assertEqual(
+            self.a.expiration_time,  # type: ignore[attr-defined]
+            expiration_time,
+        )
+
+    def test_expire_at_precomputed(self) -> None:
+        """Test with_expiration_time precomputed by with_expiration_time."""
+        self.a.expiration_delay = datetime.timedelta(days=3)
+        self.a.save()
+        expiration_time = self.a.created_at + self.a.expiration_delay
+        a = Workspace.objects.with_expiration_time().get(pk=self.a.pk)
+        self.assertEqual(
+            a.expiration_time,
+            expiration_time,
+        )
+        self.assertEqual(a.expire_at, expiration_time)
+
 
 class WorkspaceRoleTests(TestCase):
     """Tests for the WorkspaceRole class."""
@@ -1265,12 +1464,8 @@ class WorkspaceRoleTests(TestCase):
         with context.disable_permission_checks():
             cls.workspace1 = cls.playground.create_workspace(name="Workspace1")
             cls.workspace2 = cls.playground.create_workspace(name="Workspace2")
-            cls.group1 = Group.objects.create(
-                name="Group1", scope=cls.playground.get_default_scope()
-            )
-            cls.group2 = Group.objects.create(
-                name="Group2", scope=cls.playground.get_default_scope()
-            )
+            cls.group1 = cls.playground.create_group("Group1")
+            cls.group2 = cls.playground.create_group("Group2")
 
     def test_str(self) -> None:
         """Test stringification."""
@@ -1328,3 +1523,39 @@ class WorkspaceRoleTests(TestCase):
         self.assertFalse(
             WorkspaceRole.objects.filter(pk=assignment.pk).exists()
         )
+
+
+class WorkspaceDeleteTests(TestCase):
+    """Test DeleteWorkspaces."""
+
+    scenario = scenarios.UIPlayground()
+
+    def test_scan(self) -> None:
+        """Test DeleteWorkspaces scan of resources."""
+        operation = DeleteWorkspaces(
+            Workspace.objects.filter(pk=self.scenario.workspace.pk)
+        )
+
+        self.assertQuerySetEqual(
+            operation.workflow_templates, [self.scenario.template_sbuild]
+        )
+        self.assertEqual(operation.artifacts.count(), 13)
+        self.assertEqual(operation.work_requests.count(), 11)
+        self.assertEqual(operation.collection_items.count(), 7)
+        self.assertEqual(operation.collections.count(), 6)
+        self.assertEqual(operation.file_in_artifacts.count(), 16)
+        self.assertEqual(operation.artifact_relations.count(), 12)
+        self.assertEqual(operation.artifacts.count(), 13)
+        self.assertEqual(operation.asset_usages.count(), 1)
+        self.assertEqual(operation.assets.count(), 1)
+        self.assertEqual(operation.workspace_roles.count(), 1)
+
+    def test_delete(self) -> None:
+        """Test DeleteWorkspaces deletion."""
+        operation = DeleteWorkspaces(
+            Workspace.objects.filter(pk=self.scenario.workspace.pk)
+        )
+        operation.perform_deletions()
+        self.assertFalse(
+            Workspace.objects.filter(pk=self.scenario.workspace.pk).exists()
+        )
