@@ -11,6 +11,7 @@
 
 import logging
 import re
+from collections import defaultdict
 from collections.abc import Generator
 from contextlib import contextmanager
 from datetime import datetime, timedelta
@@ -26,14 +27,22 @@ from typing import (
     TypeAlias,
     TypeVar,
     assert_never,
-    overload,
 )
 
-from django.conf import settings
 from django.core.exceptions import FieldError, ValidationError
 from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models
-from django.db.models import F, JSONField, Q, QuerySet, UniqueConstraint
+from django.db.models import (
+    Exists,
+    ExpressionWrapper,
+    F,
+    IntegerField,
+    JSONField,
+    OuterRef,
+    Q,
+    QuerySet,
+    UniqueConstraint,
+)
 from django.db.models.functions import Coalesce
 from django.urls import reverse
 from django.utils import timezone
@@ -64,18 +73,16 @@ from debusine.tasks.models import (
     BaseTaskData,
     EventReaction,
     EventReactions,
-    LookupMultiple,
-    LookupSingle,
     NotificationDataEmail,
     OutputData,
     TaskTypes,
 )
-from debusine.tasks.server import TaskDatabaseInterface
 
 if TYPE_CHECKING:
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.auth import User
+    from debusine.db.models.collections import Collection
     from debusine.db.models.workers import Worker
     from debusine.server.workflows.models import (
         WorkRequestManualUnblockAction,
@@ -697,6 +704,16 @@ class WorkRequest(models.Model):
         null=True,
         blank=True,
     )
+    configured_task_data = JSONField(
+        null=True,
+        blank=True,
+        encoder=DjangoJSONEncoder,
+        help_text="task_data with task configuration applied",
+    )
+    version = models.IntegerField(
+        default=0,
+        help_text="version of the code that computed the dynamic task data",
+    )
 
     class Meta(TypedModelMeta):
         indexes = [
@@ -795,6 +812,20 @@ class WorkRequest(models.Model):
                 assert_never(unreachable)
         return WorkRequest.objects.can_unblock(user).filter(pk=self.pk).exists()
 
+    def set_current(self) -> None:
+        """Set the execution context to this scope/user/workspace."""
+        context.set_scope(self.workspace.scope)
+        context.set_user(self.created_by)
+        self.workspace.set_current()
+
+    @property
+    def used_task_data(self) -> Any:
+        """Return the task data used to run this task."""
+        if self.configured_task_data is not None:
+            return self.configured_task_data
+        else:
+            return self.task_data
+
     @property
     def output_data(self) -> OutputData | None:
         """Access output_data_json as a pydantic structure."""
@@ -834,6 +865,20 @@ class WorkRequest(models.Model):
         """Set event_reactions_json from a pydantic structure."""
         self.event_reactions_json = value.dict()
 
+    def add_event_reaction(
+        self,
+        event_name: Literal[
+            "on_creation", "on_unblock", "on_success", "on_failure"
+        ],
+        action: EventReaction,
+    ) -> None:
+        """Add an event reaction."""
+        event_reactions = self.event_reactions
+        if action not in getattr(event_reactions, event_name):
+            getattr(event_reactions, event_name).append(action)
+            self.event_reactions = event_reactions
+            self.save()
+
     @contextmanager
     def scheduling_disabled(self) -> Generator[None, None, None]:
         """Temporarily disable scheduling on changes to this work request."""
@@ -887,6 +932,9 @@ class WorkRequest(models.Model):
 
     def can_satisfy_dynamic_data(self) -> bool:
         """Check if dynamic data can still be looked up."""
+        # Import here to prevent an import loop
+        from debusine.db.models.task_database import TaskDatabase
+
         try:
             task = self.get_task()
         except TaskConfigError:
@@ -1172,7 +1220,7 @@ class WorkRequest(models.Model):
                     TaskTypes(self.task_type), self.task_name
                 )
                 task = task_cls(
-                    task_data=self.task_data,
+                    task_data=self.used_task_data,
                     dynamic_task_data=self.dynamic_task_data,
                 )
             case (TaskTypes.WORKFLOW, _):
@@ -1406,17 +1454,15 @@ class WorkRequest(models.Model):
                     item_name = CollectionItem.expand_name(
                         update.name_template, expanded_variables
                     )
-                    item_variables = None
                 else:
                     item_name = None
-                    item_variables = expanded_variables
 
                 try:
                     manager.add_artifact(
                         artifact,
                         user=self.created_by,
                         workflow=self.parent,
-                        variables=item_variables,
+                        variables=expanded_variables,
                         name=item_name,
                         replace=True,
                     )
@@ -1753,8 +1799,86 @@ class WorkRequest(models.Model):
         )
         return True
 
+    def configure_for_worker(self, worker: "Worker") -> None:
+        """Populate configured and dynamic task_data for this worker."""
+        from debusine.db.models.task_database import TaskDatabase
+        from debusine.server.collections.debusine_task_configuration import (
+            apply_configuration,
+        )
+
+        # Import here to prevent circular import
+        from debusine.server.collections.lookup import lookup_single
+
+        self.configured_task_data = None
+        self.dynamic_task_data = None
+
+        # Instantiate the unconfigured task
+        task = self.get_task(worker=worker)
+        if not (task_configuration := task.data.task_configuration):
+            # We have no configuration to apply: recompute dynamic_task_data
+            # for this worker
+            if dynamic_task_data := task.compute_dynamic_data(
+                TaskDatabase(self)
+            ):
+                self.dynamic_task_data = model_to_json_serializable_dict(
+                    dynamic_task_data, exclude_unset=True
+                )
+            return
+
+        # Compute dynamic task data once to extract subject and
+        # configuration_context
+        # TODO: these values can be computed in a separate methods if the rest
+        # of compute_dynamic_data becomes too expensive to run twice
+        early_dynamic_task_data = task.compute_dynamic_data(TaskDatabase(self))
+        # TODO: remove this check once compute_dynamic_data is changed to
+        # never return None (see !1639)
+        if early_dynamic_task_data is None:
+            return  # pragma: no cover
+
+        # Look up the debusine:task-configuration collection
+        lookup_result = lookup_single(
+            lookup=task_configuration,
+            workspace=self.workspace,
+            user=self.created_by,
+            default_category=CollectionCategory.TASK_CONFIGURATION,
+            workflow_root=self.get_workflow_root(),
+            expect_type=LookupChildType.COLLECTION,
+        )
+        config_collection = lookup_result.collection
+
+        # Compute the configured task data
+        self.configured_task_data = model_to_json_serializable_dict(
+            task.data, exclude_unset=True
+        )
+        apply_configuration(
+            self.configured_task_data,
+            config_collection,
+            task.TASK_TYPE,
+            task.name,
+            early_dynamic_task_data.subject,
+            early_dynamic_task_data.configuration_context,
+        )
+
+        # Reinstantiate using configured_task_data
+        try:
+            configured_task = self.get_task(worker=worker)
+        except TaskConfigError as exc:
+            exc.add_parent_message("Configured task data is invalid")
+            raise
+
+        if dynamic_task_data := configured_task.compute_dynamic_data(
+            TaskDatabase(self)
+        ):
+            dynamic_task_data.task_configuration_id = config_collection.pk
+            self.dynamic_task_data = model_to_json_serializable_dict(
+                dynamic_task_data, exclude_unset=True
+            )
+        else:
+            self.dynamic_task_data = None
+
     def assign_worker(self, worker: "Worker") -> None:
         """Assign worker and save it."""
+        self.configure_for_worker(worker)
         self.worker = worker
         self.save()
         notifications.notify_work_request_assigned(self)
@@ -1965,117 +2089,6 @@ class WorkRequest(models.Model):
         return task.get_label()
 
 
-class TaskDatabase(TaskDatabaseInterface):
-    """Implementation of database interaction in worker tasks."""
-
-    def __init__(self, work_request: WorkRequest) -> None:
-        """Construct a :py:class:`TaskDatabase`."""
-        self.work_request = work_request
-
-    @overload
-    def lookup_single_artifact(
-        self,
-        lookup: LookupSingle,
-        default_category: CollectionCategory | None = None,
-    ) -> int: ...
-
-    @overload
-    def lookup_single_artifact(
-        self,
-        lookup: None,
-        default_category: CollectionCategory | None = None,
-    ) -> None: ...
-
-    def lookup_single_artifact(
-        self,
-        lookup: LookupSingle | None,
-        default_category: CollectionCategory | None = None,
-    ) -> int | None:
-        """Look up a single artifact using :ref:`lookup-single`."""
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_single
-
-        return (
-            None
-            if lookup is None
-            else lookup_single(
-                lookup=lookup,
-                workspace=self.work_request.workspace,
-                user=self.work_request.created_by,
-                default_category=default_category,
-                workflow_root=self.work_request.get_workflow_root(),
-                expect_type=LookupChildType.ARTIFACT,
-            ).artifact.id
-        )
-
-    def lookup_multiple_artifacts(
-        self,
-        lookup: LookupMultiple | None,
-        default_category: CollectionCategory | None = None,
-    ) -> list[int]:
-        """Look up multiple artifacts using :ref:`lookup-multiple`."""
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_multiple
-
-        return (
-            []
-            if lookup is None
-            else [
-                result.artifact.id
-                for result in lookup_multiple(
-                    lookup=lookup,
-                    workspace=self.work_request.workspace,
-                    user=self.work_request.created_by,
-                    default_category=default_category,
-                    workflow_root=self.work_request.get_workflow_root(),
-                    expect_type=LookupChildType.ARTIFACT,
-                )
-            ]
-        )
-
-    @overload
-    def lookup_single_collection(
-        self,
-        lookup: LookupSingle,
-        default_category: CollectionCategory | None = None,
-    ) -> int: ...
-
-    @overload
-    def lookup_single_collection(
-        self,
-        lookup: None,
-        default_category: CollectionCategory | None = None,
-    ) -> None: ...
-
-    def lookup_single_collection(
-        self,
-        lookup: LookupSingle | None,
-        default_category: CollectionCategory | None = None,
-    ) -> int | None:
-        """Look up a single collection using :ref:`lookup-single`."""
-        # Import here to prevent circular imports
-        from debusine.server.collections.lookup import lookup_single
-
-        return (
-            None
-            if lookup is None
-            else lookup_single(
-                lookup=lookup,
-                workspace=self.work_request.workspace,
-                user=self.work_request.created_by,
-                default_category=default_category,
-                workflow_root=self.work_request.get_workflow_root(),
-                expect_type=LookupChildType.COLLECTION,
-            ).collection.id
-        )
-
-    def get_server_setting(self, setting: str) -> str:
-        """Look up a Django setting (strings only)."""
-        value = getattr(settings, setting)
-        assert isinstance(value, str)
-        return value
-
-
 #: Regexp matching the structure of notification channel names
 notification_channel_name_regex = re.compile(r"^[A-Za-z][A-Za-z0-9+._-]*$")
 
@@ -2208,3 +2221,121 @@ def compute_workflow_runtime_status(
     raise AssertionError(
         f"Could not compute runtime status for workflow {workflow}"
     )
+
+
+class DeleteUnused:
+    """
+    Delete work requests not currently in use.
+
+    Delete work requests that don't have unexpired child work requests.
+
+    Delete also the work requests' internal collection if present.
+    """
+
+    #: Work requests to delete
+    work_requests: set[WorkRequest]
+    #: Internal collections to delete
+    collections: set["Collection"]
+
+    def __init__(self, queryset: QuerySet[WorkRequest]) -> None:
+        """
+        Initialize object.
+
+        :param work_requests: candidate work_requests to delete
+        """
+        self.queryset = queryset
+        self.scan_work_requests()
+        self.scan_collections()
+
+    def scan_work_requests(self) -> None:
+        """Look for work requests to delete, filling self.work_requests."""
+        # to_delete: expired work requests excluding the ones referenced
+        # by a CollectionItem.created_by_workflow/removed_by_workflow
+        # that are not in the CollectionItem.parent_collection
+        #
+        # In other words: if a WorkRequest is referenced by a
+        # CollectionItem.created_by_workflow/removed_by_workflow but the
+        # CollectionItem.parent_collection is WorkRequest.internal_collection
+        # the WorkRequest can be deleted.
+        to_delete = {
+            wr
+            for wr in self.queryset.exclude(
+                Exists(
+                    # See https://stackoverflow.com/questions/67775006/django-queryset-matching-null-value-with-null-outerref  # noqa: E501
+                    # and https://code.djangoproject.com/ticket/31714
+                    CollectionItem.objects.annotate(
+                        internal_collection_id=ExpressionWrapper(
+                            OuterRef("internal_collection_id"),
+                            output_field=IntegerField(),
+                        )
+                    ).filter(
+                        (
+                            Q(created_by_workflow_id=OuterRef("id"))
+                            | Q(removed_by_workflow_id=OuterRef("id"))
+                        )
+                        & (
+                            Q(internal_collection_id__isnull=True)
+                            | ~Q(
+                                parent_collection_id=F("internal_collection_id")
+                            )
+                        )
+                    )
+                )
+            )
+        }
+
+        # Keep work requests whose parent is not in the queryset
+        to_keep: set[WorkRequest]
+        while to_delete:
+            to_keep = set()
+            for wr in to_delete:
+                if wr.parent is not None and wr.parent not in to_delete:
+                    to_keep.add(wr)
+            to_delete -= to_keep
+            if not to_keep:
+                break
+
+        # Keep work requests that have children not in the queryset
+
+        # Precache children of all nodes in to_delete
+        children = defaultdict(list)
+        for child in WorkRequest.objects.filter(
+            parent__in=to_delete
+        ).select_related("parent"):
+            children[child.parent].append(child)
+
+        while to_delete:
+            to_keep = set()
+            for wr in to_delete:
+                for child in children[wr]:
+                    if child not in to_delete:
+                        to_keep.add(wr)
+                        break
+            to_delete -= to_keep
+            if not to_keep:
+                break
+
+        self.work_requests = to_delete
+
+    def scan_collections(self) -> None:
+        """Look for collections to delete, filling self.collections."""
+        collections: set[Collection] = set()
+        for wr in self.work_requests:
+            if wr.internal_collection:
+                collections.add(wr.internal_collection)
+        self.collections = collections
+
+    def perform_deletions(self) -> None:
+        """Delete objects found in self.work_requests and self.collections."""
+        # Import here to prevent import loop
+        from debusine.db.models.collections import Collection
+
+        CollectionItem.objects.filter(
+            parent_collection__in=self.collections
+        ).delete()
+        WorkRequest.objects.filter(
+            id__in=[wr.id for wr in self.work_requests]
+        ).delete()
+        Collection.objects.filter(
+            id__in=[c.id for c in self.collections]
+        ).delete()
