@@ -11,7 +11,7 @@
 
 import re
 from collections.abc import Sequence
-from datetime import timedelta
+from datetime import datetime, timedelta
 from typing import (
     Any,
     Generic,
@@ -28,12 +28,13 @@ from django.contrib.auth.models import AnonymousUser
 from django.contrib.postgres.aggregates import ArrayAgg
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import Case, Q, QuerySet, UniqueConstraint, Value, When
+from django.db.models import F, Max, Q, QuerySet, UniqueConstraint, Value
+from django.db.models.functions import Greatest
 from django.urls import reverse
 
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import permissions
-from debusine.db.models.files import File, FileStore
+from debusine.db.models.files import File
 from debusine.db.models.permissions import (
     PartialCheckResult,
     PermissionUser,
@@ -122,6 +123,13 @@ def validate_workspace_name(value: str) -> None:
 class WorkspaceQuerySet(QuerySet["Workspace", A], Generic[A]):
     """Custom QuerySet for Workspace."""
 
+    def with_expiration_time(self) -> "WorkspaceQuerySet[Any]":
+        """Annotate the queryset with expiration times."""
+        return self.annotate(
+            expiration_time=F("expiration_delay")
+            + Greatest("created_at", Max("workrequest__completed_at"))
+        )
+
     def in_current_scope(self) -> "WorkspaceQuerySet[A]":
         """Filter to workspaces in the current scope."""
         if context.scope is None:
@@ -147,6 +155,22 @@ class WorkspaceQuerySet(QuerySet["Workspace", A], Generic[A]):
             )
         return self.filter(constraints).distinct()
 
+    @permission_filter
+    def can_configure(self, user: PermissionUser) -> "WorkspaceQuerySet[A]":
+        """Keep only Workspaces that can be configured."""
+        assert user is not None  # Enforced by decorator
+        # Workers cannot change workspace configuration
+        if context.worker_token:
+            return self.none()
+        if not user.is_authenticated:
+            return self.none()
+        constraints = WORKSPACE_ROLES(
+            user,
+            scope=[Scope.Roles.OWNER],
+            workspace=[Workspace.Roles.OWNER],
+        )
+        return self.filter(constraints).distinct()
+
     @permission_filter
     def can_create_artifacts(
         self, user: PermissionUser
@@ -191,6 +215,22 @@ class WorkspaceQuerySet(QuerySet["Workspace", A], Generic[A]):
             )
         ).distinct()
 
+    @permission_filter
+    def can_create_experiment_workspace(
+        self, user: PermissionUser
+    ) -> "WorkspaceQuerySet[A]":
+        """Workspaces from which the user can create an experiment workspace."""
+        assert user is not None  # Enforced by decorator
+        # Workers and anonymous users are not allowed
+        if context.worker_token or not user.is_authenticated:
+            return self.none()
+        constraints = Q(public=True) | WORKSPACE_ROLES(
+            user,
+            scope=[Scope.Roles.OWNER],
+            workspace=[Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR],
+        )
+        return self.filter(constraints).distinct()
+
 
 class WorkspaceManager(models.Manager["Workspace"]):
     """Manager for Workspace model."""
@@ -228,6 +268,69 @@ class WorkspaceManager(models.Manager["Workspace"]):
         return workspace
 
 
+class DeleteWorkspaces:
+    """Delete all workspaces in a queryset."""
+
+    def __init__(self, workspaces: WorkspaceQuerySet["Workspace"]) -> None:
+        """Store workspaces to delete and compute affected objects."""
+        # Import here to prevent circular dependencies
+        from debusine.db import models as dmodels
+
+        self.workspaces = workspaces
+
+        # Since we use on_delete=PROTECT on most models, there may be
+        # elements in the model interdependency graphs that we are not
+        # deleting yet.
+        #
+        # It is difficult to test this without having infrastructure to
+        # simulate a fully populated database that is kept up to date as
+        # new models get added, so for the moment we limit ourselves to
+        # adding to this as the need arises.
+        self.workflow_templates = dmodels.WorkflowTemplate.objects.filter(
+            workspace__in=workspaces
+        )
+        self.work_requests = dmodels.WorkRequest.objects.filter(
+            workspace__in=workspaces
+        )
+        self.collection_items = dmodels.CollectionItem.objects.filter(
+            parent_collection__workspace__in=workspaces
+        )
+        self.collections = dmodels.Collection.objects.filter(
+            workspace__in=workspaces
+        )
+        self.file_in_artifacts = dmodels.FileInArtifact.objects.filter(
+            artifact__workspace__in=workspaces
+        )
+        self.artifact_relations = dmodels.ArtifactRelation.objects.filter(
+            Q(artifact__workspace__in=workspaces)
+            | Q(target__workspace__in=workspaces)
+        )
+        self.artifacts = dmodels.Artifact.objects.filter(
+            workspace__in=workspaces
+        )
+        self.workspace_roles = WorkspaceRole.objects.filter(
+            resource__in=workspaces
+        )
+        self.asset_usages = dmodels.AssetUsage.objects.filter(
+            workspace__in=workspaces
+        )
+        self.assets = dmodels.Asset.objects.filter(workspace__in=workspaces)
+
+    def perform_deletions(self) -> None:
+        """Delete the workspaces and all their resources."""
+        self.workflow_templates.delete()
+        self.work_requests.delete()
+        self.collection_items.delete()
+        self.collections.delete()
+        self.file_in_artifacts.delete()
+        self.artifact_relations.delete()
+        self.artifacts.delete()
+        self.asset_usages.delete()
+        self.assets.delete()
+        self.workspace_roles.delete()
+        self.workspaces.delete()
+
+
 DEFAULT_WORKSPACE_NAME = "System"
 
 
@@ -255,12 +358,6 @@ class Workspace(models.Model):
     name = models.CharField(
         max_length=255, validators=[validate_workspace_name]
     )
-    default_file_store = models.ForeignKey(
-        FileStore, on_delete=models.PROTECT, related_name="default_workspaces"
-    )
-    other_file_stores = models.ManyToManyField(
-        FileStore, related_name="other_workspaces"
-    )
     public = models.BooleanField(default=False)
     default_expiration_delay = models.DurationField(
         default=timedelta(0),
@@ -279,6 +376,18 @@ class Workspace(models.Model):
         Scope, on_delete=models.PROTECT, related_name="workspaces"
     )
 
+    created_at = models.DateTimeField(
+        auto_now_add=True, help_text="time the workspace was created"
+    )
+    expiration_delay = models.DurationField(
+        blank=True,
+        null=True,
+        help_text=(
+            "if set, time since the last task completion time"
+            "after which the workspace can be deleted"
+        ),
+    )
+
     class Meta(TypedModelMeta):
         constraints = [
             UniqueConstraint(
@@ -291,6 +400,10 @@ class Workspace(models.Model):
         """Return an absolute URL to this workspace."""
         return reverse("workspaces:detail", kwargs={"wname": self.name})
 
+    def get_absolute_url_configure(self) -> str:
+        """Return an absolute URL to configure this workspace."""
+        return reverse("workspaces:update", kwargs={"wname": self.name})
+
     @copy_signature_from(models.Model.save)
     def save(self, **kwargs: Any) -> None:
         """Wrap save with permission checks."""
@@ -386,6 +499,31 @@ class Workspace(models.Model):
                 assert_never(unreachable)
         return Workspace.objects.can_display(user).filter(pk=self.pk).exists()
 
+    @permission_check("{user} cannot configure workspace {resource}")
+    def can_configure(self, user: PermissionUser) -> bool:
+        """Check if the workspace can be configured."""
+        assert user is not None  # enforced by decorator
+        # Workers cannot change a workspace configuration
+        if context.worker_token:
+            return False
+        if not user.is_authenticated:
+            return False
+        # Shortcut to avoid hitting the database for common cases
+        match self.context_has_role(
+            user,
+            Workspace.Roles.OWNER,
+            scope_roles=Scope.Roles.OWNER,
+        ):
+            case PartialCheckResult.ALLOW:
+                return True
+            case PartialCheckResult.DENY:
+                return False
+            case PartialCheckResult.PASS:
+                pass
+            case _ as unreachable:
+                assert_never(unreachable)
+        return Workspace.objects.can_configure(user).filter(pk=self.pk).exists()
+
     @permission_check("{user} cannot create artifacts in {resource}")
     def can_create_artifacts(self, user: PermissionUser) -> bool:
         """Check if the user can create artifacts."""
@@ -448,6 +586,40 @@ class Workspace(models.Model):
             .exists()
         )
 
+    @permission_check(
+        "{user} cannot create an experiment workspace from {resource}"
+    )
+    def can_create_experiment_workspace(self, user: PermissionUser) -> bool:
+        """Check if the user can create an experiment workspace from this."""
+        assert user is not None  # enforced by decorator
+        # Workers can not create experiment workspaces
+        if context.worker_token:
+            return False
+        # Anonymous cannot create experiment workspaces
+        if not user.is_authenticated:
+            return False
+        # Shortcut to avoid hitting the database for common cases
+        if self.public:
+            return True
+        match self.context_has_role(
+            user,
+            (Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR),
+            scope_roles=Scope.Roles.OWNER,
+        ):
+            case PartialCheckResult.ALLOW:
+                return True
+            case PartialCheckResult.DENY:
+                return False
+            case PartialCheckResult.PASS:
+                pass
+            case _ as unreachable:
+                assert_never(unreachable)
+        return (
+            Workspace.objects.can_create_experiment_workspace(user)
+            .filter(pk=self.pk)
+            .exists()
+        )
+
     def context_has_role(
         self,
         user: "User",
@@ -505,37 +677,51 @@ class Workspace(models.Model):
             )
         return cast(QuerySet["WorkspaceRole", "WorkspaceRoles"], result)
 
-    def file_stores(self, fileobj: File) -> QuerySet[FileStore]:
-        """
-        Find the file stores in this workspace that have fileobj, if any.
+    @property
+    def expire_at(self) -> datetime | None:
+        """Return computed expiration date."""
+        if self.expiration_delay is None:
+            return None
 
-        The returned query set is ordered so that the workspace's default
-        file store comes first if it has the file, followed by any of the
-        workspace's other file stores that have it.
-        """
-        return FileStore.objects.filter(
-            Q(default_workspaces=self) | Q(other_workspaces=self),
-            files=fileobj,
-        ).order_by(Case(When(default_workspaces=self, then=0), default=1).asc())
+        # Try to reuse a value precomputed by
+        # WorkspaceQuerySet.with_expiration_time()
+        # otherwise, compute it and set it
 
-    def is_file_in_workspace(self, fileobj: File) -> bool:
-        """
-        Return True if fileobj is part of this workspace.
+        if not hasattr(self, "expiration_time"):
+            query = Workspace.objects.with_expiration_time().filter(pk=self.pk)
+            # expiration_time is introduced by with_expiration_time but it's
+            # currently hard to explain to the type system
+            (expiration_time,) = query.values_list(
+                "expiration_time", flat=True
+            )  # type: ignore[misc]
+            setattr(self, "expiration_time", expiration_time)
 
-        To be considered part of a workspace, a file must be in one of its
-        stores, and must also be in an artifact which is in the workspace.
+        return cast(
+            datetime, self.expiration_time  # type: ignore[attr-defined]
+        )
+
+    def file_needs_upload(self, fileobj: File) -> bool:
+        """
+        Return True if fileobj needs to be uploaded to this workspace.
+
+        Before an artifact can be considered complete, all its files must be
+        part of the artifact's workspace.  This requires each file to be in
+        one of its stores, and also to be in some artifact in the workspace.
+        Otherwise, the file must be uploaded, even if it already exists
+        somewhere else in debusine; this prevents users obtaining
+        unauthorized access to existing file contents.
         """
         from debusine.db.models import FileInArtifact
 
-        if not self.file_stores(fileobj).exists():
-            return False
+        if not self.scope.download_file_stores(fileobj).exists():
+            return True
 
         if not FileInArtifact.objects.filter(
             artifact__workspace=self, file=fileobj, complete=True
         ).exists():
-            return False
+            return True
 
-        return True
+        return False
 
     def set_inheritance(self, chain: Sequence["Workspace"]) -> None:
         """Set the inheritance chain for this workspace."""
