@@ -10,16 +10,17 @@
 """Infrastructure to create test scenarios in the database."""
 
 import tempfile
+import textwrap
 from dataclasses import dataclass
 from datetime import timedelta
 from pathlib import Path
+from secrets import token_hex
 from typing import Any, Literal, TYPE_CHECKING, assert_never, overload
 
 from django.conf import settings
 from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
-from django.db.models import Model
-from django.test import override_settings
+from django.db.models import F, Model
 from django.utils import timezone
 
 from debusine.artifacts import LocalArtifact
@@ -28,15 +29,28 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     ArtifactData,
     BareDataCategory,
+    BaseArtifactDataModel,
     CollectionCategory,
     DebianPackageBuildLog,
 )
 from debusine.artifacts.playground import ArtifactPlayground
+from debusine.assets import (
+    AWSProviderAccountConfiguration,
+    AWSProviderAccountCredentials,
+    AWSProviderAccountData,
+    AssetCategory,
+    BaseAssetDataModel,
+    CloudProviderAccountData,
+    KeyPurpose,
+    SigningKeyData,
+)
 from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
+    Asset,
+    AssetUsage,
     Collection,
     CollectionItem,
     DEFAULT_FILE_STORE_NAME,
@@ -47,7 +61,6 @@ from debusine.db.models import (
     FileUpload,
     Group,
     Scope,
-    TaskDatabase,
     Token,
     User,
     WorkRequest,
@@ -199,7 +212,10 @@ class Playground(ArtifactPlayground):
     def get_or_create_scope(self, name: str, **kwargs: Any) -> Scope:
         """Create a scope, or return an existing scope."""
         kwargs.setdefault("label", name.capitalize())
-        return Scope.objects.get_or_create(name=name, defaults=kwargs)[0]
+        scope, created = Scope.objects.get_or_create(name=name, defaults=kwargs)
+        if created:
+            scope.file_stores.add(self.get_default_file_store())
+        return scope
 
     @context.disable_permission_checks()
     def create_user(
@@ -214,30 +230,45 @@ class Playground(ArtifactPlayground):
         )
         return user
 
+    @context.disable_permission_checks()
+    def create_group(
+        self,
+        name: str,
+        users: list[User] | None = None,
+        scope: Scope | None = None,
+        ephemeral: bool = False,
+    ) -> Group:
+        """Create a group, setting membership to users."""
+        if scope is None:
+            scope = self.get_default_scope()
+        if users is None:
+            users = []
+        group, created = Group.objects.get_or_create(
+            scope=scope, name=name, ephemeral=ephemeral
+        )
+        group.users.set(users)
+        return group
+
     @context.disable_permission_checks()
     def create_group_role(
         self,
         resource: Model,
         role: Roles | str,
-        *users: User,
+        users: list[User] | None = None,
         name: str | None = None,
     ) -> Group:
         """Create a group with the given role on a resource."""
         if name is None:
             name = f"{str(resource).replace('/', '-')}-{role}"
+        if users is None:
+            users = []
 
         role = resolve_role(getattr(resource, "Roles"), role)
         group_scope = get_resource_scope(resource)
-        group, created = Group.objects.get_or_create(
-            scope=group_scope, name=name
-        )
-        if not created:
-            roles_model = getattr(
-                resource.__class__, "objects"
-            ).get_roles_model()
-            roles_model.objects.filter(group=group).delete()
+        group = self.create_group(name, users, scope=group_scope)
+        roles_model = getattr(resource.__class__, "objects").get_roles_model()
+        roles_model.objects.filter(group=group).delete()
         group.assign_role(resource, role)
-        group.users.set(users)
         return group
 
     @context.disable_permission_checks()
@@ -267,8 +298,8 @@ class Playground(ArtifactPlayground):
 
         if worker is None:
             worker = self.create_worker()
-            worker.token = token
-            worker.save()
+        worker.token = token
+        worker.save()
 
         if enabled:
             token.enable()
@@ -298,22 +329,14 @@ class Playground(ArtifactPlayground):
 
     @context.disable_permission_checks()
     def create_workspace(
-        self,
-        *,
-        file_store: FileStore | None = None,
-        scope: Scope | None = None,
-        **kwargs: Any,
+        self, *, scope: Scope | None = None, **kwargs: Any
     ) -> Workspace:
         """Create a Workspace and return it."""
-        if file_store is None:
-            file_store = self.get_default_file_store()
         if scope is None:
             scope = self.get_default_scope()
         workspace_name = kwargs.pop("name", self.default_workspace_name)
         workspace, _ = Workspace.objects.select_related("scope").get_or_create(
-            scope=scope,
-            name=workspace_name,
-            defaults={**{"default_file_store": file_store, **kwargs}},
+            scope=scope, name=workspace_name, defaults=kwargs
         )
 
         return workspace
@@ -411,6 +434,7 @@ class Playground(ArtifactPlayground):
         result: WorkRequest.Results | None = None,
         expired: bool = False,
         workspace: Workspace | None = None,
+        task_name: str = "noop",
         task_data: BaseTaskData | dict[str, Any] | None = None,
         **kwargs: Any,
     ) -> WorkRequest:
@@ -433,7 +457,10 @@ class Playground(ArtifactPlayground):
         if workspace is None:
             workspace = self.create_workspace(public=True)
 
-        defaults: dict[str, Any] = {"workspace": workspace}
+        defaults: dict[str, Any] = {
+            "task_name": task_name,
+            "workspace": workspace,
+        }
         if "created_by" not in kwargs:
             token = self.create_user_token()
             defaults["created_by"] = token.user
@@ -493,7 +520,6 @@ class Playground(ArtifactPlayground):
             "system:cpu_cores": 4,
             "system:worker_type": worker_type,
             "sbuild:version": 1,
-            "sbuild:chroots": "bullseye-amd64",
         }
 
         if extra_dynamic_metadata:
@@ -543,17 +569,18 @@ class Playground(ArtifactPlayground):
         name: str,
         *,
         category: BareDataCategory = BareDataCategory.TEST,
-        data: dict[str, Any] | None = None,
+        data: BaseArtifactDataModel | dict[str, Any] | None = None,
         created_by_user: User | None = None,
+        created_by_workflow: WorkRequest | None = None,
     ) -> CollectionItem:
         """Create a collection item holding bare data."""
-        return CollectionItem.objects.create(
+        return CollectionItem.objects.create_from_bare_data(
             parent_collection=parent_collection,
             name=name,
-            child_type=CollectionItem.Types.BARE,
             category=category,
             data=data or {},
             created_by_user=created_by_user or self.get_default_user(),
+            created_by_workflow=created_by_workflow,
         )
 
     @context.disable_permission_checks()
@@ -576,8 +603,10 @@ class Playground(ArtifactPlayground):
 
         :param paths: list of paths to create (will contain random data)
         :param files_size: size of the test data
-        :param category: this artifact's category (see :ref:`artifacts`)
-        :param data: key-value data for this artifact (see :ref:`artifacts`)
+        :param category: this artifact's category (see
+          :ref:`artifact-reference`)
+        :param data: key-value data for this artifact (see
+          :ref:`artifact-reference`)
         :param expiration_delay: set expiration_delay field (in days)
         :param work_request: work request that created this artifact
         :param created_by: set Artifact.created_by to it
@@ -635,7 +664,9 @@ class Playground(ArtifactPlayground):
                 files_contents[path] = next(data_gen)
 
         if create_files:
-            file_backend = workspace.default_file_store.get_backend_object()
+            file_backend = workspace.scope.file_stores.order_by(
+                F("filestoreinscope__upload_priority").desc(nulls_last=True)
+            )[0].get_backend_object()
 
             for path, contents in files_contents.items():
                 if skip_add_files_in_store:
@@ -741,7 +772,6 @@ class Playground(ArtifactPlayground):
                 create_files=create_files,
             )
 
-    @context.disable_permission_checks()
     def create_minimal_binary_packages_artifact(
         self,
         srcpkg_name: str = "hello",
@@ -762,6 +792,30 @@ class Playground(ArtifactPlayground):
         )
         return artifact
 
+    def create_minimal_binary_package_artifact(
+        self,
+        srcpkg_name: str = "hello",
+        srcpkg_version: str = "1.0-1",
+        package: str = "hello",
+        version: str = "1.0-1",
+        architecture: str = "all",
+    ) -> Artifact:
+        """Create a minimal `debian:binary-package` artifact."""
+        artifact, _ = self.create_artifact(
+            category=ArtifactCategory.BINARY_PACKAGE,
+            data={
+                "srcpkg_name": srcpkg_name,
+                "srcpkg_version": srcpkg_version,
+                "deb_fields": {
+                    "Architecture": architecture,
+                    "Package": package,
+                    "Version": version,
+                },
+                "deb_control_files": [],
+            },
+        )
+        return artifact
+
     @overload
     def create_upload_artifacts(
         self,
@@ -921,6 +975,123 @@ class Playground(ArtifactPlayground):
         )
         return artifact
 
+    def create_signing_input_artifact(
+        self,
+        binary_package_name: str | None = "hello",
+        trusted_certs: list[str] | None = None,
+        workspace: Workspace | None = None,
+    ) -> Artifact:
+        """Create a minimal `debusine:signing-input` artifact."""
+        artifact, _ = self.create_artifact(
+            category=ArtifactCategory.SIGNING_INPUT,
+            data={
+                "trusted_certs": trusted_certs,
+                "binary_package_name": binary_package_name,
+            },
+            workspace=workspace,
+        )
+        return artifact
+
+    @context.disable_permission_checks()
+    def create_asset(
+        self,
+        category: AssetCategory,
+        data: BaseAssetDataModel | dict[str, Any],
+        workspace: Workspace | None = None,
+        created_by: User | None = None,
+        created_by_work_request: WorkRequest | None = None,
+    ) -> Asset:
+        """Create an Asset."""
+        if created_by is None:
+            created_by = self.get_default_user()
+        return Asset.objects.create(
+            category=category,
+            workspace=workspace,
+            data=data.dict() if isinstance(data, BaseAssetDataModel) else data,
+            created_by=created_by,
+            created_by_work_request=created_by_work_request,
+        )
+
+    def create_signing_key_asset(
+        self,
+        purpose: KeyPurpose = KeyPurpose.OPENPGP,
+        fingerprint: str | None = None,
+        public_key: str | None = None,
+        description: str | None = "Test Key",
+        workspace: Workspace | None = None,
+        created_by: User | None = None,
+        created_by_work_request: WorkRequest | None = None,
+    ) -> Asset:
+        """Create a debusine:signing-key asset."""
+        if fingerprint is None:
+            fingerprint = token_hex()
+        if public_key is None:
+            public_key = textwrap.dedent(
+                f"""\
+            -----BEGIN PGP PUBLIC KEY BLOCK-----
+
+            Comment: This isn't a valid key.
+
+            {token_hex()}
+            {token_hex()}
+            {token_hex()}
+
+            -----END PGP PUBLIC KEY BLOCK-----
+            """
+            )
+        if workspace is None:
+            workspace = self.get_default_workspace()
+        return self.create_asset(
+            category=AssetCategory.SIGNING_KEY,
+            data=SigningKeyData(
+                purpose=purpose,
+                fingerprint=fingerprint,
+                public_key=public_key,
+                description=description,
+            ),
+            workspace=workspace,
+            created_by=created_by,
+            created_by_work_request=created_by_work_request,
+        )
+
+    @context.disable_permission_checks()
+    def create_cloud_provider_account_asset(
+        self,
+        data: CloudProviderAccountData | None = None,
+        workspace: Workspace | None = None,
+        created_by: User | None = None,
+        created_by_work_request: WorkRequest | None = None,
+    ) -> Asset:
+        """Create a debusine:cloud-provider-account asset."""
+        if data is None:
+            data = AWSProviderAccountData(
+                name="test",
+                configuration=AWSProviderAccountConfiguration(
+                    region_name="test-region"
+                ),
+                credentials=AWSProviderAccountCredentials(
+                    access_key_id="access-key", secret_access_key="secret-key"
+                ),
+            )
+        return self.create_asset(
+            category=AssetCategory.CLOUD_PROVIDER_ACCOUNT,
+            data=data,
+            workspace=workspace,
+            created_by=created_by,
+            created_by_work_request=created_by_work_request,
+        )
+
+    @context.disable_permission_checks()
+    def create_asset_usage(
+        self,
+        resource: Asset,
+        workspace: Workspace | None = None,
+    ) -> AssetUsage:
+        """Create an AssetUsage model."""
+        if workspace is None:
+            workspace = self.get_default_workspace()
+        return AssetUsage.objects.create(asset=resource, workspace=workspace)
+
     @context.disable_permission_checks()
     def create_debian_environments_collection(
         self, name: str = "debian", *, workspace: Workspace | None = None
@@ -1056,28 +1227,16 @@ class Playground(ArtifactPlayground):
                 ),
             )
         else:
+            assert task_data.host_architecture != "all", (
+                "For an Sbuild task (not workflow): host_architecture must "
+                "be a real architecture (e.g. amd64)"
+            )
             return self.create_work_request(
                 task_name="sbuild",
                 task_data=task_data,
                 workspace=workspace or self.get_default_workspace(),
             )
 
-    @context.disable_permission_checks()
-    def compute_dynamic_data(
-        self, work_request: WorkRequest, worker: Worker | None = None
-    ) -> None:
-        """Compute dynamic data for the work request, and save it."""
-        task = work_request.get_task(worker=worker)
-        with override_settings(DEBUSINE_FQDN="debusine-playground.net"):
-            data = task.compute_dynamic_data(TaskDatabase(work_request))
-        if data:
-            work_request.dynamic_task_data = model_to_json_serializable_dict(
-                data, exclude_unset=True
-            )
-        else:
-            work_request.dynamic_task_data = None
-        work_request.save()
-
     @context.disable_permission_checks()
     def simulate_package_build(
         self,
@@ -1111,9 +1270,6 @@ class Playground(ArtifactPlayground):
         if worker is None:
             worker = self.create_worker()
 
-        # Resolve dynamic data
-        self.compute_dynamic_data(work_request, worker)
-
         # Assign a worker
         work_request.assign_worker(worker)
         work_request.mark_running()
