@@ -165,6 +165,9 @@ class UIPlayground(DefaultContext):
         self.user.set_password("playground")
         self.user.save()
 
+        asset = self.playground.create_signing_key_asset()
+        self.playground.create_asset_usage(asset)
+
         # Pretend the user comes from Salsa
         Identity.objects.create(
             user=self.user,
@@ -175,7 +178,7 @@ class UIPlayground(DefaultContext):
 
         # Make them owners of the playground workspace
         self.playground.create_group_role(
-            self.workspace, Workspace.Roles.OWNER, self.user
+            self.workspace, Workspace.Roles.OWNER, users=[self.user]
         )
 
         # Create a Debian scope, to test multi-scope UI elements
@@ -239,11 +242,14 @@ class UIPlayground(DefaultContext):
         self.add_results_to_suite(wr)
 
         # Create a variety of work requests
-        self.playground.create_work_request(
+        wr = self.playground.create_work_request(
             status=WorkRequest.Statuses.COMPLETED,
             result=WorkRequest.Results.SUCCESS,
             task_name="noop",
         )
+        wr.task_data = {"result": False}
+        wr.configured_task_data = {"result": True}
+        wr.save()
         self.playground.create_work_request(
             status=WorkRequest.Statuses.COMPLETED,
             result=WorkRequest.Results.FAILURE,
@@ -308,7 +314,7 @@ class UIPlayground(DefaultContext):
                 "input": {
                     "source_artifact": source.pk,
                 },
-                "backend": "schroot",
+                "backend": "unshare",
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64", "s390x"],
             },
