@@ -10,17 +10,21 @@
 """Tests for the database playground."""
 
 import django.test
+from django.conf import settings
 from django.contrib.auth.hashers import check_password
 from django.utils import timezone
 
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
+    DebianBinaryPackage,
     DebianBinaryPackages,
     DebianPackageBuildLog,
     DebianSourcePackage,
     DebianUpload,
+    DebusineSigningInput,
 )
+from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -76,7 +80,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(file_store.backend, FileStore.BackendChoices.MEMORY)
 
         workspace = playground.get_default_workspace()
-        self.assertEqual(workspace.default_file_store, file_store)
+        self.assertEqual(workspace.scope.file_stores.get(), file_store)
         self.assertEqual(workspace.name, DEFAULT_WORKSPACE_NAME)
 
     def test_user_password(self) -> None:
@@ -111,30 +115,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(playground.create_workspace(scope=testscope), ws2)
         self.assertNotEqual(ws1, ws2)
 
-    @context.disable_permission_checks()
-    def test_create_workspace_file_store(self) -> None:
-        """Test setting file store on created workspaces."""
-        playground = Playground()
-        ws1 = playground.create_workspace()
-        self.assertEqual(
-            ws1.default_file_store, playground.get_default_file_store()
-        )
-
-        custom_store = FileStore.objects.create(
-            name="external",
-            backend=FileStore.BackendChoices.EXTERNAL_DEBIAN_SUITE,
-            configuration={
-                "archive_root_url": "https://deb.debian.org/debian",
-                "suite": "bookworm",
-                "components": ["main"],
-            },
-        )
-
-        ws2 = playground.create_workspace(
-            name="custom", file_store=custom_store
-        )
-        self.assertEqual(ws2.default_file_store, custom_store)
-
     def test_create_file_in_backend(self) -> None:
         """create_file_in_backend() returns a File and writes contents to it."""
         playground = self.enterContext(Playground(memory_file_store=False))
@@ -148,40 +128,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         assert local_path is not None
         self.assertEqual(local_path.read_bytes(), contents)
 
-    def test_compute_dynamic_data_noop(self) -> None:
-        """Test compute_dynamic_data with empty result."""
-        playground = Playground()
-        wr = playground.create_work_request(
-            task_type=TaskTypes.WORKER, task_name="noop"
-        )
-        playground.compute_dynamic_data(wr)
-        self.assertIsNone(wr.dynamic_task_data)
-
-    def test_compute_dynamic_data_sbuild(self) -> None:
-        """Test compute_dynamic_data with nonempty result."""
-        playground = Playground()
-        with context.disable_permission_checks():
-            source = playground.create_source_artifact()
-            environment_item = playground.create_debian_environment()
-        assert environment_item.artifact is not None
-        environment = environment_item.artifact
-        wr = playground.create_sbuild_work_request(
-            source=source,
-            environment=environment,
-        )
-        playground.compute_dynamic_data(wr)
-        self.assertEqual(
-            wr.dynamic_task_data,
-            {
-                "environment_id": environment.pk,
-                "input_extra_binary_artifacts_ids": [],
-                "input_source_artifact_id": source.pk,
-                "binnmu_maintainer": (
-                    "Debusine <noreply@debusine-playground.net>"
-                ),
-            },
-        )
-
     def test_create_source_artifact(self) -> None:
         """Test creating a source artifact."""
         playground = Playground()
@@ -199,14 +145,13 @@ class PlaygroundTest(django.test.TestCase, TestCase):
     def test_create_minimal_binary_packages_artifact(self) -> None:
         """Test creating a binary_packages artifact."""
         playground = Playground()
-        with context.disable_permission_checks():
-            bp = playground.create_minimal_binary_packages_artifact(
-                "hello", "1.0-1", "1.0-1", "amd64"
-            )
+        bp = playground.create_minimal_binary_packages_artifact(
+            "hello", "1.0-1", "1.0-1", "amd64"
+        )
         self.assertEqual(bp.category, ArtifactCategory.BINARY_PACKAGES)
         self.assertEqual(bp.workspace, playground.get_default_workspace())
         self.assertEqual(bp.files.count(), 0)
-        self.assertEqual(bp.created_by, None)
+        self.assertIsNone(bp.created_by)
         self.assertIsNone(bp.created_by_work_request)
 
         artifact = DebianBinaryPackages(**bp.data)
@@ -216,6 +161,31 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(artifact.architecture, "amd64")
         self.assertEqual(artifact.packages, [])
 
+    def test_create_minimal_binary_package_artifact(self) -> None:
+        """Test creating a binary_package artifact."""
+        playground = Playground()
+        bp = playground.create_minimal_binary_package_artifact(
+            "hello", "1.0-1", "hello", "1.0-1", "amd64"
+        )
+        self.assertEqual(bp.category, ArtifactCategory.BINARY_PACKAGE)
+        self.assertEqual(bp.workspace, playground.get_default_workspace())
+        self.assertEqual(bp.files.count(), 0)
+        self.assertIsNone(bp.created_by)
+        self.assertIsNone(bp.created_by_work_request)
+
+        artifact = DebianBinaryPackage(**bp.data)
+        self.assertEqual(artifact.srcpkg_name, "hello")
+        self.assertEqual(artifact.srcpkg_version, "1.0-1")
+        self.assertEqual(artifact.deb_control_files, [])
+        self.assertEqual(
+            artifact.deb_fields,
+            {
+                "Architecture": "amd64",
+                "Package": "hello",
+                "Version": "1.0-1",
+            },
+        )
+
     def test_create_source_artifact_with_files(self) -> None:
         """Test creating a source artifact with its files."""
         playground = Playground()
@@ -359,8 +329,8 @@ class PlaygroundTest(django.test.TestCase, TestCase):
 
         file = buildlog.files.first()
         assert file is not None
-        backend = buildlog.workspace.default_file_store.get_backend_object()
-        with backend.get_stream(file) as fd:
+        file_backend = buildlog.workspace.scope.download_file_backend(file)
+        with file_backend.get_stream(file) as fd:
             self.assertEqual(
                 fd.read().splitlines(keepends=True)[3].decode(),
                 "Line 4 of hello_1.0-1_amd64.buildlog\n",
@@ -395,8 +365,8 @@ class PlaygroundTest(django.test.TestCase, TestCase):
 
         file = buildlog.files.first()
         assert file is not None
-        backend = buildlog.workspace.default_file_store.get_backend_object()
-        with backend.get_stream(file) as fd:
+        file_backend = buildlog.workspace.scope.download_file_backend(file)
+        with file_backend.get_stream(file) as fd:
             self.assertEqual(fd.read(), test_contents)
 
     @context.disable_permission_checks()
@@ -414,6 +384,82 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(buildlog.created_by, user)
         self.assertEqual(buildlog.created_by_work_request, work_request)
 
+    def test_create_signing_input_artifact(self) -> None:
+        playground = Playground()
+        bp = playground.create_signing_input_artifact("hello")
+        self.assertEqual(bp.category, ArtifactCategory.SIGNING_INPUT)
+        self.assertEqual(bp.workspace, playground.get_default_workspace())
+        self.assertEqual(bp.files.count(), 0)
+        self.assertIsNone(bp.created_by)
+        self.assertIsNone(bp.created_by_work_request)
+
+        artifact = DebusineSigningInput(**bp.data)
+        self.assertEqual(artifact.binary_package_name, "hello")
+        self.assertIsNone(artifact.trusted_certs)
+
+    def test_create_signing_key_asset_defaults(self) -> None:
+        """Test create_signing_key_asset with defaults."""
+        playground = Playground()
+        asset = playground.create_signing_key_asset()
+        self.assertEqual(asset.category, AssetCategory.SIGNING_KEY)
+        self.assertEqual(asset.workspace, playground.get_default_workspace())
+        self.assertEqual(asset.created_by, playground.get_default_user())
+        self.assertIsNone(asset.created_by_work_request)
+        data_model = asset.data_model
+        assert isinstance(data_model, SigningKeyData)
+        self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP)
+        self.assertTrue(len(data_model.fingerprint) > 8)
+        self.assertTrue(
+            data_model.public_key.startswith(
+                "-----BEGIN PGP PUBLIC KEY BLOCK-----"
+            )
+        )
+        self.assertEqual(data_model.description, "Test Key")
+
+    def test_create_signing_key_asset_custom(self) -> None:
+        """Test create_signing_key_asset with specified features."""
+        playground = Playground()
+        workspace = playground.create_workspace()
+        user = playground.create_user(username="test")
+        asset = playground.create_signing_key_asset(
+            purpose=KeyPurpose.UEFI,
+            fingerprint="ABCDEF",
+            public_key="PUBLIC KEY",
+            description="Custom Asset",
+            workspace=workspace,
+            created_by=user,
+        )
+        self.assertEqual(asset.workspace, workspace)
+        self.assertEqual(asset.created_by, user)
+        self.assertIsNone(asset.created_by_work_request)
+        data_model = asset.data_model
+        assert isinstance(data_model, SigningKeyData)
+        self.assertEqual(data_model.purpose, KeyPurpose.UEFI)
+        self.assertEqual(data_model.fingerprint, "ABCDEF")
+        self.assertEqual(data_model.public_key, "PUBLIC KEY")
+        self.assertEqual(data_model.description, "Custom Asset")
+
+    def test_create_asset_usage_defaults(self) -> None:
+        """Test create_asset_usage with default workspace."""
+        playground = Playground()
+        asset = playground.create_signing_key_asset()
+        asset_usage = playground.create_asset_usage(resource=asset)
+        self.assertEqual(asset_usage.asset, asset)
+        self.assertEqual(
+            asset_usage.workspace, playground.get_default_workspace()
+        )
+
+    def test_create_asset_usage_custom(self) -> None:
+        """Test create_asset_usage with specified workspace."""
+        playground = Playground()
+        asset = playground.create_signing_key_asset()
+        workspace = playground.create_workspace()
+        asset_usage = playground.create_asset_usage(
+            resource=asset, workspace=workspace
+        )
+        self.assertEqual(asset_usage.asset, asset)
+        self.assertEqual(asset_usage.workspace, workspace)
+
     def test_create_debian_env_collection_defaults(self) -> None:
         """Test create_debian_environments_collection."""
         playground = Playground()
@@ -518,7 +564,6 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 'mirror': 'https://deb.debian.org',
                 'pkglist': [],
                 "variant": "apt",
-                'variant': 'apt',
                 'vendor': 'Debian',
                 "with_dev": True,
                 'with_init': True,
@@ -588,7 +633,9 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             environment_item = playground.create_debian_environment()
             assert environment_item.artifact is not None
             wr = playground.create_sbuild_work_request(
-                source=source, environment=environment_item.artifact
+                source=source,
+                environment=environment_item.artifact,
+                architecture="amd64",
             )
 
         self.assertEqual(wr.workspace, playground.get_default_workspace())
@@ -602,9 +649,9 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             wr.task_data,
             {
                 'backend': BackendType.UNSHARE,
-                'build_components': [SbuildBuildComponent.ALL],
+                'build_components': [SbuildBuildComponent.ANY],
                 'environment': environment_item.artifact.pk,
-                'host_architecture': 'all',
+                'host_architecture': 'amd64',
                 'input': {'source_artifact': source.pk},
             },
         )
@@ -616,7 +663,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         playground = Playground()
         with context.disable_permission_checks():
             source = playground.create_source_artifact()
-            wr = playground.simulate_package_build(source)
+            wr = playground.simulate_package_build(source, architecture="amd64")
 
         binaries: list[Artifact] = []
         buildlogs: list[Artifact] = []
@@ -651,9 +698,9 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             wr.task_data,
             {
                 'backend': BackendType.UNSHARE,
-                'build_components': [SbuildBuildComponent.ALL],
+                'build_components': [SbuildBuildComponent.ANY],
                 'environment': environment.pk,
-                'host_architecture': 'all',
+                'host_architecture': 'amd64',
                 'input': {'source_artifact': source.pk},
             },
         )
@@ -662,7 +709,12 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
-                binnmu_maintainer="Debusine <noreply@debusine-playground.net>",
+                binnmu_maintainer=(
+                    f"Debusine <noreply@{settings.DEBUSINE_FQDN}>"
+                ),
+                subject="hello",
+                runtime_context="any:amd64:amd64",
+                configuration_context="bookworm",
             ),
         )
         self.assertIsNone(wr.parent)
@@ -697,7 +749,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             {
                 'deb_control_files': ['control'],
                 'deb_fields': {
-                    'Architecture': 'all',
+                    'Architecture': 'amd64',
                     'Description': 'Example description',
                     'Maintainer': 'Example Maintainer <example@example.org>',
                     'Package': 'hello',
@@ -742,6 +794,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             source,
             environment=environment,
             worker=worker,
+            architecture="amd64",
         )
 
         binaries: list[Artifact] = []
@@ -780,9 +833,9 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             wr.task_data,
             {
                 'backend': BackendType.UNSHARE,
-                'build_components': [SbuildBuildComponent.ALL],
+                'build_components': [SbuildBuildComponent.ANY],
                 'environment': environment.pk,
-                'host_architecture': 'all',
+                'host_architecture': 'amd64',
                 'input': {'source_artifact': source.pk},
             },
         )
@@ -791,7 +844,12 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
-                binnmu_maintainer="Debusine <noreply@debusine-playground.net>",
+                binnmu_maintainer=(
+                    f"Debusine <noreply@{settings.DEBUSINE_FQDN}>"
+                ),
+                subject="test",
+                runtime_context="any:amd64:amd64",
+                configuration_context="bookworm",
             ),
         )
         self.assertIsNone(wr.parent)
@@ -820,7 +878,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             {
                 'deb_control_files': ['control'],
                 'deb_fields': {
-                    'Architecture': 'all',
+                    'Architecture': 'amd64',
                     'Description': 'Example description',
                     'Maintainer': 'Example Maintainer <example@example.org>',
                     'Package': 'test',
@@ -954,6 +1012,22 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertIsNone(token.user)
         self.assertIsNotNone(token.worker)
 
+    def test_create_group(self) -> None:
+        playground = Playground()
+        scope = playground.get_default_scope()
+        user1 = playground.create_user("user1")
+        group = playground.create_group("testgroup", users=[user1], scope=scope)
+        self.assertEqual(group.name, "testgroup")
+        self.assertQuerySetEqual(group.users.all(), [user1])
+        self.assertEqual(group.scope, scope)
+
+    def test_create_group_defaults(self) -> None:
+        playground = Playground()
+        group = playground.create_group("testgroup")
+        self.assertEqual(group.name, "testgroup")
+        self.assertQuerySetEqual(group.users.all(), [])
+        self.assertEqual(group.scope, playground.get_default_scope())
+
     def test_create_group_role(self) -> None:
         """Test create_group_role."""
         playground = Playground()
@@ -983,7 +1057,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         user1 = playground.get_default_user()
         user2 = playground.create_user("user2")
         group = playground.create_group_role(
-            scope, Scope.Roles.OWNER, user1, user2
+            scope, Scope.Roles.OWNER, users=[user1, user2]
         )
         self.assertQuerySetEqual(
             group.users.all(), [user1, user2], ordered=False
@@ -998,12 +1072,16 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         user1 = playground.get_default_user()
         user2 = playground.create_user("user2")
 
-        group = playground.create_group_role(scope, Scope.Roles.OWNER, user1)
+        group = playground.create_group_role(
+            scope, Scope.Roles.OWNER, users=[user1]
+        )
         assignment = ScopeRole.objects.get(resource=scope, group=group)
         self.assertEqual(assignment.role, Scope.Roles.OWNER)
         self.assertQuerySetEqual(group.users.all(), [user1])
 
-        group2 = playground.create_group_role(scope, Scope.Roles.OWNER, user2)
+        group2 = playground.create_group_role(
+            scope, Scope.Roles.OWNER, users=[user2]
+        )
         self.assertEqual(group.pk, group2.pk)
         self.assertQuerySetEqual(group2.users.all(), [user2])
 
