@@ -5,7 +5,12 @@ When running in production, selected.py should point to this file.
 """
 
 # PostgreSQL should be used in production
-from debusine.project.settings.db_postgresql import DATABASES  # noqa: F401
+try:
+    from debusine.project.settings.db_postgresql import DATABASES  # noqa: F401
+except (ModuleNotFoundError, PermissionError):
+    from debusine.project.settings.db_postgresql_default import (  # noqa: F401
+        DATABASES,
+    )
 
 # Use paths from the package
 from debusine.project.settings.pkg_paths import *  # noqa: F401, F403, I202
