@@ -32,11 +32,10 @@ from typing import Any
 
 from django.conf import settings
 from django.contrib import admin
-from django.contrib.auth import views as auth_views
 from django.urls import URLPattern, URLResolver, include, path, re_path
 from django.views.generic import RedirectView
 
-from debusine.web.views.auth import LogoutView
+from debusine.web.views.auth import LoginView, LogoutView
 from debusine.web.views.task_status import TaskStatusView
 
 # from django.conf import settings
@@ -55,22 +54,8 @@ service_urlpatterns: list[URLPattern | URLResolver] = [
         "user/",
         include("debusine.web.urls.user", namespace="user"),
     ),
-    path(
-        "login/",
-        auth_views.LoginView.as_view(
-            template_name="account/login.html",
-            extra_context={"is_login_view": True},
-        ),
-        name="login",
-    ),
-    path(
-        "logout/",
-        LogoutView.as_view(
-            template_name="account/logged_out.html",
-            extra_context={"is_logout_view": True},
-        ),
-        name="logout",
-    ),
+    path("login/", LoginView.as_view(), name="login"),
+    path("logout/", LogoutView.as_view(), name="logout"),
     path(
         "signon/",
         include("debusine.web.urls.signon", namespace="signon"),
