@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,6 +15,7 @@ from contextlib import ExitStack
 from celery import shared_task
 from django.db import transaction
 
+from debusine.db.context import context
 from debusine.db.models import WorkRequest
 from debusine.server.tasks import BaseServerTask
 from debusine.tasks import TaskConfigError
@@ -53,6 +54,9 @@ def run_server_task(work_request_id: int) -> bool:
     task_name = work_request.task_name
     work_request.mark_running()
 
+    context.reset()
+    work_request.set_current()
+
     if work_request.task_type != TaskTypes.SERVER:
         logger.error("Task: %s cannot run on a Celery worker", task_name)
         work_request.mark_completed(WorkRequest.Results.ERROR)
