@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -31,9 +31,6 @@ from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.collections.debian_suite_lintian import (
     DebianSuiteLintianManager,
 )
-from debusine.server.collections.debian_suite_signing_keys import (
-    DebianSuiteSigningKeysManager,
-)
 from debusine.server.collections.workflow_internal import (
     WorkflowInternalManager,
 )
@@ -44,7 +41,6 @@ __all__ = [
     "DebianPackageBuildLogsManager",
     "DebianSuiteLintianManager",
     "DebianSuiteManager",
-    "DebianSuiteSigningKeysManager",
     "ItemAdditionError",
     "ItemRemovalError",
     "WorkflowInternalManager",
