@@ -13,7 +13,9 @@ from typing import Any
 
 from django.contrib.auth.models import AnonymousUser
 from django.db import IntegrityError
-from django.db.models import Q
+from django.db.models import IntegerField, Q
+from django.db.models.functions import Cast
+from django.db.models.lookups import In
 from django.utils import timezone
 
 from debusine.artifacts.models import (
@@ -224,8 +226,16 @@ class DebianPackageBuildLogsManager(CollectionManagerInterface):
             work_request_ids = {
                 item.artifact.created_by_work_request_id
                 for item in items
-                if item.artifact is not None
+                if (
+                    item.artifact is not None
+                    and item.artifact.created_by_work_request_id is not None
+                )
             }
-            return Q(data__work_request_id__in=work_request_ids)
+            return Q(
+                In(
+                    Cast("data__work_request_id", IntegerField()),
+                    work_request_ids,
+                )
+            )
         else:
             raise LookupError(f'Unexpected lookup filter format: "{key}"')
