@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -81,7 +81,7 @@ class DebianSuiteManagerTests(TestCase):
         self, name: str, version: str, paths: list[str]
     ) -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": name,
@@ -105,7 +105,7 @@ class DebianSuiteManagerTests(TestCase):
         paths: list[str],
     ) -> Artifact:
         """Create a minimal `debian:binary-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE,
             data={
                 "srcpkg_name": srcpkg_name,
@@ -214,7 +214,7 @@ class DebianSuiteManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_add_source_package_wrong_category(self) -> None:
         """`add_source_package` requires `debian:source-package` artifacts."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE
         )
 
@@ -339,7 +339,7 @@ class DebianSuiteManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_add_binary_package_wrong_category(self) -> None:
         """`add_binary_package` requires `debian:binary-package` artifacts."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE
         )
 
@@ -510,7 +510,7 @@ class DebianSuiteManagerTests(TestCase):
                 "hello_1.0.orig.tar.xz",
             ):
                 contents = f"data for {path}".encode()
-                file = self.create_file(contents)
+                file = self.playground.create_file(contents)
                 FileInArtifact.objects.create(
                     artifact=artifact, path=path, file=file
                 )
@@ -569,7 +569,7 @@ class DebianSuiteManagerTests(TestCase):
                 "hello_1.0.orig.tar.xz",
             ):
                 contents = f"data for {path} at {version}".encode()
-                file = self.create_file(contents)
+                file = self.playground.create_file(contents)
                 FileInArtifact.objects.create(
                     artifact=artifact, path=path, file=file
                 )
