@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -72,7 +72,7 @@ class DebianSuiteLintianManagerTests(TestCase):
 
     def create_source_package(self, name: str, version: str) -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": name,
@@ -87,7 +87,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         self, srcpkg_name: str, srcpkg_version: str
     ) -> Artifact:
         """Create a minimal `debian:binary-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE,
             data={
                 "srcpkg_name": srcpkg_name,
@@ -106,7 +106,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         architecture: str,
     ) -> Artifact:
         """Create a minimal `debian:binary-packages` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGES,
             data={
                 "srcpkg_name": srcpkg_name,
@@ -122,7 +122,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         self, architecture: str, filenames: list[str]
     ) -> Artifact:
         """Create a minimal `debian:upload` artifact with binaries."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.UPLOAD,
             data={
                 "type": "dpkg",
@@ -156,10 +156,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
 
         self.assertEqual(
             self.manager.get_related_source_package(lintian_artifact),
@@ -171,7 +173,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         """Analysis relates to a source package via `debian:binary-package`."""
         source_package_artifact = self.create_source_package("hello", "1.0")
         binary_package_artifact = self.create_binary_package("hello", "1.0")
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             binary_package_artifact,
             source_package_artifact,
             ArtifactRelation.Relations.BUILT_USING,
@@ -180,10 +182,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0_amd64.deb"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, binary_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, binary_package_artifact
+        )
 
         self.assertEqual(
             self.manager.get_related_source_package(lintian_artifact),
@@ -197,7 +201,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         binary_packages_artifact = self.create_binary_packages(
             "hello", "1.0", "1.0", "amd64"
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             binary_packages_artifact,
             source_package_artifact,
             ArtifactRelation.Relations.BUILT_USING,
@@ -206,10 +210,10 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0_amd64.deb"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             lintian_artifact, binary_packages_artifact
         )
 
@@ -223,7 +227,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         """Analysis relates to a source package via `debian:upload`."""
         source_package_artifact = self.create_source_package("hello", "1.0")
         binary_package_artifact = self.create_binary_package("hello", "1.0")
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             binary_package_artifact,
             source_package_artifact,
             ArtifactRelation.Relations.BUILT_USING,
@@ -231,7 +235,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         upload_artifact = self.create_binary_upload(
             "amd64", ["hello_1.0_amd64.deb"]
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             upload_artifact,
             binary_package_artifact,
             ArtifactRelation.Relations.EXTENDS,
@@ -240,10 +244,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0_amd64.deb"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, upload_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, upload_artifact
+        )
 
         self.assertEqual(
             self.manager.get_related_source_package(lintian_artifact),
@@ -318,10 +324,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
 
         collection_item = self.manager.add_artifact(
             lintian_artifact, user=self.user
@@ -343,16 +351,16 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact_1, _ = self.create_artifact(
+        lintian_artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             lintian_artifact_1, source_package_artifact
         )
-        lintian_artifact_2, _ = self.create_artifact(
+        lintian_artifact_2, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             lintian_artifact_2, source_package_artifact
         )
 
@@ -370,7 +378,7 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
 
@@ -385,7 +393,7 @@ class DebianSuiteLintianManagerTests(TestCase):
         source_package_artifact_1 = self.create_source_package("hello", "1.0")
         source_package_artifact_2 = self.create_source_package("hello", "1.1")
         binary_package_artifact = self.create_binary_package("hello", "1.1")
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             binary_package_artifact,
             source_package_artifact_2,
             ArtifactRelation.Relations.BUILT_USING,
@@ -394,13 +402,15 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             lintian_artifact, source_package_artifact_1
         )
-        self.create_artifact_relation(lintian_artifact, binary_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, binary_package_artifact
+        )
 
         with self.assertRaisesRegex(
             ItemAdditionError, "has multiple related source packages"
@@ -427,10 +437,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
 
         collection_item = self.manager.add_artifact(
             lintian_artifact,
@@ -450,17 +462,19 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
         collection_item = self.manager.add_artifact(
             lintian_artifact, user=self.user
         )
-        lintian_artifact2, _ = self.create_artifact(
+        lintian_artifact2, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             lintian_artifact2, source_package_artifact
         )
 
@@ -484,10 +498,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
 
         collection_item = self.manager.add_artifact(
             lintian_artifact, user=self.user, replace=True
@@ -504,10 +520,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {"hello": "hello_1.0.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
 
         collection_item = self.manager.add_artifact(
             lintian_artifact, user=self.user
@@ -589,7 +607,7 @@ class DebianSuiteLintianManagerTests(TestCase):
                 binary_package_artifact = self.create_binary_package(
                     name, version
                 )
-                self.create_artifact_relation(
+                self.playground.create_artifact_relation(
                     binary_package_artifact,
                     source_package_artifact,
                     ArtifactRelation.Relations.BUILT_USING,
@@ -607,10 +625,12 @@ class DebianSuiteLintianManagerTests(TestCase):
                     )
                 },
             }
-            lintian_artifact, _ = self.create_artifact(
+            lintian_artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.LINTIAN, data={"summary": summary}
             )
-            self.create_artifact_relation(lintian_artifact, lintian_target)
+            self.playground.create_artifact_relation(
+                lintian_artifact, lintian_target
+            )
             items.append(
                 self.manager.add_artifact(lintian_artifact, user=self.user)
             )
@@ -620,10 +640,12 @@ class DebianSuiteLintianManagerTests(TestCase):
             **self.basic_summary,
             "package_filename": {name: f"{name}_{version}.dsc"},
         }
-        lintian_artifact, _ = self.create_artifact(
+        lintian_artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.LINTIAN, data={"summary": summary}
         )
-        self.create_artifact_relation(lintian_artifact, source_package_artifact)
+        self.playground.create_artifact_relation(
+            lintian_artifact, source_package_artifact
+        )
         items.append(
             self.manager.add_artifact(lintian_artifact, user=self.user)
         )
