@@ -20,6 +20,7 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
     CollectionCategory,
+    DebusinePromise,
 )
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
@@ -58,10 +59,9 @@ class LookupMixin(TestCase):
         """Set up common data for tests."""
         super().setUpTestData()
         cls.user = cls.playground.get_default_user()
-        cls.owners = Group.objects.create(
-            scope=cls.playground.get_default_scope(), name="Owners"
+        cls.owners = cls.playground.create_group(
+            name="Owners", users=[cls.user]
         )
-        cls.owners.users.add(cls.user)
 
     def create_artifact_item(
         self,
@@ -736,7 +736,14 @@ class LookupMultipleTests(LookupMixin, TestCase):
             bookworm, "bare1", category=BareDataCategory.TEST
         )
         bare2 = self.playground.create_bare_data_item(
-            bookworm, "bare2", category=BareDataCategory.PROMISE
+            bookworm,
+            "bare2",
+            category=BareDataCategory.PROMISE,
+            data=DebusinePromise(
+                promise_work_request_id=2,
+                promise_workflow_id=1,
+                promise_category=ArtifactCategory.TEST,
+            ),
         )
         self.create_collection_item(bookworm, "collection")
         items = []
