@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -127,7 +127,7 @@ class WorkflowInternalManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_do_add_artifact_raise_item_addition_error(self) -> None:
         """Test do_add_artifact raise error on Integrity issue."""
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
         with mock.patch(
@@ -144,13 +144,13 @@ class WorkflowInternalManagerTests(TestCase):
         """`do_add_artifact` can replace an existing artifact."""
         workflow = self.playground.create_workflow()
 
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
         collection_item = self.manager.add_artifact(
             artifact_1, user=self.user, name="test"
         )
-        artifact_2, _ = self.create_artifact(
+        artifact_2, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
 
@@ -179,7 +179,7 @@ class WorkflowInternalManagerTests(TestCase):
 
         The artifact's variables cannot contain keys starting with promise\\_.
         """
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
 
@@ -200,7 +200,7 @@ class WorkflowInternalManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_do_add_artifact_with_variables_copied(self) -> None:
         """Test adding artifact copies the variables into CollectionItem."""
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
 
@@ -219,7 +219,7 @@ class WorkflowInternalManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_do_add_artifact_replace_nonexistent(self) -> None:
         """Replacing a nonexistent artifact is allowed."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
 
@@ -233,7 +233,7 @@ class WorkflowInternalManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_do_add_artifact_replace_bare_data(self) -> None:
         """Replacing bare data with an artifact is allowed."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST, data={}
         )
         self.manager.add_bare_data(
