@@ -15,6 +15,7 @@ from typing import ClassVar
 import responses
 
 from debusine.db.models import FileStore
+from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.file_backend.external import ExternalDebianSuiteFileBackend
 from debusine.test.django import TestCase
 from debusine.utils import NotSupportedError
@@ -43,6 +44,20 @@ class ExternalDebianSuiteFileBackendTests(TestCase):
 
         cls.file_backend = ExternalDebianSuiteFileBackend(cls.db_store)
 
+    def test_entry_str(self) -> None:
+        """ExternalFileBackendEntry.__str__ returns the URL."""
+        file = self.file_backend.add_remote_file(
+            "pool/main/c/coreutils/coreutils_9.1-1_amd64.deb",
+            _calculate_hash_from_data(b"test"),
+            len(b"test"),
+        )
+        entry = self.file_backend.get_entry(file)
+        expected_url = (
+            "http://ftp.debian.org/debian/"
+            "pool/main/c/coreutils/coreutils_9.1-1_amd64.deb"
+        )
+        self.assertEqual(str(entry), expected_url)
+
     @responses.activate
     def test_remote_file(self) -> None:  # pragma: no cover
         """Create a remote file."""
