@@ -10,7 +10,7 @@
 """Tests for LocalFileBackend."""
 
 import logging
-from pathlib import Path
+from pathlib import Path, PurePath
 from typing import ClassVar
 from unittest import mock
 
@@ -24,8 +24,12 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.db.tests.utils import _calculate_hash_from_data
-from debusine.server.file_backend.local import LocalFileBackend
+from debusine.server.file_backend.local import (
+    LocalFileBackend,
+    LocalFileBackendEntry,
+)
 from debusine.test.django import TestCase
+from debusine.test.utils import data_generator
 
 
 class LocalFileBackendTests(TestCase):
@@ -45,8 +49,9 @@ class LocalFileBackendTests(TestCase):
 
     def test_init_set_base_directory_default(self) -> None:
         """Test LocalFileBackend use setting's directory for Default store."""
-        Collection.objects.filter(name="_").delete()
+        Collection.objects.filter(name__in=("_", "default")).delete()
         Workspace.objects.all().delete()
+        self.default_file_store.scopes.clear()
         self.default_file_store.delete()
 
         file_store = FileStore.objects.create(
@@ -107,6 +112,7 @@ class LocalFileBackendTests(TestCase):
         # the local_store_directory is cleaned up
 
         fileobj_path = file_backend.get_local_path(fileobj)
+        assert fileobj_path is not None
 
         self.assertTrue(
             str(fileobj_path.parent).startswith(
@@ -114,6 +120,22 @@ class LocalFileBackendTests(TestCase):
             )
         )
 
+    def test_entry_str(self) -> None:
+        """LocalFileBackendEntry.__str__ returns the local path."""
+        file = self.playground.create_file_in_backend(self.file_backend)
+        hash_hex = file.sha256.hex()
+        entry = self.file_backend.get_entry(file)
+        self.assertEqual(
+            str(entry),
+            str(
+                Path(self.default_file_store.configuration["base_directory"])
+                / hash_hex[0:2]
+                / hash_hex[2:4]
+                / hash_hex[4:6]
+                / f"{hash_hex}-{file.size}"
+            ),
+        )
+
     def test_get_local_path_for_added_file(self) -> None:
         """LocalFileBackend.from_local_path return the already existing file."""
         temp_file_path = self.create_temporary_file()
@@ -143,64 +165,10 @@ class LocalFileBackendTests(TestCase):
         """LocalFileBackend.get_url return None."""
         fileobj = self.playground.create_file_in_backend(self.file_backend)
 
-        url = self.file_backend.get_url(  # type: ignore[func-returns-value]
-            fileobj
-        )
+        url = self.file_backend.get_url(fileobj)
         self.assertIsNone(url)
 
-    def test_add_file_from_local_path_reuse_fileobj(self) -> None:
-        """LocalFileBackend.add_file reuse the fileobj."""
-        temp_file_path = self.create_temporary_file(contents=b"This is a test")
-
-        fileobj = File.from_local_path(temp_file_path)
-
-        fileobj_before_id = fileobj.id
-
-        added_fileobj = self.file_backend.add_file(
-            temp_file_path, fileobj=fileobj
-        )
-
-        self.assertEqual(added_fileobj.id, fileobj_before_id)
-        self.assertTrue(
-            FileInStore.objects.filter(
-                store=self.file_backend.db_store, file=added_fileobj
-            ).exists()
-        )
-
-    def test_add_file_from_local_path_fileobj_size_do_not_match(self) -> None:
-        """
-        LocalFileBackend.add_file with fileobj not reused: size mismatch.
-
-        Exception is raised, file is not added in the backend.
-        """
-        original_contents = b"This is a test"
-        new_contents = b"Different content length"
-
-        temp_file_path = self.create_temporary_file(contents=original_contents)
-        fileobj = File.from_local_path(temp_file_path)
-
-        self.assertEqual(FileInStore.objects.count(), 0)
-
-        with open(temp_file_path, "wb") as f:
-            f.write(new_contents)
-
-        expected_message = (
-            "^add_file file size mismatch. Path: "
-            f"{temp_file_path} Size in disk: {len(new_contents)} "
-            f"fileobj.size: {len(original_contents)}$"
-        )
-
-        with self.assertRaisesRegex(ValueError, expected_message):
-            self.file_backend.add_file(temp_file_path, fileobj=fileobj)
-
-        # No file was added in the File table
-        self.assertEqual(File.objects.count(), 1)
-
-        # No file was added in the FileInStore (wouldn't be possible
-        # since no file was added anyway)
-        self.assertEqual(FileInStore.objects.count(), 0)
-
-    def test_add_file_from_local_path_fileobj(self) -> None:
+    def test_add_file(self) -> None:
         """LocalFileBackend.add_file copy and return the file to the backend."""
         mocked_fsync = self.patch_os_fsync()
         contents = b"this is a test2"
@@ -214,7 +182,9 @@ class LocalFileBackendTests(TestCase):
         fileobj_temp = File()
         fileobj_temp.hash_digest = _calculate_hash_from_data(contents)
         fileobj_temp.size = len(contents)
-        fileobj_temp_path = self.file_backend.get_local_path(fileobj_temp)
+        fileobj_temp_path = self.file_backend.get_entry(
+            fileobj_temp
+        ).get_local_path()
         fileobj_temp_path.parent.mkdir(parents=True, exist_ok=True)
 
         # Writes the contents twice. To test that debusine will overwrite
@@ -233,7 +203,7 @@ class LocalFileBackendTests(TestCase):
 
         # The file has been properly copied
         file_in_file_backend_hash = File.calculate_hash(
-            self.file_backend.get_local_path(file)
+            self.file_backend.get_entry(file).get_local_path()
         )
 
         self.assertEqual(file.hash_digest, file_in_file_backend_hash)
@@ -244,35 +214,7 @@ class LocalFileBackendTests(TestCase):
         )
         mocked_fsync.assert_called()
 
-    def patch_shutil_copy_raise_filenotfound(self) -> mock.MagicMock:
-        """Patch shutil.copy: raise FileNotFoundError if called."""
-        patcher = mock.patch("shutil.copy", autospec=True)
-        mocked_copy = patcher.start()
-        mocked_copy.side_effect = FileNotFoundError
-        self.addCleanup(patcher.stop)
-
-        return mocked_copy
-
-    def test_add_file_from_local_path_file_cannot_be_copied(self) -> None:
-        """LocalFileBackend.add_file fail to copy file: no File is created."""
-        temp_file_path = self.create_temporary_file(contents=b"test")
-
-        self.patch_shutil_copy_raise_filenotfound()
-
-        try:
-            self.file_backend.add_file(temp_file_path)
-        except FileNotFoundError:
-            # shutil.copy() raised FileNotFoundError:
-            # in production this could be any error: FileNotFoundError
-            # because the file is gone when it tried to copy it, no space to
-            # finish the copy, wrong permissions,
-            # Only if the file is copied should then be referenced in the DB
-            pass
-
-        # The File in db was not created
-        self.assertFalse(File.objects.all().exists())
-
-    def test_add_file_for_an_added_file(self) -> None:
+    def test_add_file_already_in_backend(self) -> None:
         """
         LocalFileBackend.add_file add a file that is already in the backend.
 
@@ -280,41 +222,14 @@ class LocalFileBackendTests(TestCase):
         """
         fileobj = self.playground.create_file_in_backend(self.file_backend)
 
-        mocked_copy = self.patch_shutil_copy_raise_filenotfound()
-
         # In this test we pretend to add the file from the backend into the
         # backend. Usually it would be from outside the backend directory.
-        file_path = self.file_backend.get_local_path(fileobj)
+        file_path = self.file_backend.get_entry(fileobj).get_local_path()
 
-        self.assertEqual(
-            self.file_backend.add_file(file_path, fileobj), fileobj
-        )
-        mocked_copy.assert_not_called()
-
-    def test_add_file_contents_mismatch(self) -> None:
-        """File exists in the backend with same hash but different contents."""
-        original_contents = b"This is a test"
-        new_contents = b"Hash collision"  # same length as original_contents
-        temp_file_path = self.create_temporary_file(contents=original_contents)
-        fileobj = File.from_local_path(temp_file_path)
-        self.file_backend.add_file(temp_file_path)
-
-        self.assertEqual(File.objects.count(), 1)
-        self.assertEqual(FileInStore.objects.count(), 1)
-
-        with open(temp_file_path, "wb") as f:
-            f.write(new_contents)
-
-        expected_message = (
-            f"add_file contents mismatch when trying to add "
-            f"{fileobj.hash_digest.hex()}"
-        )
-
-        with self.assertRaisesRegex(ValueError, expected_message):
-            self.file_backend.add_file(temp_file_path, fileobj=fileobj)
-
-        self.assertEqual(File.objects.count(), 1)
-        self.assertEqual(FileInStore.objects.count(), 1)
+        with mock.patch("shutil.copy", autospec=True):
+            self.assertEqual(
+                self.file_backend.add_file(file_path, fileobj), fileobj
+            )
 
     def test_directory_not_created_if_base_directory_do_not_exist(self) -> None:
         """
@@ -358,26 +273,28 @@ class LocalFileBackendTests(TestCase):
         with self.file_backend.get_stream(file) as f:
             self.assertEqual(f.read(), contents)
 
-    def test_remove_file(self) -> None:
-        """LocalFileBackend._remove_file delete the file from the backend."""
+    def test_entry_remove(self) -> None:
+        """LocalFileBackendEntry.remove deletes the file from the backend."""
         file = self.playground.create_file_in_backend(self.file_backend)
+        entry = self.file_backend.get_entry(file)
 
-        self.assertTrue(self.file_backend.get_local_path(file).exists())
+        self.assertTrue(entry.get_local_path().exists())
 
-        self.file_backend._remove_file(file)
+        entry.remove()
 
-        self.assertFalse(self.file_backend.get_local_path(file).exists())
+        self.assertFalse(entry.get_local_path().exists())
 
-    def test_remove_file_did_not_exist(self) -> None:
+    def test_entry_remove_did_not_exist(self) -> None:
         """
-        LocalFileBackend.remove_file try to delete a file that did not exist.
+        LocalFileBackendEntry.remove tries to delete a file that did not exist.
 
-        This could happen if remove_file() was called concurrently.
+        This could happen if remove() was called concurrently.
         """
         file = self.playground.create_file_in_backend(self.file_backend)
-        self.file_backend.get_local_path(file).unlink()
+        entry = self.file_backend.get_entry(file)
+        entry.get_local_path().unlink()
 
-        self.file_backend._remove_file(file)
+        entry.remove()
         self.assertTrue(True)  # No exception was raised
 
     def patch_os_fsync(self) -> mock.MagicMock:
@@ -388,19 +305,21 @@ class LocalFileBackendTests(TestCase):
 
         return mocked_fsync
 
-    def test_sync_file(self) -> None:
-        """LocalFileBackend._sync_file sync the file and its directory."""
+    def test_entry_sync_file(self) -> None:
+        """LocalFileBackendEntry._sync_file sync the file and its directory."""
         mocked_fsync = self.patch_os_fsync()
 
+        entry = self.file_backend.get_entry(self.playground.create_file())
         temporary_file = self.create_temporary_file()
 
-        LocalFileBackend._sync_file(temporary_file)
+        entry._sync_file(temporary_file)
 
         # Called twice: one to os.fsync the file and again for the directory
         self.assertEqual(mocked_fsync.call_count, 2)
 
-    def test_sync_file_does_not_exist(self) -> None:
+    def test_entry_sync_file_does_not_exist(self) -> None:
         """LocalFileBackend._sync_file log warning: file does not exist."""
+        entry = self.file_backend.get_entry(self.playground.create_file())
         temporary_file = self.create_temporary_file()
         temporary_file.unlink()
 
@@ -409,4 +328,29 @@ class LocalFileBackendTests(TestCase):
         with self.assertLogsContains(
             expected_message, logger="debusine", level=logging.DEBUG
         ):
-            LocalFileBackend._sync_file(temporary_file)
+            entry._sync_file(temporary_file)
+
+    def test_list_entries(self) -> None:
+        """LocalFileBackend.list_entries returns all entries in the backend."""
+        data_gen = data_generator(16)
+        files = [
+            self.playground.create_file_in_backend(
+                self.file_backend, contents=next(data_gen)
+            )
+            for _ in range(10)
+        ]
+        expected_entries: list[LocalFileBackendEntry] = []
+        for file in files:
+            hash_hex = file.sha256.hex()
+            expected_entries.append(
+                LocalFileBackendEntry(
+                    backend=self.file_backend,
+                    identifier=PurePath(hash_hex[0:2])
+                    / hash_hex[2:4]
+                    / hash_hex[4:6]
+                    / f"{hash_hex}-{file.size}",
+                )
+            )
+        self.assertCountEqual(
+            self.file_backend.list_entries(), expected_entries
+        )
