@@ -19,19 +19,22 @@ import django
 from django.core.management import CommandParser
 from django.db import connection, transaction
 from django.db.backends.utils import CursorWrapper
-from django.db.models import Exists, OuterRef, ProtectedError, Q
+from django.db.models import ProtectedError, Q, QuerySet
 from django.utils import timezone
 
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
-    Collection,
     CollectionItem,
     File,
     FileInArtifact,
     FileInStore,
+    Group,
     WorkRequest,
+    Workspace,
+    work_requests,
 )
+from debusine.db.models.workspaces import DeleteWorkspaces
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
 
 
@@ -62,6 +65,14 @@ class Command(DebusineBaseCommand):
             delete_expired_artifacts = DeleteExpiredArtifacts(operation)
             delete_expired_artifacts.run()
 
+            delete_expired_ephemeral_groups = DeleteExpiredEphemeralGroups(
+                operation
+            )
+            delete_expired_ephemeral_groups.run()
+
+            delete_expired_workspaces = DeleteExpiredWorkspaces(operation)
+            delete_expired_workspaces.run()
+
         raise SystemExit(0)
 
 
@@ -352,10 +363,7 @@ class DeleteExpiredWorkRequests:
     Delete also the work requests' internal collection if present.
     """
 
-    #: Work requests to delete
-    work_requests: set[WorkRequest]
-    #: Internal collections to delete
-    collections: set[Collection]
+    to_delete: work_requests.DeleteUnused
 
     def __init__(self, operation: DeleteOperation) -> None:
         """
@@ -365,79 +373,22 @@ class DeleteExpiredWorkRequests:
         """
         self.operation = operation
 
-    def scan_work_requests(self) -> None:
-        """Look for work requests to delete, filling self.work_requests."""
-        # to_delete: expired work requests excluding the ones referenced
-        # by a CollectionItem.created_by_workflow/removed_by_workflow
-        # that are not in the CollectionItem.parent_collection
-        #
-        # In other words: if a WorkRequest is referenced by a
-        # CollectionItem.created_by_workflow/removed_by_workflow but the
-        # CollectionItem.parent_collection is WorkRequest.internal_collection
-        # the WorkRequest can be deleted.
-        to_delete = {
-            wr
-            for wr in WorkRequest.objects.expired(
-                at=self.operation.initial_time
-            ).exclude(
-                Exists(
-                    CollectionItem.objects.filter(
-                        (
-                            Q(created_by_workflow_id=OuterRef("id"))
-                            | Q(removed_by_workflow_id=OuterRef("id"))
-                        )
-                        & ~Q(
-                            parent_collection_id=OuterRef(
-                                "internal_collection_id"
-                            )
-                        )
-                    )
-                )
-            )
-        }
-
-        # Keep work requests whose parent is not expired
-        while to_delete:
-            to_keep: set[WorkRequest] = set()
-            for wr in to_delete:
-                if wr.parent is not None and wr.parent not in to_delete:
-                    to_keep.add(wr)
-            to_delete -= to_keep
-            if not to_keep:
-                break
-
-        self.work_requests = to_delete
-
-    def scan_collections(self) -> None:
-        """Look for collections to delete, filling self.collections."""
-        collections: set[Collection] = set()
-        for wr in self.work_requests:
-            if wr.internal_collection:
-                collections.add(wr.internal_collection)
-        self.collections = collections
-
     def perform_deletions(self) -> None:
         """Delete objects found in self.work_requests and self.collections."""
-        CollectionItem.objects.filter(
-            parent_collection__in=self.collections
-        ).delete()
-        WorkRequest.objects.filter(
-            id__in=[wr.id for wr in self.work_requests]
-        ).delete()
-        Collection.objects.filter(
-            id__in=[c.id for c in self.collections]
-        ).delete()
+        self.to_delete.perform_deletions()
 
     def run(self) -> None:
         """Find and optionally delete the expired work requests."""
         with transaction.atomic():
-            self.scan_work_requests()
-            self.scan_collections()
-
-            if self.work_requests:
+            self.to_delete = work_requests.DeleteUnused(
+                WorkRequest.objects.expired(at=self.operation.initial_time)
+            )
+            if self.to_delete.work_requests:
                 self.operation.verbose(
-                    f"Deleting {len(self.work_requests)} expired work requests"
-                    f" and {len(self.collections)} expired collections\n"
+                    f"Deleting {len(self.to_delete.work_requests)}"
+                    " expired work requests"
+                    f" and {len(self.to_delete.collections)}"
+                    " expired collections\n"
                 )
 
                 # FIXME: do we enumerate each deleted work request to stdout?
@@ -446,3 +397,78 @@ class DeleteExpiredWorkRequests:
                     self.perform_deletions()
             else:
                 self.operation.verbose("There were no expired work requests\n")
+
+
+class DeleteExpiredEphemeralGroups:
+    """Delete expired exphemeral groups."""
+
+    groups: QuerySet[Group]
+
+    def __init__(self, operation: DeleteOperation) -> None:
+        """
+        Initialize object.
+
+        :param operation: class used for output
+        """
+        self.operation = operation
+
+    def perform_deletions(self) -> None:
+        """Delete previously found unused ephemeral groups."""
+        self.groups.delete()
+
+    def run(self) -> None:
+        """Find and optionally delete the expired work requests."""
+        with transaction.atomic():
+            self.groups = Group.objects.unused_ephemeral()
+            if count := len(self.groups):
+                self.operation.verbose(
+                    f"Deleting {count} unused ephemeral groups\n"
+                )
+                if not self.operation.dry_run:
+                    self.perform_deletions()
+            else:
+                self.operation.verbose(
+                    "There were no unused ephemeral groups\n"
+                )
+
+
+class DeleteExpiredWorkspaces:
+    """
+    Delete expired workspaces.
+
+    Delete workspaces that have expiration_delay set, ``expiration_delay`` days
+    after ``max(created_at, last_task_completion_time)``
+    """
+
+    #: Work requests to delete
+    to_delete: DeleteWorkspaces
+
+    def __init__(self, operation: DeleteOperation) -> None:
+        """
+        Initialize object.
+
+        :param operation: class used for output
+        """
+        self.operation = operation
+
+    def perform_deletions(self) -> None:
+        """Delete objects found in self.work_requests and self.collections."""
+        self.to_delete.perform_deletions()
+
+    def run(self) -> None:
+        """Find and optionally delete the expired work requests."""
+        with transaction.atomic():
+            self.to_delete = DeleteWorkspaces(
+                Workspace.objects.exclude(expiration_delay__isnull=True)
+                .with_expiration_time()
+                .filter(expiration_time__lt=self.operation.initial_time)
+            )
+            if self.to_delete.workspaces:
+                self.operation.verbose(
+                    f"Deleting {len(self.to_delete.workspaces)}"
+                    " expired workspaces\n"
+                )
+                if not self.operation.dry_run:
+                    self.perform_deletions()
+            else:
+                self.operation.verbose("There were no expired workspaces\n")
