@@ -15,7 +15,7 @@ from django.core.management import CommandError
 from django.test import override_settings
 
 from debusine.artifacts.models import SINGLETON_COLLECTION_CATEGORIES
-from debusine.db.models import FileStore, Group, Scope, Workspace
+from debusine.db.models import Group, Scope, Workspace
 from debusine.db.models.workspaces import WorkspaceRole
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
@@ -33,7 +33,7 @@ class CreateWorkspaceCommandTests(TestCase):
         """Set up common test data."""
         super().setUpTestData()
         cls.scope = cls.playground.get_or_create_scope(name="scope")
-        cls.admin_group, _ = Group.objects.get_or_create(
+        cls.admin_group = cls.playground.create_group(
             scope=cls.scope, name="Owners-test"
         )
 
@@ -93,40 +93,6 @@ class CreateWorkspaceCommandTests(TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
 
-    def test_create_workspace_with_default_file_store(self) -> None:
-        """Create a workspace with a different default file store."""
-        name = "test"
-        default_file_store = FileStore.objects.create(
-            name="different", backend=FileStore.BackendChoices.LOCAL
-        )
-        stdout, stderr, exit_code = call_command(
-            "create_workspace",
-            "--default-file-store",
-            "different",
-            self.scope.name + "/" + name,
-        )
-
-        workspace = Workspace.objects.get(scope=self.scope, name=name)
-
-        self.assertFalse(workspace.public)
-        self.assertEqual(workspace.default_file_store, default_file_store)
-        self.assertEqual(stderr, "")
-        self.assertEqual(exit_code, 0)
-
-    def test_create_workspace_with_default_file_store_not_found(self) -> None:
-        """create_workspace: default file store not found."""
-        with self.assertRaisesRegex(
-            CommandError, 'File store "nonexistent" not found'
-        ) as exc:
-            call_command(
-                "create_workspace",
-                "--default-file-store",
-                "nonexistent",
-                self.scope.name + "/test",
-            )
-
-        self.assertEqual(exc.exception.returncode, 3)
-
     def test_create_workspace_with_invalid_name(self) -> None:
         """Create a workspace with an invalid name."""
         with self.assertRaisesRegex(
@@ -146,7 +112,7 @@ class CreateWorkspaceCommandTests(TestCase):
         )
 
         scope = Scope.objects.create(name="different")
-        admin_group, _ = Group.objects.get_or_create(
+        admin_group = self.playground.create_group(
             scope=scope, name="Owners-test"
         )
         stdout, stderr, exit_code = call_command(
@@ -191,7 +157,7 @@ class CreateWorkspaceCommandTests(TestCase):
     def test_create_workspace_with_specific_owners_group(self) -> None:
         """Create a workspace with a specific owners group."""
         name = "test"
-        admin_group, _ = Group.objects.get_or_create(
+        admin_group = self.playground.create_group(
             scope=self.scope, name="MyOwners"
         )
         stdout, stderr, exit_code = call_command(
