@@ -21,11 +21,7 @@ from django.db import OperationalError, connection, transaction
 from django.db.models.manager import Manager
 from django.utils import timezone
 
-from debusine.artifacts.models import (
-    ArtifactCategory,
-    BareDataCategory,
-    CollectionCategory,
-)
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -36,11 +32,13 @@ from debusine.db.models import (
     FileInArtifact,
     FileInStore,
     FileStore,
+    Group,
     WorkRequest,
     WorkflowTemplate,
     default_file_store,
     default_workspace,
 )
+from debusine.db.playground import scenarios
 from debusine.db.tests.utils import (
     RunInParallelTransaction,
     RunInThreadAndCloseDBConnections,
@@ -49,7 +47,9 @@ from debusine.django.management.tests import call_command
 from debusine.server.collections import DebianEnvironmentsManager
 from debusine.server.management.commands.delete_expired import (
     DeleteExpiredArtifacts,
+    DeleteExpiredEphemeralGroups,
     DeleteExpiredWorkRequests,
+    DeleteExpiredWorkspaces,
     DeleteOperation,
 )
 from debusine.test.django import TestCase, TransactionTestCase
@@ -133,6 +133,14 @@ class DeleteExpiredCommandTests(TestCase):
                 internal_collection=collection_1,
             )
 
+            create_time = timezone.now() - timedelta(days=30)
+            with mock.patch(
+                "django.utils.timezone.now", return_value=create_time
+            ):
+                self.playground.create_workspace(
+                    name="ws", expiration_delay=timedelta(days=7)
+                )
+
         num_artifacts = Artifact.objects.count()
         num_artifact_relations = ArtifactRelation.objects.count()
         num_files_in_store = FileInStore.objects.all().count()
@@ -150,6 +158,8 @@ class DeleteExpiredCommandTests(TestCase):
             + _format_deleted_artifacts(
                 {artifact_id_1, artifact_id_2, artifact_id_3}
             )
+            + "There were no unused ephemeral groups\n"
+            + "Deleting 1 expired workspaces\n"
             + "dry-run mode: no changes were made\n",
         )
 
@@ -195,20 +205,30 @@ class DeleteExpiredCommandTests(TestCase):
             expiration_delay=timedelta(days=1),
             internal_collection=collection_1,
         )
+        create_time = timezone.now() - timedelta(days=30)
+        with mock.patch("django.utils.timezone.now", return_value=create_time):
+            self.playground.create_workspace(
+                name="ws", expiration_delay=timedelta(days=7)
+            )
         self.assertEqual(WorkRequest.objects.count(), 1)
         num_collections = Collection.objects.count()
 
+        group = self.playground.create_group(name="test", ephemeral=True)
+
         stdout, stderr, retval = call_command("delete_expired", verbosity=2)
 
         self.assertEqual(
             stdout,
             _format_deleted_work_requests([work_request_1], [collection_1])
-            + _format_deleted_artifacts({artifact_id_1, artifact_id_2}),
+            + _format_deleted_artifacts({artifact_id_1, artifact_id_2})
+            + "Deleting 1 unused ephemeral groups\n"
+            + "Deleting 1 expired workspaces\n",
         )
         self.assertEqual(Artifact.objects.count(), 0)
         self.assertEqual(ArtifactRelation.objects.count(), 0)
         self.assertEqual(WorkRequest.objects.count(), 0)
         self.assertEqual(Collection.objects.count(), num_collections - 1)
+        self.assertQuerySetEqual(Group.objects.filter(pk=group.pk), [])
 
     def test_delete_expired_verbose_nothing_deleted(self) -> None:
         """Test delete_expired verbose: print no artifact expired."""
@@ -218,7 +238,9 @@ class DeleteExpiredCommandTests(TestCase):
         self.assertEqual(
             stdout,
             _format_deleted_work_requests([], [])
-            + "There were no expired artifacts\n",
+            + "There were no expired artifacts\n"
+            + "There were no unused ephemeral groups\n"
+            + "There were no expired workspaces\n",
         )
 
 
@@ -982,8 +1004,8 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         ):
             self.delete_expired.run()
 
-        self.assertEqual(self.delete_expired.work_requests, set())
-        self.assertEqual(self.delete_expired.collections, set())
+        self.assertEqual(self.delete_expired.to_delete.work_requests, set())
+        self.assertEqual(self.delete_expired.to_delete.collections, set())
         perform_deletions.assert_not_called()
         verbose.assert_called_once_with("There were no expired work requests\n")
 
@@ -996,8 +1018,8 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         ):
             self.delete_expired.run()
 
-        self.assertEqual(self.delete_expired.work_requests, {wr1})
-        self.assertEqual(self.delete_expired.collections, set())
+        self.assertEqual(self.delete_expired.to_delete.work_requests, {wr1})
+        self.assertEqual(self.delete_expired.to_delete.collections, set())
         perform_deletions.assert_called()
         verbose.assert_called_once_with(
             "Deleting 1 expired work requests and 0 expired collections\n"
@@ -1015,176 +1037,208 @@ class DeleteExpiredWorkRequestsTests(TestCase):
         ):
             self.delete_expired.run()
 
-        self.assertEqual(self.delete_expired.work_requests, {wr1, wr2, wr3})
-        self.assertEqual(self.delete_expired.collections, set())
+        self.assertEqual(
+            self.delete_expired.to_delete.work_requests, {wr1, wr2, wr3}
+        )
+        self.assertEqual(self.delete_expired.to_delete.collections, set())
         perform_deletions.assert_called()
         verbose.assert_called_once_with(
             "Deleting 3 expired work requests and 0 expired collections\n"
         )
         self.assert_dry_run_does_not_delete()
 
-    def test_expired_but_referenced_by_collection_item(self) -> None:
-        """
-        Work requests do not expire if referenced by a CollectionItem.
 
-        Keep work requests expired and that:
-        - CollectionItem.created_by_workflow -> FK WorkRequest
-        - CollectionItem.removed_by_workflow -> FK WorkRequest
-        - CollectionItem.id != WorkRequest.internal_collection.id
-        """
-        collection_1 = self.playground.create_collection(
-            "internal-1", CollectionCategory.WORKFLOW_INTERNAL
-        )
-        collection_2 = self.playground.create_collection(
-            "internal-2", CollectionCategory.WORKFLOW_INTERNAL
+class DeleteExpiredEphemeralGroupsTest(TestCase):
+    """Tests for the DeleteExpiredEphemeralGroups class."""
+
+    scenario = scenarios.DefaultScopeUser()
+
+    def setUp(self) -> None:
+        """Initialize test."""
+        self.out = io.StringIO()
+        self.err = io.StringIO()
+        self.operation = DeleteOperation(
+            out=self.out, err=self.err, dry_run=False
         )
+        self.operation.initial_time = timezone.now()
+        self.delete_expired = DeleteExpiredEphemeralGroups(self.operation)
 
-        work_request_keep = self.playground.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection_2,
-        )
+    @contextlib.contextmanager
+    def mock_perform_deletions(self) -> Iterator[mock.MagicMock]:
+        """Shortcut to mock perform_deletions."""
+        with mock.patch(
+            "debusine.server.management.commands.delete_expired"
+            ".DeleteExpiredEphemeralGroups.perform_deletions"
+        ) as perform_deletions:
+            yield perform_deletions
 
-        work_request_delete = self.playground.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection_1,
-        )
+    @contextlib.contextmanager
+    def mock_verbose(self) -> Iterator[mock.MagicMock]:
+        """Shortcut to mock operation.verbose."""
+        with mock.patch(
+            "debusine.server.management.commands.delete_expired"
+            ".DeleteOperation.verbose"
+        ) as verbose:
+            yield verbose
 
-        # Ensure all work requests are deleted (they are not
-        # referenced by any CollectionItem yet)
-        with self.mock_perform_deletions():
-            self.delete_expired.run()
+    def assert_dry_run_does_not_delete(self) -> None:
+        """Make sure that, with dry_run, perform_deletions is not called."""
+        old_dry_run = self.operation.dry_run
+        self.operation.dry_run = True
+        try:
+            with self.mock_perform_deletions() as perform_deletions:
+                self.delete_expired.run()
+            perform_deletions.assert_not_called()
+        finally:
+            self.operation.dry_run = old_dry_run
 
-        self.assertEqual(
-            self.delete_expired.work_requests,
-            {work_request_delete, work_request_keep},
-        )
-
-        # Make work_request_keep to be kept: referenced by a
-        # CollectionItem (via created_by_workflow) and
-        # WorkRequest.internal_collection != collection_1
-        collection_1.manager.add_bare_data(
-            name="Item-1",
-            user=self.playground.get_default_user(),
-            category=BareDataCategory.TEST,
-            workflow=work_request_keep,
-        )
-
-        # work_request_delete is in a collection but deleted: referenced
-        # by a CollectionItem, but
-        # WorkRequest.internal-collection == collection_2
-        collection_1.manager.add_bare_data(
-            name="Item-2",
-            user=self.playground.get_default_user(),
-            category=BareDataCategory.TEST,
-            workflow=work_request_delete,
-        )
-
-        with self.mock_perform_deletions():
-            self.delete_expired.run()
-
-        self.assertEqual(
-            self.delete_expired.work_requests, {work_request_delete}
-        )
-
-        # work_request_keep still kept: now is referenced by a
-        # CollectionItem.removed_by_workflow
-        collection_1.manager.remove_bare_data(
-            name="Item-1",
-            user=self.playground.get_default_user(),
-            workflow=work_request_keep,
-        )
-
-        with self.mock_perform_deletions():
-            self.delete_expired.run()
-
-        self.assertEqual(
-            self.delete_expired.work_requests, {work_request_delete}
-        )
-
-        # Check again if no CollectionItem referenced the work requests:
-        # they are deleted
-        CollectionItem.objects.all().delete()
-        with self.mock_perform_deletions():
-            self.delete_expired.run()
-
-        self.assertEqual(
-            self.delete_expired.work_requests,
-            {work_request_keep, work_request_delete},
-        )
-
-    def test_expired_but_valid_parent(self) -> None:
-        """Keep work requests with non-expired parent."""
-        wr1 = self.playground.create_work_request()
-        self.playground.create_work_request(parent=wr1, expired=True)
-        wr3 = self.playground.create_work_request(expired=True)
+    def test_no_unused(self) -> None:
+        """No unused ephemeral groups."""
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
         ):
             self.delete_expired.run()
 
-        self.assertEqual(self.delete_expired.work_requests, {wr3})
-        self.assertEqual(self.delete_expired.collections, set())
-        perform_deletions.assert_called()
+        self.assertQuerySetEqual(self.delete_expired.groups, [])
+        perform_deletions.assert_not_called()
         verbose.assert_called_once_with(
-            "Deleting 1 expired work requests and 0 expired collections\n"
+            "There were no unused ephemeral groups\n"
         )
-        self.assert_dry_run_does_not_delete()
 
-    def test_expired_with_collection(self) -> None:
-        """Also delete internal collections."""
-        c1 = self.playground.create_collection(
-            "Collection-1", CollectionCategory.TEST
-        )
-        wr1 = self.playground.create_work_request(
-            internal_collection=c1, expired=True
-        )
+    def test_one_unused(self) -> None:
+        """One unused ephemeral group."""
+        group = self.playground.create_group(name="test", ephemeral=True)
         with (
             self.mock_perform_deletions() as perform_deletions,
             self.mock_verbose() as verbose,
         ):
             self.delete_expired.run()
 
-        self.assertEqual(self.delete_expired.work_requests, {wr1})
-        self.assertEqual(self.delete_expired.collections, {c1})
+        self.assertQuerySetEqual(self.delete_expired.groups, [group])
         perform_deletions.assert_called()
-        verbose.assert_called_once_with(
-            "Deleting 1 expired work requests and 1 expired collections\n"
-        )
+        verbose.assert_called_once_with("Deleting 1 unused ephemeral groups\n")
         self.assert_dry_run_does_not_delete()
 
-    def test_delete_expired_with_internal_collection(self) -> None:
-        """Delete a work request referenced by an internal collection item."""
-        collection = self.playground.create_collection(
-            "internal", CollectionCategory.WORKFLOW_INTERNAL
-        )
-
-        work_request = self.playground.create_work_request(
-            created_at=timezone.now() - timedelta(days=365),
-            expiration_delay=timedelta(days=1),
-            internal_collection=collection,
-        )
-
-        collection.manager.add_bare_data(
-            name="Item",
-            user=self.playground.get_default_user(),
-            category=BareDataCategory.TEST,
-            workflow=work_request,
-        )
-
-        # Work request is now expired but referenced by an item in an internal
-        # collection: it should be deleted
-        with self.mock_perform_deletions():
+    def test_ephemeral_but_used(self) -> None:
+        """One ephemeral group in use."""
+        group = self.playground.create_group(name="test", ephemeral=True)
+        group.assign_role(self.scenario.scope, self.scenario.scope.Roles.OWNER)
+        with (
+            self.mock_perform_deletions() as perform_deletions,
+            self.mock_verbose() as verbose,
+        ):
             self.delete_expired.run()
-        self.assertEqual(
-            self.delete_expired.work_requests,
-            {work_request},
+
+        self.assertQuerySetEqual(self.delete_expired.groups, [])
+        perform_deletions.assert_not_called()
+        verbose.assert_called_once_with(
+            "There were no unused ephemeral groups\n"
         )
 
-        # Check that it is actually deleted
-        self.delete_expired.run()
+
+class DeleteExpiredWorkspacesTests(TestCase):
+    """Tests for the DeleteExpiredWorkspaces class."""
+
+    def setUp(self) -> None:
+        """Initialize test."""
+        self.out = io.StringIO()
+        self.err = io.StringIO()
+        self.operation = DeleteOperation(
+            out=self.out, err=self.err, dry_run=False
+        )
+        self.operation.initial_time = timezone.now()
+        self.delete_expired = DeleteExpiredWorkspaces(self.operation)
+
+    @contextlib.contextmanager
+    def mock_perform_deletions(self) -> Iterator[mock.MagicMock]:
+        """Shortcut to mock perform_deletions."""
+        with mock.patch(
+            "debusine.server.management.commands.delete_expired"
+            ".DeleteExpiredWorkspaces.perform_deletions"
+        ) as perform_deletions:
+            yield perform_deletions
+
+    @contextlib.contextmanager
+    def mock_verbose(self) -> Iterator[mock.MagicMock]:
+        """Shortcut to mock operation.verbose."""
+        with mock.patch(
+            "debusine.server.management.commands.delete_expired"
+            ".DeleteOperation.verbose"
+        ) as verbose:
+            yield verbose
+
+    def assert_dry_run_does_not_delete(self) -> None:
+        """Make sure that, with dry_run, perform_deletions is not called."""
+        old_dry_run = self.operation.dry_run
+        self.operation.dry_run = True
+        try:
+            with self.mock_perform_deletions() as perform_deletions:
+                self.delete_expired.run()
+            perform_deletions.assert_not_called()
+        finally:
+            self.operation.dry_run = old_dry_run
+
+    def test_no_expired(self) -> None:
+        """No expired workspaces."""
+        with (
+            self.mock_perform_deletions() as perform_deletions,
+            self.mock_verbose() as verbose,
+        ):
+            self.delete_expired.run()
+
+        self.assertQuerySetEqual(self.delete_expired.to_delete.workspaces, [])
+        perform_deletions.assert_not_called()
+        verbose.assert_called_once_with("There were no expired workspaces\n")
+
+    def test_one_expired(self) -> None:
+        """One expired workspace."""
+        create_time = timezone.now() - timedelta(days=30)
+        with mock.patch("django.utils.timezone.now", return_value=create_time):
+            ws = self.playground.create_workspace(
+                name="ws", expiration_delay=timedelta(days=7)
+            )
+
+        with (
+            self.mock_perform_deletions() as perform_deletions,
+            self.mock_verbose() as verbose,
+        ):
+            self.delete_expired.run()
+
+        self.assertQuerySetEqual(
+            self.delete_expired.to_delete.workspaces.all(), [ws]
+        )
+        perform_deletions.assert_called()
+        verbose.assert_called_once_with("Deleting 1 expired workspaces\n")
+        self.assert_dry_run_does_not_delete()
+
+    def test_multiple_expired(self) -> None:
+        """Multiple expired workspaces."""
+        create_time = timezone.now() - timedelta(days=30)
+        with mock.patch("django.utils.timezone.now", return_value=create_time):
+            ws1 = self.playground.create_workspace(
+                name="ws1", expiration_delay=timedelta(days=7)
+            )
+            ws2 = self.playground.create_workspace(
+                name="ws2", expiration_delay=timedelta(days=7)
+            )
+            ws3 = self.playground.create_workspace(
+                name="ws3", expiration_delay=timedelta(days=7)
+            )
+        with (
+            self.mock_perform_deletions() as perform_deletions,
+            self.mock_verbose() as verbose,
+        ):
+            self.delete_expired.run()
+
+        self.assertQuerySetEqual(
+            self.delete_expired.to_delete.workspaces,
+            [ws1, ws2, ws3],
+            ordered=False,
+        )
+        perform_deletions.assert_called()
+        verbose.assert_called_once_with("Deleting 3 expired workspaces\n")
+        self.assert_dry_run_does_not_delete()
 
 
 def _format_deleted_work_requests(
