@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -23,7 +23,6 @@ from debusine.artifacts.models import (
 from debusine.db.context import context
 from debusine.db.models import (
     ArtifactRelation,
-    FileStore,
     Group,
     Scope,
     WorkRequest,
@@ -81,19 +80,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
                 "workspace", "create", "localtest/my ws", "Owners-myws"
             )
 
-        with self.assertRaisesRegex(
-            CommandError,
-            'File store "nonexistent" not found',
-        ):
-            call_command(
-                "workspace",
-                "create",
-                "localtest/myws",
-                "Owners-myws",
-                "--default-file-store",
-                "nonexistent",
-            )
-
         with self.assertRaisesRegex(
             CommandError, "Group 'Owners-myws' not found in scope 'localtest'"
         ):
@@ -118,7 +104,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             self.assertEqual(role.role, WorkspaceRole.Roles.OWNER)
             self.assertFalse(workspace.public)
             self.assertEqual(workspace.default_expiration_delay, timedelta(30))
-            self.assertEqual(workspace.default_file_store, FileStore.default())
             self.assertQuerySetEqual(
                 workspace.collections.values_list("category", flat=True),
                 SINGLETON_COLLECTION_CATEGORIES,
@@ -126,9 +111,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             )
 
         # Set parameters
-        different_file_store = FileStore.objects.create(
-            name="different", backend=FileStore.BackendChoices.LOCAL
-        )
         call_command(
             "workspace",
             "define",
@@ -137,22 +119,16 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             "--public",
             "--default-expiration-delay",
             "0",
-            "--default-file-store",
-            "different",
             "--no-singleton-collections",
         )
         workspace = Workspace.objects.get(scope=self.scope, name="myws-new")
         self.assertTrue(workspace.public)
         self.assertEqual(workspace.default_expiration_delay, timedelta(0))
-        self.assertEqual(workspace.default_file_store, different_file_store)
         self.assertQuerySetEqual(
             workspace.collections.values_list("category", flat=True), []
         )
 
         # Change parameters
-        different_file_store = FileStore.objects.create(
-            name="different2", backend=FileStore.BackendChoices.LOCAL
-        )
         call_command(
             "workspace",
             "define",
@@ -161,13 +137,10 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             "--private",
             "--default-expiration-delay",
             "10",
-            "--default-file-store",
-            "different2",
         )
         workspace.refresh_from_db()
         self.assertFalse(workspace.public)
         self.assertEqual(workspace.default_expiration_delay, timedelta(10))
-        self.assertEqual(workspace.default_file_store, different_file_store)
         role = WorkspaceRole.objects.get(
             resource=workspace, group__name="Owners-myws2"
         )
@@ -181,7 +154,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         )
         self.assertFalse(workspace.public)
         self.assertEqual(workspace.default_expiration_delay, timedelta(10))
-        self.assertEqual(workspace.default_file_store, different_file_store)
 
     def test_rename_scenario(self) -> None:
         """Test rename command."""
@@ -250,12 +222,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             "workspace", "create", "localtest/public", "Owners-myws", "--public"
         )
         call_command("workspace", "create", "localtest/otherfs", "Owners-myws")
-        different_file_store = FileStore.objects.create(
-            name="different", backend=FileStore.BackendChoices.LOCAL
-        )
-        Workspace.objects.get(
-            scope=self.scope, name="otherfs"
-        ).other_file_stores.add(different_file_store)
         call_command(
             "workspace",
             "create",
@@ -280,38 +246,29 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         )
         self.assertEqual(output.col(1), ["True", "True", "False", "False"])
         self.assertEqual(output.col(2), ["Never", "30", "30", "Never"])
-        self.assertEqual(output.col(4), ["0", "0", "1", "0"])
 
         # --yaml
         stdout, stderr, _ = call_command("workspace", "list", "--yaml")
         data = yaml.safe_load(stdout)
         expected_data = [
             {
-                'default_file_store': 'Default (Memory)',
                 'expiration': 'Never',
                 'name': 'debusine/System',
-                'other_file_store': 0,
                 'public': True,
             },
             {
-                'default_file_store': 'Default (Memory)',
                 'expiration': 30,
                 'name': 'localtest/public',
-                'other_file_store': 0,
                 'public': True,
             },
             {
-                'default_file_store': 'Default (Memory)',
                 'expiration': 30,
                 'name': 'localtest/otherfs',
-                'other_file_store': 1,
                 'public': False,
             },
             {
-                'default_file_store': 'Default (Memory)',
                 'expiration': 'Never',
                 'name': 'localtest/noexpire',
-                'other_file_store': 0,
                 'public': False,
             },
         ]
@@ -395,13 +352,19 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         """Create a test workspace."""
         scope = self.playground.get_or_create_scope("scope")
         workspace = self.playground.create_workspace(name="Test", scope=scope)
+        for architecture in ("amd64", "s390x"):
+            self.playground.create_debian_environment(
+                codename="bookworm",
+                architecture=architecture,
+                workspace=workspace,
+            )
         work_request = self.playground.create_work_request(
             workspace=workspace, task_name="noop"
         )
         collection = self.playground.create_collection(
             "test", CollectionCategory.WORKFLOW_INTERNAL, workspace=workspace
         )
-        artifact_hello, _ = self.create_artifact(
+        artifact_hello, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": "hello",
@@ -421,7 +384,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         artifact_hello.created_by_work_request = work_request
         artifact_hello.save()
 
-        artifact_hello_traditional, _ = self.create_artifact(
+        artifact_hello_traditional, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": "hello-traditional",
@@ -438,7 +401,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             create_files=True,
             skip_add_files_in_store=True,
         )
-        self.create_artifact_relation(
+        self.playground.create_artifact_relation(
             artifact_hello_traditional,
             artifact_hello,
             ArtifactRelation.Relations.RELATES_TO,
@@ -477,7 +440,6 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
                 "input": {
                     "source_artifact": udev.pk,
                 },
-                "backend": "schroot",
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64", "s390x"],
             },
