@@ -10,7 +10,8 @@
 """Mapping database models to REST Api support."""
 
 from collections.abc import Callable
-from typing import Any, Never, TYPE_CHECKING, TypeAlias
+from copy import copy
+from typing import Any, Never, NoReturn, TYPE_CHECKING, TypeAlias
 
 from django.db.models import QuerySet
 from django.http import HttpRequest
@@ -24,13 +25,16 @@ from rest_framework import serializers
 from rest_framework.exceptions import ErrorDetail, ValidationError
 
 from debusine.artifacts.models import ArtifactCategory
+from debusine.assets import AssetCategory, asset_data_model
 from debusine.client.models import (
     LookupChildType,
     model_to_json_serializable_dict,
 )
+from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
+    Asset,
     Collection,
     CollectionItem,
     DEFAULT_WORKSPACE_NAME,
@@ -195,6 +199,30 @@ class WorkRequestSerializer(
         ]
 
 
+class WorkRequestSerializerWithConfiguredTaskData(WorkRequestSerializer):
+    """Specialization of WorkRequestSerializer to feed task data to workers."""
+
+    def update(self, instance: Any, validated_data: dict[str, Any]) -> NoReturn:
+        """Disable update."""
+        raise NotImplementedError('`update()` must be implemented.')
+
+    def create(self, validated_data: dict[str, Any]) -> NoReturn:
+        """Disable create."""
+        raise NotImplementedError('`create()` must be implemented.')
+
+    def to_representation(self, instance: WorkRequest) -> dict[str, Any]:
+        """
+        Use configured_task_data instead of task_data.
+
+        This is intended to feed configured_task_data to workers without
+        breaking the worker API.
+        """
+        res = super().to_representation(instance)
+        if instance.configured_task_data:
+            res["task_data"] = instance.configured_task_data
+        return res
+
+
 class WorkRequestUpdateSerializer(WorkRequestSerializer):
     """Serializer for updating work requests."""
 
@@ -420,9 +448,7 @@ class ArtifactSerializerResponse(serializers.ModelSerializer[Artifact]):
         for file_in_artifact in artifact.fileinartifact_set.all().order_by(
             "path"
         ):
-            if not artifact.workspace.is_file_in_workspace(
-                file_in_artifact.file
-            ):
+            if artifact.workspace.file_needs_upload(file_in_artifact.file):
                 files_to_upload.append(file_in_artifact.path)
 
         return files_to_upload
@@ -497,6 +523,74 @@ class ArtifactRelationResponseSerializer(
         validators: list[Callable[[ArtifactRelation], None]] = []
 
 
+class AssetSerializer(DebusineSerializerMixin, serializers.Serializer[Asset]):
+    """Serializer to deserialize the client request to create an asset."""
+
+    validation_error_title = "Cannot deserialize asset"
+
+    id = serializers.IntegerField(required=False, read_only=True)
+    created_at = serializers.DateTimeField(read_only=True)
+    category = serializers.CharField(max_length=255)
+    workspace = WorkspaceInCurrentScopeField(slug_field="name")
+    # Deliberately incompatible with Serializer.data.
+    data = serializers.DictField()  # type: ignore[assignment]
+    work_request = serializers.IntegerField(
+        allow_null=True, required=False, source="created_by_work_request_id"
+    )
+
+    def validate(self, data: dict[str, Any]) -> dict[str, Any]:
+        """Validate category and data."""
+        try:
+            # Python 3.11 doesn't support X in StrEnum
+            AssetCategory(data["category"])
+        except ValueError:
+            raise serializers.ValidationError(
+                {"category": f"{data['category']} is not a known category."}
+            )
+
+        try:
+            asset_data_model(data["category"], data["data"])
+        except ValueError as e:
+            raise serializers.ValidationError(
+                {"data": f"invalid asset data: {e}"}
+            ) from e
+        return data
+
+    def create(self, validated_data: dict[str, Any]) -> Asset:
+        """Create an Asset model from deserialized data."""
+        data = copy(validated_data)
+        if not data.get("created_by_work_request_id"):
+            data["created_by"] = context.require_user()
+        return Asset.objects.create(**data)
+
+
+class AssetPermissionCheckRequest(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
+    """Serializer for a permission check on an asset."""
+
+    validation_error_title = "Cannot deserialize permission check request"
+
+    artifact_id = serializers.IntegerField()
+    work_request_id = serializers.IntegerField()
+    workspace = serializers.CharField()
+
+
+class AssetPermissionCheckResponse(
+    DebusineSerializerMixin, serializers.Serializer[Never]
+):
+    """Serializer for a permission check result."""
+
+    validation_error_title = "Cannot serialize permission check result"
+
+    has_permission = serializers.BooleanField()
+    username = serializers.CharField(max_length=150, required=False)
+    user_id = serializers.IntegerField(required=False)
+    resource = serializers.DictField(
+        child=serializers.CharField(max_length=255), required=False
+    )
+
+
 class LookupExpectTypeChoiceField(serializers.ChoiceField):
     """
     A field for `Lookup*.expect_type`.
