@@ -50,7 +50,7 @@ class TestDebusineSignon(TestCase):
         """Set up common test data."""
         super().setUpTestData()
         cls.scope = cls.playground.get_default_scope()
-        cls.group = Group.objects.create(name="group", scope=cls.scope)
+        cls.group = cls.playground.create_group(name="group")
         cls.identity = Identity.objects.create(
             issuer="test",
             subject="test@debian.org",
