@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -22,7 +22,7 @@ from urllib.parse import parse_qs, urlparse
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
 from django.http import HttpRequest
-from django.test import RequestFactory, TestCase, override_settings
+from django.test import Client, RequestFactory, TestCase, override_settings
 from django.urls import resolve, reverse
 from django.utils import timezone
 
@@ -132,7 +132,7 @@ class SalsaSignonViews(TestCase):
         identity = self._make_identity(user=user)
 
         # Simulate a session with a logged in user
-        url = reverse("user:logout")
+        url = reverse("logout")
         request = self._make_post_request(url)
         assert isinstance(request, RequestSignonProtocol)
         request.signon.activate_identity(identity)
@@ -154,7 +154,7 @@ class SalsaSignonViews(TestCase):
         identity = self._make_identity(user=user)
 
         # Simulate a session with a logged in user
-        url = reverse("user:logout")
+        url = reverse("logout")
         request = self._make_post_request(url)
         assert isinstance(request, RequestSignonProtocol)
         request.signon.activate_identity(identity)
@@ -378,6 +378,48 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(Identity.objects.count(), 0)
         self.assertFalse(response.wsgi_request.user.is_authenticated)
 
+    def test_oidc_callback_redirect_url(self) -> None:
+        """Test redirect after successful OIDC callback."""
+        user = get_user_model().objects.create_user(
+            "test", email="test@example.org"
+        )
+        ident = self._make_identity(
+            user=user,
+            subject="123",
+            last_used=timezone.now() - datetime.timedelta(days=1),
+        )
+
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": ident.issuer})
+
+        # Default redirect
+        client = Client()
+        with self.setup_mock_auth(identity=ident, profile="profile_url"):
+            response = client.get(cb_url)
+        self.assertRedirects(response, reverse("homepage:homepage"))
+
+        # Redirect comes from SIGNON_DEFAULT_REDIRECT
+        with override_settings(SIGNON_DEFAULT_REDIRECT="https://example.org"):
+            client = Client()
+            with self.setup_mock_auth(identity=ident, profile="profile_url"):
+                response = client.get(cb_url)
+            self.assertRedirects(
+                response, "https://example.org", fetch_redirect_response=False
+            )
+
+        # Redirect from the session
+        client = Client()
+        session = client.session
+        session["sso_callback_next_url"] = "https://session.example.org"
+        session.save()
+        with self.setup_mock_auth(identity=ident, profile="profile_url"):
+            response = client.get(cb_url)
+        self.assertRedirects(
+            response,
+            "https://session.example.org",
+            fetch_redirect_response=False,
+        )
+        self.assertNotIn("sso_callback_next_url", client.session)
+
     def test_bindidentity(self) -> None:
         """Bind identity redirects correctly."""
         user = get_user_model().objects.create_user(
