@@ -12,14 +12,17 @@ Server-side Debusine tasks.
 
 The tasks in this package run in a Celery worker.
 """
-
 from debusine.server.tasks.base import BaseServerTask  # isort: split
 
 # Sub-tasks need to be imported in order to be available to BaseTask
 # (e.g. for BaseTask.is_valid_task_name). They are registered via
 # BaseTask.__init_subclass__.
 from debusine.server.tasks.aptmirror import APTMirror
+from debusine.server.tasks.base import ServerTaskPermissionDenied
 from debusine.server.tasks.copy_collection_items import CopyCollectionItems
+from debusine.server.tasks.create_experiment_workspace import (
+    CreateExperimentWorkspace,
+)
 from debusine.server.tasks.noop import ServerNoop
 from debusine.server.tasks.package_upload import PackageUpload
 from debusine.server.tasks.update_suite_lintian_collection import (
@@ -30,7 +33,9 @@ __all__ = [
     "APTMirror",
     "BaseServerTask",
     "CopyCollectionItems",
+    "CreateExperimentWorkspace",
     "PackageUpload",
     "ServerNoop",
+    "ServerTaskPermissionDenied",
     "UpdateSuiteLintianCollection",
 ]
