@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,9 +11,10 @@
 
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import ServerNoopData
+from debusine.tasks.models import BaseDynamicTaskData
 
 
-class ServerNoop(BaseServerTask[ServerNoopData]):
+class ServerNoop(BaseServerTask[ServerNoopData, BaseDynamicTaskData]):
     """Task that runs on server-side Celery workers and returns a boolean."""
 
     TASK_VERSION = 1
@@ -23,3 +24,7 @@ class ServerNoop(BaseServerTask[ServerNoopData]):
         if self.data.exception:
             raise RuntimeError("Client requested an exception")
         return self.data.result
+
+    def get_label(self) -> str:
+        """Return the task label."""
+        return "noop"
