@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -80,7 +80,7 @@ class APTMirrorTests(TestCase):
         self, name: str, version: str, paths: list[str]
     ) -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SOURCE_PACKAGE,
             data={
                 "name": name,
@@ -104,7 +104,7 @@ class APTMirrorTests(TestCase):
         paths: list[str],
     ) -> Artifact:
         """Create a minimal `debian:binary-package` artifact."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.BINARY_PACKAGE,
             data={
                 "srcpkg_name": srcpkg_name,
