@@ -9,6 +9,7 @@
 
 """Unit tests for base server-side task classes."""
 
+import re
 from pathlib import Path
 from typing import ClassVar
 from unittest import mock
@@ -16,7 +17,11 @@ from unittest import mock
 from debusine.artifacts.local_artifact import WorkRequestDebugLogs
 from debusine.db.context import context
 from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
-from debusine.server.tasks.base import BaseServerTask
+from debusine.db.models.permissions import format_permission_check_error
+from debusine.server.tasks.base import (
+    BaseServerTask,
+    ServerTaskPermissionDenied,
+)
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, WorkerType
 from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test.django import TestCase
@@ -152,15 +157,14 @@ class BaseServerTaskTests(TestCase):
         self.assertEqual(artifact.workspace, work_request.workspace)
         self.assertEqual(artifact.data, {})
         self.assertEqual(artifact.created_by_work_request, work_request)
-        file_store = (
-            work_request.workspace.default_file_store.get_backend_object()
-        )
         for file_in_artifact, (name, contents) in zip(
             artifact.fileinartifact_set.order_by("id"), expected_files
         ):
             self.assertEqual(file_in_artifact.path, name)
-            assert file_in_artifact.file is not None
-            with file_store.get_stream(file_in_artifact.file) as file:
+            file_backend = work_request.workspace.scope.download_file_backend(
+                file_in_artifact.file
+            )
+            with file_backend.get_stream(file_in_artifact.file) as file:
                 self.assertEqual(file.read(), contents)
 
         return artifact
@@ -212,3 +216,23 @@ class BaseServerTaskTests(TestCase):
                 category=WorkRequestDebugLogs._category
             ).exists()
         )
+
+    def test_enforce(self) -> None:
+        """Test enforcing permissions."""
+        work_request = self.playground.create_work_request()
+        work_request.set_current()
+        self.task.set_work_request(work_request)
+
+        workspace = self.playground.get_default_workspace()
+
+        self.task.enforce(workspace.can_display)
+
+        with self.assertRaisesRegex(
+            ServerTaskPermissionDenied,
+            re.escape(
+                format_permission_check_error(
+                    workspace.can_create_work_requests, work_request.created_by
+                )
+            ),
+        ):
+            self.task.enforce(workspace.can_create_work_requests)
