@@ -25,6 +25,7 @@ from debusine.db.models import (
     Collection,
     CollectionItem,
     FileStore,
+    Scope,
     User,
     WorkRequest,
     Workspace,
@@ -36,7 +37,7 @@ from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
 )
-from debusine.tasks.models import LookupMultiple, TaskTypes
+from debusine.tasks.models import LookupMultiple, TaskTypes, WorkerType
 from debusine.test.django import TestCase
 
 
@@ -44,8 +45,10 @@ class CopyCollectionItemsTests(TestCase):
     """Tests for :py:class:`CopyCollectionItems`."""
 
     user: ClassVar[User]
+    source_scope: ClassVar[Scope]
     source_workspace: ClassVar[Workspace]
     source_build_logs: ClassVar[Collection]
+    target_scope: ClassVar[Scope]
     target_workspace: ClassVar[Workspace]
     target_build_logs: ClassVar[Collection]
 
@@ -55,15 +58,21 @@ class CopyCollectionItemsTests(TestCase):
         """Set up common data for tests."""
         super().setUpTestData()
         cls.user = cls.playground.get_default_user()
+        cls.source_scope = cls.playground.get_or_create_scope(
+            name="source", label="Source"
+        )
         cls.source_workspace = cls.playground.create_workspace(
-            name="source", public=True
+            scope=cls.source_scope, name="source", public=True
         )
         cls.source_build_logs = cls.playground.create_singleton_collection(
             CollectionCategory.PACKAGE_BUILD_LOGS,
             workspace=cls.source_workspace,
         )
+        cls.target_scope = cls.playground.get_or_create_scope(
+            name="target", label="Target"
+        )
         cls.target_workspace = cls.playground.create_workspace(
-            name="target", public=True
+            scope=cls.target_scope, name="target", public=True
         )
         cls.target_build_logs = cls.playground.create_singleton_collection(
             CollectionCategory.PACKAGE_BUILD_LOGS,
@@ -251,7 +260,7 @@ class CopyCollectionItemsTests(TestCase):
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
-        source_file_store = self.source_workspace.file_stores(
+        source_file_store = self.source_scope.download_file_stores(
             build_log_file
         ).get()
 
@@ -291,7 +300,7 @@ class CopyCollectionItemsTests(TestCase):
         self.assertEqual(copied_artifact.original_artifact, artifact)
         self.assertEqual(copied_artifact.files.get(), build_log_file)
         # The file in the copied artifact is in the same store.
-        target_file_store = self.target_workspace.file_stores(
+        target_file_store = self.target_scope.upload_file_stores(
             build_log_file
         ).get()
         self.assertEqual(source_file_store, target_file_store)
@@ -435,7 +444,7 @@ class CopyCollectionItemsTests(TestCase):
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
-        self.source_workspace.default_file_store.files.clear()
+        self.source_scope.file_stores.get().files.clear()
 
         with self.assertRaisesRegex(
             CannotCopy,
@@ -448,13 +457,18 @@ class CopyCollectionItemsTests(TestCase):
 
     @context.disable_permission_checks()
     def test_copy_item_artifact_different_file_store(self) -> None:
-        """Copy artifact file contents from a different file store."""
-        self.source_workspace.default_file_store = FileStore.objects.create(
+        """
+        Copy artifact file contents from a different file store.
+
+        We copy between scopes in order to exercise the case where the
+        download and upload file stores are different.
+        """
+        file_store = FileStore.objects.create(
             name="test",
             backend=FileStore.BackendChoices.LOCAL,
             configuration={"base_directory": settings.DEBUSINE_STORE_DIRECTORY},
         )
-        self.source_workspace.save()
+        self.source_scope.file_stores.set([file_store])
         work_request = self.playground.create_work_request(
             workspace=self.source_workspace
         )
@@ -469,7 +483,7 @@ class CopyCollectionItemsTests(TestCase):
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
-        source_file_store = self.source_workspace.file_stores(
+        source_file_store = self.source_scope.download_file_stores(
             build_log_file
         ).get()
 
@@ -494,7 +508,7 @@ class CopyCollectionItemsTests(TestCase):
         self.assertNotEqual(copied_artifact, artifact)
         self.assertQuerySetEqual(copied_artifact.files.all(), [build_log_file])
         # The file in the copied artifact is in a different store.
-        target_file_store = self.target_workspace.file_stores(
+        target_file_store = self.target_scope.upload_file_stores(
             build_log_file
         ).get()
         self.assertNotEqual(source_file_store, target_file_store)
@@ -590,7 +604,8 @@ class CopyCollectionItemsTests(TestCase):
                 ]
             ),
         )
-        self.playground.compute_dynamic_data(copy_collection_items)
+        worker = self.playground.create_worker(worker_type=WorkerType.CELERY)
+        copy_collection_items.assign_worker(worker)
         task = copy_collection_items.get_task()
         assert isinstance(task, CopyCollectionItems)
         task.set_work_request(copy_collection_items)
