@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,13 +14,14 @@ import os
 from pathlib import Path
 from unittest import mock
 
-from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.models import ArtifactCategory, DebianUpload
 from debusine.db.context import context
 from debusine.db.models import Artifact, File
 from debusine.server.tasks import PackageUpload
 from debusine.server.tasks.models import PackageUploadDynamicData
 from debusine.server.tasks.tests.ftp_server import FakeFTPServerProcess
 from debusine.server.tasks.tests.sftp_server import FakeSFTPServerProcess
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import FakeTaskDatabase, TaskHelperMixin
 from debusine.test.django import TestCase
 
@@ -73,7 +74,17 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
         task_db = FakeTaskDatabase(
             single_lookups={
                 # input.upload
-                ("internal@collections/name:upload-source", None): 10
+                ("internal@collections/name:upload-source", None): ArtifactInfo(
+                    id=10,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "source",
+                            "Files": [{"name": "hello_1.0.dsc"}],
+                        },
+                    ),
+                )
             }
         )
 
@@ -441,16 +452,15 @@ class PackageUploadTests(TaskHelperMixin[PackageUpload], TestCase):
 
         self.assertTrue(self.task.execute())
 
-        file_backend = (
-            self.task.workspace.default_file_store.get_backend_object()
-        )
         debug_logs_artifact = Artifact.objects.get(
             category=ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             created_by_work_request=self.task.work_request,
         )
         package_upload_log = debug_logs_artifact.fileinartifact_set.get()
         self.assertEqual(package_upload_log.path, "package-upload.log")
-        with file_backend.get_stream(package_upload_log.file) as file:
+        fileobj = package_upload_log.file
+        file_backend = self.task.workspace.scope.download_file_backend(fileobj)
+        with file_backend.get_stream(fileobj) as file:
             self.assertEqual(
                 file.read().decode(),
                 "".join(
