@@ -95,8 +95,10 @@ class RunServerTaskTests(TestCase):
     def test_bad_task_name(self) -> None:
         """The Celery task fails if the work request has a bad task name."""
         work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER, task_name="nonexistent"
+            task_type=TaskTypes.SERVER, task_name="servernoop"
         )
+        work_request.task_name = "nonexistent"
+        work_request.save()
         with (
             self.assertLogsContains(
                 "Task: nonexistent does not exist",
@@ -117,10 +119,10 @@ class RunServerTaskTests(TestCase):
     def test_bad_task_data(self) -> None:
         """The Celery task fails if the work request has bad task data."""
         work_request = self.create_assigned_work_request(
-            task_type=TaskTypes.SERVER,
-            task_name="servernoop",
-            task_data={"nonexistent": True},
+            task_type=TaskTypes.SERVER, task_name="servernoop"
         )
+        work_request.task_data = {"nonexistent": True}
+        work_request.save()
         with (
             self.assertLogsContains(
                 "Task: servernoop failed to configure",
