@@ -1,4 +1,4 @@
-# Copyright 2021-2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -36,12 +36,6 @@ from debusine.server.middlewares.token_last_seen_at import (
     TokenLastSeenAtMiddlewareChannels,
 )
 from debusine.server.routing import websocket_urlpatterns
-from debusine.tasks.models import (
-    BackendType,
-    SbuildBuildComponent,
-    SbuildData,
-    SbuildInput,
-)
 from debusine.test.django import (
     BaseDjangoTestCase,
     TestCase,
@@ -94,7 +88,6 @@ class WorkerConsumerMixin(BaseDjangoTestCase):
                 "cpu_cores": 4,
                 "sbuild:version": 1,
                 "sbuild:available": True,
-                "sbuild:chroots": "bullseye-amd64",
             }
         )
         return worker
@@ -350,7 +343,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         with context.disable_permission_checks():
             work_request: WorkRequest = await database_sync_to_async(
                 self.playground.create_work_request
-            )(task_name="sbuild")
+            )()
 
         await database_sync_to_async(work_request.assign_worker)(worker)
         await database_sync_to_async(work_request.mark_running)()
@@ -483,9 +476,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         communicator, worker = await self.connect_to_new_worker()
 
         def assign_work_request(worker: Worker) -> None:
-            work_request = self.playground.create_work_request(
-                task_name="sbuild"
-            )
+            work_request = self.playground.create_work_request()
             work_request.assign_worker(worker)
 
         with context.disable_permission_checks():
@@ -675,7 +666,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             work_request_notified = await database_sync_to_async(
                 self.playground.create_work_request
             )(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -687,7 +677,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         work_request_to_notify = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             assign_new_worker=True,
             mark_running=True,
             result=result,
@@ -730,7 +719,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         with context.disable_permission_checks():
             await database_sync_to_async(self.playground.create_work_request)(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -765,7 +753,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
             work_request_last_created_1 = await database_sync_to_async(
                 self.playground.create_work_request
             )(
-                task_name="sbuild",
                 assign_new_worker=True,
                 mark_running=True,
                 result=result,
@@ -777,7 +764,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         work_request_last_created_2 = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             assign_new_worker=True,
             mark_running=True,
             result=result,
@@ -821,28 +807,15 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
 
         await communicator.disconnect()
 
-    async def create_sbuild_work_request(
-        self, workspace: Workspace
-    ) -> WorkRequest:
-        """Create an sbuild work request."""
+    async def create_work_request(self, workspace: Workspace) -> WorkRequest:
+        """Create a work request."""
         work_request = await database_sync_to_async(
             self.playground.create_work_request
         )(
-            task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
             mark_running=True,
             result=WorkRequest.Results.SUCCESS,
-            task_data=SbuildData(
-                input=SbuildInput(source_artifact=5),
-                distribution="stable",
-                backend=BackendType.SCHROOT,
-                host_architecture="amd64",
-                build_components=[
-                    SbuildBuildComponent.ANY,
-                    SbuildBuildComponent.ALL,
-                ],
-            ),
         )
         assert isinstance(work_request, WorkRequest)
         return work_request
@@ -865,7 +838,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         # the work request
         self.assertTrue(await communicator.receive_nothing())
 
-        work_request = await self.create_sbuild_work_request(workspace)
+        work_request = await self.create_work_request(workspace)
         assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
@@ -908,7 +881,6 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
         result = WorkRequest.Results.SUCCESS
 
         await database_sync_to_async(self.playground.create_work_request)(
-            task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
             mark_running=True,
@@ -929,7 +901,7 @@ class WorkRequestCompletedConsumerTests(TransactionTestCase):
                 self.playground.create_workspace
             )()
 
-        work_request = await self.create_sbuild_work_request(workspace)
+        work_request = await self.create_work_request(workspace)
         assert work_request.completed_at is not None
 
         actual = await communicator.receive_json_from()
