@@ -11,9 +11,10 @@
 
 import inspect
 import logging
+import re
 from collections.abc import Callable
 from datetime import timedelta
-from typing import Any, ClassVar, TYPE_CHECKING, cast
+from typing import Any, ClassVar, TYPE_CHECKING
 from unittest import mock
 
 from channels.db import database_sync_to_async
@@ -23,8 +24,9 @@ from django.utils import timezone
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
-    KeyPurpose,
+    DebianSourcePackage,
 )
+from debusine.assets import KeyPurpose
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -58,23 +60,42 @@ from debusine.tasks.models import (
     TaskTypes,
     WorkerType,
 )
-from debusine.tasks.tests.helper_mixin import FakeTaskDatabase, TestTaskMixin
-from debusine.test.django import TestCase, TransactionTestCase
-from debusine.test.utils import preserve_task_registry
+from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.tests.helper_mixin import (
+    FakeTaskDatabaseBase,
+    TestTaskMixin,
+)
+from debusine.test.django import (
+    PlaygroundTestCase,
+    TestCase,
+    TransactionTestCase,
+)
+from debusine.test.utils import (
+    create_system_tarball_data,
+    preserve_task_registry,
+)
 
 if TYPE_CHECKING:
     from debusine.test.playground import Playground
 
 
-class HelperMixin:
+class FakeTaskDatabase(FakeTaskDatabaseBase, TaskDatabase):
+    """Use TaskDatabase's configure."""
+
+    def __init__(self, *, work_request: WorkRequest, **kwargs: Any):
+        """Also store a WorkRequest reference."""
+        super().__init__(**kwargs)
+        self.work_request = work_request
+
+
+class SchedulerTestCase(PlaygroundTestCase):
     """Additional methods to help test scheduler related functions."""
 
     SAMPLE_TASK_DATA = {
         "input": {
             "source_artifact": 10,
         },
-        "distribution": "unstable",
-        "backend": "schroot",
+        "environment": "debian/match:codename=sid",
         "host_architecture": "amd64",
         "build_components": [
             "any",
@@ -82,24 +103,46 @@ class HelperMixin:
         ],
     }
 
+    single_lookups: dict[
+        tuple[LookupSingle, CollectionCategory | None], ArtifactInfo
+    ] = {
+        # source_artifact
+        (10, None): ArtifactInfo(
+            id=10,
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data=DebianSourcePackage(
+                name="hello",
+                version="1.0-1",
+                type="dpkg",
+                dsc_fields={"Package": "hello", "Version": "1.0-1"},
+            ),
+        ),
+        # environment
+        (
+            "debian/match:codename=sid:architecture=amd64:format=tarball:"
+            "backend=unshare",
+            CollectionCategory.ENVIRONMENTS,
+        ): ArtifactInfo(
+            id=2,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        ),
+    }
+
     playground: "Playground"
 
-    def patch_task_database(
-        self,
-        single_lookups: (
-            dict[tuple[LookupSingle, str | None], int] | None
-        ) = None,
-    ) -> None:
+    def patch_task_database(self) -> None:
         """Patch :py:class:`debusine.server.scheduler.TaskDatabase`."""
         task_database_patcher = mock.patch(
-            "debusine.server.scheduler.TaskDatabase",
-            side_effect=lambda _: FakeTaskDatabase(
-                single_lookups=single_lookups or {(10, None): 10},
+            "debusine.db.models.task_database.TaskDatabase",
+            side_effect=lambda work_request: FakeTaskDatabase(
+                work_request=work_request,
+                single_lookups=self.single_lookups,
                 settings={"DEBUSINE_FQDN": "debusine.example.net"},
             ),
         )
         task_database_patcher.start()
-        cast(TestCase, self).addCleanup(task_database_patcher.stop)
+        self.addCleanup(task_database_patcher.stop)
 
     @classmethod
     def _create_worker(
@@ -109,17 +152,13 @@ class HelperMixin:
         token = Token.objects.create(enabled=enabled)
         worker = Worker.objects.create_with_fqdn('worker.lan', token)
 
-        sbuild_chroots = "{}-{}".format(
-            cls.SAMPLE_TASK_DATA['distribution'],
-            cls.SAMPLE_TASK_DATA['host_architecture'],
-        )
         worker.dynamic_metadata = {
             "system:worker_type": worker_type,
             "system:host_architecture": "amd64",
             "system:architectures": ["amd64"],
             "sbuild:version": Sbuild.TASK_VERSION,
             "sbuild:available": True,
-            "sbuild:chroots": [sbuild_chroots],
+            "executor:unshare:available": True,
         }
 
         worker.mark_connected()
@@ -178,7 +217,9 @@ class TaskDatabaseTests(TestCase):
         )
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.SYSTEM_TARBALL,
-            data={"codename": "bookworm", "architecture": "amd64"},
+            data=create_system_tarball_data(
+                codename="bookworm", architecture="amd64"
+            ),
         )
         collection.manager.add_artifact(
             artifact, user=self.playground.get_default_user()
@@ -189,14 +230,18 @@ class TaskDatabaseTests(TestCase):
             task_db.lookup_single_artifact(
                 "debian@debian:environments/match:codename=bookworm"
             ),
-            artifact.id,
+            ArtifactInfo(
+                id=artifact.id, category=artifact.category, data=artifact.data
+            ),
         )
         self.assertEqual(
             task_db.lookup_single_artifact(
                 "debian/match:codename=bookworm",
                 default_category=CollectionCategory.ENVIRONMENTS,
             ),
-            artifact.id,
+            ArtifactInfo(
+                id=artifact.id, category=artifact.category, data=artifact.data
+            ),
         )
         with self.assertRaisesRegex(
             LookupError,
@@ -215,7 +260,9 @@ class TaskDatabaseTests(TestCase):
         for codename in "bookworm", "trixie":
             artifact, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                data={"codename": codename, "architecture": "amd64"},
+                data=create_system_tarball_data(
+                    codename=codename, architecture="amd64"
+                ),
             )
             collection.manager.add_artifact(
                 artifact, user=self.playground.get_default_user()
@@ -229,14 +276,28 @@ class TaskDatabaseTests(TestCase):
                     {"collection": "debian@debian:environments"}
                 )
             ),
-            [artifact.id for artifact in artifacts],
+            [
+                ArtifactInfo(
+                    id=artifact.id,
+                    category=artifact.category,
+                    data=artifact.data,
+                )
+                for artifact in artifacts
+            ],
         )
         self.assertCountEqual(
             task_db.lookup_multiple_artifacts(
                 LookupMultiple.parse_obj({"collection": "debian"}),
                 default_category=CollectionCategory.ENVIRONMENTS,
             ),
-            [artifact.id for artifact in artifacts],
+            [
+                ArtifactInfo(
+                    id=artifact.id,
+                    category=artifact.category,
+                    data=artifact.data,
+                )
+                for artifact in artifacts
+            ],
         )
         with self.assertRaisesRegex(
             LookupError,
@@ -290,7 +351,7 @@ class TaskDatabaseTests(TestCase):
         )
 
 
-class SchedulerTests(HelperMixin, TestCase):
+class SchedulerTests(SchedulerTestCase, TestCase):
     """Test schedule() function."""
 
     def setUp(self) -> None:
@@ -485,7 +546,7 @@ class SchedulerTests(HelperMixin, TestCase):
         self.assertEqual(work_request.status, WorkRequest.Statuses.RUNNING)
 
 
-class ScheduleForWorkerTests(HelperMixin, TestCase):
+class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
     """Test schedule_for_worker()."""
 
     def setUp(self) -> None:
@@ -535,8 +596,7 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
         self.create_sample_sbuild_work_request()
         # The request is for unstable-amd64, so can_run_on will return False
         self.worker.dynamic_metadata = {
-            "system.worker_type": WorkerType.EXTERNAL,
-            "sbuild:chroots": ["stable-amd64"],
+            "system.worker_type": WorkerType.EXTERNAL
         }
         self.worker.save()
 
@@ -585,7 +645,6 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
         """schedule_for_worker() handles failure to compute dynamic data."""
         work_request = self.create_sample_sbuild_work_request()
         work_request.task_data["backend"] = "unshare"
-        del work_request.task_data["distribution"]
         work_request.task_data["environment"] = "bad-lookup"
         work_request.save()
         self.worker.dynamic_metadata["executor:unshare:available"] = True
@@ -596,13 +655,18 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
             result = schedule_for_worker(self.worker)
         work_request.refresh_from_db()
 
-        self.assertIn(
-            f"WorkRequest {work_request.id} failed to pre-process task data, "
-            f"aborting it. "
-            f"Task data: {work_request.task_data} "
-            f"Error: ('bad-lookup', {CollectionCategory.ENVIRONMENTS!r})",
-            "\n".join(log.output),
+        lines = re.split(r"(?= [A-Z])", log.output[0])
+        self.assertEqual(
+            lines,
+            [
+                "WARNING:debusine.server.scheduler:"
+                f"WorkRequest {work_request.id}"
+                " failed to pre-process task data, aborting it.",
+                f" Task data: {work_request.task_data}",
+                f" Error: ('bad-lookup', {CollectionCategory.ENVIRONMENTS!r})",
+            ],
         )
+
         self.assertIsNone(result)
         self.assertEqual(work_request.status, WorkRequest.Statuses.COMPLETED)
         self.assertEqual(work_request.result, WorkRequest.Results.ERROR)
@@ -626,16 +690,27 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
                 environment="debian/match:codename=bookworm",
             ),
         )
-        self.patch_task_database(
-            single_lookups={
-                (1, None): 1,
-                (
-                    "debian/match:codename=bookworm:architecture=i386:"
-                    "format=tarball:backend=unshare",
-                    CollectionCategory.ENVIRONMENTS,
-                ): 2,
-            }
-        )
+        self.single_lookups = {
+            (1, None): ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name="hello",
+                    version="1.0-1",
+                    type="dpkg",
+                    dsc_fields={"Package": "hello", "Version": "1.0-1"},
+                ),
+            ),
+            (
+                "debian/match:codename=bookworm:architecture=i386:"
+                "format=tarball:backend=unshare",
+                CollectionCategory.ENVIRONMENTS,
+            ): ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+        }
 
         result = schedule_for_worker(self.worker)
 
@@ -648,6 +723,9 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
                 "input_source_artifact_id": 1,
                 "input_binary_artifacts_ids": [],
                 "environment_id": 2,
+                "subject": "hello",
+                "runtime_context": "binary-all+binary-any+source:sid",
+                "configuration_context": "sid",
             },
         )
 
@@ -800,7 +878,7 @@ class ScheduleForWorkerTests(HelperMixin, TestCase):
         self.assertTrue(_method_has_kwargs(_worker_changed))
 
 
-class ScheduleForWorkerTransactionTests(HelperMixin, TransactionTestCase):
+class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
     """Test schedule_for_worker()."""
 
     def setUp(self) -> None:
@@ -1004,8 +1082,31 @@ class ScheduleForWorkerTransactionTests(HelperMixin, TransactionTestCase):
         wr.refresh_from_db()
         self.assertEqual(wr.status, WorkRequest.Statuses.RUNNING)
 
-    def test_populate_not_called_for_subworkflow(self) -> None:
-        """schedule() does not dispatch sub-workflows to Celery."""
+    def test_sub_workflow(self) -> None:
+        """schedule() orchestrates pending sub-workflows via their root."""
+        with context.disable_permission_checks():
+            root_wr = self.playground.create_workflow(task_name="noop")
+            sub_wr = self.playground.create_workflow(parent=root_wr)
+            # Mark the root workflow running, to simulate the situation
+            # where it can only do part of its work and has to be called
+            # back later.
+            root_wr.mark_running()
+
+        with mock.patch(
+            "debusine.server.scheduler.run_workflow_task.apply_async",
+            side_effect=lambda *args, **kwargs: sub_wr.mark_running(),
+        ) as mock_apply_async:
+            schedule()
+
+        self.assert_dispatches_celery_task(
+            mock_apply_async, root_wr, "workflow_noop"
+        )
+        # The (fake) orchestrator was called and marked the sub-workflow as
+        # running.
+        self.assertEqual(sub_wr.status, WorkRequest.Statuses.RUNNING)
+
+    def test_groups_workflows_by_root(self) -> None:
+        """schedule() groups pending workflows by their root workflow."""
         with context.disable_permission_checks():
             root_wr = self.playground.create_workflow(task_name="noop")
             sub_wr = self.playground.create_workflow(parent=root_wr)
@@ -1015,11 +1116,10 @@ class ScheduleForWorkerTransactionTests(HelperMixin, TransactionTestCase):
         ) as mock_apply_async:
             schedule()
 
+        # Only the root workflow's orchestrator is called.
         self.assert_dispatches_celery_task(
             mock_apply_async, root_wr, "workflow_noop"
         )
-
-        # Only the root workflow's work request is marked as running.
         root_wr.refresh_from_db()
         sub_wr.refresh_from_db()
         self.assertEqual(root_wr.status, WorkRequest.Statuses.RUNNING)
