@@ -1,4 +1,4 @@
-# Copyright 2021-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -16,7 +16,12 @@ from django.test import RequestFactory, override_settings
 from django.urls import reverse
 from rest_framework.exceptions import ErrorDetail
 
-from debusine.client.models import LookupChildType
+from debusine.artifacts.models import RuntimeStatistics
+from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
+from debusine.client.models import (
+    LookupChildType,
+    model_to_json_serializable_dict,
+)
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -32,6 +37,7 @@ from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.serializers import (
     ArtifactSerializer,
     ArtifactSerializerResponse,
+    AssetSerializer,
     FileSerializer,
     LookupMultipleSerializer,
     LookupSingleSerializer,
@@ -40,7 +46,7 @@ from debusine.server.serializers import (
     WorkerRegisterSerializer,
     WorkflowTemplateSerializer,
 )
-from debusine.tasks.models import SbuildData, SbuildInput
+from debusine.tasks.models import OutputData, SbuildData, SbuildInput
 from debusine.test import utils
 from debusine.test.django import TestCase
 
@@ -115,8 +121,12 @@ class WorkRequestSerializerTests(TestCase):
         self,
     ) -> None:
         """Assert serialized WorkRequest include artifact ids created by it."""
-        artifact_1, _ = self.create_artifact(work_request=self.work_request)
-        artifact_2, _ = self.create_artifact(work_request=self.work_request)
+        artifact_1, _ = self.playground.create_artifact(
+            work_request=self.work_request
+        )
+        artifact_2, _ = self.playground.create_artifact(
+            work_request=self.work_request
+        )
         serialized = self.work_request_serializer.data
         self.assertEqual(
             serialized["artifacts"], sorted([artifact_1.id, artifact_2.id])
@@ -280,7 +290,7 @@ class WorkRequestCompletedSerializerTests(TestCase):
     """Test for WorkRequestCompletedSerializer class."""
 
     def test_accept_success(self) -> None:
-        """Serializer accept {"result": "success" }."""
+        """Serializer accepts {"result": "success"}."""
         work_request_completed_serializer = WorkRequestCompletedSerializer(
             data={'result': 'success'}
         )
@@ -290,12 +300,40 @@ class WorkRequestCompletedSerializerTests(TestCase):
             'success',
         )
 
+    def test_accept_success_with_output_data(self) -> None:
+        """Serializer accepts output data."""
+        work_request_completed_serializer = WorkRequestCompletedSerializer(
+            data={
+                "result": "success",
+                "output_data": model_to_json_serializable_dict(
+                    OutputData(
+                        runtime_statistics=RuntimeStatistics(duration=60)
+                    ),
+                    exclude_unset=True,
+                ),
+            }
+        )
+        self.assertTrue(work_request_completed_serializer.is_valid())
+        self.assertEqual(
+            work_request_completed_serializer.validated_data['result'],
+            'success',
+        )
+
     def test_not_accept_unknown_result(self) -> None:
         """Serializer does not accept unrecognised result."""
         work_request_completed_serializer = WorkRequestCompletedSerializer(
             data={'result': 'something'}
         )
         self.assertFalse(work_request_completed_serializer.is_valid())
+        self.assertIn("result", work_request_completed_serializer.errors)
+
+    def test_not_accept_invalid_output_data(self) -> None:
+        """Serializer does not accept invalid output data."""
+        work_request_completed_serializer = WorkRequestCompletedSerializer(
+            data={"result": "success", "output_data": {"nonsense": True}}
+        )
+        self.assertFalse(work_request_completed_serializer.is_valid())
+        self.assertIn("output_data", work_request_completed_serializer.errors)
 
 
 def serialized_file() -> dict[str, Any]:
@@ -515,6 +553,93 @@ class ArtifactSerializerResponseTests(TestCase):
         self.assertEqual(actual, expected)
 
 
+class AssetSerializerTests(TestCase):
+    """Tests for AssetSerializer."""
+
+    def setUp(self) -> None:
+        """Configure a context, required by the context."""
+        super().setUp()
+        self.scope = self.playground.get_default_scope()
+        self.user = self.playground.get_default_user()
+        self.workspace = self.playground.get_default_workspace()
+        context.set_scope(self.scope)
+
+    def test_unknown_asset_category(self) -> None:
+        """Deserialize an asset of an unknown category."""
+        serializer = AssetSerializer(
+            data={
+                "category": "unknown",
+                "workspace": self.workspace.name,
+                "data": {},
+                "work_request": 12,
+            },
+        )
+        self.assertFalse(serializer.is_valid())
+        self.assertIn("category", serializer.errors)
+        self.assertEqual(
+            serializer.errors["category"][0], "unknown is not a known category."
+        )
+
+    def test_invalid_data(self) -> None:
+        """Deserialize an asset with invalid data."""
+        serializer = AssetSerializer(
+            data={
+                "category": AssetCategory.SIGNING_KEY,
+                "workspace": self.workspace.name,
+                "data": {},
+                "work_request": 12,
+            },
+        )
+        self.assertFalse(serializer.is_valid())
+        self.assertIn("data", serializer.errors)
+        self.assertTrue(
+            serializer.errors["data"][0].startswith("invalid asset data:")
+        )
+
+    def test_valid_data_work_request(self) -> None:
+        """Deserialize an asset with valid data, created by a work request."""
+        work_request = self.playground.create_work_request()
+        serializer = AssetSerializer(
+            data={
+                "category": AssetCategory.SIGNING_KEY,
+                "workspace": self.workspace.name,
+                "data": SigningKeyData(
+                    description="A Test Key",
+                    fingerprint="ABC123",
+                    public_key="PUBLIC KEY",
+                    purpose=KeyPurpose.OPENPGP,
+                ).dict(),
+                "work_request": work_request.id,
+            },
+        )
+        self.assertTrue(serializer.is_valid())
+        with context.disable_permission_checks():
+            asset = serializer.create(serializer.validated_data)
+        self.assertEqual(asset.created_by_work_request, work_request)
+        self.assertIsNone(asset.created_by)
+
+    def test_valid_data_user(self) -> None:
+        """Deserialize an asset with valid data, created by a user."""
+        context.set_user(self.user)
+        serializer = AssetSerializer(
+            data={
+                "category": AssetCategory.SIGNING_KEY,
+                "workspace": self.workspace.name,
+                "data": SigningKeyData(
+                    description="A Test Key",
+                    fingerprint="ABC123",
+                    public_key="PUBLIC KEY",
+                    purpose=KeyPurpose.OPENPGP,
+                ).dict(),
+            },
+        )
+        self.assertTrue(serializer.is_valid())
+        with context.disable_permission_checks():
+            asset = serializer.create(serializer.validated_data)
+        self.assertIsNone(asset.created_by_work_request)
+        self.assertEqual(asset.created_by, self.user)
+
+
 class LookupSingleSerializerTests(TestCase):
     """Tests for LookupSingleSerializer."""
 
