@@ -151,20 +151,18 @@ class TarArtifactTests(TestCase):
             file_paths, create_files=True
         )
 
-        file_backend = (
-            artifact.workspace.default_file_store.get_backend_object()
-        )
-
         fileobj = artifact.fileinartifact_set.all()[0].file
+        file_backend = artifact.workspace.scope.download_file_backend(fileobj)
 
-        local_path = file_backend.get_local_path(fileobj)
-        assert local_path is not None
-        local_path.unlink()
+        entry = file_backend.get_entry(fileobj)
+        entry.remove()
 
         tar_artifact = TarArtifact(artifact=artifact)
 
         with (
-            self.assertLogsContains(str(local_path), expected_count=2),
+            self.assertLogsContains(
+                str(entry.get_local_path()), expected_count=2
+            ),
             self.assertRaises(StopIteration),
         ):
             tar_artifact.__next__()
