@@ -10,16 +10,13 @@
 """Views for the server application: artifacts."""
 
 import binascii
-import copy
 import logging
 import tempfile
 from collections.abc import Mapping
 from pathlib import Path
-from typing import Any, Generic, IO, Literal, TypeVar
+from typing import Any, IO, Literal
 
 from django.conf import settings
-from django.core.exceptions import ObjectDoesNotExist
-from django.db.models import Model
 from django.http import Http404
 from django.utils import timezone
 from rest_framework import status
@@ -51,12 +48,12 @@ from debusine.server.serializers import (
 from debusine.server.views.base import (
     BaseAPIView,
     CanDisplayFilterBackend,
-    CreateAPIViewBase,
     DestroyAPIViewBase,
     GenericAPIViewBase,
-    IsTokenAuthenticated,
+    GetOrCreateAPIView,
     ListAPIViewBase,
 )
+from debusine.server.views.rest import IsTokenAuthenticated
 from debusine.utils import parse_content_range_header
 
 logger = logging.getLogger(__name__)
@@ -111,7 +108,7 @@ class ArtifactView(GenericAPIViewBase[Artifact], BaseAPIView):
             artifact=artifact, file=fileobj, path=path, complete=False
         )
 
-        if created or not artifact.workspace.is_file_in_workspace(fileobj):
+        if created or artifact.workspace.file_needs_upload(fileobj):
             if file_size == 0:
                 # Create the file upload and add the file in the store
                 temp_empty_file = tempfile.NamedTemporaryFile(
@@ -467,8 +464,8 @@ class UploadFileView(BaseAPIView):
                 status_code=status.HTTP_409_CONFLICT,
             )
 
-        workspace = file_in_artifact.artifact.workspace
-        file_backend = workspace.default_file_store.get_backend_object()
+        scope = file_in_artifact.artifact.workspace.scope
+        file_backend = scope.upload_file_backend(file_in_artifact.file)
 
         file_backend.add_file(file_path, fileobj=file_in_artifact.file)
         file_in_artifact.complete = True
@@ -477,54 +474,6 @@ class UploadFileView(BaseAPIView):
         return Response(status=status.HTTP_201_CREATED)
 
 
-T = TypeVar("T", bound=Model, covariant=True)
-
-
-class GetOrCreateAPIView(CreateAPIViewBase[T], BaseAPIView, Generic[T]):
-    """Class to create a new object (return 201 created) or return 200."""
-
-    def get_object(self) -> T:
-        """Override to return more API-friendly errors."""
-        try:
-            return super().get_object()
-        except Http404 as exc:
-            raise DebusineAPIException(
-                title=str(exc), status_code=status.HTTP_404_NOT_FOUND
-            )
-
-    def create(self, request: Request, *args: Any, **kwargs: Any) -> Response:
-        """
-        Return the object with HTTP 201 (created) or HTTP 200 (already existed).
-
-        It is a modification of CreateAPIView.create().
-
-        The default behaviour in rest_framework when an object cannot be
-        inserted because it already exists is to return HTTP 400. In
-        GetOrCreateApiView it returns HTTP 200 and the client knows that
-        the object that wanted to create was already in the database.
-        """
-        serializer = self.get_serializer(data=request.data)
-        serializer.is_valid(raise_exception=True)
-
-        response_status: int
-        try:
-            obj = self.get_queryset().get(**serializer.validated_data)
-            response_status = status.HTTP_200_OK
-            pk = obj.pk
-        except ObjectDoesNotExist:
-            self.perform_create(serializer)
-            response_status = status.HTTP_201_CREATED
-            assert serializer.instance is not None
-            pk = serializer.instance.pk
-
-        headers = self.get_success_headers(serializer.data)
-
-        response_data = copy.deepcopy(serializer.data)
-        response_data["id"] = pk
-
-        return Response(response_data, status=response_status, headers=headers)
-
-
 class ArtifactRelationsView(
     GetOrCreateAPIView[ArtifactRelation],
     DestroyAPIViewBase[ArtifactRelation],
