@@ -26,7 +26,12 @@ from django.utils import timezone
 from django.utils.http import urlencode
 from rest_framework import status
 
-from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    ArtifactData,
+    DebianSourcePackage,
+    DebianUpload,
+)
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
@@ -90,7 +95,11 @@ class WorkRequestTestCase(TestCase):
                 'duration': work_request.duration,
                 'worker': work_request.worker_id,
                 'task_type': work_request.task_type,
-                'task_data': work_request.task_data,
+                'task_data': (
+                    work_request.configured_task_data
+                    if work_request.configured_task_data
+                    else work_request.task_data
+                ),
                 'dynamic_task_data': (
                     dynamic_task_data
                     if work_request.dynamic_task_data is None
@@ -231,7 +240,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
-        cls.artifact, _ = cls.playground.create_artifact()
+        cls.artifact = cls.playground.create_source_artifact()
 
     def create_test_work_request(
         self, workspace: Workspace | None = None
@@ -423,7 +432,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
         """Work requests in private workspaces 200 to the authorized."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         work_request = self.create_test_work_request(
             workspace=private_workspace
@@ -457,8 +466,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
                     'source_artifact': input_artifact_id or self.artifact.id
                 },
                 'host_architecture': 'amd64',
-                'backend': 'schroot',
-                'distribution': 'bookworm',
+                'environment': 'debian/match:codename=bookworm',
             },
         }
 
@@ -504,10 +512,15 @@ class WorkRequestViewTests(WorkRequestTestCase):
             workspace.name = "Testing"
             workspace.save()
             self.playground.create_group_role(
-                workspace, Workspace.Roles.OWNER, self.scenario.user
+                workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
+            )
+            self.playground.create_debian_environment(
+                codename="bookworm", workspace=workspace
             )
 
-            artifact, _ = self.playground.create_artifact(workspace=workspace)
+            artifact = self.playground.create_source_artifact(
+                workspace=workspace
+            )
         self.verify_post_success(
             workspace=workspace, input_artifact_id=artifact.id
         )
@@ -516,6 +529,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
     def test_post_success_workspace_default(self) -> None:
         """Client POST a WorkRequest without specifying a workspace."""
         self.scenario.workspace_owners.users.add(self.scenario.user)
+        self.playground.create_debian_environment(codename="bookworm")
         self.verify_post_success()
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=False)
@@ -524,6 +538,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
         self.token = self.create_superuser_token()
         assert self.token.user is not None
         self.scenario.workspace_owners.users.add(self.token.user)
+        self.playground.create_debian_environment(codename="bookworm")
         self.verify_post_success()
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=False)
@@ -541,7 +556,10 @@ class WorkRequestViewTests(WorkRequestTestCase):
 
         for workspace in (workspace1, workspace2):
             self.playground.create_group_role(
-                workspace, Workspace.Roles.OWNER, self.scenario.user
+                workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
+            )
+            self.playground.create_debian_environment(
+                codename="bookworm", workspace=workspace
             )
             self.verify_post_success(workspace=workspace)
 
@@ -860,7 +878,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
         # A user with permission to see the private workspace sees its work
         # requests.
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         response = self.client.get(
             reverse("api:work-requests"),
@@ -1035,7 +1053,7 @@ class WorkRequestViewTests(WorkRequestTestCase):
         """Work requests in private workspaces 200 to the authorized."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         work_request = self.create_test_work_request(
             workspace=private_workspace
@@ -1305,7 +1323,7 @@ class WorkRequestRetryViewTests(TestCase):
         """Retrying in a private workspace succeeds with `can_display`."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         self.work_request.workspace = private_workspace
         self.work_request.status = WorkRequest.Statuses.ABORTED
@@ -1607,7 +1625,7 @@ class WorkRequestUnblockViewTests(WorkRequestTestCase):
         self.work_request.workspace = workspace
         self.work_request.save()
         self.playground.create_group_role(
-            workspace, Workspace.Roles.OWNER, self.scenario.user
+            workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
 
         response = self.post(
@@ -1652,7 +1670,7 @@ class WorkRequestUnblockViewTests(WorkRequestTestCase):
         """Unblocking in a private workspace succeeds with `can_display`."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         self.work_request.workspace = private_workspace
         self.work_request.save()
@@ -1686,6 +1704,13 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
         cls.unsigned, _ = cls.playground.create_artifact(
             paths=cls.unsigned_files,
             category=ArtifactCategory.UPLOAD,
+            data=DebianUpload(
+                type="dpkg",
+                changes_fields={
+                    "Architecture": "source",
+                    "Files": [{"name": name} for name in cls.unsigned_files],
+                },
+            ),
             create_files=True,
             skip_add_files_in_store=True,
         )
@@ -1776,6 +1801,7 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
         cls,
         paths: list[str] | dict[str, bytes] | None = None,
         category: ArtifactCategory = ArtifactCategory.UPLOAD,
+        data: ArtifactData | None = None,
     ) -> Artifact:
         """
         Create a signed upload artifact.
@@ -1786,16 +1812,24 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
         :param category: the upload artifact category, defaulting to
           `debian:upload`
         """
+        if paths is None:
+            paths = {
+                **cls.unsigned_files,
+                "foo_1.0.dsc": b"signed dsc",
+                "foo_1.0_source.changes": b"signed changes",
+            }
+        if data is None:
+            data = DebianUpload(
+                type="dpkg",
+                changes_fields={
+                    "Architecture": "source",
+                    "Files": [{"name": name} for name in paths],
+                },
+            )
         signed, _ = cls.playground.create_artifact(
-            paths=(
-                paths
-                or {
-                    **cls.unsigned_files,
-                    "foo_1.0.dsc": b"signed dsc",
-                    "foo_1.0_source.changes": b"signed changes",
-                }
-            ),
+            paths=paths,
             category=category,
+            data=data,
             create_files=True,
             skip_add_files_in_store=True,
         )
@@ -1948,7 +1982,7 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
         """Work requests in private workspaces 200 to the authorized."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         self.work_request.workspace = private_workspace
         self.work_request.save()
@@ -2066,7 +2100,10 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
     def test_post_wrong_category(self) -> None:
         """A `POST` request checks the category of the signed artifact."""
         signed = self.create_signed_artifact(
-            category=ArtifactCategory.SOURCE_PACKAGE
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data=DebianSourcePackage(
+                name="hello", version="1.0", type="dpkg", dsc_fields={}
+            ),
         )
 
         response = self.post(signed=signed)
@@ -2236,7 +2273,7 @@ class WorkRequestExternalDebsignViewTests(WorkRequestTestCase):
         """`POST` in a private workspace succeeds with `can_display`."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         self.work_request.workspace = private_workspace
         self.work_request.save()
