@@ -11,8 +11,11 @@
 
 from functools import cached_property
 
-from debusine.artifacts.models import ArtifactCategory
-from debusine.db.models import WorkRequest
+from debusine.artifacts import SourcePackage
+from debusine.artifacts.models import ArtifactCategory, DebianSourcePackage
+from debusine.client.models import LookupChildType
+from debusine.db.models import Artifact, WorkRequest
+from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows.base import Workflow, WorkflowValidationError
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
@@ -24,6 +27,7 @@ from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class AutopkgtestWorkflow(
@@ -99,7 +103,7 @@ class AutopkgtestWorkflow(
             fail_on=self.data.fail_on,
             timeout=self.data.timeout,
         )
-        wr = self.work_request.create_child(
+        wr = self.work_request_ensure_child(
             task_name="autopkgtest",
             task_data=task_data,
             workflow_data=WorkRequestWorkflowData(
@@ -107,6 +111,7 @@ class AutopkgtestWorkflow(
                 step=f"autopkgtest-{architecture}",
             ),
         )
+        self.requires_artifact(wr, self.data.source_artifact)
         self.requires_artifact(wr, filtered_binary_artifacts)
         self.requires_artifact(wr, filtered_context_artifacts)
         self.provides_artifact(
@@ -136,6 +141,34 @@ class AutopkgtestWorkflow(
             for architecture in new_architectures:
                 self._populate_single(architecture)
 
+    # TODO refactor with QaWorkflow, DebianPipelineWorkflow and SbuildWorkflow
+    @cached_property
+    def source_package(self) -> Artifact:
+        """Source package artifact."""
+        artifact = lookup_single(
+            self.data.source_artifact,
+            self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT,
+        ).artifact
+        return self.locate_debian_source_package("source_artifact", artifact)
+
+    # TODO refactor with QaWorkflow, DebianPipelineWorkflow and SbuildWorkflow
+    @cached_property
+    def source_package_data(self) -> DebianSourcePackage:
+        """Parsed source package artifact data."""
+        return SourcePackage.create_data(self.source_package.data)
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        source_data = self.source_package_data
+        return BaseDynamicTaskData(
+            subject=source_data.name,
+            parameter_summary=f"{source_data.name}_{source_data.version}",
+        )
+
     def get_label(self) -> str:
         """Return the task label."""
         # TODO: copy the source package information in dynamic task data and
