@@ -13,9 +13,9 @@ See :ref:`explanation-workflows` for a high level explanation of concepts used
 here.
 """
 import logging
-import typing
 from abc import ABCMeta, abstractmethod
-from collections.abc import Iterable
+from collections.abc import Collection as AbcCollection
+from collections.abc import Iterable, Sequence
 from typing import Any, TypeVar, TypedDict, cast
 
 from django.contrib.auth.models import AnonymousUser
@@ -26,7 +26,10 @@ from debusine.artifacts.models import (
     CollectionCategory,
     DebianBinaryPackage,
     DebianBinaryPackages,
+    DebianPackageBuildLog,
+    DebianSourcePackage,
     DebianUpload,
+    get_source_package_name,
 )
 from debusine.client.models import (
     LookupChildType,
@@ -105,7 +108,7 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
     def __init__(self, work_request: "WorkRequest"):
         """Instantiate a Workflow with its database instance."""
         super().__init__(
-            task_data=work_request.task_data,
+            task_data=work_request.used_task_data,
             dynamic_task_data=work_request.dynamic_task_data,
         )
         self.set_work_request(work_request)
@@ -243,38 +246,42 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         # work_request is part of a workflow
         assert work_request.parent is not None
 
-        work_request.process_update_collection_with_data(
-            [
-                ActionUpdateCollectionWithData(
-                    collection="internal@collections",
-                    name_template=name,
-                    category=BareDataCategory.PROMISE,
-                    data={
-                        "promise_work_request_id": work_request.id,
-                        "promise_workflow_id": work_request.parent.id,
-                        "promise_category": category,
-                        **(data or {}),
-                    },
-                )
-            ]
-        )
-
-        event_reactions = work_request.event_reactions
+        try:
+            lookup_single(
+                f"internal@collections/{name}",
+                work_request.workspace,
+                user=work_request.created_by or AnonymousUser(),
+                workflow_root=work_request.get_workflow_root(),
+                expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+            )
+        except KeyError:
+            work_request.process_update_collection_with_data(
+                [
+                    ActionUpdateCollectionWithData(
+                        collection="internal@collections",
+                        name_template=name,
+                        category=BareDataCategory.PROMISE,
+                        data={
+                            "promise_work_request_id": work_request.id,
+                            "promise_workflow_id": work_request.parent.id,
+                            "promise_category": category,
+                            **(data or {}),
+                        },
+                    )
+                ]
+            )
 
         artifact_filters = artifact_filters or {}
-
-        event_reactions.on_success.append(
+        work_request.add_event_reaction(
+            "on_success",
             ActionUpdateCollectionWithArtifacts(
                 collection="internal@collections",
                 name_template=name,
                 variables=data,
                 artifact_filters={**artifact_filters, "category": category},
-            )
+            ),
         )
 
-        work_request.event_reactions = event_reactions
-        work_request.save()
-
     @staticmethod
     def requires_artifact(
         work_request: WorkRequest, lookup: LookupSingle | LookupMultiple
@@ -350,10 +357,16 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
                 task_type=task_type,
                 # If we're creating a sub-workflow, then the parent must
                 # already be running and therefore the child should be
-                # running too.  Any dependencies will be handled at the
-                # level of individual non-workflow work requests.
+                # unblocked.  In most cases dependencies will be handled at
+                # the level of individual non-workflow work requests;
+                # however, in some cases the sub-workflow itself needs a
+                # dependency because it needs more information before being
+                # able to create its own child work requests, so we leave
+                # the sub-workflow pending to give the parent orchestrator a
+                # chance to add those dependencies before marking the
+                # sub-workflow running.
                 status=(
-                    WorkRequest.Statuses.RUNNING
+                    WorkRequest.Statuses.PENDING
                     if task_type == TaskTypes.WORKFLOW
                     else WorkRequest.Statuses.BLOCKED
                 ),
@@ -471,17 +484,6 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         """Unused abstract method from BaseTask: populate() is used instead."""
         raise NotImplementedError()
 
-    @staticmethod
-    def ensure_artifact_categories(
-        artifact: Artifact, categories: typing.Collection[ArtifactCategory]
-    ) -> None:
-        """Ensure that artifact has a category in categories."""
-        if artifact.category not in categories:
-            raise TaskConfigError(
-                f'Category of "{artifact}" is not one of '
-                f"{[category.value for category in categories]}"
-            )
-
     @staticmethod
     def follow_artifact_relation(
         artifact: Artifact,
@@ -507,15 +509,18 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         return relation.target
 
     @classmethod
-    def locate_debian_source_package(cls, artifact: Artifact) -> Artifact:
+    def locate_debian_source_package(
+        cls, configuration_key: str, artifact: Artifact
+    ) -> Artifact:
         """
         Accept a debian:upload or debian:source-package in a workflow.
 
         Resolve to the debian:source-package.
         """
         cls.ensure_artifact_categories(
-            artifact,
-            [ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
+            configuration_key=configuration_key,
+            category=artifact.category,
+            expected=[ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD],
         )
         match artifact.category:
             case ArtifactCategory.SOURCE_PACKAGE:
@@ -532,10 +537,10 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
                 )
 
     def locate_debian_source_package_lookup(
-        self, lookup: LookupSingle
+        self, configuration_key: str, lookup: LookupSingle
     ) -> LookupSingle:
         """
-        Return a lookup to a debian:source-packge.
+        Return a lookup to a debian:source-package.
 
         If the specified lookup returns a debian:source-package, return it.
         If it returns a debian:upload, find the related debian:source-package
@@ -549,13 +554,97 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
         if artifact.category == ArtifactCategory.UPLOAD:
-            source_package = self.locate_debian_source_package(artifact)
+            source_package = self.locate_debian_source_package(
+                configuration_key, artifact
+            )
             return f"{source_package.id}@artifacts"
         self.ensure_artifact_categories(
-            artifact, [ArtifactCategory.SOURCE_PACKAGE]
+            configuration_key=configuration_key,
+            category=artifact.category,
+            expected=[ArtifactCategory.SOURCE_PACKAGE],
         )
         return lookup
 
+    @classmethod
+    def get_source_package_names(
+        cls,
+        results: Sequence[LookupResult],
+        *,
+        configuration_key: str,
+        artifact_expected_categories: AbcCollection[ArtifactCategory],
+    ) -> list[str]:
+        """
+        Return a sorted list of source package names from results.
+
+        It ensures that:
+
+        - The `LookupResult` objects contain either an artifact or promise.
+        - Artifacts belong to the artifact_expected_categories.
+        - If `LookupResult` is a promise: extracts the name from the promise
+          data "source_package_name".
+
+        :param results: A sequence of `LookupResult` objects representing
+          artifacts to be processed. Each entry is expected to be either an
+          artifact or a promise.
+        :param configuration_key: A string used by
+          `cls.ensure_artifact_categories` for the exception message.
+        :param artifact_expected_categories: valid `ArtifactCategory` that
+          artifacts must belong to.
+        :return: A sorted list of source package names.
+        """
+        source_package_names = set()
+
+        for result in results:
+            # lookup_multiple expect_type: only artifacts or promises
+            match result.result_type:
+                case CollectionItem.Types.ARTIFACT:
+                    assert result.artifact is not None
+                    category = result.artifact.category
+
+                    cls.ensure_artifact_categories(
+                        configuration_key=configuration_key,
+                        category=category,
+                        expected=artifact_expected_categories,
+                    )
+                    artifact_data = result.artifact.create_data()
+
+                    assert isinstance(
+                        artifact_data,
+                        (
+                            DebianSourcePackage,
+                            DebianUpload,
+                            DebianBinaryPackage,
+                            DebianBinaryPackages,
+                            DebianPackageBuildLog,
+                        ),
+                    )
+                    source_package_names.add(
+                        get_source_package_name(artifact_data)
+                    )
+
+                case _:
+                    # Makes coverage happy
+                    # It's a promise.
+                    assert result.result_type == CollectionItem.Types.BARE
+                    assert result.collection_item is not None
+
+                    cls.ensure_artifact_categories(
+                        configuration_key=configuration_key,
+                        category=result.collection_item.data[
+                            "promise_category"
+                        ],
+                        expected=artifact_expected_categories,
+                    )
+
+                    if (
+                        package_name := result.collection_item.data.get(
+                            "source_package_name"
+                        )
+                    ) is not None:
+                        source_package_names.add(package_name)
+
+        return sorted(source_package_names)
+
 
 def orchestrate_workflow(work_request: WorkRequest) -> None:
     """
