@@ -12,6 +12,7 @@
 import logging
 
 from celery import shared_task
+from django.db import transaction
 
 from debusine.db.models import WorkRequest
 from debusine.server.workflows.base import orchestrate_workflow
@@ -23,6 +24,7 @@ logger = logging.getLogger(__name__)
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
 @shared_task  # type: ignore[misc]
+@transaction.atomic
 def run_workflow_task(work_request_id: int) -> bool:
     """Run a workflow callback or workflow in Celery."""
     try:
