@@ -19,7 +19,8 @@ except ImportError:
     import pydantic  # type: ignore
 
 from debusine.artifacts import SourcePackage
-from debusine.artifacts.models import DebianSourcePackage, KeyPurpose
+from debusine.artifacts.models import DebianSourcePackage
+from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact
 from debusine.server.collections.lookup import lookup_single
@@ -62,7 +63,7 @@ class DebianPipelineWorkflow(
             user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
-        return self.locate_debian_source_package(artifact)
+        return self.locate_debian_source_package("source_artifact", artifact)
 
     @cached_property
     def source_package_data(self) -> DebianSourcePackage:
@@ -75,7 +76,8 @@ class DebianPipelineWorkflow(
         """Compute dynamic data for this workflow."""
         source_data = self.source_package_data
         return BaseDynamicTaskData(
-            parameter_summary=f"{source_data.name}_{source_data.version}"
+            subject=source_data.name,
+            parameter_summary=f"{source_data.name}_{source_data.version}",
         )
 
     def populate(self) -> None:
@@ -219,7 +221,7 @@ class DebianPipelineWorkflow(
                 step="sbuild",
             ),
         )
-
+        wr.mark_running()
         orchestrate_workflow(wr)
 
     def _populate_qa(
@@ -272,9 +274,10 @@ class DebianPipelineWorkflow(
                 step="qa",
             ),
         )
-
-        self.requires_artifact(wr, binary_artifacts)
-
+        # The qa workflow's children will have dependencies on the work
+        # requests creating binary_artifacts, but the qa workflow itself
+        # doesn't need that in order to populate itself.
+        wr.mark_running()
         orchestrate_workflow(wr)
 
     def _populate_make_signed_source(
@@ -285,7 +288,7 @@ class DebianPipelineWorkflow(
         binary_artifacts: LookupMultiple,
         architectures: list[str],
         purpose: KeyPurpose,
-        key: LookupSingle,
+        key: str,
         sbuild_backend: BackendType,
         signing_template_names: dict[str, list[str]],
     ) -> None:
@@ -308,14 +311,13 @@ class DebianPipelineWorkflow(
 
                 signing_artifacts.append(signing_artifact)
 
+        signing_template_artifacts = LookupMultiple.parse_obj(signing_artifacts)
         wr = self.work_request_ensure_child(
             task_name="make_signed_source",
             task_type=TaskTypes.WORKFLOW,
             task_data=MakeSignedSourceWorkflowData(
                 binary_artifacts=binary_artifacts,
-                signing_template_artifacts=LookupMultiple.parse_obj(
-                    signing_artifacts
-                ),
+                signing_template_artifacts=signing_template_artifacts,
                 vendor=vendor,
                 codename=codename,
                 architectures=architectures,
@@ -327,10 +329,11 @@ class DebianPipelineWorkflow(
                 display_name="make signed source", step="make_signed_source"
             ),
         )
-
-        self.requires_artifact(wr, binary_artifacts)
-        self.requires_artifact(wr, LookupMultiple.parse_obj(signing_artifacts))
-
+        # The make_signed_source workflow's children will have dependencies
+        # on the work requests creating binary_artifacts and
+        # signing_artifacts, but the make_signed_source workflow itself
+        # doesn't need that in order to populate itself.
+        wr.mark_running()
         orchestrate_workflow(wr)
 
     def _populate_package_upload(
@@ -342,7 +345,7 @@ class DebianPipelineWorkflow(
         include_binaries: bool,
         merge_uploads: bool,
         since_version: str | None,
-        key: LookupSingle | None,
+        key: str | None,
         require_signature: bool,
         target: str,
         delayed_days: int | None,
@@ -378,9 +381,11 @@ class DebianPipelineWorkflow(
                 display_name="package upload", step="package_upload"
             ),
         )
-
-        self.requires_artifact(wr, binary_artifacts)
-
+        # The package_upload workflow's children will have dependencies on
+        # the work requests creating binary_artifacts, but the
+        # package_upload workflow itself doesn't need that in order to
+        # populate itself.
+        wr.mark_running()
         orchestrate_workflow(wr)
 
     def get_label(self) -> str:
