@@ -8,7 +8,13 @@
 # contained in the LICENSE file.
 
 """lintian workflow."""
-from debusine.db.models import WorkRequest
+from functools import cached_property
+
+from debusine.artifacts import SourcePackage
+from debusine.artifacts.models import DebianSourcePackage
+from debusine.client.models import LookupChildType
+from debusine.db.models import Artifact, WorkRequest
+from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import (
     LintianWorkflowData,
@@ -24,6 +30,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     LookupSingle,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class LintianWorkflow(Workflow[LintianWorkflowData, BaseDynamicTaskData]):
@@ -101,6 +108,34 @@ class LintianWorkflow(Workflow[LintianWorkflowData, BaseDynamicTaskData]):
 
         return wr
 
+    # TODO refactor with QaWorkflow, DebianPipelineWorkflow and SbuildWorkflow
+    @cached_property
+    def source_package(self) -> Artifact:
+        """Source package artifact."""
+        artifact = lookup_single(
+            self.data.source_artifact,
+            self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT,
+        ).artifact
+        return self.locate_debian_source_package("source_artifact", artifact)
+
+    # TODO refactor with QaWorkflow, DebianPipelineWorkflow and SbuildWorkflow
+    @cached_property
+    def source_package_data(self) -> DebianSourcePackage:
+        """Parsed source package artifact data."""
+        return SourcePackage.create_data(self.source_package.data)
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        source_data = self.source_package_data
+        return BaseDynamicTaskData(
+            subject=source_data.name,
+            parameter_summary=f"{source_data.name}_{source_data.version}",
+        )
+
     def get_label(self) -> str:
         """Return the task label."""
         return "run lintian"
