@@ -9,10 +9,13 @@
 
 """Make signed source workflow."""
 
-from debusine.artifacts.models import ArtifactCategory, KeyPurpose
+from debusine.artifacts.models import ArtifactCategory
+from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import WorkRequest
+from debusine.server.collections.lookup import lookup_multiple
 from debusine.server.workflows import Workflow
+from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     MakeSignedSourceWorkflowData,
     SbuildWorkflowData,
@@ -32,6 +35,7 @@ from debusine.tasks.models import (
     SbuildInput,
     TaskTypes,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class MakeSignedSourceWorkflow(
@@ -45,6 +49,31 @@ class MakeSignedSourceWorkflow(
         """Instantiate a Workflow with its database instance."""
         super().__init__(work_request)
 
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data."""
+        binaries = lookup_multiple(
+            self.data.binary_artifacts,
+            workflow_root=self.work_request.get_workflow_root(),
+            workspace=self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+
+        source_package_names = self.get_source_package_names(
+            binaries,
+            configuration_key="binary_artifacts",
+            artifact_expected_categories=(
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.BINARY_PACKAGES,
+                ArtifactCategory.UPLOAD,
+            ),
+        )
+        return BaseDynamicTaskData(
+            subject=" ".join(source_package_names),
+        )
+
     def populate(self) -> None:
         """Create work requests and sub-workflows."""
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
@@ -70,7 +99,7 @@ class MakeSignedSourceWorkflow(
                 # single lookups.
                 assert not isinstance(signing_template_lookup, LookupDict)
 
-                extracted_work_request, extracted_lookup = (
+                extract_for_signing_wr, extracted_lookup = (
                     self._populate_extract_for_signing(
                         template_artifact=signing_template_lookup,
                         binary_artifacts=binary_artifacts_lookups,
@@ -79,11 +108,11 @@ class MakeSignedSourceWorkflow(
                     )
                 )
 
-                signed = self._populate_sign(
+                sign_wr, signed = self._populate_sign(
                     purpose=self.data.purpose,
                     unsigned=extracted_lookup,
                     key=self.data.key,
-                    extracted=extracted_work_request,
+                    extract_for_signing_wr=extract_for_signing_wr,
                     architecture=arch,
                 )
 
@@ -91,6 +120,7 @@ class MakeSignedSourceWorkflow(
                     environment=environment,
                     template=signing_template_lookup,
                     signed=signed,
+                    sign_wr=sign_wr,
                     architecture=arch,
                 )
 
@@ -136,8 +166,8 @@ class MakeSignedSourceWorkflow(
         self.requires_artifact(wr, template_artifact)
         self.requires_artifact(wr, binary_artifacts)
 
-        event_reactions = wr.event_reactions
-        event_reactions.on_success.append(
+        wr.add_event_reaction(
+            "on_success",
             ActionUpdateCollectionWithArtifacts(
                 collection="internal@collections",
                 name_template=(
@@ -150,10 +180,8 @@ class MakeSignedSourceWorkflow(
                     "architecture": architecture,
                 },
                 artifact_filters={"category": ArtifactCategory.SIGNING_INPUT},
-            )
+            ),
         )
-        wr.event_reactions = event_reactions
-        wr.save()
         lookup = LookupMultiple.parse_obj(
             {
                 "collection": "internal@collections",
@@ -170,14 +198,15 @@ class MakeSignedSourceWorkflow(
         *,
         purpose: KeyPurpose,
         unsigned: LookupMultiple,
-        key: LookupSingle,
-        extracted: WorkRequest,
+        key: str,
+        extract_for_signing_wr: WorkRequest,
         architecture: str,
-    ) -> LookupSingle:
+    ) -> tuple[WorkRequest, LookupMultiple]:
         """
-        Create work request for Signing.
+        Create work request for Sign.
 
-        :returns: Lookup of the signed artifact in the collection.
+        :returns: A tuple of the Sign work request and a lookup of the
+          signed artifacts in the collection.
         """
         wr = self.work_request_ensure_child(
             task_type=TaskTypes.SIGNING,
@@ -189,20 +218,40 @@ class MakeSignedSourceWorkflow(
             ),
         )
 
-        wr.add_dependency(extracted)
+        wr.add_dependency(extract_for_signing_wr)
 
-        artifact_name = f"signed-{architecture}"
-        self.provides_artifact(
-            wr, ArtifactCategory.SIGNING_OUTPUT, artifact_name
+        wr.add_event_reaction(
+            "on_success",
+            ActionUpdateCollectionWithArtifacts(
+                collection="internal@collections",
+                name_template="signed-{architecture}-{binary_package_name}",
+                variables={
+                    # Extract the binary package name from the artifact
+                    # data, since the workflow doesn't know it up-front.
+                    "$binary_package_name": "binary_package_name",
+                    "architecture": architecture,
+                },
+                artifact_filters={"category": ArtifactCategory.SIGNING_OUTPUT},
+            ),
         )
-        return f"internal@collections/name:{artifact_name}"
+        lookup = LookupMultiple.parse_obj(
+            {
+                "collection": "internal@collections",
+                "child_type": LookupChildType.ARTIFACT,
+                "category": ArtifactCategory.SIGNING_OUTPUT,
+                "name__startswith": "signed-",
+                "data__architecture": architecture,
+            }
+        )
+        return wr, lookup
 
     def _populate_assemble_signed_source(
         self,
         *,
         environment: str,
         template: LookupSingle,
-        signed: LookupSingle,
+        signed: LookupMultiple,
+        sign_wr: WorkRequest,
         architecture: str,
     ) -> LookupSingle:
         """
@@ -213,9 +262,7 @@ class MakeSignedSourceWorkflow(
         wr = self.work_request_ensure_child(
             task_name="assemblesignedsource",
             task_data=AssembleSignedSourceData(
-                environment=environment,
-                template=template,
-                signed=LookupMultiple.parse_obj([signed]),
+                environment=environment, template=template, signed=signed
             ),
             workflow_data=WorkRequestWorkflowData(
                 display_name=f"Assemble signed source {architecture}",
@@ -223,7 +270,7 @@ class MakeSignedSourceWorkflow(
             ),
         )
 
-        self.requires_artifact(wr, signed)
+        wr.add_dependency(sign_wr)
 
         artifact_name = f"signed-source-{architecture}"
         self.provides_artifact(
@@ -258,6 +305,10 @@ class MakeSignedSourceWorkflow(
 
         self.requires_artifact(wr, assembled)
 
+        if wr.status == WorkRequest.Statuses.PENDING:
+            wr.mark_running()
+            orchestrate_workflow(wr)
+
     def get_label(self) -> str:
         """Return the task label."""
         return "run sign source"
