@@ -9,9 +9,9 @@
 
 """Package publishing workflow."""
 
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.client.models import LookupChildType
-from debusine.server.collections.lookup import lookup_single
+from debusine.server.collections.lookup import lookup_multiple, lookup_single
 from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
@@ -22,6 +22,7 @@ from debusine.server.workflows.models import (
     WorkRequestWorkflowData,
 )
 from debusine.tasks.models import BaseDynamicTaskData, LookupMultiple, TaskTypes
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class PackagePublishWorkflow(
@@ -101,6 +102,28 @@ class PackagePublishWorkflow(
         )
         self.requires_artifact(wr, copies[0].source_items)
 
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        binaries = lookup_multiple(
+            self.data.binary_artifacts,
+            workflow_root=self.work_request.get_workflow_root(),
+            workspace=self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+
+        source_package_names = self.get_source_package_names(
+            binaries,
+            configuration_key="binary_artifacts",
+            artifact_expected_categories=(ArtifactCategory.UPLOAD,),
+        )
+
+        return BaseDynamicTaskData(
+            subject=" ".join(source_package_names),
+        )
+
     def get_label(self) -> str:
         """Return the task label."""
         return f"publish to {self.data.target_suite}"
