@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -44,6 +44,7 @@ from debusine.tasks.models import (
     TaskTypes,
     empty_lookup_multiple,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class PackageUploadWorkflow(
@@ -141,6 +142,52 @@ class PackageUploadWorkflow(
 
         self._populate_package_uploads()
 
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        binaries = lookup_multiple(
+            self.data.binary_artifacts,
+            workflow_root=self.work_request.get_workflow_root(),
+            workspace=self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+        source_package_names = self.get_source_package_names(
+            binaries,
+            configuration_key="binary_artifacts",
+            artifact_expected_categories=(
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.BINARY_PACKAGES,
+                ArtifactCategory.UPLOAD,
+            ),
+        )
+
+        if self.data.source_artifact is not None:
+            source_artifact = lookup_single(
+                self.data.source_artifact,
+                workflow_root=self.work_request.get_workflow_root(),
+                workspace=self.workspace,
+                user=self.work_request.created_by,
+                expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+            )
+
+            source_artifact_names = self.get_source_package_names(
+                [source_artifact],
+                configuration_key="source_artifact",
+                artifact_expected_categories=(
+                    ArtifactCategory.SOURCE_PACKAGE,
+                    ArtifactCategory.UPLOAD,
+                ),
+            )
+            source_package_names = sorted(
+                set(source_package_names) | set(source_artifact_names)
+            )
+
+        return BaseDynamicTaskData(
+            subject=" ".join(source_package_names),
+        )
+
     def _populate_make_source_package_upload(
         self,
         lookup: LookupSingle,
@@ -179,7 +226,7 @@ class PackageUploadWorkflow(
         self,
         signer_identifier: str,
         unsigned_artifact: LookupSingle,
-        key: LookupSingle | None,
+        key: str | None,
     ) -> LookupSingle:
         """
         Create a signer: Debsign or ExternalDebsign depending on self.data.
