@@ -8,6 +8,9 @@
 # contained in the LICENSE file.
 
 """Piuparts workflow."""
+from debusine.artifacts.models import ArtifactCategory
+from debusine.client.models import LookupChildType
+from debusine.server.collections.lookup import lookup_multiple
 from debusine.server.workflows import Workflow
 from debusine.server.workflows.models import (
     PiupartsWorkflowData,
@@ -18,6 +21,7 @@ from debusine.tasks.models import (
     PiupartsData,
     PiupartsDataInput,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
@@ -98,6 +102,31 @@ class PiupartsWorkflow(Workflow[PiupartsWorkflowData, BaseDynamicTaskData]):
             # an architecture field in their data.
             self.requires_artifact(wr, arch_subset_binary_artifacts)
 
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        binaries = lookup_multiple(
+            self.data.binary_artifacts,
+            workflow_root=self.work_request.get_workflow_root(),
+            workspace=self.workspace,
+            user=self.work_request.created_by,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+        source_package_names = self.get_source_package_names(
+            binaries,
+            configuration_key="binary_artifacts",
+            artifact_expected_categories=(
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.BINARY_PACKAGES,
+                ArtifactCategory.UPLOAD,
+            ),
+        )
+
+        return BaseDynamicTaskData(
+            subject=" ".join(source_package_names),
+        )
+
     def get_label(self) -> str:
         """Return the task label."""
         return "run piuparts"
