@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,9 +12,13 @@
 from collections.abc import Iterable
 from typing import Any
 
-from debusine.artifacts.models import ArtifactCategory, BareDataCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    BareDataCategory,
+    DebusinePromise,
+)
 from debusine.db.context import context
-from debusine.db.models import Artifact, WorkRequest
+from debusine.db.models import Artifact, TaskDatabase, WorkRequest
 from debusine.server.workflows import (
     AutopkgtestWorkflow,
     Workflow,
@@ -30,6 +34,7 @@ from debusine.tasks.models import (
     AutopkgtestNeedsInternet,
     BackendType,
     BaseDynamicTaskData,
+    ExtraRepository,
     LookupMultiple,
     SbuildData,
     SbuildInput,
@@ -69,7 +74,7 @@ class AutopkgtestWorkflowTests(TestCase):
         self, architecture: str, filenames: list[str]
     ) -> Artifact:
         """Create a minimal `debian:upload` artifact with binaries."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.UPLOAD,
             data={
                 "type": "dpkg",
@@ -218,6 +223,11 @@ class AutopkgtestWorkflowTests(TestCase):
             w.work_request.internal_collection,
             "build-amd64",
             category=BareDataCategory.PROMISE,
+            data=DebusinePromise(
+                promise_work_request_id=w.work_request.id + 1,
+                promise_workflow_id=w.work_request.id,
+                promise_category=ArtifactCategory.UPLOAD,
+            ),
         )
 
         with self.assertRaisesRegex(
@@ -229,6 +239,7 @@ class AutopkgtestWorkflowTests(TestCase):
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
         architectures = ("amd64", "i386")
+        source_artifact = self.playground.create_source_artifact()
 
         with preserve_task_registry():
 
@@ -243,7 +254,9 @@ class AutopkgtestWorkflowTests(TestCase):
                         task_type=TaskTypes.WORKFLOW,
                         task_name="sbuild",
                         task_data=SbuildWorkflowData(
-                            input=SbuildInput(source_artifact=1),
+                            input=SbuildInput(
+                                source_artifact=source_artifact.id
+                            ),
                             target_distribution="debian:sid",
                             architectures=["all", *architectures],
                         ),
@@ -252,7 +265,9 @@ class AutopkgtestWorkflowTests(TestCase):
                         child = sbuild.create_child(
                             task_name="sbuild",
                             task_data=SbuildData(
-                                input=SbuildInput(source_artifact=1),
+                                input=SbuildInput(
+                                    source_artifact=source_artifact.id
+                                ),
                                 host_architecture=architecture,
                                 environment="debian/match:codename=sid",
                             ),
@@ -268,7 +283,7 @@ class AutopkgtestWorkflowTests(TestCase):
                         task_type=TaskTypes.WORKFLOW,
                         task_name="autopkgtest",
                         task_data=AutopkgtestWorkflowData(
-                            source_artifact=1,
+                            source_artifact=source_artifact.id,
                             binary_artifacts=LookupMultiple.parse_obj(
                                 [
                                     f"internal@collections/"
@@ -278,6 +293,15 @@ class AutopkgtestWorkflowTests(TestCase):
                             ),
                             vendor="debian",
                             codename="sid",
+                            extra_repositories=[
+                                ExtraRepository.parse_obj(
+                                    {
+                                        "url": "http://example.com/",
+                                        "suite": "bookworm",
+                                        "components": ["main"],
+                                    }
+                                )
+                            ],
                         ),
                     )
                     AutopkgtestWorkflow(autopkgtest).populate()
@@ -322,6 +346,13 @@ class AutopkgtestWorkflowTests(TestCase):
                         },
                         "host_architecture": architecture,
                         "environment": "debian/match:codename=sid",
+                        "extra_repositories": [
+                            {
+                                "url": "http://example.com/",
+                                "suite": "bookworm",
+                                "components": ["main"],
+                            }
+                        ],
                         "backend": "unshare",
                         "include_tests": [],
                         "exclude_tests": [],
@@ -377,6 +408,31 @@ class AutopkgtestWorkflowTests(TestCase):
             children = list(WorkRequest.objects.filter(parent=autopkgtest))
             self.assertEqual(len(children), len(architectures))
 
+    def test_compute_dynamic_data(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        wr = self.playground.create_workflow(
+            task_name="autopkgtest",
+            task_data=AutopkgtestWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = AutopkgtestWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(
+                subject="hello", parameter_summary="hello_1.0-1"
+            ),
+        )
+
     def test_label(self) -> None:
         """Test get_label."""
         w = self.create_autopkgtest_workflow()
