@@ -13,7 +13,8 @@ import logging
 from unittest import mock
 
 from debusine.artifacts.models import ArtifactCategory
-from debusine.db.models import ArtifactRelation, WorkRequest
+from debusine.db.models import ArtifactRelation, CollectionItem, WorkRequest
+from debusine.server.collections.lookup import LookupResult
 from debusine.server.workflows.base import (
     Workflow,
     WorkflowRunError,
@@ -38,7 +39,9 @@ class WorkflowClassTests(TestCase):
             category=ArtifactCategory.TEST
         )
         Workflow.ensure_artifact_categories(
-            artifact, [ArtifactCategory.TEST, ArtifactCategory.UPLOAD]
+            configuration_key="test",
+            category=artifact.category,
+            expected=[ArtifactCategory.TEST, ArtifactCategory.UPLOAD],
         )
 
     def test_ensure_artifact_categories_mismatches(self) -> None:
@@ -46,14 +49,17 @@ class WorkflowClassTests(TestCase):
         artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.TEST
         )
+
         with self.assertRaisesRegex(
             TaskConfigError,
-            r'Category of ".* debusine:test .*" is not one of '
-            r"\['debian:system-tarball', 'debian:system-image'\]",
+            r"^test: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: \['debian:system-image', "
+            r"'debian:system-tarball'\]$",
         ):
             Workflow.ensure_artifact_categories(
-                artifact,
-                [
+                configuration_key="test",
+                category=artifact.category,
+                expected=[
                     ArtifactCategory.SYSTEM_TARBALL,
                     ArtifactCategory.SYSTEM_IMAGE,
                 ],
@@ -122,13 +128,17 @@ class WorkflowClassTests(TestCase):
     def test_locate_debian_source_package_with_srcpkg(self) -> None:
         """Test with a debian:source-package."""
         source_artifact = self.playground.create_source_artifact()
-        result = Workflow.locate_debian_source_package(source_artifact)
+        result = Workflow.locate_debian_source_package(
+            "source_artifact", source_artifact
+        )
         self.assertEqual(result, source_artifact)
 
     def test_locate_debian_source_package_with_upload(self) -> None:
         """Test with a debian:source-package."""
         upload_artifacts = self.playground.create_upload_artifacts()
-        result = Workflow.locate_debian_source_package(upload_artifacts.upload)
+        result = Workflow.locate_debian_source_package(
+            "upload_artifact", upload_artifacts.upload
+        )
         self.assertEqual(result, upload_artifacts.source)
 
     def test_locate_debian_source_package_with_junk(self) -> None:
@@ -138,10 +148,123 @@ class WorkflowClassTests(TestCase):
         )
         with self.assertRaisesRegex(
             TaskConfigError,
-            r'Category of ".* debusine:test .*" is not one of '
-            r"\['debian:source-package', 'debian:upload'\]",
+            r"^test_artifact: unexpected artifact category: "
+            r"'debusine:test'. Valid categories: "
+            r"\['debian:source-package', 'debian:upload'\]$",
         ):
-            Workflow.locate_debian_source_package(artifact)
+            Workflow.locate_debian_source_package("test_artifact", artifact)
+
+    def test_get_source_package_names(self) -> None:
+        binary_artifact_1 = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        binary_artifact_2 = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        binaries_artifact = (
+            self.playground.create_minimal_binary_packages_artifact(
+                srcpkg_name="linux-base"
+            )
+        )
+        upload_artifact = self.playground.create_upload_artifacts(
+            src_name="firefox"
+        ).upload
+
+        lookup_result = [
+            LookupResult(
+                result_type=CollectionItem.Types.ARTIFACT,
+                artifact=binary_artifact_1,
+            ),
+            LookupResult(
+                result_type=CollectionItem.Types.ARTIFACT,
+                artifact=binary_artifact_2,
+            ),
+            LookupResult(
+                result_type=CollectionItem.Types.ARTIFACT,
+                artifact=binaries_artifact,
+            ),
+            LookupResult(
+                result_type=CollectionItem.Types.ARTIFACT,
+                artifact=upload_artifact,
+            ),
+            LookupResult(
+                result_type=CollectionItem.Types.BARE,
+                collection_item=CollectionItem(
+                    data={
+                        "promise_category": ArtifactCategory.BINARY_PACKAGE,
+                        "source_package_name": "python3",
+                    }
+                ),
+            ),
+        ]
+
+        self.assertEqual(
+            Workflow.get_source_package_names(
+                lookup_result,
+                configuration_key="testing",
+                artifact_expected_categories=(
+                    ArtifactCategory.BINARY_PACKAGE,
+                    ArtifactCategory.BINARY_PACKAGES,
+                    ArtifactCategory.UPLOAD,
+                ),
+            ),
+            ["firefox", "hello", "linux-base", "python3"],
+        )
+
+    def test_get_source_package_name_invalid_artifact_category(self) -> None:
+        source_artifact = self.playground.create_source_artifact(name="hello")
+
+        lookup_result = [
+            LookupResult(
+                result_type=CollectionItem.Types.ARTIFACT,
+                artifact=source_artifact,
+            )
+        ]
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^testing: unexpected artifact category: 'debian:source-package'. "
+            r"Valid categories: "
+            r"\['debian:binary-package', 'debian:binary-packages'\]$",
+        ):
+            Workflow.get_source_package_names(
+                lookup_result,
+                configuration_key="testing",
+                artifact_expected_categories=(
+                    ArtifactCategory.BINARY_PACKAGE,
+                    ArtifactCategory.BINARY_PACKAGES,
+                ),
+            )
+
+    def test_get_source_package_name_invalid_artifact_in_promise_category(
+        self,
+    ) -> None:
+        lookup_result = [
+            LookupResult(
+                result_type=CollectionItem.Types.BARE,
+                collection_item=CollectionItem(
+                    data={
+                        "promise_category": ArtifactCategory.TEST,
+                        "source_package_name": "python3",
+                    }
+                ),
+            )
+        ]
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^testing: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: \['debian:binary-package'\]$",
+        ):
+            Workflow.get_source_package_names(
+                lookup_result,
+                configuration_key="testing",
+                artifact_expected_categories=(ArtifactCategory.BINARY_PACKAGE,),
+            )
 
 
 class WorkflowInstanceTests(TestCase):
@@ -158,7 +281,7 @@ class WorkflowInstanceTests(TestCase):
         source_artifact = self.playground.create_source_artifact()
         self.assertEqual(
             self.workflow.locate_debian_source_package_lookup(
-                source_artifact.id
+                "source_artifact", source_artifact.id
             ),
             source_artifact.id,
         )
@@ -168,7 +291,7 @@ class WorkflowInstanceTests(TestCase):
         artifacts = self.playground.create_upload_artifacts()
         self.assertEqual(
             self.workflow.locate_debian_source_package_lookup(
-                artifacts.upload.id
+                "source_artifact", artifacts.upload.id
             ),
             f"{artifacts.source.id}@artifacts",
         )
