@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,13 +9,9 @@
 
 """Unit tests for the debian pipeline workflow."""
 from typing import Any, ClassVar
-from unittest.mock import call, patch
 
-from debusine.artifacts.models import (
-    ArtifactCategory,
-    CollectionCategory,
-    KeyPurpose,
-)
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
@@ -25,11 +21,11 @@ from debusine.server.workflows.base import (
     orchestrate_workflow,
 )
 from debusine.server.workflows.models import DebianPipelineWorkflowData
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     BackendType,
     BaseDynamicTaskData,
     CollectionItemMatcherKind,
-    LookupMultiple,
     TaskTypes,
 )
 from debusine.test.django import TestCase
@@ -103,12 +99,16 @@ class DebianPipelineWorkflowTests(TestCase):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(parameter_summary="hello_1.0-1"),
+            BaseDynamicTaskData(
+                subject="hello", parameter_summary="hello_1.0-1"
+            ),
         )
 
     def test_compute_dynamic_data_source_artifact_wrong_category(self) -> None:
-        """`source_artifact` must be a `debian:source-package`."""
-        artifact, _ = self.create_artifact(category=ArtifactCategory.TEST)
+        """`source_artifact` must be a source-package or upload."""
+        artifact, _ = self.playground.create_artifact(
+            category=ArtifactCategory.TEST
+        )
         wr = self.playground.create_workflow(
             task_name="debian_pipeline",
             task_data=DebianPipelineWorkflowData(
@@ -118,12 +118,26 @@ class DebianPipelineWorkflowTests(TestCase):
         workflow = DebianPipelineWorkflow(wr)
 
         with self.assertRaisesRegex(
-            ValueError,
-            f"source artifact is type '{ArtifactCategory.TEST}' instead of "
-            f"{ArtifactCategory.SOURCE_PACKAGE}",
+            TaskConfigError,
+            r"^source_artifact: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
         ):
             workflow.compute_dynamic_data(TaskDatabase(wr))
 
+    def test_source_package_upload(self) -> None:
+        """`source_artifact` can be an upload."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        wr = self.playground.create_workflow(
+            task_name="debian_pipeline",
+            task_data=DebianPipelineWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        workflow = DebianPipelineWorkflow(wr)
+        self.assertEqual(workflow.source_package, upload_artifacts.source)
+
     def test_populate_use_available_architectures(self) -> None:
         """
         Test populate use available architectures.
@@ -138,7 +152,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 workspace=self.playground.get_default_workspace(),
             )
             for arch in ["amd64", "i386"]:
-                artifact, _ = self.create_artifact(
+                artifact, _ = self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
                     data={"codename": "bookworm", "architecture": arch},
                 )
@@ -183,6 +197,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "architectures": ["amd64"],
                 "backend": BackendType.AUTO,
                 "environment_variant": None,
+                "extra_repositories": None,
                 "input": {"source_artifact": self.source_artifact.id},
                 "target_distribution": "debian:bookworm",
                 "signing_template_names": {},
@@ -216,21 +231,16 @@ class DebianPipelineWorkflowTests(TestCase):
             qa.workflow_data_json, {"display_name": "QA", "step": "qa"}
         )
 
-        self.assertQuerySetEqual(
-            qa.dependencies.all(),
-            list(
-                WorkRequest.objects.filter(
-                    task_type=TaskTypes.WORKER, task_name="sbuild"
-                )
-            ),
-        )
-
         # QaWorkflow.populate() was called and created its tasks
         self.assertTrue(qa.children.exists())
         for child in qa.children.all():
             # Children are sub-workflows, so go straight to RUNNING.
             self.assertEqual(child.status, WorkRequest.Statuses.RUNNING)
 
+        self.assertQuerySetEqual(
+            qa.children.earliest("id").dependencies.all(), []
+        )
+
     def test_populate_qa_with_sbuild_subset(self) -> None:
         """`sbuild` may only create a subset of the requested architectures."""
         self.assertEqual(
@@ -242,6 +252,13 @@ class DebianPipelineWorkflowTests(TestCase):
                 "architectures": ["all", "amd64"],
                 "source_artifact": f"{self.source_artifact.id}@artifacts",
                 "enable_lintian": True,
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
             }
         )
 
@@ -261,6 +278,13 @@ class DebianPipelineWorkflowTests(TestCase):
                 },
                 "target_distribution": "debian:bookworm",
                 "signing_template_names": {},
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
             },
         )
         self.assertEqual(
@@ -286,6 +310,13 @@ class DebianPipelineWorkflowTests(TestCase):
                     }
                 ],
                 "codename": "bookworm",
+                "extra_repositories": [
+                    {
+                        "url": "http://example.com/",
+                        "suite": "bookworm",
+                        "components": ["main"],
+                    }
+                ],
                 "enable_autopkgtest": False,
                 "enable_lintian": True,
                 "enable_piuparts": False,
@@ -325,6 +356,7 @@ class DebianPipelineWorkflowTests(TestCase):
                     }
                 ],
                 "codename": "bookworm",
+                "extra_repositories": None,
                 "enable_autopkgtest": False,
                 "enable_lintian": True,
                 "enable_piuparts": False,
@@ -365,6 +397,7 @@ class DebianPipelineWorkflowTests(TestCase):
                     }
                 ],
                 "codename": "bookworm",
+                "extra_repositories": None,
                 "enable_autopkgtest": True,
                 "enable_lintian": False,
                 "enable_piuparts": False,
@@ -405,6 +438,7 @@ class DebianPipelineWorkflowTests(TestCase):
                     }
                 ],
                 "codename": "bookworm",
+                "extra_repositories": None,
                 "enable_autopkgtest": False,
                 "enable_lintian": False,
                 "enable_piuparts": True,
@@ -439,6 +473,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "architectures": ["i386"],
                 "backend": BackendType.AUTO,
                 "environment_variant": None,
+                "extra_repositories": None,
                 "input": {"source_artifact": self.source_artifact.id},
                 "target_distribution": "debian:bookworm",
                 "signing_template_names": {},
@@ -484,23 +519,16 @@ class DebianPipelineWorkflowTests(TestCase):
 
     def test_populate_make_signed_source(self) -> None:
         """Populate create a make_signed_source workflow."""
-        with patch.object(
-            DebianPipelineWorkflow,
-            "requires_artifact",
-            wraps=DebianPipelineWorkflow.requires_artifact,
-        ) as requires_artifact_wrapper:
-            workflow = self.orchestrate(
-                extra_data={
-                    "architectures": ["amd64"],
-                    "source_artifact": self.source_artifact.id,
-                    "enable_make_signed_source": True,
-                    "make_signed_source_purpose": KeyPurpose.OPENPGP,
-                    "signing_template_names": {
-                        "amd64": ["hello"],
-                    },
-                    "make_signed_source_key": 80,
-                }
-            )
+        workflow = self.orchestrate(
+            extra_data={
+                "architectures": ["amd64"],
+                "source_artifact": self.source_artifact.id,
+                "enable_make_signed_source": True,
+                "make_signed_source_purpose": KeyPurpose.OPENPGP,
+                "signing_template_names": {"amd64": ["hello"]},
+                "make_signed_source_key": "ABC123",
+            }
+        )
 
         expected_binary_artifacts = [
             {
@@ -521,7 +549,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "architectures": ["amd64"],
                 "binary_artifacts": expected_binary_artifacts,
                 "codename": "bookworm",
-                "key": 80,
+                "key": "ABC123",
                 "purpose": "openpgp",
                 "sbuild_backend": BackendType.AUTO,
                 "signing_template_artifacts": [
@@ -538,54 +566,50 @@ class DebianPipelineWorkflowTests(TestCase):
                 "step": "make_signed_source",
             },
         )
+        self.assertQuerySetEqual(make_signed_source.dependencies.all(), [])
 
-        # There are two calls to self.requires_artifact:
-        # self.requires_artifact(wr, binary_artifacts)
-        # self.requires_artifact(wr, LookupMultiple.parse_obj...)
-        # But both of them are adding the same WorkRequest to depend on:
-        # the sbuild work request (which generates the binary artifacts and the
-        # signing artifacts
+        # MakeSignedSource.populate() was called and created its tasks.
+        # They are all blocked, with the first one depending on sbuild and
+        # the others depending on the previous one in sequence.
         self.assertQuerySetEqual(
-            make_signed_source.dependencies.all(),
-            list(
-                WorkRequest.objects.filter(
-                    task_type=TaskTypes.WORKER, task_name="sbuild"
-                )
+            make_signed_source.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
             ),
-        )
-
-        # Assert that the correct calls were made
-        requires_artifact_wrapper.assert_has_calls(
             [
-                call(
-                    make_signed_source,
-                    LookupMultiple.parse_obj(expected_binary_artifacts),
-                ),
-                call(
-                    make_signed_source,
-                    LookupMultiple.parse_obj(
-                        [
-                            "internal@collections/"
-                            "name:signing-template-amd64-hello"
-                        ]
-                    ),
-                ),
-            ]
-        )
-
-        # MakeSignedSource.populate() was called and created its tasks
-        self.assertTrue(make_signed_source.children.exists())
-        for child in make_signed_source.children.filter():
-            # The sbuild sub-workflow is running.  Everything else is
-            # blocked.
-            self.assertEqual(
-                child.status,
                 (
-                    WorkRequest.Statuses.RUNNING
-                    if child.task_type == TaskTypes.WORKFLOW
-                    else WorkRequest.Statuses.BLOCKED
+                    TaskTypes.WORKER,
+                    "extractforsigning",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
                 ),
-            )
+                (
+                    TaskTypes.SIGNING,
+                    "sign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "extractforsigning",
+                ),
+                (
+                    TaskTypes.WORKER,
+                    "assemblesignedsource",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.SIGNING,
+                    "sign",
+                ),
+                (
+                    TaskTypes.WORKFLOW,
+                    "sbuild",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "assemblesignedsource",
+                ),
+            ],
+        )
 
     def test_populate_package_upload(self) -> None:
         """Populate create an upload_package workflow."""
@@ -596,7 +620,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_make_signed_source": True,
                 "make_signed_source_purpose": KeyPurpose.OPENPGP,
                 "signing_template_names": {"amd64": ["hello"]},
-                "make_signed_source_key": 80,
+                "make_signed_source_key": "ABC123",
                 "enable_upload": True,
                 "vendor": "debian",
                 "codename": "trixie",
@@ -641,29 +665,62 @@ class DebianPipelineWorkflowTests(TestCase):
             package_upload.workflow_data_json,
             {"display_name": "package upload", "step": "package_upload"},
         )
+        self.assertQuerySetEqual(package_upload.dependencies.all(), [])
 
+        # PackageUpload.populate() was called and created its tasks.
+        # MakeSourcePackageUpload only requires the source artifact and so
+        # can start immediately.  Everything else is blocked.
         self.assertQuerySetEqual(
-            package_upload.dependencies.all(),
-            list(
-                WorkRequest.objects.filter(
-                    task_type=TaskTypes.WORKER, task_name="sbuild"
-                )
+            package_upload.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
             ),
-        )
-
-        # PackageUpload.populate() was called and created its tasks
-        self.assertTrue(package_upload.children.exists())
-        for child in package_upload.children.all():
-            # MakeSourcePackageUpload only requires the source artifact and
-            # so can start immediately.  Everything else is blocked.
-            self.assertEqual(
-                child.status,
+            # Work requests to construct, sign, and upload the source package.
+            [
                 (
-                    WorkRequest.Statuses.PENDING
-                    if child.task_name == "makesourcepackageupload"
-                    else WorkRequest.Statuses.BLOCKED
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                    WorkRequest.Statuses.PENDING,
+                    None,
+                    None,
                 ),
-            )
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            # Work requests to sign and upload each of the two binary packages.
+            + [
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            * 2,
+        )
 
     def test_populate_upload_package_no_include_source_binaries(self) -> None:
         """Populate create upload_workflow without source neither binaries."""
@@ -674,7 +731,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_make_signed_source": True,
                 "make_signed_source_purpose": KeyPurpose.OPENPGP,
                 "signing_template_names": {"amd64": ["hello"]},
-                "make_signed_source_key": 80,
+                "make_signed_source_key": "ABC123",
                 "enable_upload": True,
                 "vendor": "debian",
                 "codename": "trixie",
@@ -763,29 +820,60 @@ class DebianPipelineWorkflowTests(TestCase):
             package_upload.workflow_data_json,
             {"display_name": "package upload", "step": "package_upload"},
         )
+        self.assertQuerySetEqual(package_upload.dependencies.all(), [])
 
+        # PackageUpload.populate() was called and created its tasks.
+        # MakeSourcePackageUpload only requires the source artifact and so
+        # can start immediately.  Everything else is blocked.
         self.assertQuerySetEqual(
-            package_upload.dependencies.all(),
-            list(
-                WorkRequest.objects.filter(
-                    task_type=TaskTypes.WORKER, task_name="sbuild"
-                )
+            package_upload.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
             ),
-        )
-
-        # PackageUpload.populate() was called and created its tasks
-        self.assertTrue(package_upload.children.exists())
-        for child in package_upload.children.all():
-            # MakeSourcePackageUpload only requires the source artifact and
-            # so can start immediately.  Everything else is blocked.
-            self.assertEqual(
-                child.status,
+            [
                 (
-                    WorkRequest.Statuses.PENDING
-                    if child.task_name == "makesourcepackageupload"
-                    else WorkRequest.Statuses.BLOCKED
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                    WorkRequest.Statuses.PENDING,
+                    None,
+                    None,
                 ),
-            )
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            + [
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            * 2,
+        )
 
     def test_get_label(self) -> None:
         """Test get_label."""
