@@ -9,24 +9,33 @@
 
 """Unit tests for the make_signed_source workflow."""
 from collections.abc import Sequence
-from itertools import chain
 from typing import Any
 
-from debusine.artifacts.models import ArtifactCategory, KeyPurpose
+from django.test import override_settings
+
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    DebianSourcePackage,
+    DebusineSigningInput,
+    DebusineSigningOutput,
+)
+from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
-from debusine.db.models import Artifact, WorkRequest
+from debusine.db.models import Artifact, TaskDatabase, WorkRequest
+from debusine.server.collections.lookup import lookup_multiple, lookup_single
+from debusine.server.scheduler import schedule
 from debusine.server.workflows import MakeSignedSourceWorkflow
 from debusine.server.workflows.base import Workflow, orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
     MakeSignedSourceWorkflowData,
     SbuildWorkflowData,
+    WorkRequestWorkflowData,
 )
 from debusine.tasks.models import (
     BaseDynamicTaskData,
     LookupMultiple,
-    SbuildData,
     SbuildInput,
     TaskTypes,
 )
@@ -54,7 +63,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
             "vendor": "debian",
             "codename": "bookworm",
             "purpose": "uefi",
-            "key": 80,
+            "key": "ABC123",
         }
         task_data.update(extra_task_data)
         wr = self.playground.create_workflow(
@@ -98,10 +107,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
           creates an sbuild child promising a binary SIGNING_INPUT and
           BINARY_PACKAGE promising artifacts or only the artifacts (see
           create_promises / create_artifacts)
-        :param create_sbuild_promises: if True, create an `sbuild` children work
-          request that provides, for each architecture an artifact named
-          `build-{arch}` with data={"architecture": arch}
+        :param create_sbuild_promises: if True, create an `sbuild`
+          sub-workflow
         """
+        source_artifact = self.playground.create_source_artifact()
 
         class ExamplePipeline(
             TestTaskMixin, Workflow[BaseWorkflowData, BaseDynamicTaskData]
@@ -111,49 +120,42 @@ class MakeSignedSourceWorkflowTests(TestCase):
             def populate(self) -> None:
                 """Populate the pipeline."""
                 if create_sbuild_promises:
-                    sbuild = self.work_request.create_child(
+                    sbuild_architectures = [
+                        *architectures,
+                        *(extra_sbuild_architectures or []),
+                    ]
+                    sbuild = self.work_request_ensure_child(
                         task_type=TaskTypes.WORKFLOW,
                         task_name="sbuild",
                         task_data=SbuildWorkflowData(
-                            input=SbuildInput(source_artifact=1),
+                            input=SbuildInput(
+                                source_artifact=source_artifact.id
+                            ),
                             target_distribution="debian:bookworm",
-                            architectures=[
-                                *architectures,
-                                *(extra_sbuild_architectures or []),
-                            ],
+                            architectures=sbuild_architectures,
+                            signing_template_names={
+                                arch: [f"hello-{arch}-signed"]
+                                for arch in sbuild_architectures
+                            },
+                        ),
+                        workflow_data=WorkRequestWorkflowData(
+                            display_name="sbuild", step="sbuild"
                         ),
                     )
-                    for arch in chain(
-                        architectures, extra_sbuild_architectures or []
-                    ):
-                        child = sbuild.create_child(
-                            task_name="sbuild",
-                            task_data=SbuildData(
-                                input=SbuildInput(source_artifact=1),
-                                host_architecture=arch,
-                                environment="debian/match:codename=bookworm",
-                            ),
-                        )
+                    sbuild.mark_running()
+                    orchestrate_workflow(sbuild)
 
-                        self.provides_artifact(
-                            child,
-                            ArtifactCategory.BINARY_PACKAGE,
-                            f"build-{arch}",
-                            data={"architecture": arch},
-                        )
-                        self.provides_artifact(
-                            child,
-                            ArtifactCategory.SIGNING_INPUT,
-                            f"signing-{arch}",
-                            data={"architecture": arch},
-                        )
-
-                make_signed_source = self.work_request.create_child(
+                make_signed_source = self.work_request_ensure_child(
                     task_type=TaskTypes.WORKFLOW,
                     task_name="make_signed_source",
                     task_data=task_data,
+                    workflow_data=WorkRequestWorkflowData(
+                        display_name="make signed source",
+                        step="make_signed_source",
+                    ),
                 )
-                MakeSignedSourceWorkflow(make_signed_source).populate()
+                make_signed_source.mark_running()
+                orchestrate_workflow(make_signed_source)
 
         root = self.playground.create_workflow(task_name="examplepipeline")
 
@@ -162,6 +164,40 @@ class MakeSignedSourceWorkflowTests(TestCase):
 
         return root
 
+    def test_compute_dynamic_data(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_packages_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        upload_artifact = self.playground.create_upload_artifacts(
+            src_name="linux-base"
+        ).upload
+        signing_artifact = self.create_signing_template_artifact("amd64")
+
+        wr = self.playground.create_workflow(
+            task_name="make_signed_source",
+            task_data=MakeSignedSourceWorkflowData(
+                binary_artifacts=LookupMultiple.parse_obj(
+                    [binary_artifact.id, upload_artifact.id]
+                ),
+                vendor="debian",
+                codename="trixie",
+                signing_template_artifacts=LookupMultiple.parse_obj(
+                    [signing_artifact.id]
+                ),
+                architectures=["amd64"],
+                purpose=KeyPurpose("uefi"),
+                key="ABC123",
+            ),
+        )
+        workflow = MakeSignedSourceWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(subject="hello linux-base"),
+        )
+
     @preserve_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
@@ -180,24 +216,29 @@ class MakeSignedSourceWorkflowTests(TestCase):
                 ),
                 signing_template_artifacts=LookupMultiple.parse_obj(
                     [
-                        f"internal@collections/name:signing-{arch}"
+                        {
+                            "collection": "internal@collections",
+                            "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                            "name__startswith": f"signing-template-{arch}-",
+                        }
                         for arch in architectures
                     ]
                 ),
                 vendor="debian",
                 codename="bookworm",
                 purpose=KeyPurpose.UEFI,
-                key=80,
+                key="ABC123",
             ),
             architectures=architectures,
             create_sbuild_promises=True,
             extra_sbuild_architectures=("arm64",),
         )
 
-        make_signed_source = WorkRequest.objects.get(
-            task_type=TaskTypes.WORKFLOW,
-            task_name="make_signed_source",
-            parent=root,
+        initial_sbuild_workflow = root.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="sbuild"
+        )
+        make_signed_source = root.children.get(
+            task_type=TaskTypes.WORKFLOW, task_name="make_signed_source"
         )
 
         extract_for_signings = make_signed_source.children.filter(
@@ -223,8 +264,8 @@ class MakeSignedSourceWorkflowTests(TestCase):
         for arch in architectures:
             collection_id = root.internal_collection_id
             extract_for_signing = extract_for_signings.get(
-                task_data__input__template_artifact=(
-                    f"{collection_id}@collections/name:signing-{arch}"
+                task_data__input__template_artifact__startswith=(
+                    f"{collection_id}@collections/name:signing-template-{arch}-"
                 )
             )
             self.assertEqual(
@@ -236,8 +277,8 @@ class MakeSignedSourceWorkflowTests(TestCase):
                             f"{collection_id}@collections/name:build-{arch}"
                         ],
                         "template_artifact": (
-                            f"{collection_id}@"
-                            f"collections/name:signing-{arch}"
+                            f"{collection_id}@collections/"
+                            f"name:signing-template-{arch}-hello-{arch}-signed"
                         ),
                     },
                 },
@@ -283,15 +324,19 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     "name__startswith": "extracted-for-signing-",
                     "data__architecture": arch,
                 }
-            ).dict(exclude_unset=True)["__root__"]
-            sign = signs.get(task_data__unsigned=unsigned)
+            )
+            sign = signs.get(
+                task_data__unsigned=unsigned.dict(exclude_unset=True)[
+                    "__root__"
+                ]
+            )
             self.assertEqual(
                 {
                     key: value
                     for key, value in sign.task_data.items()
                     if key != "unsigned"
                 },
-                {"key": 80, "purpose": "uefi"},
+                {"key": "ABC123", "purpose": "uefi"},
             )
             self.assertEqual(
                 sign.workflow_data_json,
@@ -310,25 +355,43 @@ class MakeSignedSourceWorkflowTests(TestCase):
                                 "category": "debusine:signing-output"
                             },
                             "collection": "internal@collections",
-                            "name_template": f"signed-{arch}",
-                            "variables": None,
+                            "name_template": (
+                                "signed-{architecture}-{binary_package_name}"
+                            ),
+                            "variables": {
+                                "$binary_package_name": "binary_package_name",
+                                "architecture": arch,
+                            },
                         }
                     ],
                     "on_unblock": [],
                 },
             )
 
+            signed = LookupMultiple.parse_obj(
+                {
+                    "collection": "internal@collections",
+                    "child_type": LookupChildType.ARTIFACT,
+                    "category": ArtifactCategory.SIGNING_OUTPUT,
+                    "name__startswith": "signed-",
+                    "data__architecture": arch,
+                }
+            )
             assemble_sign_source = assemble_sign_sources.get(
-                task_data__signed=[f"internal@collections/name:signed-{arch}"]
+                task_data__signed=signed.dict(exclude_unset=True)["__root__"]
             )
 
             self.assertEqual(
-                assemble_sign_source.task_data,
+                {
+                    key: value
+                    for key, value in assemble_sign_source.task_data.items()
+                    if key != "signed"
+                },
                 {
                     "environment": "debian/match:codename=bookworm",
-                    "signed": [f"internal@collections/name:signed-{arch}"],
                     "template": (
-                        f"{collection_id}@collections/name:signing-{arch}"
+                        f"{collection_id}@collections/"
+                        f"name:signing-template-{arch}-hello-{arch}-signed"
                     ),
                 },
             )
@@ -364,6 +427,9 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     f"internal@collections/name:signed-source-{arch}"
                 )
             )
+            self.assertEqual(
+                sbuild_workflow.status, WorkRequest.Statuses.BLOCKED
+            )
 
             self.assertEqual(
                 sbuild_workflow.task_data,
@@ -385,6 +451,103 @@ class MakeSignedSourceWorkflowTests(TestCase):
             )
             self.assertEqual(sbuild_workflow.event_reactions_json, {})
 
+            # Completing each work request adds items to the internal
+            # collection that can be used by the next work request.
+            initial_sbuild = initial_sbuild_workflow.children.get(
+                task_data__host_architecture=arch
+            )
+            initial_sbuild.mark_completed(WorkRequest.Results.SUCCESS)
+            signing_input, _ = self.playground.create_artifact(
+                category=ArtifactCategory.SIGNING_INPUT,
+                data=DebusineSigningInput(binary_package_name="hello"),
+                workspace=extract_for_signing.workspace,
+                work_request=extract_for_signing,
+            )
+            extract_for_signing.refresh_from_db()
+            extract_for_signing.mark_completed(WorkRequest.Results.SUCCESS)
+            self.assertEqual(
+                [
+                    result.artifact
+                    for result in lookup_multiple(
+                        unsigned,
+                        sign.workspace,
+                        user=sign.created_by,
+                        workflow_root=root,
+                        expect_type=LookupChildType.ARTIFACT,
+                    )
+                ],
+                [signing_input],
+            )
+
+            signing_output, _ = self.playground.create_artifact(
+                category=ArtifactCategory.SIGNING_OUTPUT,
+                data=DebusineSigningOutput(
+                    purpose=KeyPurpose.UEFI, fingerprint="ABC123", results=[]
+                ),
+                workspace=sign.workspace,
+                work_request=sign,
+            )
+            sign.refresh_from_db()
+            sign.mark_completed(WorkRequest.Results.SUCCESS)
+            self.assertEqual(
+                [
+                    result.artifact
+                    for result in lookup_multiple(
+                        LookupMultiple.parse_obj(
+                            assemble_sign_source.task_data["signed"]
+                        ),
+                        assemble_sign_source.workspace,
+                        user=assemble_sign_source.created_by,
+                        workflow_root=root,
+                        expect_type=LookupChildType.ARTIFACT,
+                    )
+                ],
+                [signing_output],
+            )
+
+            source_package, _ = self.playground.create_artifact(
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=DebianSourcePackage(
+                    name=f"hello-{arch}-signed",
+                    version="1.0.0",
+                    type="dpkg",
+                    dsc_fields={"Architecture": arch},
+                ),
+                workspace=assemble_sign_source.workspace,
+                work_request=assemble_sign_source,
+            )
+            assemble_sign_source.refresh_from_db()
+            assemble_sign_source.mark_completed(WorkRequest.Results.SUCCESS)
+            self.assertEqual(
+                lookup_single(
+                    sbuild_workflow.task_data["input"]["source_artifact"],
+                    sbuild_workflow.workspace,
+                    user=sbuild_workflow.created_by,
+                    workflow_root=root,
+                    expect_type=LookupChildType.ARTIFACT,
+                ).artifact,
+                source_package,
+            )
+
+            # The sbuild sub-workflow is now marked pending, and will be
+            # picked up by the scheduler.
+            sbuild_workflow.refresh_from_db()
+            self.assertEqual(
+                sbuild_workflow.status, WorkRequest.Statuses.PENDING
+            )
+
+            # The scheduler would run on commit, but run it explicitly now
+            # so that we can test what it does.
+            with self.captureOnCommitCallbacks() as on_commit:
+                self.assertEqual(schedule(), [root])
+            with override_settings(CELERY_TASK_ALWAYS_EAGER=True):
+                for callback in on_commit:
+                    callback()
+            sbuild_workflow.refresh_from_db()
+            self.assertEqual(
+                sbuild_workflow.status, WorkRequest.Statuses.RUNNING
+            )
+
     @preserve_task_registry()
     def test_populate_with_artifacts(self) -> None:
         """
@@ -395,11 +558,16 @@ class MakeSignedSourceWorkflowTests(TestCase):
         with context.disable_permission_checks():
             architectures = ["amd64"]
 
-            binary_artifact = (
+            binary_artifact_1 = (
                 self.playground.create_minimal_binary_packages_artifact(
                     "hello", "1.0.0", "1.0.0", "amd64"
                 )
             )
+            binary_artifact_2 = (
+                self.playground.create_minimal_binary_package_artifact(
+                    "coreutils", "9.1", "9.1-1", "amd64"
+                )
+            )
             signing_artifact = self.create_signing_template_artifact("amd64")
 
             self.playground.create_minimal_binary_packages_artifact(
@@ -409,14 +577,16 @@ class MakeSignedSourceWorkflowTests(TestCase):
 
         root = self.orchestrate(
             task_data=MakeSignedSourceWorkflowData(
-                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                binary_artifacts=LookupMultiple.parse_obj(
+                    [binary_artifact_1.id, binary_artifact_2.id]
+                ),
                 signing_template_artifacts=LookupMultiple.parse_obj(
                     [signing_artifact.id]
                 ),
                 vendor="debian",
                 codename="bookworm",
                 purpose=KeyPurpose.UEFI,
-                key=80,
+                key="ABC123",
                 architectures=architectures,
             ),
             architectures=architectures,
