@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,6 +11,7 @@
 
 from debusine.server.workflows.models import (
     DebianPipelineWorkflowData,
+    ExperimentWorkspaceData,
     PackagePublishWorkflowData,
     QAWorkflowData,
     SbuildWorkflowData,
@@ -148,3 +149,18 @@ class PackagePublishWorkflowDataTests(TestCase):
                 binary_artifacts=empty_lookup_multiple(),
                 target_suite="bookworm@debian:suite",
             )
+
+
+class ExperimentWorkspaceDataTests(TestCase):
+    """Tests for :py:class:`ExperimentWorkspaceData`."""
+
+    def test_validate_experiment_name(self) -> None:
+        """Test experiment name validation."""
+        for name in ["test-test", "0test", "test@test", ".test", "+test"]:
+            with (
+                self.subTest(name=name),
+                self.assertRaisesRegex(
+                    ValueError, "experiment name contains invalid characters"
+                ),
+            ):
+                ExperimentWorkspaceData(experiment_name="test-test")
