@@ -14,7 +14,13 @@ from typing import Any, ClassVar
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
-from debusine.db.models import Collection, User, WorkRequest, Workspace
+from debusine.db.models import (
+    Collection,
+    TaskDatabase,
+    User,
+    WorkRequest,
+    Workspace,
+)
 from debusine.server.workflows import PackagePublishWorkflow, Workflow
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
@@ -245,11 +251,11 @@ class PackagePublishWorkflowTests(TestCase):
         with context.disable_permission_checks():
             source_workspace = self.playground.create_workspace(name="source")
             self.playground.create_group_role(
-                source_workspace, Workspace.Roles.OWNER, self.user
+                source_workspace, Workspace.Roles.OWNER, users=[self.user]
             )
             target_workspace = self.playground.create_workspace(name="target")
             self.playground.create_group_role(
-                target_workspace, Workspace.Roles.OWNER, self.user
+                target_workspace, Workspace.Roles.OWNER, users=[self.user]
             )
         source_workspace.set_inheritance([target_workspace])
         target_suite = self.create_suite_collection(
@@ -324,6 +330,25 @@ class PackagePublishWorkflowTests(TestCase):
             },
         )
 
+    def test_compute_dynamic_data_binary_package(self) -> None:
+        upload_artifact = (
+            self.playground.create_upload_artifacts(src_name="hello")
+        ).upload
+
+        wr = self.playground.create_workflow(
+            task_name="package_publish",
+            task_data=PackagePublishWorkflowData(
+                binary_artifacts=LookupMultiple.parse_obj([upload_artifact.id]),
+                target_suite=f"bookworm@{CollectionCategory.SUITE}",
+            ),
+        )
+        workflow = PackagePublishWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(subject="hello"),
+        )
+
     def test_get_label(self) -> None:
         """Test get_label()."""
         w = self.create_package_publish_workflow(
