@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,21 +11,29 @@
 from typing import Any
 from unittest.mock import call, patch
 
+try:
+    import pydantic.v1 as pydantic
+except ImportError:
+    import pydantic as pydantic  # type: ignore
+
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     CollectionItem,
+    TaskDatabase,
     WorkRequest,
     WorkflowTemplate,
     default_workspace,
 )
+from debusine.server.tasks.models import PackageUploadTarget
 from debusine.server.workflows import (
     PackageUploadWorkflow,
     WorkflowValidationError,
 )
 from debusine.server.workflows.base import orchestrate_workflow
-from debusine.tasks.models import LookupMultiple, TaskTypes
+from debusine.server.workflows.models import PackageUploadWorkflowData
+from debusine.tasks.models import BaseDynamicTaskData, LookupMultiple, TaskTypes
 from debusine.test.django import TestCase
 
 
@@ -49,10 +57,16 @@ class PackageUploadWorkflowTests(TestCase):
         )
         return PackageUploadWorkflow(wr)
 
-    def create_source_package(self) -> Artifact:
+    def create_source_package(self, *, name: str = "hello") -> Artifact:
         """Create a minimal `debian:source-package` artifact."""
         artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.SOURCE_PACKAGE, data={}
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data={
+                "name": name,
+                "version": "1.0",
+                "type": "dpkg",
+                "dsc_fields": {},
+            },
         )
         return artifact
 
@@ -280,10 +294,7 @@ class PackageUploadWorkflowTests(TestCase):
 
     def test_populate_source_is_package_upload(self) -> None:
         """Category source package is UPLOAD: skip makesourcepackageupload."""
-        with context.disable_permission_checks():
-            source = self.create_source_package()
-            source.category = ArtifactCategory.UPLOAD
-            source.save()
+        source = self.playground.create_upload_artifacts().upload
 
         wr = self.orchestrate(
             data={
@@ -409,7 +420,6 @@ class PackageUploadWorkflowTests(TestCase):
         """Upload signed artifacts."""
         with context.disable_permission_checks():
             source, binary = self.create_source_binary_artifacts()
-            key, _ = self.create_artifact(category=ArtifactCategory.SIGNING_KEY)
 
         wr = self.orchestrate(
             data={
@@ -418,7 +428,7 @@ class PackageUploadWorkflowTests(TestCase):
                 "require_signature": False,
                 "target_distribution": "debian/match:codename=bookworm",
                 "merge_uploads": True,
-                "key": key.id,
+                "key": "ABC123",
                 "vendor": "debian",
                 "codename": "bookworm",
             },
@@ -430,7 +440,7 @@ class PackageUploadWorkflowTests(TestCase):
         self.assertEqual(
             signer.task_data,
             {
-                "key": key.id,
+                "key": "ABC123",
                 "unsigned": "internal@collections/name:package-upload-merged",
             },
         )
@@ -619,6 +629,55 @@ class PackageUploadWorkflowTests(TestCase):
 
         self.assertQuerySetEqual(wr.children.all(), children, ordered=False)
 
+    def test_compute_dynamic_data_source_artifact(self) -> None:
+        source_artifact = self.create_source_package(name="hello")
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact=source_artifact.id,
+                target=pydantic.parse_obj_as(
+                    PackageUploadTarget,
+                    "sftp://example.org",
+                ),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = PackageUploadWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(subject="hello"),
+        )
+
+    def test_compute_dynamic_data_binary_artifacts(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                source_artifact=None,
+                target=pydantic.parse_obj_as(
+                    PackageUploadTarget,
+                    "sftp://example.org",
+                ),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = PackageUploadWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(subject="hello"),
+        )
+
     @context.disable_permission_checks()
     def test_get_label(self) -> None:
         """Test get_label()."""
