@@ -104,6 +104,7 @@ class QAWorkflowTests(TestCase):
                             data={
                                 "binary_names": ["hello"],
                                 "architecture": arch,
+                                "source_package_name": "hello",
                             },
                         )
 
@@ -149,7 +150,6 @@ class QAWorkflowTests(TestCase):
         return artifact
 
     def test_compute_dynamic_data(self) -> None:
-        """Dynamic data includes a parameter summary."""
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
         )
@@ -168,7 +168,9 @@ class QAWorkflowTests(TestCase):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(parameter_summary="hello_1.0-1"),
+            BaseDynamicTaskData(
+                subject="hello", parameter_summary="hello_1.0-1"
+            ),
         )
 
     def test_compute_dynamic_data_source_artifact_wrong_category(self) -> None:
@@ -191,8 +193,8 @@ class QAWorkflowTests(TestCase):
 
         with self.assertRaisesRegex(
             TaskConfigError,
-            r'Category of ".* debusine:test .*" is not one of '
-            r"\['debian:source-package', 'debian:upload'\]",
+            r"^source_artifact: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
         ):
             workflow.compute_dynamic_data(TaskDatabase(wr))
 
@@ -270,21 +272,27 @@ class QAWorkflowTests(TestCase):
             autopkgtest.workflow_data_json,
             {"display_name": "autopkgtest", "step": "autopkgtest"},
         )
-
-        self.assertQuerySetEqual(
-            autopkgtest.dependencies.all(),
-            set(
-                WorkRequest.objects.filter(
-                    task_name="sbuild", task_data__host_architecture="amd64"
-                )
-            ),
-            ordered=False,
-        )
+        self.assertQuerySetEqual(autopkgtest.dependencies.all(), [])
 
         # AutopkgtestWorkflow.populate() was called and created its tasks
-        self.assertTrue(autopkgtest.children.exists())
-        for child in autopkgtest.children.all():
-            self.assertEqual(child.status, WorkRequest.Statuses.BLOCKED)
+        self.assertQuerySetEqual(
+            autopkgtest.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            [
+                (
+                    TaskTypes.WORKER,
+                    "autopkgtest",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                )
+            ],
+        )
 
         self.assertFalse(
             workflow.children.filter(
@@ -316,20 +324,26 @@ class QAWorkflowTests(TestCase):
             lintian.workflow_data_json,
             {"display_name": "lintian", "step": "lintian"},
         )
-
-        self.assertQuerySetEqual(
-            lintian.dependencies.all(),
-            set(
-                WorkRequest.objects.filter(
-                    task_name="sbuild", task_data__host_architecture="amd64"
-                )
-            ),
-            ordered=False,
-        )
+        self.assertQuerySetEqual(lintian.dependencies.all(), [])
         # LintianWorkflow.populate() was called and created its tasks
-        self.assertTrue(lintian.children.exists())
-        for child in lintian.children.all():
-            self.assertEqual(child.status, WorkRequest.Statuses.BLOCKED)
+        self.assertQuerySetEqual(
+            lintian.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            [
+                (
+                    TaskTypes.WORKER,
+                    "lintian",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                )
+            ],
+        )
 
         piuparts = workflow.children.get(
             task_name="piuparts", task_type=TaskTypes.WORKFLOW
@@ -363,21 +377,27 @@ class QAWorkflowTests(TestCase):
             piuparts.workflow_data_json,
             {"display_name": "piuparts", "step": "piuparts"},
         )
-
-        self.assertQuerySetEqual(
-            piuparts.dependencies.all(),
-            set(
-                WorkRequest.objects.filter(
-                    task_name="sbuild", task_data__host_architecture="amd64"
-                )
-            ),
-            ordered=False,
-        )
+        self.assertQuerySetEqual(piuparts.dependencies.all(), [])
 
         # PiupartsWorkflow.populate() was called and created its tasks
-        self.assertTrue(piuparts.children.exists())
-        for child in piuparts.children.all():
-            self.assertEqual(child.status, WorkRequest.Statuses.BLOCKED)
+        self.assertQuerySetEqual(
+            piuparts.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            [
+                (
+                    TaskTypes.WORKER,
+                    "piuparts",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                )
+            ],
+        )
 
     @preserve_task_registry()
     def test_populate_use_available_architectures(self) -> None:
@@ -562,16 +582,7 @@ class QAWorkflowTests(TestCase):
                 "step": "reverse-dependencies-autopkgtest",
             },
         )
-
-        self.assertQuerySetEqual(
-            rdep_autopkgtest.dependencies.all(),
-            set(
-                WorkRequest.objects.filter(
-                    task_name="sbuild", task_data__host_architecture="amd64"
-                )
-            ),
-            ordered=False,
-        )
+        self.assertQuerySetEqual(rdep_autopkgtest.dependencies.all(), [])
 
         # ReverseDependenciesAutopkgtestWorkflow.populate() was called and
         # created its sub-workflow.
@@ -581,6 +592,24 @@ class QAWorkflowTests(TestCase):
             autopkgtest.task_data["source_artifact"],
             "sid@debian:suite/source-version:depends_1",
         )
+        self.assertQuerySetEqual(
+            autopkgtest.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            [
+                (
+                    TaskTypes.WORKER,
+                    "autopkgtest",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                )
+            ],
+        )
 
     @preserve_task_registry()
     def test_populate_architectures_allowed(self) -> None:
