@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -22,7 +22,7 @@ from debusine.artifacts.models import (
 )
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
-from debusine.db.models import Artifact, Collection, WorkRequest
+from debusine.db.models import Artifact, Collection, TaskDatabase, WorkRequest
 from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows import (
@@ -40,10 +40,12 @@ from debusine.server.workflows.reverse_dependencies_autopkgtest import (
     _BinaryPackage,
     _SourcePackage,
 )
+from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     AutopkgtestNeedsInternet,
     BackendType,
     BaseDynamicTaskData,
+    ExtraRepository,
     LookupMultiple,
     LookupSingle,
     SbuildData,
@@ -199,7 +201,12 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             workflow.work_request.internal_collection,
             "build-amd64",
             category=BareDataCategory.PROMISE,
-            data={"binary_names": ["hello", "hello-dev"]},
+            data={
+                "promise_work_request_id": workflow.work_request.id + 1,
+                "promise_workflow_id": workflow.work_request.id,
+                "promise_category": ArtifactCategory.UPLOAD,
+                "binary_names": ["hello", "hello-dev"],
+            },
         )
 
         self.assertEqual(workflow.get_binary_names(), {"hello", "hello-dev"})
@@ -292,9 +299,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
         )
 
         with self.assertRaisesRegex(
-            ValueError,
-            f"Expected source_artifact to be of category "
-            f"{ArtifactCategory.SOURCE_PACKAGE}; got {ArtifactCategory.TEST}",
+            TaskConfigError,
+            r"^source_artifact: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
         ):
             workflow.get_reverse_dependencies()
 
@@ -517,6 +524,34 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             },
         )
 
+        upload_artifacts = self.playground.create_upload_artifacts(
+            src_name="python-urllib3",
+            version="2.0.7-2",
+            binaries=["python3-urllib3"],
+        )
+        workflow = self.create_rdep_autopkgtest_workflow(
+            extra_task_data={
+                "source_artifact": upload_artifacts.upload.id,
+                "binary_artifacts": [upload_artifacts.binaries[0].id],
+                "suite_collection": sid.id,
+            }
+        )
+        self.assert_reverse_dependencies(
+            workflow,
+            {
+                "sid/source-version:pre-depends_1.0": (
+                    "sid/binary-version:python3-pre-depends_1:1.0_all",
+                ),
+                "sid/source-version:depends_2.32.3+dfsg-1": (
+                    "sid/binary-version:python3-depends_2.32.3+dfsg-1_all",
+                    "sid/binary-version:depends-doc_2.32.3+dfsg-1_all",
+                ),
+                "sid/source-version:trigger_2": (
+                    "sid/binary-version:trigger_2_all",
+                ),
+            },
+        )
+
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
         sid = self.playground.create_collection(
@@ -606,6 +641,15 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                             suite_collection=sid.id,
                             vendor="debian",
                             codename="sid",
+                            extra_repositories=[
+                                ExtraRepository.parse_obj(
+                                    {
+                                        "url": "http://example.com/",
+                                        "suite": "bookworm",
+                                        "components": ["main"],
+                                    }
+                                )
+                            ],
                         ),
                     )
                     ReverseDependenciesAutopkgtestWorkflow(
@@ -653,6 +697,13 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                         "architectures": [],
                         "arch_all_host_architecture": "amd64",
                         "debug_level": 0,
+                        "extra_repositories": [
+                            {
+                                "url": "http://example.com/",
+                                "suite": "bookworm",
+                                "components": ["main"],
+                            }
+                        ],
                     },
                 )
 
@@ -685,6 +736,13 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                         "exclude_tests": [],
                         "debug_level": 0,
                         "extra_environment": {},
+                        "extra_repositories": [
+                            {
+                                "url": "http://example.com/",
+                                "suite": "bookworm",
+                                "components": ["main"],
+                            }
+                        ],
                         "needs_internet": AutopkgtestNeedsInternet.RUN,
                         "fail_on": {},
                         "timeout": None,
@@ -732,6 +790,32 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             children = list(WorkRequest.objects.filter(parent=rdep_autopkgtest))
             self.assertEqual(len(children), 2)
 
+    def test_compute_dynamic_data(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        wr = self.playground.create_workflow(
+            task_name="reverse_dependencies_autopkgtest",
+            task_data=ReverseDependenciesAutopkgtestWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                vendor="debian",
+                codename="trixie",
+                suite_collection="sid@debian:suite",
+            ),
+        )
+        workflow = ReverseDependenciesAutopkgtestWorkflow(wr)
+
+        self.assertEqual(
+            workflow.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(
+                subject="hello", parameter_summary="hello_1.0-1"
+            ),
+        )
+
     def test_label(self) -> None:
         """Test get_label."""
         w = self.create_rdep_autopkgtest_workflow()
