@@ -37,6 +37,7 @@ from debusine.tasks.models import (
     TaskTypes,
 )
 from debusine.test.django import TestCase
+from debusine.test.utils import create_system_tarball_data
 
 
 class SbuildWorkflowTests(CollectionTestMixin, TestCase):
@@ -113,7 +114,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(parameter_summary="hello_1.0-1"),
+            BaseDynamicTaskData(
+                subject="hello", parameter_summary="hello_1.0-1"
+            ),
         )
 
     @context.disable_permission_checks()
@@ -148,14 +151,11 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
         o.validate_input()
 
         # environment_variant
-        data = {
-            "codename": "bookworm",
-            "architecture": "amd64",
-            "variant": "buildd",
-        }
         tarball, _ = self.playground.create_artifact(
             category=ArtifactCategory.SYSTEM_TARBALL,
-            data={**data, "with_dev": True},
+            data=create_system_tarball_data(
+                codename="bookworm", variant="buildd"
+            ),
         )
         # add with explicit :variant
         tarball_item = self.playground.create_debian_environment(
@@ -230,136 +230,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                         ):
                             o.architectures
 
-    def test_create_work_request_schroot(self) -> None:
-        """Test creating a Sbuild workflow workrequest with schroot."""
-        with context.disable_permission_checks():
-            default_workspace().collections.filter(
-                category=CollectionCategory.PACKAGE_BUILD_LOGS
-            ).delete()
-
-            # Create a source package
-            source = self.create_source_package(
-                "hello",
-                "1.0",
-                dsc_fields={
-                    "Binary": "hello, hello-dev",
-                    "Architecture": "all amd64 s390x",
-                },
-            )
-
-            template = self.create_sbuild_template()
-            wr = WorkRequest.objects.create_workflow(
-                template=template,
-                data={
-                    "prefix": "signed-source/",
-                    "input": {
-                        "source_artifact": source.pk,
-                    },
-                    "backend": "schroot",
-                    "target_distribution": "debian:bookworm",
-                    "architectures": ["all", "amd64", "s390x"],
-                },
-                created_by=self.playground.get_default_user(),
-            )
-            self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
-            wr.mark_running()
-        orchestrate_workflow(wr)
-
-        expected_children = [
-            (
-                "all",
-                {
-                    "build_components": ["all"],
-                    "build_profiles": None,
-                    "host_architecture": "amd64",
-                },
-            ),
-            (
-                "amd64",
-                {
-                    "build_components": ["any"],
-                    "build_profiles": None,
-                    "host_architecture": "amd64",
-                },
-            ),
-            (
-                "s390x",
-                {
-                    "build_components": ["any"],
-                    "build_profiles": None,
-                    "host_architecture": "s390x",
-                },
-            ),
-        ]
-
-        children = list(WorkRequest.objects.filter(parent=wr).order_by("id"))
-        self.assertEqual(len(children), len(expected_children))
-
-        for child, expected_child in zip(children, expected_children):
-            expected_arch, expected_data = expected_child
-            self.assertEqual(child.status, WorkRequest.Statuses.PENDING)
-            self.assertEqual(child.task_type, TaskTypes.WORKER)
-            self.assertEqual(child.task_name, "sbuild")
-            self.assertEqual(
-                child.task_data,
-                {
-                    'backend': 'schroot',
-                    'distribution': "bookworm",
-                    'environment': None,
-                    'input': {'source_artifact': source.pk},
-                    'binnmu': None,
-                    'extra_repositories': None,
-                    **expected_data,
-                },
-            )
-            promise = CollectionItem.objects.get(
-                parent_collection=wr.internal_collection,
-                child_type=CollectionItem.Types.BARE,
-                category=BareDataCategory.PROMISE,
-                name=f"signed-source/build-{expected_arch}",
-            )
-            self.assertEqual(
-                promise.data,
-                {
-                    "promise_work_request_id": child.id,
-                    "promise_workflow_id": wr.id,
-                    "promise_category": ArtifactCategory.UPLOAD,
-                    "binary_names": ["hello", "hello-dev"],
-                    "architecture": expected_arch,
-                },
-            )
-            self.assert_work_request_event_reactions(
-                child,
-                on_success=[
-                    {
-                        "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
-                        "artifact_filters": {
-                            "category": ArtifactCategory.UPLOAD
-                        },
-                        "collection": "internal@collections",
-                        "name_template": f"signed-source/build-{expected_arch}",
-                        "variables": {
-                            "binary_names": ["hello", "hello-dev"],
-                            "architecture": expected_arch,
-                        },
-                    },
-                ],
-            )
-            self.assertQuerySetEqual(child.dependencies.all(), [])
-            self.assertEqual(
-                child.workflow_data_json,
-                {
-                    "display_name": f"Build {expected_arch}",
-                    "step": f"build-{expected_arch}",
-                },
-            )
-
-        # populate() idempotence
-        orchestrator = SbuildWorkflow(wr)
-        orchestrator.populate()
-        children = list(WorkRequest.objects.filter(parent=wr))
-        self.assertEqual(len(children), len(expected_children))
-
     def test_create_work_request_unshare(self) -> None:
         """Test creating a Sbuild workflow workrequest with unshare."""
         with context.disable_permission_checks():
@@ -369,12 +239,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             tarballs = {
                 arch: self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
-                    data={
-                        "codename": "bookworm",
-                        "architecture": arch,
-                        "variant": "apt",
-                        "with_dev": True,
-                    },
+                    data=create_system_tarball_data(
+                        codename="bookworm", variant="apt", architecture=arch
+                    ),
                 )[0]
                 for arch in ("amd64", "s390x")
             }
@@ -455,7 +322,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 {
                     'backend': 'unshare',
                     'environment': 'debian/match:codename=bookworm',
-                    'distribution': None,
                     'input': {'source_artifact': source.pk},
                     'binnmu': None,
                     'extra_repositories': None,
@@ -475,6 +341,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                     "promise_workflow_id": wr.id,
                     "promise_category": ArtifactCategory.UPLOAD,
                     "binary_names": ["hello", "hello-dev"],
+                    "source_package_name": "hello",
                     "architecture": expected_arch,
                 },
             )
@@ -490,6 +357,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                         "name_template": f"build-{expected_arch}",
                         "variables": {
                             "binary_names": ["hello", "hello-dev"],
+                            "source_package_name": "hello",
                             "architecture": expected_arch,
                         },
                     }
@@ -519,12 +387,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             tarballs = {
                 arch: self.playground.create_artifact(
                     category=ArtifactCategory.SYSTEM_TARBALL,
-                    data={
-                        "codename": "bookworm",
-                        "architecture": arch,
-                        "variant": "apt",
-                        "with_dev": True,
-                    },
+                    data=create_system_tarball_data(
+                        codename="bookworm", variant="apt", architecture=arch
+                    ),
                 )[0]
                 for arch in ("amd64", "s390x")
             }
@@ -593,12 +458,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             )
             tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                data={
-                    "codename": "bookworm",
-                    "architecture": "amd64",
-                    "variant": "apt",
-                    "with_dev": True,
-                },
+                data=create_system_tarball_data(
+                    codename="bookworm", variant="apt", architecture="amd64"
+                ),
             )
             environments.manager.add_artifact(
                 tarball,
@@ -644,7 +506,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             {
                 "input": {"source_artifact": source.pk},
                 "host_architecture": "amd64",
-                "distribution": None,
                 "build_components": ["any"],
                 "build_profiles": None,
                 "binnmu": None,
@@ -667,6 +528,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 "promise_category": ArtifactCategory.UPLOAD,
                 "binary_names": ["hello", "hello-dev"],
                 "architecture": "amd64",
+                "source_package_name": "hello",
             },
         )
         self.assert_work_request_event_reactions(
@@ -697,6 +559,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                     "variables": {
                         "binary_names": ["hello", "hello-dev"],
                         "architecture": "amd64",
+                        "source_package_name": "hello",
                     },
                 },
                 {
@@ -732,12 +595,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             )
             tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                data={
-                    "codename": "bookworm",
-                    "architecture": "amd64",
-                    "variant": "apt",
-                    "with_dev": True,
-                },
+                data=create_system_tarball_data(
+                    codename="bookworm", variant="apt", architecture="amd64"
+                ),
             )
             environments.manager.add_artifact(
                 tarball,
@@ -777,7 +637,6 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             {
                 "input": {"source_artifact": source.pk},
                 "host_architecture": "amd64",
-                "distribution": None,
                 "build_components": ["any"],
                 "build_profiles": None,
                 "binnmu": None,
@@ -795,6 +654,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 "variables": {
                     "binary_names": ["hello", "hello-dev"],
                     "architecture": "amd64",
+                    "source_package_name": "hello",
                 },
             },
             child.event_reactions_json["on_success"],
@@ -815,12 +675,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             )
             tarball, _ = self.playground.create_artifact(
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                data={
-                    "codename": "bookworm",
-                    "architecture": "amd64",
-                    "variant": "apt",
-                    "with_dev": True,
-                },
+                data=create_system_tarball_data(
+                    codename="bookworm", variant="apt", architecture="amd64"
+                ),
             )
             environments.manager.add_artifact(
                 tarball,
@@ -864,6 +721,7 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 "variables": {
                     "binary_names": ["hello-unsigned"],
                     "architecture": "amd64",
+                    "source_package_name": "hello",
                 },
             },
             child.event_reactions_json["on_success"],
@@ -916,8 +774,9 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
         orchestrator = create_orchestrator(invalid_artifact)
         with self.assertRaisesRegex(
             TaskConfigError,
-            r'Category of ".* debusine:test .*" is not one of '
-            r"\['debian:source-package', 'debian:upload'\]",
+            r"^input.source_artifact: unexpected artifact category: "
+            r"'debusine:test'. Valid categories: "
+            r"\['debian:source-package', 'debian:upload'\]$",
         ):
             orchestrator.source_package
 
