@@ -15,11 +15,16 @@ from django.test import TestCase
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
-from debusine.db.models import WorkRequest, WorkflowTemplate, default_workspace
+from debusine.db.models import (
+    TaskDatabase,
+    WorkRequest,
+    WorkflowTemplate,
+    default_workspace,
+)
 from debusine.server.collections.tests.utils import CollectionTestMixin
 from debusine.server.workflows import UpdateEnvironmentsWorkflow
 from debusine.server.workflows.base import orchestrate_workflow
-from debusine.tasks.models import TaskTypes
+from debusine.tasks.models import BaseDynamicTaskData, TaskTypes
 
 
 class UpdateEnvironmentsWorkflowTests(CollectionTestMixin, TestCase):
@@ -287,6 +292,27 @@ class UpdateEnvironmentsWorkflowTests(CollectionTestMixin, TestCase):
         assert item is not None
         self.assertEqual(item.artifact, tarball)
 
+    def test_compute_dynamic_data(self) -> None:
+        wr = WorkRequest(
+            task_data={
+                "vendor": "debian",
+                "targets": [
+                    {
+                        "codenames": ["bookworm"],
+                        "architectures": ["amd64"],
+                        "mmdebstrap_template": {},
+                    }
+                ],
+            },
+            workspace=default_workspace(),
+        )
+        w = UpdateEnvironmentsWorkflow(wr)
+
+        self.assertEqual(
+            w.compute_dynamic_data(TaskDatabase(wr)),
+            BaseDynamicTaskData(subject="debian"),
+        )
+
     def test_label(self) -> None:
         """Test get_label."""
         wr = WorkRequest(
