@@ -16,6 +16,7 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
     DebianSourcePackage,
+    DebusinePromise,
 )
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
@@ -139,12 +140,11 @@ class WorkflowTests(TestCase):
             child, category, name, data=data, artifact_filters=artifact_filters
         )
 
-        promise = CollectionItem.objects.get(
-            parent_collection=workflow_root.internal_collection,
-            child_type=CollectionItem.Types.BARE,
-            category=BareDataCategory.PROMISE,
-            name=name,
+        promise = CollectionItem.active_objects.get(
+            parent_collection=workflow_root.internal_collection, name=name
         )
+        self.assertEqual(promise.child_type, CollectionItem.Types.BARE)
+        self.assertEqual(promise.category, BareDataCategory.PROMISE)
         self.assertEqual(
             promise.data,
             {
@@ -154,22 +154,44 @@ class WorkflowTests(TestCase):
                 **data,
             },
         )
-        self.assert_work_request_event_reactions(
-            child,
-            on_success=[
-                {
-                    "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
-                    "collection": "internal@collections",
-                    "name_template": name,
-                    "variables": data,
-                    "artifact_filters": {
-                        **artifact_filters,
-                        "category": category,
-                    },
-                }
-            ],
+        on_success = [
+            {
+                "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
+                "collection": "internal@collections",
+                "name_template": name,
+                "variables": data,
+                "artifact_filters": {
+                    **artifact_filters,
+                    "category": category,
+                },
+            }
+        ]
+        self.assert_work_request_event_reactions(child, on_success=on_success)
+
+        # provides_artifact is idempotent.
+        artifact, _ = self.playground.create_artifact()
+        assert workflow_root.internal_collection is not None
+        workflow_root.internal_collection.manager.add_artifact(
+            artifact,
+            user=child.created_by,
+            workflow=workflow_root,
+            variables={},
+            name=name,
+            replace=True,
         )
 
+        workflow.provides_artifact(
+            child, category, name, data=data, artifact_filters=artifact_filters
+        )
+
+        item = CollectionItem.active_objects.get(
+            parent_collection=workflow_root.internal_collection, name=name
+        )
+        self.assertEqual(item.child_type, CollectionItem.Types.ARTIFACT)
+        self.assertEqual(item.category, ArtifactCategory.TEST)
+        self.assertEqual(item.artifact, artifact)
+        self.assert_work_request_event_reactions(child, on_success=on_success)
+
     @preserve_task_registry()
     def test_provides_artifact_raise_value_error(self) -> None:
         r"""Test requires_artifact() LookupError: key starts with promise\_."""
@@ -327,7 +349,12 @@ class WorkflowTests(TestCase):
             collection,
             "test",
             category=BareDataCategory.PROMISE,
-            data={"architecture": "amd64"},
+            data={
+                "promise_work_request_id": 2,
+                "promise_workflow_id": 1,
+                "promise_category": ArtifactCategory.TEST,
+                "architecture": "amd64",
+            },
         )
         result = LookupResult(
             result_type=CollectionItem.Types.BARE, collection_item=promise
@@ -341,7 +368,14 @@ class WorkflowTests(TestCase):
             "test", CollectionCategory.TEST
         )
         promise = self.playground.create_bare_data_item(
-            collection, "test", category=BareDataCategory.PROMISE
+            collection,
+            "test",
+            category=BareDataCategory.PROMISE,
+            data=DebusinePromise(
+                promise_work_request_id=2,
+                promise_workflow_id=1,
+                promise_category=ArtifactCategory.TEST,
+            ),
         )
         result = LookupResult(
             result_type=CollectionItem.Types.BARE, collection_item=promise
@@ -548,7 +582,12 @@ class WorkflowTests(TestCase):
             collection,
             "test",
             category=BareDataCategory.PROMISE,
-            data={"architecture": 000},
+            data={
+                "promise_work_request_id": 2,
+                "promise_workflow_id": 1,
+                "promise_category": ArtifactCategory.TEST,
+                "architecture": 000,
+            },
         )
         result = LookupResult(
             result_type=CollectionItem.Types.BARE, collection_item=promise
