@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,11 +24,11 @@ from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BaseDynamicTaskData,
     BaseTaskData,
-    EventReactions,
     MmDebstrapData,
     SystemImageBuildData,
     TaskTypes,
 )
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 @dataclass(frozen=True)
@@ -169,24 +169,29 @@ class UpdateEnvironmentsWorkflow(
         step = f"{task_name}-{codename}-{architecture}"
         display_name = f"{display_name_prefix} for {codename}/{architecture}"
 
-        self.work_request.create_child(
+        wr = self.work_request_ensure_child(
             task_name=task_name,
             task_data=task_data_class(**task_data),
             workflow_data=WorkRequestWorkflowData(
                 step=step, display_name=display_name, group=group
             ),
-            event_reactions=EventReactions(
-                on_success=[
-                    ActionUpdateCollectionWithArtifacts(
-                        collection=(
-                            f"{vendor}@{CollectionCategory.ENVIRONMENTS}"
-                        ),
-                        variables=variables.to_dict(),
-                        artifact_filters={"category": category},
-                    )
-                    for variables in action_variables
-                ]
-            ),
+        )
+        for variables in action_variables:
+            wr.add_event_reaction(
+                "on_success",
+                ActionUpdateCollectionWithArtifacts(
+                    collection=(f"{vendor}@{CollectionCategory.ENVIRONMENTS}"),
+                    variables=variables.to_dict(),
+                    artifact_filters={"category": category},
+                ),
+            )
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Compute dynamic data for this workflow."""
+        return BaseDynamicTaskData(
+            subject=self.data.vendor,
         )
 
     def populate(self) -> None:
