@@ -23,8 +23,13 @@ from typing import Any
 
 from debusine import utils
 from debusine.artifacts import Upload
-from debusine.artifacts.models import ArtifactCategory, KeyPurpose
-from debusine.client.models import FilesResponseType, RelationType
+from debusine.artifacts.models import ArtifactCategory
+from debusine.assets import AssetCategory, KeyPurpose
+from debusine.client.models import (
+    AssetPermissionCheckResponse,
+    FilesResponseType,
+    RelationType,
+)
 from debusine.signing.db.models import Key
 from debusine.signing.models import SigningMode
 from debusine.signing.tasks import BaseSigningTask
@@ -48,6 +53,7 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
         self._files: FilesResponseType | None = None
         self._key: Key | None = None
         self._signed_changes_path: Path | None = None
+        self._permission_check: AssetPermissionCheckResponse | None = None
 
     @classmethod
     def analyze_worker(cls) -> dict[str, Any]:
@@ -78,13 +84,23 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
         return DebsignDynamicData(
             unsigned_id=task_database.lookup_single_artifact(
                 self.data.unsigned
-            ),
-            key_id=task_database.lookup_single_artifact(self.data.key),
+            ).id,
         )
 
+    def get_source_artifacts_ids(self) -> list[int]:
+        """Return the list of source artifact IDs used by this task."""
+        if not self.dynamic_data:
+            return []
+        return [
+            self.dynamic_data.unsigned_id,
+        ]
+
     def fetch_input(self, destination: Path) -> bool:
         """Fetch artifacts and database objects needed by the task."""
+        assert self.debusine is not None
         assert self.dynamic_data is not None
+        assert self.work_request_id is not None
+        assert self.workspace_name is not None
 
         unsigned = self.fetch_artifact(
             self.dynamic_data.unsigned_id, destination
@@ -100,18 +116,34 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
             return False
         self._files = unsigned.files
 
-        key_artifact = self.fetch_artifact(
-            self.dynamic_data.key_id, destination
+        asset_slug = f"openpgp:{self.data.key}"
+        permission_check = self.debusine.asset_permission_check(
+            asset_category=AssetCategory.SIGNING_KEY,
+            asset_slug=asset_slug,
+            permission_name="sign_with",
+            artifact_id=unsigned.id,
+            work_request_id=self.work_request_id,
+            workspace=self.workspace_name,
         )
-        fingerprint = key_artifact.data["fingerprint"]
+        self._permission_check = permission_check
+        if not permission_check.has_permission:
+            self.append_to_log_file(
+                "fetch_input.log",
+                [
+                    f"User {permission_check.username} does not have "
+                    f"permission to sign {unsigned.id} with {asset_slug}."
+                ],
+            )
+            return False
+
         try:
             self._key = Key.objects.get(
-                purpose=KeyPurpose.OPENPGP, fingerprint=fingerprint
+                purpose=KeyPurpose.OPENPGP, fingerprint=self.data.key
             )
         except Key.DoesNotExist:
             self.append_to_log_file(
                 "fetch_input.log",
-                [f"Signing key openpgp:{fingerprint} does not exist"],
+                [f"Signing key openpgp:{self.data.key} does not exist"],
             )
             return False
 
@@ -141,6 +173,7 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
         assert self.work_request_id is not None
         assert self._files is not None
         assert self._key is not None
+        assert self._permission_check is not None
 
         with self.open_debug_log_file("cmd-output.log", mode="wb") as log_file:
             # Upload.files_contain_changes checks that there is exactly one
@@ -157,6 +190,9 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
                     signature_path,
                     SigningMode.DEBSIGN,
                     self.work_request_id,
+                    username=self._permission_check.username,
+                    user_id=self._permission_check.user_id,
+                    resource=self._permission_check.resource,
                     log_file=log_file,
                 )
             except Exception as e:
@@ -190,11 +226,6 @@ class Debsign(BaseSigningTask[DebsignData, DebsignDynamicData]):
             self.dynamic_data.unsigned_id,
             RelationType.RELATES_TO,
         )
-        self.debusine.relation_create(
-            uploaded_upload_artifact.id,
-            self.dynamic_data.key_id,
-            RelationType.RELATES_TO,
-        )
 
     def get_label(self) -> str:
         """Return the task label."""
