@@ -10,6 +10,8 @@
 """Unit tests for base signing task classes."""
 
 from pathlib import Path
+from tempfile import TemporaryDirectory
+from unittest import mock
 
 from django.db import connections
 from django.test import TestCase, TransactionTestCase
@@ -79,4 +81,11 @@ class BaseSigningTaskTransactionTests(TransactionTestCase):
     def test_runs_in_transaction(self) -> None:
         """Tasks are executed in a transaction."""
         task = TestBaseSigningTask({}, {})
-        self.assertTrue(task.execute())
+        task._debug_log_files_directory = TemporaryDirectory(
+            prefix="debusine-tests-"
+        )
+        self.addCleanup(task._debug_log_files_directory.cleanup)
+        with mock.patch.object(
+            task, "_upload_work_request_debug_logs", autospec=True
+        ):
+            self.assertTrue(task.execute())
