@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -23,6 +23,7 @@ from debusine.tasks._task import (  # isort: split
     BaseExternalTask,
     BaseTask,
     BaseTaskWithExecutor,
+    ExtraRepositoryMixin,
     RunCommandTask,
     TaskConfigError,
     get_environment,
@@ -56,6 +57,7 @@ __all__ = [
     "BaseTaskWithExecutor",
     "Blhc",
     "DebDiff",
+    "ExtraRepositoryMixin",
     "ExtractForSigning",
     "Lintian",
     "MakeSourcePackageUpload",
