@@ -30,8 +30,9 @@ import tempfile
 import traceback
 from abc import ABCMeta, abstractmethod
 from collections import defaultdict
-from collections.abc import Generator
+from collections.abc import Collection, Generator
 from contextlib import contextmanager
+from datetime import datetime, timezone
 from pathlib import Path
 from typing import (
     Any,
@@ -41,6 +42,7 @@ from typing import (
     Generic,
     IO,
     Literal,
+    Self,
     TYPE_CHECKING,
     TextIO,
     TypeVar,
@@ -52,7 +54,7 @@ if TYPE_CHECKING:
     from _typeshed import OpenBinaryModeWriting, OpenTextModeWriting
 
 from debusine.artifacts import WorkRequestDebugLogs
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks.executors import (
@@ -77,7 +79,7 @@ from debusine.tasks.models import (
     parse_key_value_lookup_segment,
     parse_lookup_string_segments,
 )
-from debusine.tasks.server import TaskDatabaseInterface
+from debusine.tasks.server import ArtifactInfo, TaskDatabaseInterface
 from debusine.utils import extract_generic_type_arguments
 
 
@@ -97,6 +99,13 @@ class TaskConfigError(Exception):
         super().__init__(message)
         self.original_exception = original_exception
 
+    def add_parent_message(self, msg: str) -> None:
+        """Prepend the error message with one from the containing scope."""
+        if self.args[0]:
+            self.args = (f"{msg}: {self.args[0]}",)
+        else:
+            self.args = (msg,)
+
     def __str__(self) -> str:
         """
         Return a string representation.
@@ -249,6 +258,18 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
             tempfile.TemporaryDirectory[str]
         ) = None
 
+    def instantiate_with_new_data(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> Self:
+        """Create a new task like this one, with different data."""
+        new_task = self.__class__(task_data, dynamic_task_data)
+        new_task.work_request_id = self.work_request_id
+        new_task.workspace_name = self.workspace_name
+        new_task.worker_host_architecture = self.worker_host_architecture
+        return new_task
+
     def append_to_log_file(self, filename: str, lines: list[str]) -> None:
         """
         Open log file and write contents into it.
@@ -259,6 +280,29 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         with self.open_debug_log_file(filename) as file:
             file.writelines([line + "\n" for line in lines])
 
+    @staticmethod
+    def ensure_artifact_categories(
+        *,
+        configuration_key: str,
+        category: str,
+        expected: Collection[ArtifactCategory],
+    ) -> None:
+        """
+        Validate that the artifact's category is one of the expected categories.
+
+        :param configuration_key: Optional key to identify the source of the
+          artifact. Provides additional context in error messages.
+        :param category: The category to validate.
+        :param expected: A collection of valid artifact categories.
+        """
+        valid_categories = [cat.value for cat in sorted(expected)]
+        if category not in valid_categories:
+            raise TaskConfigError(
+                f"{configuration_key}: unexpected artifact "
+                f"category: '{category}'. "
+                f"Valid categories: {valid_categories}"
+            )
+
     @overload
     def open_debug_log_file(
         self, filename: str, *, mode: "OpenTextModeWriting" = "a"
@@ -587,6 +631,13 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         :return: None if no label could be computed from task data
         """
 
+    def get_subject(self) -> str | None:
+        """Return the subject if known or None."""
+        if self.dynamic_data is not None:
+            return self.dynamic_data.subject
+
+        return None
+
     def get_event_reactions(
         self,
         event_name: Literal[  # noqa: U100
@@ -735,7 +786,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         """Clean up after running the task."""
         # Do nothing by default.
 
-    def _execute(self) -> bool:
+    def _execute(self) -> bool:  # noqa: C901
         """
         Fetch the required input, execute the command and upload artifacts.
 
@@ -756,18 +807,25 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         with (
             self._temporary_directory() as execute_directory,
             self._temporary_directory() as download_directory,
+            self.open_debug_log_file("stages.log") as stages_log,
         ):
-            self.logger.info(
-                "Work request %s: Fetching input", self.work_request_id
-            )
+
+            def log_stage(message: str) -> None:
+                """Log a stage, both to the worker log and to a WR debug log."""
+                self.logger.info(
+                    "Work request %s: %s", self.work_request_id, message
+                )
+                print(
+                    f"{datetime.now(timezone.utc).isoformat()} {message}",
+                    file=stages_log,
+                )
+
+            log_stage("Fetching input")
             if not self.fetch_input(download_directory):
                 return False
 
             try:
-                self.logger.info(
-                    "Work request %s: Configuring for execution",
-                    self.work_request_id,
-                )
+                log_stage("Configuring for execution")
                 if not self.configure_for_execution(download_directory):
                     return False
             except Exception as exc:
@@ -776,15 +834,10 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
 
             try:
                 try:
-                    self.logger.info(
-                        "Work request %s: Preparing to run",
-                        self.work_request_id,
-                    )
+                    log_stage("Preparing to run")
                     self.prepare_to_run(download_directory, execute_directory)
 
-                    self.logger.info(
-                        "Work request %s: Running", self.work_request_id
-                    )
+                    log_stage("Running")
 
                     execution_succeeded = self.run(execute_directory)
                 except Exception as exc:
@@ -792,9 +845,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     return False
 
                 try:
-                    self.logger.info(
-                        "Work request %s: Checking output", self.work_request_id
-                    )
+                    log_stage("Checking output")
                     if errors := self.check_directory_for_consistency_errors(
                         execute_directory
                     ):
@@ -803,10 +854,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                         )
                         return False
 
-                    self.logger.info(
-                        "Work request %s: Uploading artifacts",
-                        self.work_request_id,
-                    )
+                    log_stage("Uploading artifacts")
                     self.upload_artifacts(
                         execute_directory, execution_success=execution_succeeded
                     )
@@ -815,15 +863,19 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     return False
             finally:
                 try:
-                    self.logger.info(
-                        "Work request %s: Cleaning up", self.work_request_id
-                    )
+                    log_stage("Cleaning up")
                     self.cleanup()
                 except Exception as exc:
                     self._log_exception(exc, stage="post_execution")
                     return False
 
-            return execution_succeeded
+        if execution_succeeded:
+            # We mainly care about stages.log for failures.  It's rather
+            # boring for successes.
+            assert self._debug_log_files_directory is not None
+            (Path(self._debug_log_files_directory.name) / "stages.log").unlink()
+
+        return execution_succeeded
 
     def _upload_work_request_debug_logs(self) -> None:
         """
@@ -867,7 +919,7 @@ def get_environment(
     backend: str | None,
     default_category: CollectionCategory | None = None,
     image_category: ExecutorImageCategory | None = None,
-) -> int:
+) -> ArtifactInfo:
     """
     Get an environment.
 
@@ -884,7 +936,7 @@ def get_environment(
     :param image_category: try to use an environment with this image
       category; defaults to the image category needed by the executor for
       `self.backend`
-    :return: the ID of a suitable environment artifact
+    :return: the ArtifactInfo of a suitable environment artifact
     """
     if (
         isinstance(lookup, str)
@@ -952,7 +1004,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
         default_category: CollectionCategory | None = None,
         image_category: ExecutorImageCategory | None = None,
         set_backend: bool = True,
-    ) -> int:
+    ) -> ArtifactInfo:
         """
         Get an environment for an executor-capable task.
 
@@ -969,7 +1021,7 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
           for `self.backend`
         :param set_backend: if True (default), try to use an environment
           matching `self.backend`
-        :return: the ID of a suitable environment artifact
+        :return: the ArtifactInfo of a suitable environment artifact
         """
         return get_environment(
             task_database=task_database,
