@@ -31,8 +31,10 @@ from debusine.artifacts import (
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
-    KeyPurpose,
+    DebianBinaryPackage,
+    get_binary_package_name,
 )
+from debusine.assets import KeyPurpose
 from debusine.client.models import ArtifactResponse, RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
@@ -95,18 +97,27 @@ class AssembleSignedSource(
         self, task_database: TaskDatabaseInterface
     ) -> AssembleSignedSourceDynamicData:
         """Resolve artifact lookups for this task."""
+        template = task_database.lookup_single_artifact(self.data.template)
+
+        self.ensure_artifact_categories(
+            configuration_key="template",
+            category=template.category,
+            expected=(ArtifactCategory.BINARY_PACKAGE,),
+        )
+        assert isinstance(template.data, DebianBinaryPackage)
+        package_name = get_binary_package_name(template.data)
+
         return AssembleSignedSourceDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            template_id=task_database.lookup_single_artifact(
-                self.data.template
-            ),
+            ).id,
+            template_id=template.id,
             signed_ids=task_database.lookup_multiple_artifacts(
                 self.data.signed
-            ),
+            ).get_ids(),
+            subject=package_name,
         )
 
     def _check_category(
