@@ -22,11 +22,17 @@ except ImportError:
 from debusine import utils
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
 from debusine.artifacts.models import (
+    ArtifactCategory,
     CollectionCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResult,
     DebianAutopkgtestResultStatus,
     DebianAutopkgtestSource,
+    DebianSourcePackage,
+    DebianSystemImage,
+    DebianSystemTarball,
+    DebianUpload,
+    get_source_package_name,
 )
 from debusine.tasks import (
     BaseTaskWithExecutor,
@@ -95,21 +101,49 @@ class Autopkgtest(
         self, task_database: TaskDatabaseInterface
     ) -> AutopkgtestDynamicData:
         """Resolve artifact lookups for this task."""
+        input_source_artifact = task_database.lookup_single_artifact(
+            self.data.input.source_artifact
+        )
+
+        self.ensure_artifact_categories(
+            configuration_key="input.source_artifact",
+            category=input_source_artifact.category,
+            expected=(ArtifactCategory.SOURCE_PACKAGE, ArtifactCategory.UPLOAD),
+        )
+        assert isinstance(
+            input_source_artifact.data, (DebianSourcePackage, DebianUpload)
+        )
+        package_name = get_source_package_name(input_source_artifact.data)
+
+        environment = self.get_environment(
+            task_database,
+            self.data.environment,
+            default_category=CollectionCategory.ENVIRONMENTS,
+        )
+        self.ensure_artifact_categories(
+            configuration_key="environment",
+            category=environment.category,
+            expected=(
+                ArtifactCategory.SYSTEM_TARBALL,
+                ArtifactCategory.SYSTEM_IMAGE,
+            ),
+        )
+        assert isinstance(
+            environment.data, (DebianSystemTarball, DebianSystemImage)
+        )
+
         return AutopkgtestDynamicData(
-            environment_id=self.get_environment(
-                task_database,
-                self.data.environment,
-                default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
+            environment_id=environment.id,
+            input_source_artifact_id=input_source_artifact.id,
             input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
                 self.data.input.binary_artifacts
-            ),
+            ).get_ids(),
             input_context_artifacts_ids=task_database.lookup_multiple_artifacts(
                 self.data.input.context_artifacts
-            ),
+            ).get_ids(),
+            subject=package_name,
+            runtime_context=f"{self.host_architecture()}:{self.backend}",
+            configuration_context=environment.data.codename,
         )
 
     def check_directory_for_consistency_errors(
@@ -408,6 +442,8 @@ class Autopkgtest(
 
     def get_label(self) -> str:
         """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "autopkgtest"
+        return (
+            f"autopkgtest {subject}"
+            if (subject := self.get_subject())
+            else "autopkgtest"
+        )
