@@ -14,7 +14,12 @@ from typing import Any
 
 from debusine import utils
 from debusine.artifacts import BlhcArtifact
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianPackageBuildLog,
+    get_source_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import BlhcData, BlhcDynamicData
@@ -46,6 +51,18 @@ class Blhc(
         self, task_database: TaskDatabaseInterface
     ) -> BlhcDynamicData:
         """Resolve artifact lookups for this task."""
+        input_artifact = task_database.lookup_single_artifact(
+            self.data.input.artifact
+        )
+
+        self.ensure_artifact_categories(
+            configuration_key="input.artifact",
+            category=input_artifact.category,
+            expected=(ArtifactCategory.PACKAGE_BUILD_LOG,),
+        )
+        assert isinstance(input_artifact.data, DebianPackageBuildLog)
+        package_name = get_source_package_name(input_artifact.data)
+
         return BlhcDynamicData(
             environment_id=(
                 None
@@ -54,11 +71,10 @@ class Blhc(
                     task_database,
                     self.data.environment,
                     default_category=CollectionCategory.ENVIRONMENTS,
-                )
-            ),
-            input_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.artifact
+                ).id
             ),
+            input_artifact_id=input_artifact.id,
+            subject=package_name,
         )
 
     def _cmdline(self) -> list[str]:
