@@ -14,7 +14,13 @@ from typing import Any
 
 from debusine import utils
 from debusine.artifacts import DebDiffArtifact
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSourcePackage,
+    DebianUpload,
+    get_source_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import DebDiffData, DebDiffDynamicData
@@ -47,28 +53,72 @@ class DebDiff(
         self, task_database: TaskDatabaseInterface
     ) -> DebDiffDynamicData:
         """Resolve artifact lookups for this task."""
+        subject = None
+        source_artifacts_ids = []
+
+        if self.data.input.source_artifacts:
+            source_artifacts = [
+                task_database.lookup_single_artifact(artifact)
+                for artifact in self.data.input.source_artifacts
+            ]
+
+            for source_artifact in source_artifacts:
+                # TODO: improve configuration_key including the index?
+                self.ensure_artifact_categories(
+                    configuration_key="input.source_artifacts",
+                    category=source_artifact.category,
+                    expected=(ArtifactCategory.SOURCE_PACKAGE,),
+                )
+
+                if subject is None:
+                    assert isinstance(source_artifact.data, DebianSourcePackage)
+
+                    package_name = get_source_package_name(source_artifact.data)
+                    subject = f"source:{package_name}"
+
+                source_artifacts_ids.append(source_artifact.id)
+
+        binary_artifacts_ids = []
+
+        if self.data.input.binary_artifacts:
+            binary_artifacts = [
+                task_database.lookup_multiple_artifacts(artifact)
+                for artifact in self.data.input.binary_artifacts
+            ]
+
+            source_package_names = set()
+
+            for binary_artifact in binary_artifacts:
+                binary_artifacts_ids.append(binary_artifact.get_ids())
+
+                for artifact in binary_artifact:
+                    # TODO: improve configuration_key including the index?
+                    self.ensure_artifact_categories(
+                        configuration_key="input.binary_artifacts",
+                        category=artifact.category,
+                        expected=(ArtifactCategory.UPLOAD,),
+                    )
+
+                    assert isinstance(
+                        artifact.data,
+                        (DebianUpload,),
+                    )
+                    source_package_names.add(
+                        get_source_package_name(artifact.data)
+                    )
+
+            if subject is None and len(source_package_names) == 1:
+                subject = f"binary:{source_package_names.pop()}"
+
         return DebDiffDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_source_artifacts_ids=(
-                None
-                if self.data.input.source_artifacts is None
-                else [
-                    task_database.lookup_single_artifact(artifact)
-                    for artifact in self.data.input.source_artifacts
-                ]
-            ),
-            input_binary_artifacts_ids=(
-                None
-                if self.data.input.binary_artifacts is None
-                else [
-                    task_database.lookup_multiple_artifacts(artifact)
-                    for artifact in self.data.input.binary_artifacts
-                ]
-            ),
+            ).id,
+            input_source_artifacts_ids=source_artifacts_ids or None,
+            input_binary_artifacts_ids=binary_artifacts_ids or None,
+            subject=subject,
         )
 
     def _cmdline(self) -> list[str]:
