@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -32,6 +32,7 @@ from tempfile import TemporaryDirectory
 from typing import Any, AnyStr, ClassVar, Literal, overload
 
 import yaml
+from prometheus_client.parser import text_string_to_metric_families
 
 from debusine import utils
 from debusine.client.debusine import Debusine
@@ -39,6 +40,7 @@ from debusine.client.models import ArtifactResponse
 from debusine.tasks.executors.base import (
     ExecutorImageCategory,
     ExecutorInterface,
+    ExecutorStatistics,
     ImageImportError,
     InstanceInterface,
     InstanceNotRunning,
@@ -463,6 +465,71 @@ class IncusExecutorMixin:
         """Remove incus images from the cache."""
         run_incus_cmd("image", "rm", f"artifact/{artifact_id}")
 
+    @classmethod
+    def get_statistics(
+        cls, instance: InstanceInterface | None
+    ) -> ExecutorStatistics:
+        """Return statistics for this executor."""
+        statistics = ExecutorStatistics()
+
+        if instance is not None and instance.is_started():
+            assert isinstance(instance, IncusInstance)
+            assert instance._instance is not None
+
+            free_disk_space: int | None = None
+            free_memory: int | None = None
+            for family in text_string_to_metric_families(
+                subprocess.run(
+                    ["incus", "query", "/1.0/metrics"],
+                    capture_output=True,
+                    check=True,
+                    text=True,
+                ).stdout
+            ):
+
+                def sum_relevant_samples() -> float:
+                    return sum(
+                        sample.value
+                        for sample in family.samples
+                        if sample.labels.get("name") == instance._instance
+                    )
+
+                match family.name:
+                    case "incus_cpu_effective_total":
+                        statistics.cpu_count = int(sum_relevant_samples())
+                    case "incus_filesystem_avail_bytes":
+                        free_disk_space = int(sum_relevant_samples())
+                    case "incus_filesystem_size_bytes":
+                        statistics.available_disk_space = int(
+                            sum_relevant_samples()
+                        )
+                    case "incus_memory_MemFree_bytes":
+                        free_memory = int(sum_relevant_samples())
+                    case "incus_memory_MemTotal_bytes":
+                        statistics.available_memory = int(
+                            sum_relevant_samples()
+                        )
+
+            if (
+                statistics.available_disk_space is not None
+                and free_disk_space is not None
+            ):
+                statistics.disk_space = (
+                    statistics.available_disk_space - free_disk_space
+                )
+            if (
+                statistics.available_memory is not None
+                and free_memory is not None
+            ):
+                # TODO: This is an approximation; in particular, it doesn't take
+                # swap into account.  Incus's metrics don't seem to include
+                # available swap space, so we'd probably need to do something
+                # more complicated to sum all the relevant categories of used
+                # memory.
+                statistics.memory = statistics.available_memory - free_memory
+
+        return statistics
+
 
 class IncusLXCExecutor(
     IncusExecutorMixin, ExecutorInterface, backend_name="incus-lxc"
