@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -19,8 +19,10 @@ from debusine.client.models import ArtifactResponse
 from debusine.tasks.executors.base import (
     ExecutorImageCategory,
     ExecutorInterface,
+    ExecutorStatistics,
     ImageImportError,
     ImageNotDownloadedError,
+    InstanceInterface,
 )
 from debusine.tasks.executors.images import ImageCache
 
@@ -97,3 +99,13 @@ class QemuExecutor(ExecutorInterface, backend_name="qemu"):
             self.system_image.data["architecture"],
             str(self._local_path),
         ]
+
+    @classmethod
+    def get_statistics(
+        cls, instance: InstanceInterface | None  # noqa: U100
+    ) -> ExecutorStatistics:
+        """Return statistics for this executor."""
+        # TODO: The autopkgtest-virt-server interface makes this very
+        # difficult to implement.  See
+        # https://salsa.debian.org/freexian-team/debusine/-/issues/671.
+        return ExecutorStatistics()
