@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,6 +15,7 @@ from pathlib import Path
 from shutil import rmtree
 from subprocess import CalledProcessError, CompletedProcess, DEVNULL, PIPE
 from tempfile import NamedTemporaryFile, mkdtemp
+from textwrap import dedent
 from typing import Any, cast
 from unittest.mock import ANY, MagicMock, call, create_autospec, patch
 
@@ -26,6 +27,7 @@ from debusine.tasks import Noop
 from debusine.tasks.executors.base import (
     ExecutorImageCategory,
     ExecutorInterface,
+    ExecutorStatistics,
     ImageImportError,
     InstanceNotRunning,
     executor_class,
@@ -650,6 +652,97 @@ class IncusExecutorCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
         self.executor_cls.clean_up_image(42)
         run_incus_cmd.assert_called_once_with("image", "rm", "artifact/42")
 
+    def test_get_statistics_without_instance(self) -> None:
+        """`get_statistics` returns empty statistics without an instance."""
+        self.assertEqual(
+            self.executor_cls.get_statistics(None), ExecutorStatistics()
+        )
+
+    def test_get_statistics_with_instance(self) -> None:
+        """`get_statistics` returns all statistics with an instance."""
+        instance = IncusInstance("image-name")
+        instance._instance = "instance"
+        run_result = MagicMock()
+        run_result.returncode = 0
+        run_result.stdout = dedent(
+            """\
+            # HELP incus_cpu_effective_total The total number of effective CPUs.
+            # TYPE incus_cpu_effective_total gauge
+            incus_cpu_effective_total{name="instance"} 2
+            incus_cpu_effective_total{name="other-instance"} 4
+            # HELP incus_filesystem_avail_bytes The number of available space in bytes.
+            # TYPE incus_filesystem_avail_bytes gauge
+            incus_filesystem_avail_bytes{mountpoint="/",name="instance"} 1e+10
+            incus_filesystem_avail_bytes{mountpoint="/build",name="instance"} 6.7e+10
+            incus_filesystem_avail_bytes{mountpoint="/",name="other-instance"} 1e+10
+            # HELP incus_filesystem_size_bytes The size of the filesystem in bytes.
+            # TYPE incus_filesystem_size_bytes gauge
+            incus_filesystem_size_bytes{mountpoint="/",name="instance"} 2e+10
+            incus_filesystem_size_bytes{mountpoint="/build",name="instance"} 7.4e+10
+            incus_filesystem_size_bytes{mountpoint="/",name="other-instance"} 2e+10
+            # HELP incus_memory_MemFree_bytes The amount of free memory.
+            # TYPE incus_memory_MemFree_bytes gauge
+            incus_memory_MemFree_bytes{name="instance"} 2.0e+9
+            incus_memory_MemFree_bytes{name="other-instance"} 3.1e+9
+            # HELP incus_memory_MemTotal_bytes The amount of used memory.
+            # TYPE incus_memory_MemTotal_bytes gauge
+            incus_memory_MemTotal_bytes{name="instance"} 2.1e+9
+            incus_memory_MemTotal_bytes{name="other-instance"} 3.2e+9
+            # HELP incus_warnings_total The number of active warnings.
+            # TYPE incus_warnings_total counter
+            incus_warnings_total 0
+            """  # noqa: E501
+        )
+
+        with patch(
+            "subprocess.run", autospec=True, return_value=run_result
+        ) as mock_run:
+            statistics = self.executor_cls.get_statistics(instance)
+
+        mock_run.assert_called_once_with(
+            ["incus", "query", "/1.0/metrics"],
+            capture_output=True,
+            check=True,
+            text=True,
+        )
+        self.assertEqual(
+            statistics,
+            ExecutorStatistics(
+                memory=100_000_000,
+                disk_space=17_000_000_000,
+                available_memory=2_100_000_000,
+                available_disk_space=94_000_000_000,
+                cpu_count=2,
+            ),
+        )
+
+    def test_get_statistics_with_instance_limited_details(self) -> None:
+        """`get_statistics` only returns statistics that are available."""
+        instance = IncusInstance("image-name")
+        instance._instance = "instance"
+        run_result = MagicMock()
+        run_result.returncode = 0
+        run_result.stdout = dedent(
+            """\
+            # HELP incus_cpu_effective_total The total number of effective CPUs.
+            # TYPE incus_cpu_effective_total gauge
+            incus_cpu_effective_total{name="instance"} 2
+            """  # noqa: E501
+        )
+
+        with patch(
+            "subprocess.run", autospec=True, return_value=run_result
+        ) as mock_run:
+            statistics = self.executor_cls.get_statistics(instance)
+
+        mock_run.assert_called_once_with(
+            ["incus", "query", "/1.0/metrics"],
+            capture_output=True,
+            check=True,
+            text=True,
+        )
+        self.assertEqual(statistics, ExecutorStatistics(cpu_count=2))
+
 
 class IncusLXCExecutorTests(IncusExecutorCommonTests):
     """Unit tests for IncusLXCExecutor."""
