@@ -19,7 +19,12 @@ from pathlib import Path, PurePath
 from typing import Any
 
 from debusine.artifacts import SigningInputArtifact
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianBinaryPackage,
+    get_binary_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
@@ -81,18 +86,29 @@ class ExtractForSigning(
         self, task_database: TaskDatabaseInterface
     ) -> ExtractForSigningDynamicData:
         """Resolve artifact lookups for this task."""
+        template_artifact = task_database.lookup_single_artifact(
+            self.data.input.template_artifact
+        )
+
+        self.ensure_artifact_categories(
+            configuration_key="input.template_artifact",
+            category=template_artifact.category,
+            expected=(ArtifactCategory.BINARY_PACKAGE,),
+        )
+        assert isinstance(template_artifact.data, DebianBinaryPackage)
+        binary_package_name = get_binary_package_name(template_artifact.data)
+
         return ExtractForSigningDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_template_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.template_artifact
-            ),
+            ).id,
+            input_template_artifact_id=template_artifact.id,
             input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
                 self.data.input.binary_artifacts
-            ),
+            ).get_ids(),
+            subject=binary_package_name,
         )
 
     def fetch_input(self, destination: Path) -> bool:
