@@ -16,7 +16,17 @@ from typing import Any
 from debian import debfile
 from debusine import utils
 from debusine.artifacts import LintianArtifact
-from debusine.artifacts.models import CollectionCategory, DebianLintianSummary
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianBinaryPackage,
+    DebianBinaryPackages,
+    DebianLintianSummary,
+    DebianSourcePackage,
+    DebianSystemTarball,
+    DebianUpload,
+    get_source_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
@@ -90,22 +100,92 @@ class Lintian(
         self, task_database: TaskDatabaseInterface
     ) -> LintianDynamicData:
         """Resolve artifact lookups for this task."""
-        return LintianDynamicData(
-            environment_id=(
-                None
-                if self.data.environment is None
-                else self.get_environment(
-                    task_database,
-                    self.data.environment,
-                    default_category=CollectionCategory.ENVIRONMENTS,
+        input_source_artifact = task_database.lookup_single_artifact(
+            self.data.input.source_artifact
+        )
+        input_binary_artifacts = task_database.lookup_multiple_artifacts(
+            self.data.input.binary_artifacts
+        )
+
+        subject = None
+        if input_source_artifact is not None:
+            self.ensure_artifact_categories(
+                configuration_key="input.source_artifact",
+                category=input_source_artifact.category,
+                expected=(
+                    ArtifactCategory.SOURCE_PACKAGE,
+                    ArtifactCategory.UPLOAD,
+                ),
+            )
+            assert isinstance(
+                input_source_artifact.data, (DebianSourcePackage, DebianUpload)
+            )
+            subject = get_source_package_name(input_source_artifact.data)
+
+        # Validate input_binary_artifacts and get subject
+        if input_binary_artifacts:
+            for binary_artifact in input_binary_artifacts:
+                self.ensure_artifact_categories(
+                    configuration_key="input.binary_artifact",
+                    category=binary_artifact.category,
+                    expected=(
+                        ArtifactCategory.BINARY_PACKAGE,
+                        ArtifactCategory.BINARY_PACKAGES,
+                        ArtifactCategory.UPLOAD,
+                    ),
                 )
+                assert isinstance(
+                    binary_artifact.data,
+                    (DebianBinaryPackage, DebianBinaryPackages, DebianUpload),
+                )
+
+                if subject is None:
+                    subject = get_source_package_name(binary_artifact.data)
+
+        what = {"source", "binary-all", "binary-any"}
+
+        output = self.data.output
+        if not output.source_analysis:
+            what.remove("source")
+        if not output.binary_all_analysis:
+            what.remove("binary-all")
+        if not output.binary_any_analysis:
+            what.remove("binary-any")
+
+        what_str = "+".join(sorted(what))
+
+        runtime_context = None
+        configuration_context = None
+        environment_id = None
+        if self.data.environment is not None:
+            environment = self.get_environment(
+                task_database,
+                self.data.environment,
+                default_category=CollectionCategory.ENVIRONMENTS,
+            )
+            environment_id = environment.id
+
+            self.ensure_artifact_categories(
+                configuration_key="environment",
+                category=environment.category,
+                expected=(ArtifactCategory.SYSTEM_TARBALL,),
+            )
+            assert isinstance(environment.data, DebianSystemTarball)
+
+            runtime_context = f"{what_str}:{environment.data.codename}"
+            configuration_context = environment.data.codename
+
+        return LintianDynamicData(
+            environment_id=environment_id,
+            input_source_artifact_id=(
+                None
+                if input_source_artifact is None
+                else input_source_artifact.id
             ),
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
-            input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
-                self.data.input.binary_artifacts
-            ),
+            input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
+            subject=subject,
+            runtime_context=runtime_context,
+            configuration_context=configuration_context,
         )
 
     @classmethod
@@ -604,6 +684,8 @@ class Lintian(
 
     def get_label(self) -> str:
         """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "lintian"
+        return (
+            f"lintian {subject}"
+            if (subject := self.get_subject())
+            else "lintian"
+        )
