@@ -13,7 +13,12 @@ from typing import Any
 
 import debusine.utils
 from debusine.artifacts import Upload
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSourcePackage,
+    get_source_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import (
@@ -50,15 +55,26 @@ class MakeSourcePackageUpload(
         self, task_database: TaskDatabaseInterface
     ) -> MakeSourcePackageUploadDynamicData:
         """Resolve artifact lookups for this task."""
+        input_source_artifact = task_database.lookup_single_artifact(
+            self.data.input.source_artifact
+        )
+
+        self.ensure_artifact_categories(
+            configuration_key="input.source_artifact",
+            category=input_source_artifact.category,
+            expected=(ArtifactCategory.SOURCE_PACKAGE,),
+        )
+        assert isinstance(input_source_artifact.data, DebianSourcePackage)
+        package_name = get_source_package_name(input_source_artifact.data)
+
         return MakeSourcePackageUploadDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
+            ).id,
+            input_source_artifact_id=input_source_artifact.id,
+            subject=package_name,
         )
 
     def fetch_input(self, destination: Path) -> bool:
