@@ -14,7 +14,12 @@ from typing import Any
 
 import debusine.utils
 from debusine.artifacts import Upload
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianUpload,
+    get_source_package_name,
+)
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
 from debusine.tasks.models import MergeUploadsData, MergeUploadsDynamicData
@@ -53,15 +58,37 @@ class MergeUploads(
         self, task_database: TaskDatabaseInterface
     ) -> MergeUploadsDynamicData:
         """Resolve artifact lookups for this task."""
+        upload_artifacts = task_database.lookup_multiple_artifacts(
+            self.data.input.uploads
+        )
+
+        source_package_names = set()
+        for upload_artifact in upload_artifacts:
+            self.ensure_artifact_categories(
+                configuration_key="input.uploads",
+                category=upload_artifact.category,
+                expected=(ArtifactCategory.UPLOAD,),
+            )
+            assert isinstance(upload_artifact.data, DebianUpload)
+
+            source_package_names.add(
+                get_source_package_name(upload_artifact.data)
+            )
+
+        subject = (
+            source_package_names.pop()
+            if len(source_package_names) == 1
+            else None
+        )
+
         return MergeUploadsDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_uploads_ids=task_database.lookup_multiple_artifacts(
-                self.data.input.uploads
-            ),
+            ).id,
+            input_uploads_ids=upload_artifacts.get_ids(),
+            subject=subject,
         )
 
     def fetch_input(self, destination: Path) -> bool:
