@@ -10,12 +10,12 @@
 """Models used by debusine tasks."""
 
 import re
-from collections.abc import Callable, Iterator
+from collections.abc import Iterator
 from datetime import datetime
 from enum import Enum, StrEnum, auto
 from itertools import groupby
 from pathlib import Path
-from typing import Annotated, Any, Generic, Literal, TypeAlias, TypeVar, Union
+from typing import Annotated, Any, Literal, TypeAlias, Union
 
 import debian.deb822 as deb822
 
@@ -26,33 +26,12 @@ except ImportError:
 
 import yaml
 
-from debusine.artifacts.models import BareDataCategory
+from debusine.artifacts.models import BareDataCategory, RuntimeStatistics
 from debusine.client.models import LookupChildType
-
-_Get = TypeVar("_Get", covariant=True)
+from debusine.utils import DjangoChoicesEnum
 
 
-# taken from functional.py in python3-django 3:4.2.11-1 to avoid the dependency
-class classproperty(Generic[_Get]):
-    """
-    classproperty definition.
-
-    Decorator that converts a method with a single cls argument into a property
-    that can be accessed directly from the class.
-    """
-
-    def __init__(self, method: Callable[[Any], _Get]) -> None:
-        """Define __init__."""
-        self.fget = method
-
-    def __get__(
-        self, instance: Any | None, cls: type[Any] | None = None  # noqa: U100
-    ) -> _Get:
-        """Define __get__."""
-        return self.fget(cls)
-
-
-class TaskTypes(StrEnum):
+class TaskTypes(DjangoChoicesEnum):
     """Possible values for task_type."""
 
     WORKER = "Worker"
@@ -62,31 +41,20 @@ class TaskTypes(StrEnum):
     SIGNING = "Signing"
     WAIT = "Wait"
 
-    @classproperty
-    def choices(cls) -> list[tuple[str, str]]:
-        """To be compatible with django.db.models.TextChoices."""
-        return [(item, item) for item in cls]
 
-
-class WorkerType(StrEnum):
+class WorkerType(DjangoChoicesEnum):
     """The type of a Worker."""
 
     EXTERNAL = "external"
     CELERY = "celery"
     SIGNING = "signing"
 
-    @classproperty
-    def choices(cls) -> list[tuple[str, str]]:
-        """To be compatible with django.db.models.TextChoices."""
-        return [(item, item) for item in cls]
-
 
 class BackendType(StrEnum):
     """Possible values for backend."""
 
     AUTO = "auto"
     UNSHARE = "unshare"
-    SCHROOT = "schroot"
     INCUS_LXC = "incus-lxc"
     INCUS_VM = "incus-vm"
     QEMU = "qemu"
@@ -565,7 +533,8 @@ class BaseTaskData(BaseTaskDataModel):
     validation.
     """
 
-    pass
+    #: debusine:task-configuration collection to use to configure the task
+    task_configuration: LookupSingle | None = None
 
 
 class BaseTaskDataWithExecutor(BaseTaskData):
@@ -593,6 +562,26 @@ class BaseDynamicTaskData(BaseTaskDataModel):
     #: work request.
     parameter_summary: str | None = None
 
+    #: An abstract string value representing the subject of the task, for
+    #: the purpose of recording statistics.  It is meant to group possible
+    #: inputs into groups that we expect to behave similarly.
+    subject: str | None = None
+
+    #: An abstract string value representing the runtime context in which
+    #: the task is executed, for the purpose of recording statistics.  It is
+    #: meant to represent some of the task parameters that can significantly
+    #: alter the runtime behaviour of the task.
+    runtime_context: str | None = None
+
+    #: Result of the collection lookup of the task_configuration field
+    task_configuration_id: int | None = None
+
+    #: Name of the configuration context
+    # This is a stable string representation of the main input artifacts and
+    # parameters, that does not change across similar future invocations of the
+    # same task, and is used to look up configuration parameters.
+    configuration_context: str | None = None
+
 
 class BaseDynamicTaskDataWithExecutor(BaseDynamicTaskData):
     """Dynamic task data for executors."""
@@ -600,31 +589,6 @@ class BaseDynamicTaskDataWithExecutor(BaseDynamicTaskData):
     environment_id: int | None = None
 
 
-class RuntimeStatistics(BaseTaskDataModel):
-    """Runtime statistics about a completed task."""
-
-    #: Runtime duration of task, in seconds.
-    duration: int | None = None
-
-    #: Amount of CPU time used (both user and system), in seconds.
-    cpu_time: int | None = None
-
-    #: Maximum disk space used, in bytes.
-    disk_space: int | None = None
-
-    #: Maximum RAM used, in bytes.
-    memory: int | None = None
-
-    #: Available disk space when the task started, in bytes; may be rounded.
-    available_disk_space: int | None = None
-
-    #: Available RAM when the task started, in bytes; may be rounded.
-    available_memory: int | None = None
-
-    #: Number of CPU cores on the worker that ran the task.
-    cpu_count: int | None = None
-
-
 class OutputDataError(BaseTaskDataModel):
     """An error encountered when running a task."""
 
@@ -1103,40 +1067,14 @@ class SbuildData(BaseTaskDataWithExecutor, BaseTaskDataWithExtraRepositories):
 
     input: SbuildInput
     host_architecture: str
-    distribution: str | None = None
     build_components: list[SbuildBuildComponent] = pydantic.Field(
         default_factory=lambda: [SbuildBuildComponent.ANY],
     )
     binnmu: SbuildBinNMU | None = None
     build_profiles: list[str] | None = None
-
-    @pydantic.root_validator
-    @classmethod
-    def check_one_of_backend_or_environment(
-        cls, values: dict[str, Any]
-    ) -> dict[str, Any]:
-        """Make sure that backends have the arguments they need."""
-        if values.get("backend") == BackendType.SCHROOT:
-            if values.get("distribution") is None:
-                raise ValueError(
-                    'The backend "schroot" requires "distribution" to be set'
-                )
-            if values.get("environment") is not None:
-                raise ValueError(
-                    'The backend "schroot" requires "environment" to be unset'
-                )
-        else:
-            if values.get("environment") is None:
-                raise ValueError(
-                    f'''The backend "{values.get('backend')}"'''
-                    ' requires "environment" to be set'
-                )
-            if values.get("distribution") is not None:
-                raise ValueError(
-                    f'''The backend "{values.get('backend')}"'''
-                    ' requires "distribution" to be unset'
-                )
-        return values
+    # BaseTaskDataWithExecutor declares this as optional, but it's required
+    # here.
+    environment: LookupSingle
 
     @pydantic.root_validator
     @classmethod
