@@ -24,7 +24,14 @@ from pathlib import Path
 from typing import Any
 
 import debusine.utils
-from debusine.artifacts.models import CollectionCategory, DebianSystemTarball
+from debusine.artifacts import BinaryPackages
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSystemTarball,
+    DebianUpload,
+    get_source_package_name,
+)
 from debusine.tasks import (
     BaseTaskWithExecutor,
     ExtraRepositoryMixin,
@@ -64,22 +71,53 @@ class Piuparts(
         self, task_database: TaskDatabaseInterface
     ) -> PiupartsDynamicData:
         """Resolve artifact lookups for this task."""
+        input_binary_artifacts = task_database.lookup_multiple_artifacts(
+            self.data.input.binary_artifacts
+        )
+        base_tgz = self.get_environment(
+            task_database,
+            self.data.base_tgz,
+            default_category=CollectionCategory.ENVIRONMENTS,
+            image_category=ExecutorImageCategory.TARBALL,
+            set_backend=False,
+        )
+
+        self.ensure_artifact_categories(
+            configuration_key="base_tgz",
+            category=base_tgz.category,
+            expected=(ArtifactCategory.SYSTEM_TARBALL,),
+        )
+        assert isinstance(base_tgz.data, DebianSystemTarball)
+
+        package_names = []
+        for binary_artifact in input_binary_artifacts:
+            self.ensure_artifact_categories(
+                configuration_key="input.binary_artifact",
+                category=binary_artifact.category,
+                expected=(
+                    ArtifactCategory.BINARY_PACKAGES,
+                    ArtifactCategory.UPLOAD,
+                ),
+            )
+            assert isinstance(
+                binary_artifact.data, (BinaryPackages, DebianUpload)
+            )
+            package_names.append(get_source_package_name(binary_artifact.data))
+
+        runtime_context = f"{base_tgz.data.codename}:{self.host_architecture()}"
+        configuration_context = base_tgz.data.codename
+
         return PiupartsDynamicData(
             environment_id=self.get_environment(
                 task_database,
                 self.data.environment,
                 default_category=CollectionCategory.ENVIRONMENTS,
-            ),
-            input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
-                self.data.input.binary_artifacts
-            ),
-            base_tgz_id=self.get_environment(
-                task_database,
-                self.data.base_tgz,
-                default_category=CollectionCategory.ENVIRONMENTS,
-                image_category=ExecutorImageCategory.TARBALL,
-                set_backend=False,
-            ),
+            ).id,
+            input_binary_artifacts_ids=input_binary_artifacts.get_ids(),
+            base_tgz_id=base_tgz.id,
+            subject=" ".join(sorted(package_names)),
+            runtime_context=runtime_context,
+            configuration_context=configuration_context,
         )
 
     def fetch_input(self, destination: Path) -> bool:
@@ -325,6 +363,8 @@ class Piuparts(
 
     def get_label(self) -> str:
         """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "piuparts"
+        return (
+            f"piuparts {subject}"
+            if (subject := self.get_subject())
+            else "piuparts"
+        )
