@@ -16,7 +16,6 @@ https://freexian-team.pages.debian.net/debusine/reference/tasks/ontology-generic
 
 import email.utils
 import os
-import subprocess
 from pathlib import Path
 from typing import Any, cast
 
@@ -39,14 +38,17 @@ from debusine.artifacts import (
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
+    DebianSourcePackage,
+    DebianSystemImage,
+    DebianSystemTarball,
     DoseDistCheck,
+    get_source_package_name,
 )
 from debusine.client.models import RelationType, RemoteArtifact
 from debusine.tasks import (
     BaseTaskWithExecutor,
     ExtraRepositoryMixin,
     RunCommandTask,
-    TaskConfigError,
 )
 from debusine.tasks.models import SbuildData, SbuildDynamicData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
@@ -82,26 +84,63 @@ class Sbuild(
         self, task_database: TaskDatabaseInterface
     ) -> SbuildDynamicData:
         """Resolve artifact lookups for this task."""
+        input_source_artifact = task_database.lookup_single_artifact(
+            self.data.input.source_artifact
+        )
+        self.ensure_artifact_categories(
+            configuration_key="input.source_artifact",
+            category=input_source_artifact.category,
+            expected=(ArtifactCategory.SOURCE_PACKAGE,),
+        )
+        assert isinstance(input_source_artifact.data, DebianSourcePackage)
+
+        environment = self.get_environment(
+            task_database,
+            self.data.environment,
+            default_category=CollectionCategory.ENVIRONMENTS,
+        )
+        self.ensure_artifact_categories(
+            configuration_key="environment",
+            category=environment.category,
+            expected=(
+                ArtifactCategory.SYSTEM_TARBALL,
+                ArtifactCategory.SYSTEM_IMAGE,
+            ),
+        )
+        assert isinstance(
+            environment.data, (DebianSystemTarball, DebianSystemImage)
+        )
+
         DEBUSINE_FQDN = task_database.get_server_setting("DEBUSINE_FQDN")
+
+        if self._build_options():
+            # Do not record stats for non-standard builds
+            runtime_context = None
+        else:
+            build_components = "+".join(sorted(self.data.build_components))
+
+            # Currently defining build_architecture for Sbuild is not
+            # supported. It defaults to the host_architecture
+            build_architecture = self.data.host_architecture
+
+            runtime_context = (
+                f"{build_components}:"
+                f"{self.data.host_architecture}:"
+                f"{build_architecture}"
+            )
+
         return SbuildDynamicData(
-            environment_id=(
-                None
-                if self.data.environment is None
-                else self.get_environment(
-                    task_database,
-                    self.data.environment,
-                    default_category=CollectionCategory.ENVIRONMENTS,
-                )
-            ),
-            input_source_artifact_id=task_database.lookup_single_artifact(
-                self.data.input.source_artifact
-            ),
+            environment_id=environment.id,
+            input_source_artifact_id=input_source_artifact.id,
             input_extra_binary_artifacts_ids=(
                 task_database.lookup_multiple_artifacts(
                     self.data.input.extra_binary_artifacts
-                )
+                ).get_ids()
             ),
             binnmu_maintainer=f"Debusine <noreply@{DEBUSINE_FQDN}>",
+            subject=get_source_package_name(input_source_artifact.data),
+            runtime_context=runtime_context,
+            configuration_context=environment.data.codename,
         )
 
     def get_source_artifacts_ids(self) -> list[int]:
@@ -120,22 +159,6 @@ class Sbuild(
         result.extend(self.dynamic_data.input_extra_binary_artifacts_ids)
         return result
 
-    @property
-    def chroot_name(self) -> str:
-        """Build name of required chroot."""
-        return "{}-{}".format(
-            self.data.distribution,
-            self.data.host_architecture,
-        )
-
-    @staticmethod
-    def _call_dpkg_architecture() -> str:  # pragma: no cover
-        return (
-            subprocess.check_output(["dpkg", "--print-architecture"])
-            .decode("utf-8")
-            .strip()
-        )
-
     @classmethod
     def analyze_worker(cls) -> dict[str, Any]:
         """Report metadata for this task on this worker."""
@@ -144,10 +167,6 @@ class Sbuild(
         available_key = cls.prefix_with_task_name("available")
         metadata[available_key] = debusine.utils.is_command_available("sbuild")
 
-        if debusine.utils.is_command_available("schroot"):
-            chroots_key = cls.prefix_with_task_name("chroots")
-            metadata[chroots_key] = cls._list_chroots()
-
         return metadata
 
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
@@ -159,69 +178,15 @@ class Sbuild(
         if not worker_metadata.get(available_key, False):
             return False
 
-        if self.backend == "schroot":
-            chroot_key = self.prefix_with_task_name("chroots")
-            if self.chroot_name not in worker_metadata.get(chroot_key, []):
+        executor_available_key = f"executor:{self.backend}:available"
+        if not worker_metadata.get(executor_available_key, False):
+            return False
+        if self.backend != "unshare":
+            if not worker_metadata.get("autopkgtest:available", False):
                 return False
-        else:
-            executor_available_key = f"executor:{self.backend}:available"
-            if not worker_metadata.get(executor_available_key, False):
-                return False
-            if self.backend != "unshare":
-                if not worker_metadata.get("autopkgtest:available", False):
-                    return False
 
         return True
 
-    @staticmethod
-    def _call_schroot_list() -> str:  # pragma: no cover
-        return (
-            subprocess.check_output(["schroot", "--list"])
-            .decode("utf-8")
-            .strip()
-        )
-
-    @classmethod
-    def _list_chroots(cls) -> list[str]:
-        """
-        Provide support for finding available chroots.
-
-        Ensure that aliases are supported as the DSC may explicitly refer to
-        <codename>-security (or -backports) etc.
-
-        Return the list of detected chroots.
-        """
-        chroots = []
-        output = cls._call_schroot_list()
-        for line in output.split("\n"):
-            if line.startswith("chroot:") and line.endswith("-sbuild"):
-                chroots.append(line[7:-7])
-        return chroots
-
-    def _update_chroots_list(self) -> None:
-        """
-        Populate the self.chroots list, if the list is empty.
-
-        No return value, this is a find, not a get.
-        """
-        if self.chroots is not None:
-            return
-        self.chroots = self._list_chroots()
-
-    def _verify_schroot(self) -> bool:
-        """Verify a suitable schroot exists."""
-        self._update_chroots_list()
-
-        if not self.chroots:
-            self.logger.error("No sbuild chroots found")
-            return False
-
-        if self.chroot_name in self.chroots:
-            return True
-
-        self.logger.error("No suitable chroot found for %s", self.chroot_name)
-        return False
-
     def fetch_input(self, destination: Path) -> bool:
         """Download the source artifact."""
         if not self.debusine:
@@ -269,8 +234,7 @@ class Sbuild(
                 return False
             self.fetch_artifact(artifact_id, destination)
 
-        if self.backend != "schroot":
-            self._prepare_executor()
+        self._prepare_executor()
 
         self.write_extra_repository_config(
             codename=self._environment_codename(), destination=destination
@@ -280,13 +244,6 @@ class Sbuild(
 
     def _environment_codename(self) -> str:
         """Return the codename of the environment we're building in."""
-        if self.backend == "schroot":
-            if self.data.distribution is None:
-                raise AssertionError(
-                    "distribution must be set if schroot is in use"
-                )
-            return self.data.distribution
-
         if self.executor is None:
             raise ValueError(
                 "self.executor not set, call _prepare_executor() first"
@@ -342,8 +299,6 @@ class Sbuild(
         """Generate command line arguments for the backend and environment."""
         distribution = self._environment_codename()
         args: list[str] = [f"--dist={distribution}"]
-        if self.backend == "schroot":
-            return args
 
         assert self.executor
         if self.backend == "unshare":
@@ -415,14 +370,16 @@ class Sbuild(
         args.append(f"--maintainer={maintainer}")
         return args
 
-    def _cmd_env(self) -> dict[str, str] | None:
-        """Set DEB_BUILD_OPTIONS for build profiles."""
-        build_options = [
+    def _build_options(self) -> list[str]:
+        return [
             profile
             for profile in self.data.build_profiles or []
             if profile in ("nocheck", "nodoc")
         ]
-        if build_options:
+
+    def _cmd_env(self) -> dict[str, str] | None:
+        """Set DEB_BUILD_OPTIONS for build profiles."""
+        if build_options := self._build_options():
             env = dict(os.environ)
             env["DEB_BUILD_OPTIONS"] = " ".join(
                 os.environ.get("DEB_BUILD_OPTIONS", "").split() + build_options
@@ -430,20 +387,6 @@ class Sbuild(
             return env
         return None
 
-    def execute(self) -> bool:
-        """
-        Verify task can be executed and super().execute().
-
-        :raises: TaskConfigError.
-        """  # noqa: D402
-        if self.backend == "schroot" and not self._verify_schroot():
-            raise TaskConfigError(
-                "No suitable schroot for"
-                f" {self.data.distribution}-{self.data.host_architecture}"
-            )
-
-        return super().execute()
-
     def task_succeeded(
         self,
         returncode: int | None,  # noqa: U100
@@ -769,6 +712,8 @@ class Sbuild(
 
     def get_label(self) -> str:
         """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "build a package"
+        return (
+            f"build {subject}"
+            if (subject := self.get_subject())
+            else "build a package"
+        )
