@@ -17,12 +17,30 @@ interface that is implemented by server code.
 """
 
 from abc import ABC, abstractmethod
+from dataclasses import dataclass
 from typing import overload
 
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import ArtifactData, CollectionCategory
 from debusine.tasks.models import LookupMultiple, LookupSingle
 
 
+@dataclass
+class ArtifactInfo:
+    """Information about an artifact."""
+
+    id: int
+    category: str
+    data: ArtifactData
+
+
+class MultipleArtifactInfo(tuple[ArtifactInfo, ...]):
+    """Information about multiple artifacts."""
+
+    def get_ids(self) -> list[int]:
+        """Return the ID of each artifact."""
+        return [item.id for item in self]
+
+
 class TaskDatabaseInterface(ABC):
     """Interface for interacting with the database from worker tasks."""
 
@@ -32,7 +50,7 @@ class TaskDatabaseInterface(ABC):
         self,
         lookup: LookupSingle,
         default_category: CollectionCategory | None = None,
-    ) -> int: ...
+    ) -> ArtifactInfo: ...
 
     @overload
     @abstractmethod
@@ -47,7 +65,7 @@ class TaskDatabaseInterface(ABC):
         self,
         lookup: LookupSingle | None,
         default_category: CollectionCategory | None = None,
-    ) -> int | None:
+    ) -> ArtifactInfo | None:
         """Look up a single artifact using :ref:`lookup-single`."""
 
     @abstractmethod
@@ -55,7 +73,7 @@ class TaskDatabaseInterface(ABC):
         self,
         lookup: LookupMultiple | None,
         default_category: CollectionCategory | None = None,
-    ) -> list[int]:
+    ) -> MultipleArtifactInfo:
         """Look up multiple artifacts using :ref:`lookup-multiple`."""
 
     @overload
