@@ -37,7 +37,11 @@ from debusine.client.models import (
 from debusine.tasks import BaseExternalTask, BaseTask
 from debusine.tasks.executors import ExecutorInterface, UnshareInstance
 from debusine.tasks.models import LookupMultiple, LookupSingle
-from debusine.tasks.server import TaskDatabaseInterface
+from debusine.tasks.server import (
+    ArtifactInfo,
+    MultipleArtifactInfo,
+    TaskDatabaseInterface,
+)
 from debusine.utils import extract_generic_type_arguments
 
 TaskClass = TypeVar("TaskClass", bound=BaseTask[Any, Any])
@@ -54,16 +58,25 @@ class TaskTestProtocol(Protocol, Generic[TaskClass]):
     task: TaskClass
 
 
-class FakeTaskDatabase(TaskDatabaseInterface):
+class FakeTaskDatabaseBase(TaskDatabaseInterface):
     """Fake implementation of database interaction in worker tests."""
 
     def __init__(
         self,
+        *,
         single_lookups: (
-            Mapping[tuple[LookupSingle, str | None], int | None] | None
+            Mapping[
+                tuple[LookupSingle, CollectionCategory | None],
+                ArtifactInfo | None,
+            ]
+            | None
         ) = None,
         multiple_lookups: (
-            Mapping[tuple[LookupMultiple, str | None], list[int]] | None
+            Mapping[
+                tuple[LookupMultiple, CollectionCategory | None],
+                list[ArtifactInfo],
+            ]
+            | None
         ) = None,
         settings: dict[str, str] | None = None,
     ) -> None:
@@ -77,7 +90,7 @@ class FakeTaskDatabase(TaskDatabaseInterface):
         self,
         lookup: LookupSingle,
         default_category: CollectionCategory | None = None,  # noqa: U100
-    ) -> int: ...
+    ) -> ArtifactInfo: ...
 
     @overload
     def lookup_single_artifact(
@@ -90,7 +103,7 @@ class FakeTaskDatabase(TaskDatabaseInterface):
         self,
         lookup: LookupSingle | None,
         default_category: CollectionCategory | None = None,  # noqa: U100
-    ) -> int | None:
+    ) -> ArtifactInfo | None:
         """Pretend to look up a single artifact."""
         return (
             None
@@ -102,11 +115,13 @@ class FakeTaskDatabase(TaskDatabaseInterface):
         self,
         lookup: LookupMultiple | None,
         default_category: CollectionCategory | None = None,  # noqa: U100
-    ) -> list[int]:
+    ) -> MultipleArtifactInfo:
         """Pretend to look up multiple artifacts."""
         if not lookup:
-            return []
-        return self.multiple_lookups[(lookup, default_category)]
+            return MultipleArtifactInfo()
+        return MultipleArtifactInfo(
+            self.multiple_lookups[(lookup, default_category)]
+        )
 
     @overload
     def lookup_single_collection(
@@ -139,6 +154,18 @@ class FakeTaskDatabase(TaskDatabaseInterface):
         return self.settings[setting]
 
 
+class FakeTaskDatabase(FakeTaskDatabaseBase):
+    """
+    Fake task database with empty configure implementation.
+
+    This is used for tests that do not perform database access
+    """
+
+    def configure(self, task: "BaseTask[Any, Any]") -> None:
+        """Perform server-side task configuration."""
+        raise NotImplementedError()
+
+
 class TaskHelperMixin(Generic[TaskClass]):
     """Helper mixin for Task tests."""
 
