@@ -23,12 +23,19 @@ except ImportError:
 
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
 from debusine.artifacts.models import (
+    ArtifactCategory,
     CollectionCategory,
     DebianAutopkgtest,
     DebianAutopkgtestResult,
 )
 from debusine.artifacts.models import DebianAutopkgtestResultStatus as rs
-from debusine.artifacts.models import DebianAutopkgtestSource
+from debusine.artifacts.models import (
+    DebianAutopkgtestSource,
+    DebianBinaryPackage,
+    DebianBinaryPackages,
+    DebianSourcePackage,
+    DebianUpload,
+)
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
@@ -45,11 +52,13 @@ from debusine.tasks.models import (
     LookupMultiple,
     WorkerType,
 )
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
+from debusine.test.utils import create_system_tarball_data
 
 
 class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
@@ -508,6 +517,28 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             )
         )
 
+    def create_binary_packages_data(
+        self, srcpkg_name: str
+    ) -> DebianBinaryPackages:
+        """Create sample `debian:binary-packages` artifact data."""
+        return DebianBinaryPackages(
+            srcpkg_name=srcpkg_name,
+            srcpkg_version="1.0",
+            version="1.0",
+            architecture="amd64",
+            packages=[],
+        )
+
+    def create_upload_data(self, name: str) -> DebianUpload:
+        """Create sample `debian:upload` artifact data."""
+        return DebianUpload(
+            type="dpkg",
+            changes_fields={
+                "Architecture": "amd64",
+                "Files": [{"name": f"{name}_1.0_amd64.deb"}],
+            },
+        )
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
         binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
@@ -518,13 +549,34 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.source_artifact
-                (5, None): 5,
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [10, 11]
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
             },
         )
 
@@ -535,6 +587,9 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                 input_source_artifact_id=5,
                 input_binary_artifacts_ids=[10, 11],
                 input_context_artifacts_ids=[],
+                subject="hello",
+                runtime_context="amd64:unshare",
+                configuration_context="sid",
             ),
         )
 
@@ -551,15 +606,47 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.source_artifact
-                (5, None): 5,
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [10, 11],
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
                 # input.context_artifacts
-                (self.task.data.input.context_artifacts, None): [2, 3],
+                (self.task.data.input.context_artifacts, None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("context"),
+                    ),
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("context"),
+                    ),
+                ],
             },
         )
 
@@ -570,9 +657,84 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                 input_source_artifact_id=5,
                 input_binary_artifacts_ids=[10, 11],
                 input_context_artifacts_ids=[2, 3],
+                subject="hello",
+                runtime_context="amd64:unshare",
+                configuration_context="sid",
             ),
         )
 
+    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        source_artifact's category is unexpected.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
+        self.task.data.input.context_artifacts = LookupMultiple.parse_obj(
+            {"collection": "bookworm@debian:suite"}
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="hello",
+                        srcpkg_version="1.0",
+                        deb_fields={},
+                        deb_control_files=[],
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
+                # input.context_artifacts
+                (self.task.data.input.context_artifacts, None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=self.create_binary_packages_data("context"),
+                    ),
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("context"),
+                    ),
+                ],
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.source_artifact: unexpected artifact "
+            "category: 'debian:binary-package'. Valid categories: "
+            r"\['debian:source-package', 'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_cmdline_executor_not_set(self) -> None:
         """Raise AssertionError: self.executor is not set."""
         msg = (
@@ -1154,6 +1316,25 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             [],
         )
 
-    def test_label(self) -> None:
-        """Test get_label."""
+    def test_get_label_dynamic_data_is_none(self) -> None:
+        """Test get_label if dynamic_data.subject is None."""
         self.assertEqual(self.task.get_label(), "autopkgtest")
+
+    def test_get_label_dynamic_data_subject_is_none(self) -> None:
+        """Test get_label if dynamic_data is None."""
+        self.task.dynamic_data = AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=2,
+            input_binary_artifacts_ids=[],
+        )
+        self.assertEqual(self.task.get_label(), "autopkgtest")
+
+    def test_get_label_dynamic_data_subject_is_hello(self) -> None:
+        """Test get_label if dynamic_data.subject is set."""
+        self.task.dynamic_data = AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=2,
+            input_binary_artifacts_ids=[],
+            subject="hello",
+        )
+        self.assertEqual(self.task.get_label(), "autopkgtest hello")
