@@ -13,23 +13,32 @@ from pathlib import Path
 from unittest.mock import call
 
 from debusine.artifacts import DebDiffArtifact
-from debusine.artifacts.models import CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianBinaryPackage,
+    DebianSourcePackage,
+    DebianUpload,
+    EmptyArtifactData,
+)
 from debusine.client.models import (
     LookupResultType,
     LookupSingleResponse,
     RemoteArtifact,
 )
 from debusine.tasks import DebDiff, TaskConfigError
-from debusine.tasks.models import DebDiffDynamicData
+from debusine.tasks.models import DebDiffDynamicData, LookupMultiple
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
+from debusine.test.utils import create_system_tarball_data
 
 
-class DebDiffTaskTests(ExternalTaskHelperMixin[DebDiff], TestCase):
-    """Test the DebDiff Task class."""
+class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
+    """Test the DebDiff class."""
 
     SAMPLE_TASK_DATA = {
         "input": {"source_artifacts": [421, 123]},
@@ -105,10 +114,26 @@ class DebDiffTaskTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
-                # input.artifact
-                (421, None): 421,
-                (123, None): 123,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifacts
+                (421, None): ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+                (123, None): ArtifactInfo(
+                    id=123,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
             }
         )
 
@@ -118,9 +143,226 @@ class DebDiffTaskTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 environment_id=1,
                 input_source_artifacts_ids=[421, 123],
                 input_binary_artifacts_ids=None,
+                subject="source:hello",
             ),
         )
 
+    def test_compute_dynamic_subject_binary_artifacts(self) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # binary artifacts
+                (LookupMultiple.parse_obj([500]), None): [
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
+                    ),
+                ],
+                (LookupMultiple.parse_obj([501]), None): [
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
+                    ),
+                ],
+            },
+        )
+
+        self.configure_task(
+            override={"input": {"binary_artifacts": [[500], [501]]}}
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            DebDiffDynamicData(
+                environment_id=1,
+                input_source_artifacts_ids=None,
+                input_binary_artifacts_ids=[[500], [501]],
+                subject="binary:hello",
+            ),
+        )
+
+    def test_compute_dynamic_subject_is_none(self) -> None:
+        """
+        Test compute_dynamic_data set subject is None.
+
+        There is no source package, and binary packages are from different
+        source packages.
+        """
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # binary artifacts
+                (LookupMultiple.parse_obj([500]), None): [
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "hello",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
+                    ),
+                ],
+                (LookupMultiple.parse_obj([501]), None): [
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Source": "linux-base",
+                                "Files": [{"name": "hello.deb"}],
+                            },
+                        ),
+                    ),
+                ],
+            },
+        )
+
+        self.configure_task(
+            override={"input": {"binary_artifacts": [[500], [501]]}}
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            DebDiffDynamicData(
+                environment_id=1,
+                input_source_artifacts_ids=None,
+                input_binary_artifacts_ids=[[500], [501]],
+                subject=None,
+            ),
+        )
+
+    def test_compute_dynamic_data_raise_task_config_error_source(self) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifacts
+                (421, None): ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.BINARY_PACKAGES,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+                (123, None): ArtifactInfo(
+                    id=123,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            }
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^input.source_artifacts: unexpected artifact category: "
+            r"'debian:binary-packages'. "
+            r"Valid categories: \['debian:source-package'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
+    def test_compute_dynamic_data_raise_task_config_error_binary(self) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # binary artifacts
+                (LookupMultiple.parse_obj([500]), None): [
+                    ArtifactInfo(
+                        id=500,
+                        category=ArtifactCategory.SOURCE_PACKAGE,
+                        data=EmptyArtifactData(),
+                    ),
+                ],
+                (LookupMultiple.parse_obj([501]), None): [
+                    ArtifactInfo(
+                        id=501,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="linux-base",
+                            srcpkg_version="1.0",
+                            deb_fields={},
+                            deb_control_files=[],
+                        ),
+                    ),
+                ],
+            },
+        )
+
+        self.configure_task(
+            override={"input": {"binary_artifacts": [[500], [501]]}}
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^input.binary_artifacts: unexpected artifact category: "
+            r"'debian:source-package'. Valid categories: \['debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db),
+
     def test_fetch_input_source(self) -> None:
         """Test fetching different source inputs."""
         self.configure_task(override={"input": {'source_artifacts': [1, 2]}})
