@@ -19,22 +19,32 @@ from typing import Any, AnyStr
 from unittest import mock
 
 from debusine.artifacts import LocalArtifact, SigningInputArtifact
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianBinaryPackage,
+    EmptyArtifactData,
+)
 from debusine.artifacts.playground import ArtifactPlayground
 from debusine.client.models import ArtifactResponse, RemoteArtifact
-from debusine.tasks import ExtractForSigning
+from debusine.tasks import ExtractForSigning, TaskConfigError
 from debusine.tasks.extract_for_signing import ExtractError
 from debusine.tasks.models import (
     ExtractForSigningDynamicData,
     LookupMultiple,
     WorkerType,
 )
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import create_artifact_response, create_file_response
+from debusine.test.utils import (
+    create_artifact_response,
+    create_file_response,
+    create_system_tarball_data,
+)
 
 
 def _mock_run(
@@ -110,13 +120,47 @@ class ExtractForSigningTests(
                     "debian/match:codename=bookworm:format=tarball:"
                     "backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.template_artifact
-                (2, None): 2,
+                (2, None): ArtifactInfo(
+                    id=2,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="template",
+                        srcpkg_version="1.0",
+                        deb_fields={"Package": "hello"},
+                        deb_control_files=[],
+                    ),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (LookupMultiple.parse_obj([3, 4]), None): [3, 4]
+                (LookupMultiple.parse_obj([3, 4]), None): [
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": "unsigned-1"},
+                            deb_control_files=[],
+                        ),
+                    ),
+                    ArtifactInfo(
+                        id=4,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": "unsigned-2"},
+                            deb_control_files=[],
+                        ),
+                    ),
+                ]
             },
         )
 
@@ -126,9 +170,67 @@ class ExtractForSigningTests(
                 environment_id=1,
                 input_template_artifact_id=2,
                 input_binary_artifacts_ids=[3, 4],
+                subject="hello",
             ),
         )
 
+    def test_compute_dynamic_data_raise_task_config_error_template(
+        self,
+    ) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:format=tarball:"
+                    "backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.template_artifact
+                (2, None): ArtifactInfo(
+                    id=2,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (LookupMultiple.parse_obj([3, 4]), None): [
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": "unsigned-1"},
+                            deb_control_files=[],
+                        ),
+                    ),
+                    ArtifactInfo(
+                        id=4,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": "unsigned-2"},
+                            deb_control_files=[],
+                        ),
+                    ),
+                ]
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^input.template_artifact: unexpected artifact category: "
+            r"'debian:source-package'. "
+            r"Valid categories: \['debian:binary-package'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_fetch_input_template_wrong_category(self) -> None:
         """fetch_input checks the category of the template artifact."""
         self.task.work_request_id = 147
