@@ -24,7 +24,16 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from debusine.artifacts import LintianArtifact
-from debusine.artifacts.models import CollectionCategory, DebianLintianSummary
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianBinaryPackage,
+    DebianBinaryPackages,
+    DebianLintianSummary,
+    DebianSourcePackage,
+    DebianUpload,
+    EmptyArtifactData,
+)
 from debusine.client.models import (
     ArtifactResponse,
     FileResponse,
@@ -35,12 +44,18 @@ from debusine.client.models import (
 from debusine.tasks import Lintian, TaskConfigError
 from debusine.tasks.executors import InstanceInterface
 from debusine.tasks.lintian import LintianVersionParseError
-from debusine.tasks.models import LintianDynamicData, LintianPackageType
+from debusine.tasks.models import (
+    LintianDynamicData,
+    LintianPackageType,
+    LookupMultiple,
+)
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
+from debusine.test.utils import create_system_tarball_data
 
 
 class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
@@ -644,7 +659,6 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         self.configure_task(override={"input": {"binary_artifacts": [6, 7]}})
 
     def test_compute_dynamic_data(self) -> None:
-        """Dynamic data receives relevant artifact IDs."""
         task_db = FakeTaskDatabase(
             single_lookups={
                 # environment
@@ -652,17 +666,371 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # input.source_artifact
-                (5, None): 5,
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello",
+                        version="1.0",
+                        type="dpkg",
+                        dsc_fields={},
+                    ),
+                ),
             }
         )
 
         self.assertEqual(
             self.task.compute_dynamic_data(task_db),
-            LintianDynamicData(environment_id=1, input_source_artifact_id=5),
+            LintianDynamicData(
+                environment_id=1,
+                input_source_artifact_id=5,
+                subject="hello",
+                runtime_context="binary-all+binary-any+source:sid",
+                configuration_context="sid",
+            ),
         )
 
+    def test_compute_dynamic_data_subject_debian_upload(self) -> None:
+        """compute_dynamic_data subject is package name of source artifact."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Source": "hello",
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                        },
+                    ),
+                ),
+            }
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db).subject, "hello"
+        )
+
+    def test_compute_dynamic_data_subject_runtime_context(self) -> None:
+        """compute_dynamic_data runtime_context source only."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Source": "hello",
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                        },
+                    ),
+                ),
+            }
+        )
+
+        self.configure_task(
+            override={
+                "output": {
+                    "source_analysis": True,
+                    "binary_all_analysis": False,
+                    "binary_any_analysis": False,
+                }
+            }
+        )
+
+        dynamic_data = self.task.compute_dynamic_data(task_db)
+
+        self.assertEqual(dynamic_data.subject, "hello")
+        self.assertEqual(dynamic_data.runtime_context, "source:sid")
+
+    def test_compute_dynamic_data_debian_binary_package(self) -> None:
+        """
+        compute_dynamic_data subject is package name of source artifact.
+
+        There is a binary artifact with other package names.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([6])
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Source": "hello",
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                        },
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=6,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="hello-bin",
+                            srcpkg_version="1.0",
+                            deb_fields={
+                                "Package": "hello-bin",
+                                "Version": "1.0",
+                                "Architecture": "amd64",
+                            },
+                            deb_control_files=[],
+                        ),
+                    ),
+                ],
+            },
+        )
+
+        self.configure_task(
+            override={
+                "input": {"source_artifact": 5, "binary_artifacts": [6]},
+                "output": {"source_analysis": False},
+            }
+        )
+
+        dynamic_data = self.task.compute_dynamic_data(task_db)
+        self.assertEqual(dynamic_data.subject, "hello")
+        self.assertEqual(
+            dynamic_data.runtime_context, "binary-all+binary-any:sid"
+        )
+
+    def test_compute_dynamic_data_source_artifact_subject_no_binaries(
+        self,
+    ) -> None:
+        """compute_dynamic_data set to source artifact without binaries."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Source": "hello",
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                        },
+                    ),
+                ),
+            },
+        )
+
+        self.configure_task(remove=["environment"])
+
+        dynamic_data = self.task.compute_dynamic_data(task_db)
+        self.assertEqual(dynamic_data.subject, "hello")
+        self.assertIsNone(dynamic_data.runtime_context)
+
+    def test_compute_dynamic_data_debian_binary_packages_subject(self) -> None:
+        """
+        compute_dynamic_data set subject to binary package name.
+
+        There are no source packages.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([6])
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=6,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=DebianBinaryPackages(
+                            srcpkg_name="hello",
+                            srcpkg_version="1.0",
+                            version="1.0",
+                            architecture="amd64",
+                            packages=["hello"],
+                        ),
+                    )
+                ],
+            },
+        )
+
+        self.configure_task(override={"input": {"binary_artifacts": [6]}})
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db).subject, "hello"
+        )
+
+    def test_compute_dynamic_raise_config_task_error_wrong_environment(
+        self,
+    ) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        environment artifact category is unexpected.
+        """
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Source": "hello",
+                            "Architecture": "amd64",
+                            "Files": [{"name": "hello_1.0_amd64.deb"}],
+                        },
+                    ),
+                ),
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^environment: unexpected artifact category: "
+            "'debian:source-package'. Valid categories: "
+            r"\['debian:system-tarball'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
+    def test_compute_dynamic_raise_config_task_error_wrong_source_artifact(
+        self,
+    ) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        source artifact category is unexpected.
+        """
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=EmptyArtifactData(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.BINARY_PACKAGES,
+                    data=EmptyArtifactData(),
+                ),
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.source_artifact: unexpected artifact category: "
+            "'debian:binary-packages'. "
+            r"Valid categories: \['debian:source-package', 'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
+    def test_compute_dynamic_data_raise_task_config_error(
+        self,
+    ) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        binary_artifact's data is a wrong type.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([6])
+        task_db = FakeTaskDatabase(
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=6,
+                        category=ArtifactCategory.SOURCE_PACKAGE,
+                        data=EmptyArtifactData(),
+                    )
+                ],
+            },
+        )
+
+        self.configure_task(
+            override={"input": {"binary_artifacts": [6]}},
+            remove=["environment"],
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.binary_artifact: unexpected artifact category: "
+            "'debian:source-package'. Valid categories: "
+            r"\['debian:binary-package', 'debian:binary-packages', "
+            r"'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_cmdline_minimum_options(self) -> None:
         """Test Lintian._cmdline minimum options."""
         minimum_cmd = [
@@ -1323,6 +1691,16 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         ).read_text()
         self.assertEqual(log_file_contents, f"{dsc} is not a valid .dsc file\n")
 
-    def test_label(self) -> None:
-        """Test get_label."""
+    def test_label_dynamic_data_is_none(self) -> None:
+        """Test get_label if dynamic_data.subject is None."""
         self.assertEqual(self.task.get_label(), "lintian")
+
+    def test_label_dynamic_data_subject_is_none(self) -> None:
+        """Test get_label if dynamic_data is None."""
+        self.task.dynamic_data = LintianDynamicData()
+        self.assertEqual(self.task.get_label(), "lintian")
+
+    def test_label_dynamic_data_subject_is_hello(self) -> None:
+        """Test get_label if dynamic_data.subject is set."""
+        self.task.dynamic_data = LintianDynamicData(subject="hello")
+        self.assertEqual(self.task.get_label(), "lintian hello")
