@@ -19,25 +19,6 @@ import debusine.tasks.models as task_models
 from debusine.artifacts.models import ArtifactCategory
 
 
-class TaskTypeTests(TestCase):
-    """Tests for TaskTypes (to exercise classproperty)."""
-
-    def test_choices(self) -> None:
-        """Test that choices returns the expected list."""
-        TaskTypes = task_models.TaskTypes
-        self.assertEqual(
-            TaskTypes.choices,
-            [
-                (TaskTypes.WORKER, TaskTypes.WORKER),
-                (TaskTypes.SERVER, TaskTypes.SERVER),
-                (TaskTypes.INTERNAL, TaskTypes.INTERNAL),
-                (TaskTypes.WORKFLOW, TaskTypes.WORKFLOW),
-                (TaskTypes.SIGNING, TaskTypes.SIGNING),
-                (TaskTypes.WAIT, TaskTypes.WAIT),
-            ],
-        )
-
-
 class LookupTests(TestCase):
     """Tests for collection item lookups."""
 
