@@ -29,6 +29,8 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianSystemTarball,
+    DebianUpload,
+    EmptyArtifactData,
 )
 from debusine.client.models import (
     ArtifactResponse,
@@ -38,15 +40,17 @@ from debusine.client.models import (
 )
 from debusine.tasks import Piuparts, TaskConfigError
 from debusine.tasks.models import LookupMultiple, PiupartsDynamicData
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
+from debusine.test.utils import create_system_tarball_data
 
 
-class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
-    """Test the Piuparts Task class."""
+class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
+    """Test the Piuparts class."""
 
     SAMPLE_TASK_DATA = {
         "input": {"binary_artifacts": [421]},
@@ -55,6 +59,15 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         "base_tgz": 44,
     }
 
+    _binary_upload_data = DebianUpload(
+        type="dpkg",
+        changes_fields={
+            "Architecture": "amd64",
+            "Files": [{"name": "hello_1.0_amd64.deb"}],
+            "Source": "hello",
+        },
+    )
+
     def setUp(self) -> None:  # noqa: D102
         self.task = Piuparts(self.SAMPLE_TASK_DATA)
         self.task.dynamic_data = PiupartsDynamicData(
@@ -97,13 +110,27 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # base_tgz
-                (44, CollectionCategory.ENVIRONMENTS): 44,
+                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
+                    id=44,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [421]
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self._binary_upload_data,
+                    )
+                ]
             },
         )
 
@@ -113,6 +140,9 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
                 base_tgz_id=44,
+                subject="hello",
+                runtime_context="sid:amd64",
+                configuration_context="sid",
             ),
         )
 
@@ -132,17 +162,31 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     CollectionCategory.ENVIRONMENTS,
-                ): 1,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
                 # base_tgz
                 (
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball",
                     CollectionCategory.ENVIRONMENTS,
-                ): 44,
+                ): ArtifactInfo(
+                    id=44,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(codename="bookworm"),
+                ),
             },
             multiple_lookups={
                 # input.binary_artifacts
-                (binary_artifacts_lookup, None): [421]
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self._binary_upload_data,
+                    )
+                ]
             },
         )
 
@@ -152,9 +196,107 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
                 base_tgz_id=44,
+                subject="hello",
+                runtime_context="bookworm:amd64",
+                configuration_context="bookworm",
             ),
         )
 
+    def test_compute_dynamic_raise_config_task_error_wrong_environment(
+        self,
+    ) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        environment artifact category is unexpected.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # base_tgz
+                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
+                    id=44,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=EmptyArtifactData(),
+                    )
+                ]
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^base_tgz: unexpected artifact category: 'debian:source-package'. "
+            r"Valid categories: \['debian:system-tarball'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db),
+
+    def test_compute_dynamic_raise_config_task_error_wrong_binary_artifact(
+        self,
+    ) -> None:
+        """
+        Test compute_dynamic_data raise TaskConfigError.
+
+        binary_artifact artifact category is unexpected.
+        """
+        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # base_tgz
+                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
+                    id=44,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.SYSTEM_TARBALL,
+                        data=EmptyArtifactData(),
+                    )
+                ]
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.binary_artifact: unexpected artifact category: "
+            r"'debian:system-tarball'. Valid categories: "
+            r"\['debian:binary-packages', 'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_cmdline_without_usr_sbin(self) -> None:
         """_cmdline adds `/usr/sbin` to `PATH` if it isn't there."""
         with mock.patch.dict(os.environ, {"PATH": "/usr/bin:/bin"}):
@@ -611,6 +753,23 @@ class PiupartsTaskTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
         self.task.upload_artifacts(download_directory, execution_success=True)
 
-    def test_label(self) -> None:
-        """Test get_label."""
+    def test_get_label_dynamic_data_is_none(self) -> None:
+        """Test get_label if dynamic_data.subject is None."""
         self.assertEqual(self.task.get_label(), "piuparts")
+
+    def test_get_label_dynamic_data_subject_is_none(self) -> None:
+        """Test get_label if dynamic_data is None."""
+        self.task.dynamic_data = PiupartsDynamicData(
+            environment_id=1, input_binary_artifacts_ids=[], base_tgz_id=2
+        )
+        self.assertEqual(self.task.get_label(), "piuparts")
+
+    def test_get_label_dynamic_data_subject_is_hello(self) -> None:
+        """Test get_label if dynamic_data.subject is set."""
+        self.task.dynamic_data = PiupartsDynamicData(
+            environment_id=1,
+            input_binary_artifacts_ids=[],
+            base_tgz_id=2,
+            subject="hello",
+        )
+        self.assertEqual(self.task.get_label(), "piuparts hello")
