@@ -29,92 +29,42 @@ from debusine.artifacts.models import (
     CollectionCategory,
     DebianBinaryPackage,
     DebianBinaryPackages,
+    DebianSourcePackage,
     DoseDistCheck,
+    EmptyArtifactData,
 )
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse, RemoteArtifact
 from debusine.tasks import Sbuild, TaskConfigError
 from debusine.tasks.executors import ExecutorInterface
 from debusine.tasks.models import SbuildDynamicData, WorkerType
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import create_artifact_response
+from debusine.test.utils import (
+    create_artifact_response,
+    create_system_tarball_data,
+)
 
 
-class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
-    """Test the SbuildTask class."""
+class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
+    """Test the Sbuild class."""
 
     SAMPLE_TASK_DATA = {
         "input": {
             "source_artifact": 5,
         },
-        "distribution": "bullseye",
+        "environment": "debian/match:codename=bullseye",
         "host_architecture": "amd64",
         "build_components": [
             "any",
             "all",
         ],
-        "backend": "schroot",
     }
 
-    def test_compute_dynamic_data(self) -> None:
-        """Dynamic data receives relevant artifact IDs."""
-        self.configure_task(
-            override={
-                "backend": "unshare",
-                "environment": "debian/match:codename=bookworm",
-            },
-            remove=["distribution"],
-        )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare",
-                    CollectionCategory.ENVIRONMENTS,
-                ): 1,
-                # input.source_artifact
-                (5, None): 5,
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
-            },
-        )
-
-        self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
-            SbuildDynamicData(
-                environment_id=1,
-                input_source_artifact_id=5,
-                input_extra_binary_artifacts_ids=[],
-                binnmu_maintainer="Debusine <noreply@debusine.example.com>",
-            ),
-        )
-
-    def test_compute_dynamic_data_environment(self) -> None:
-        """Dynamic data includes environment ID if relevant."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # input.source_artifact
-                (5, None): 5
-            },
-            settings={
-                "DEBUSINE_FQDN": "debusine.example.com",
-            },
-        )
-
-        self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
-            SbuildDynamicData(
-                input_source_artifact_id=5,
-                binnmu_maintainer="Debusine <noreply@debusine.example.com>",
-            ),
-        )
-
     def setUp(self) -> None:
         """
         Set a path to the ontology files used in the debusine tests.
@@ -124,22 +74,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         """
         self.configure_task(self.configuration("amd64", ["any"]))
 
-        task_call_schroot_list_patcher = mock.patch.object(
-            Sbuild, "_call_schroot_list"
-        )
-        self.schroot_list_mock = task_call_schroot_list_patcher.start()
-        self.schroot_list_mock.return_value = "chroot:bullseye-amd64-sbuild"
-        self.addCleanup(task_call_schroot_list_patcher.stop)
-
-        task_call_dpkg_architecture_patcher = mock.patch.object(
-            Sbuild, "_call_dpkg_architecture"
-        )
-        self.dpkg_architecture_mock = (
-            task_call_dpkg_architecture_patcher.start()
-        )
-        self.dpkg_architecture_mock.return_value = "amd64"
-        self.addCleanup(task_call_dpkg_architecture_patcher.stop)
-
     def configure_task(
         self,
         task_data: dict[str, Any] | None = None,
@@ -175,6 +109,131 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         self.cmdline_mock.return_value = cmdline
         self.addCleanup(patcher.stop)
 
+    def test_compute_dynamic_data(self) -> None:
+        """Dynamic data receives relevant artifact IDs."""
+        self.configure_task(
+            override={
+                "backend": "unshare",
+                "environment": "debian/match:codename=bookworm",
+            },
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            },
+            settings={
+                "DEBUSINE_FQDN": "debusine.example.com",
+            },
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            SbuildDynamicData(
+                environment_id=1,
+                input_source_artifact_id=5,
+                input_extra_binary_artifacts_ids=[],
+                binnmu_maintainer="Debusine <noreply@debusine.example.com>",
+                subject="hello",
+                runtime_context="all+any:amd64:amd64",
+                configuration_context="sid",
+            ),
+        )
+
+    def test_compute_dynamic_data_build_profiles_runtime_context(self) -> None:
+        """
+        Dynamic data includes build_profiles.
+
+        If build_profiles are included then the runtime_context is None.
+        """
+        self.configure_task(
+            override={
+                "build_profiles": ["nodoc", "nocheck"],
+                "backend": "unshare",
+                "environment": "debian/match:codename=bookworm",
+            },
+        )
+
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            },
+            settings={
+                "DEBUSINE_FQDN": "debusine.example.com",
+            },
+        )
+
+        self.assertIsNone(
+            self.task.compute_dynamic_data(task_db).runtime_context
+        )
+
+    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
+        """Raise TaskConfigError: invalid input_source_artifact."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.BINARY_PACKAGES,
+                    data=EmptyArtifactData(),
+                ),
+            },
+            settings={
+                "DEBUSINE_FQDN": "debusine.example.com",
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            "^input.source_artifact: unexpected artifact category: "
+            r"'debian:binary-packages'. Valid categories: "
+            r"\['debian:source-package'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_configure_with_unknown_data(self) -> None:
         """Configure fails if a non-recognised key is in task_data."""
         with self.assertRaises(TaskConfigError):
@@ -182,7 +241,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_configure_fails_with_missing_required_data(self) -> None:
         """Configure fails with missing required keys in task_data."""
-        for key in ("input", "distribution", "host_architecture"):
+        for key in ("input", "host_architecture"):
             with self.subTest(f"Configure with key {key} missing"):
                 with self.assertRaises(TaskConfigError):
                     self.configure_task(remove=[key])
@@ -227,121 +286,11 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 }
             )
 
-    def test_configure_fails_distribution_and_environment(self) -> None:
-        """Configure fails: "distribution" and "environment" both set."""
-        error_msg = 'The backend "auto" requires "distribution" to be unset'
-        with self.assertRaisesRegex(TaskConfigError, error_msg):
-            self.configure_task(
-                override={
-                    "backend": "auto",
-                    "distribution": "bookworm",
-                    "environment": 42,
-                }
-            )
-
-        error_msg = 'The backend "schroot" requires "environment" to be unset'
-        with self.assertRaisesRegex(TaskConfigError, error_msg):
-            self.configure_task(
-                override={
-                    "backend": "schroot",
-                    "distribution": "bookworm",
-                    "environment": 42,
-                }
-            )
-
-        error_msg = 'The backend "schroot" requires "distribution" to be set'
-        with self.assertRaisesRegex(TaskConfigError, error_msg):
-            self.configure_task(
-                override={
-                    "backend": "schroot",
-                    "distribution": None,
-                    "environment": None,
-                }
-            )
-
-        error_msg = 'The backend "auto" requires "environment" to be set'
-        with self.assertRaisesRegex(TaskConfigError, error_msg):
-            self.configure_task(
-                override={
-                    "backend": "auto",
-                    "distribution": None,
-                    "environment": None,
-                }
-            )
-
-    def test_configure_fails_backend_schroot_no_distribution(self) -> None:
-        """Configure fails: backend "schroot" requires "distribution"."""
-        msg = 'The backend "schroot" requires "distribution" to be set'
-        with self.assertRaisesRegex(TaskConfigError, msg):
-            self.configure_task(
-                override={"environment": 42, "backend": "schroot"},
-                remove=["distribution"],
-            )
-
-    def test_configure_fails_backend_unshare_no_environment(self) -> None:
-        """Configure fails: backend "unshare" requires "environment"."""
-        backend = "unshare"
-        msg = f'The backend "{backend}" requires "environment" to be set'
-        with self.assertRaisesRegex(TaskConfigError, msg):
-            self.configure_task(
-                override={"distribution": "bookworm", "backend": backend},
-                remove=["environment"],
-            )
-
-    def test_update_chroots_list_skips_non_desired_chroots(self) -> None:
-        """source:foo and non foo-sbuild chroots are ignored."""
-        self.schroot_list_mock.return_value = (
-            "source:jessie-amd64-sbuild\nchroot:buster-amd64\n"
-            "chroot:wanted-sbuild"
-        )
-
-        self.task._update_chroots_list()
-        self.task._update_chroots_list()  # second call triggers no-op branch
-
-        self.assertEqual(self.task.chroots, ["wanted"])
-
     def test_fetch_input(self) -> None:
         """Test fetch_input: call fetch_artifact(artifact_id, directory)."""
         directory = self.create_temporary_directory()
         source_artifact = self.fake_debian_source_package_artifact()
-        self.configure_task(
-            override={
-                "input": {
-                    "source_artifact": source_artifact.id,
-                },
-            }
-        )
-        self.task.work_request_id = 5
-        self.task.dynamic_data = SbuildDynamicData(
-            input_source_artifact_id=source_artifact.id
-        )
-        debusine_mock = self.mock_debusine()
-        debusine_mock.artifact_get.return_value = source_artifact
-
-        with (
-            mock.patch.object(
-                self.task, "fetch_artifact", autospec=True, return_value=True
-            ) as fetch_artifact_mocked,
-            mock.patch.object(
-                self.task, "_prepare_executor", autospec=True
-            ) as prepare_executor_mocked,
-        ):
-            result = self.task.fetch_input(directory)
-
-        self.assertTrue(result)
-        fetch_artifact_mocked.assert_called_once_with(
-            source_artifact.id, directory
-        )
-        prepare_executor_mocked.assert_not_called()
-
-    def test_fetch_input_executor(self) -> None:
-        """Test fetch_input with an executor."""
-        directory = self.create_temporary_directory()
-        source_artifact = self.fake_debian_source_package_artifact()
-        self.configure_task(
-            override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
-        )
+        self.configure_task(override={"environment": 42})
         self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             environment_id=42, input_source_artifact_id=source_artifact.id
@@ -481,9 +430,22 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         debusine_mock.artifact_get.side_effect = artifact_get
 
-        with mock.patch.object(
-            self.task, "fetch_artifact", autospec=True, return_value=True
-        ) as fetch_artifact_mocked:
+        def mock_executor_executor_instance() -> None:
+            self.task.executor = MagicMock()
+            self.task.executor.system_image = MagicMock(spec=ArtifactResponse)
+            self.task.executor.system_image.data = {"codename": "bookworm"}
+
+        with (
+            mock.patch.object(
+                self.task, "fetch_artifact", autospec=True, return_value=True
+            ) as fetch_artifact_mocked,
+            mock.patch.object(
+                self.task,
+                "_prepare_executor",
+                autospec=True,
+                side_effect=mock_executor_executor_instance,
+            ) as prepare_executor_mocked,
+        ):
             result = self.task.fetch_input(directory)
 
         self.assertTrue(result)
@@ -493,6 +455,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 call(binary_artifact.id, directory),
             ]
         )
+        prepare_executor_mocked.assert_called_once_with()
 
     def test_fetch_input_extra_binaries_wrong_category(self) -> None:
         """Test fetch_input: checks the categories of extra_binary_artifacts."""
@@ -533,22 +496,9 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             ),
         )
 
-    def test_environment_codename_schroot(self) -> None:
-        """Test _environment_codename gets the schroot distribution."""
-        self.configure_task(
-            override={
-                "backend": "schroot",
-                "distribution": "bookworm",
-            }
-        )
-        self.assertEqual(self.task._environment_codename(), "bookworm")
-
     def test_environment_codename_executor(self) -> None:
         """Test _environment_codename gets the executor distribution."""
-        self.configure_task(
-            override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
-        )
+        self.configure_task(override={"environment": 42})
         self.task.dynamic_data = SbuildDynamicData(
             environment_id=42,
             input_source_artifact_id=1,
@@ -563,10 +513,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_environment_codename_executor_unconfigured(self) -> None:
         """Test _environment_codename raises ValueError without executor."""
-        self.configure_task(
-            override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
-        )
+        self.configure_task(override={"environment": 42})
         with self.assertRaisesRegex(
             ValueError,
             r"self\.executor not set, call _prepare_executor\(\) first",
@@ -586,7 +533,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_contains_arch_parameter(self) -> None:
         """Ensure --arch parameter is computed from data."""
-        self.schroot_list_mock.return_value = "chroot:bullseye-mipsel-sbuild"
         self.configure_task(override={"host_architecture": "mipsel"})
         self.patch_executor()
         cmdline = self.task._cmdline()
@@ -595,9 +541,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_contains_dist_parameter(self) -> None:
         """Ensure --dist parameter is computed from data."""
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
-        self.configure_task(override={"distribution": "jessie"})
-        self.patch_executor()
+        self.configure_task()
+        self.patch_executor(codename="jessie")
 
         cmdline = self.task._cmdline()
 
@@ -605,11 +550,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_extra_repositories_jessie(self) -> None:
         """Ensure the --extra-repository= parameter is passed to sbuild."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
         self.configure_task(
             override={
-                "distribution": "jessie",
                 "extra_repositories": [
                     {
                         "url": "http://example.com/",
@@ -623,6 +565,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsNone(env)
 
@@ -637,11 +580,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_extra_repositories_deb822_keyring(self) -> None:
         """Ensure the --pre-build-commands= parameter is passed to sbuild."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:bookworm-amd64-sbuild"
         self.configure_task(
             override={
-                "distribution": "bullseye",
                 "extra_repositories": [
                     {
                         "url": "http://example.com/",
@@ -651,6 +591,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.patch_executor(codename="bullseye")
         env = self.task._cmd_env()
         self.assertIsNone(env)
 
@@ -678,11 +619,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_extra_repositories_deb822_inline(self) -> None:
         """Ensure the --pre-build-commands= parameter is passed to sbuild."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:bookworm-amd64-sbuild"
         self.configure_task(
             override={
-                "distribution": "bookworm",
                 "extra_repositories": [
                     {
                         "url": "http://example.com/",
@@ -692,6 +630,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 ],
             }
         )
+        self.patch_executor(codename="bookworm")
         env = self.task._cmd_env()
         self.assertIsNone(env)
 
@@ -710,11 +649,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_binnmu(self) -> None:
         """Check binnmu command line."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
         self.configure_task(
             override={
-                "distribution": "jessie",
                 "binnmu": {
                     "changelog": "Rebuild for 64bit time_t",
                     "suffix": "+b1",
@@ -722,6 +658,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "build_components": ["any"],
             }
         )
+        self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--make-binNMU=Rebuild for 64bit time_t", cmdline)
         self.assertIn("--append-to-version=+b1", cmdline)
@@ -731,7 +668,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.configure_task(
             override={
-                "distribution": "jessie",
                 "binnmu": {
                     "changelog": "Rebuild for 64bit time_t",
                     "suffix": "+b1",
@@ -741,6 +677,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "build_components": ["any"],
             }
         )
+        self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--make-binNMU=Rebuild for 64bit time_t", cmdline)
         self.assertIn("--append-to-version=+b1", cmdline)
@@ -752,67 +689,40 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_no_build_profiles(self) -> None:
         """Check that missing build_profiles key is handled."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
-        self.configure_task(
-            override={
-                "distribution": "jessie",
-            }
-        )
+        self.configure_task()
+        self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
             self.assertFalse(argument.startswith("--profiles="))
 
     def test_cmdline_empty_build_profiles(self) -> None:
         """Check that empty build_profiles don't appear on the command line."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
-        self.configure_task(
-            override={
-                "distribution": "jessie",
-                "build_profiles": [],
-            }
-        )
+        self.configure_task(override={"build_profiles": []})
+        self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         for argument in cmdline:
             self.assertFalse(argument.startswith("--profiles="))
 
     def test_cmdline_build_profiles(self) -> None:
         """Check that build_profiles make it onto the command line."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
-        self.configure_task(
-            override={
-                "distribution": "jessie",
-                "build_profiles": ["nodoc", "nocheck"],
-            }
-        )
+        self.configure_task(override={"build_profiles": ["nodoc", "nocheck"]})
+        self.patch_executor(codename="jessie")
         cmdline = self.task._cmdline()
         self.assertIn("--profiles=nodoc,nocheck", cmdline)
 
     def test_cmd_env_no_build_profiles(self) -> None:
         """Check that environment isn't modified, without build_profiles."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
-        self.configure_task(
-            override={
-                "distribution": "jessie",
-                "build_profiles": ["cross"],
-            }
-        )
+        self.configure_task(override={"build_profiles": ["cross"]})
+        self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsNone(env)
 
     def test_cmd_env_build_profiles(self) -> None:
         """Check that build_profiles are specified in DEB_BUILD_OPTIONS."""
-        self.patch_executor()
-        self.schroot_list_mock.return_value = "chroot:jessie-amd64-sbuild"
         self.configure_task(
-            override={
-                "distribution": "jessie",
-                "build_profiles": ["nodoc", "nocheck", "cross"],
-            }
+            override={"build_profiles": ["nodoc", "nocheck", "cross"]}
         )
+        self.patch_executor(codename="jessie")
         env = self.task._cmd_env()
         self.assertIsInstance(env, dict)
         assert env
@@ -823,7 +733,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_cmdline_translation_of_build_components(self) -> None:
         """Test handling of build components."""
-        self.patch_executor()
         option_mapping = {
             "any": ["--arch-any", "--no-arch-any"],
             "all": ["--arch-all", "--no-arch-all"],
@@ -839,6 +748,7 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 self.configure_task(
                     override={"build_components": list(combination)}
                 )
+                self.patch_executor()
                 cmdline = self.task._cmdline()
                 for key, value in option_mapping.items():
                     if key in combination:
@@ -848,21 +758,10 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                         self.assertNotIn(value[0], cmdline)
                         self.assertIn(value[1], cmdline)
 
-    def test_cmdline_contains_chmod_schroot_build_args(self) -> None:
-        """Ensure cmdline has the arguments for schroot backend."""
-        distribution = "bookworm"
-        self.configure_task(
-            override={"backend": "schroot", "distribution": distribution}
-        )
-        cmdline = self.task._cmdline()
-        self.assertIn(f"--dist={distribution}", cmdline)
-
     def test_cmdline_contains_extra_packages(self) -> None:
         """Ensure cmdline has the arguments --extra-package=."""
-        distribution = "bookworm"
-        self.configure_task(
-            override={"backend": "schroot", "distribution": distribution}
-        )
+        self.configure_task()
+        self.patch_executor()
         self.task._extra_packages = [Path("/tmp/extra.deb")]
         cmdline = self.task._cmdline()
         self.assertIn("--extra-package=/tmp/extra.deb", cmdline)
@@ -874,7 +773,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 "backend": "unshare",
                 "environment": "debian/match:codename=bookworm",
             },
-            remove=["distribution"],
         )
         self.task.dynamic_data = SbuildDynamicData(
             environment_id=42,
@@ -906,7 +804,6 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         data = self.configuration("amd64", ["any"])
         data["backend"] = "incus-lxc"
         data["environment"] = 1
-        del data["distribution"]
         self.configure_task(data)
 
         executor_mocked = self.patch_executor()
@@ -936,36 +833,16 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             expected_args,
         )
 
-    def test_execute_fails_with_unsupported_schroot_distribution(self) -> None:
-        """execute() fails when the requested distribution is unsupported."""
-        # Default mocked setup only support bullseye
-        self.configure_task(override={"distribution": "jessie"})
-        with self.assertRaises(TaskConfigError):
-            self.task.execute()
-
-    def test_execute_fails_with_no_available_chroots(self) -> None:
-        """execute() fails when no sbuild chroots are available."""
-        self.schroot_list_mock.return_value = ""
-        self.configure_task()
-        with self.assertRaises(TaskConfigError):
-            self.task.execute()
-
     def test_execute_succeeds_backend_unshare(self) -> None:
-        """execute() backend "unshare": does not call _verify_schroots()."""
+        """execute() backend "unshare" succeeds."""
         self.configure_task(
             task_data=self.configuration("amd64", ["any"]),
             override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
         )
         self.task.work_request_id = 5
         self.patch_sbuild_debusine()
 
-        with mock.patch.object(
-            self.task, "_verify_schroot", autospec=True
-        ) as mocked:
-            self.run_execute(["true"])
-
-        mocked.assert_not_called()
+        self.run_execute(["true"])
 
     def patch_temporary_directory(self, directories: int = 1) -> list[Path]:
         """
@@ -1179,13 +1056,20 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         return mocked
 
-    def patch_executor(self) -> mock.Mock:
-        """Patch self.task._executor."""
+    def patch_executor(self, codename: str = "bookworm") -> mock.Mock:
+        """
+        Patch self.task._executor.
+
+        :param codename: Provide a mock system image with this codename.
+        """
         patcher = mock.patch.object(
             self.task, "executor", autospec=ExecutorInterface
         )
         mocked = patcher.start()
         self.addCleanup(patcher.stop)
+        assert self.task.executor is not None
+        self.task.executor.system_image = MagicMock(spec=ArtifactResponse)
+        self.task.executor.system_image.data = {"codename": codename}
         return mocked
 
     def patch_task_succeeded(self, *, return_value: bool) -> mock.Mock:
@@ -1300,36 +1184,19 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
-        self.mock_is_command_available({"sbuild": True, "schroot": True})
-        self.schroot_list_mock.return_value = "chroot:jessie-arm64-sbuild"
-        self.dpkg_architecture_mock.return_value = "mipsel"
+        self.mock_is_command_available({"sbuild": True})
 
         metadata = self.task.analyze_worker()
 
         self.assertEqual(metadata["sbuild:available"], True)
-        self.assertEqual(metadata["sbuild:chroots"], ["jessie-arm64"])
 
     def test_analyze_worker_sbuild_not_available(self) -> None:
         """analyze_worker() handles sbuild not being available."""
-        self.mock_is_command_available({"sbuild": False, "schroot": True})
-        self.schroot_list_mock.return_value = "chroot:jessie-arm64-sbuild"
-        self.dpkg_architecture_mock.return_value = "mipsel"
+        self.mock_is_command_available({"sbuild": False})
 
         metadata = self.task.analyze_worker()
 
         self.assertEqual(metadata["sbuild:available"], False)
-        self.assertEqual(metadata["sbuild:chroots"], ["jessie-arm64"])
-
-    def test_analyze_worker_schroot_not_available(self) -> None:
-        """analyze_worker() handles schroot not being available."""
-        self.mock_is_command_available({"sbuild": True, "schroot": False})
-        self.schroot_list_mock.return_value = "chroot:jessie-arm64-sbuild"
-        self.dpkg_architecture_mock.return_value = "mipsel"
-
-        metadata = self.task.analyze_worker()
-
-        self.assertEqual(metadata["sbuild:available"], True)
-        self.assertNotIn("sbuild:chroots", metadata)
 
     def worker_metadata(self) -> dict[str, Any]:
         """Return worker_metadata with sbuild:version=self.task.TASK_VERSION."""
@@ -1339,16 +1206,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             "system:host_architecture": "amd64",
             "sbuild:version": self.task.TASK_VERSION,
             "sbuild:available": True,
-            "sbuild:chroots": ["bullseye-amd64"],
         }
 
-    def test_can_run_schroot_exists(self) -> None:
-        """Ensure can_run_on returns True: schroot and exists."""
-        worker_metadata = self.worker_metadata()
-        self.configure_task(override={"backend": "schroot"})
-
-        self.assertTrue(self.task.can_run_on(worker_metadata))
-
     def test_can_run_mismatched_task_version(self) -> None:
         """Ensure can_run_on returns False for mismatched versions."""
         worker_metadata = self.worker_metadata()
@@ -1365,34 +1224,18 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
 
-    def test_can_run_chroot_not_available(self) -> None:
-        """Ensure can_run_on returns False when needed chroot is not there."""
+    def test_can_run_return_true_unshare(self) -> None:
+        """can_run_on returns True: using unshare."""
         worker_metadata = self.worker_metadata()
-        worker_metadata["sbuild:chroots"] = ["jessie-arm64"]
-        self.configure_task(override={"backend": "schroot"})
-
-        self.assertFalse(self.task.can_run_on(worker_metadata))
-
-    def test_can_run_return_true_missing_schroot_but_unshare(self) -> None:
-        """can_run_on returns True: missing distribution but using unshare."""
-        worker_metadata = self.worker_metadata()
-        worker_metadata["sbuild:chroots"] = ["jessie-arm64"]
         worker_metadata["executor:unshare:available"] = True
-        self.configure_task(
-            override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
-        )
+        self.configure_task(override={"backend": "unshare", "environment": 42})
 
         self.assertTrue(self.task.can_run_on(worker_metadata))
 
     def test_can_run_on_unshare_requires_unshare(self) -> None:
         """can_run_on returns False: missing unshare executor."""
         worker_metadata = self.worker_metadata()
-        worker_metadata["sbuild:chroots"] = ["bookworm-arm64"]
-        self.configure_task(
-            override={"backend": "unshare", "environment": 42},
-            remove=["distribution"],
-        )
+        self.configure_task(override={"backend": "unshare", "environment": 42})
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
 
@@ -1401,10 +1244,8 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         worker_metadata = self.worker_metadata()
         worker_metadata["executor:incus-lxc:available"] = True
         worker_metadata["autopkgtest:available"] = True
-        worker_metadata["sbuild:chroots"] = ["bookworm-arm64"]
         self.configure_task(
-            override={"backend": "incus-lxc", "environment": 42},
-            remove=["distribution"],
+            override={"backend": "incus-lxc", "environment": 42}
         )
 
         self.assertTrue(self.task.can_run_on(worker_metadata))
@@ -1413,22 +1254,12 @@ class SbuildTaskTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         """can_run_on returns False: without autopkgtest."""
         worker_metadata = self.worker_metadata()
         worker_metadata["executor:incus-lxc:available"] = True
-        worker_metadata["sbuild:chroots"] = ["bookworm-arm64"]
         self.configure_task(
-            override={"backend": "incus-lxc", "environment": 42},
-            remove=["distribution"],
+            override={"backend": "incus-lxc", "environment": 42}
         )
 
         self.assertFalse(self.task.can_run_on(worker_metadata))
 
-    def test_can_run_missing_distribution(self) -> None:
-        """can_run_on returns False if sbuild:chroots is not in the metadata."""
-        worker_metadata = self.worker_metadata()
-        del worker_metadata["sbuild:chroots"]
-        self.configure_task(override={"backend": "schroot"})
-
-        self.assertFalse(self.task.can_run_on(worker_metadata))
-
     def test_task_succeeded_returncode_aborted(self) -> None:
         """task_succeeded returns False if sbuild was aborted."""
         directory = self.create_temporary_directory()
@@ -2346,10 +2177,9 @@ broken-packages: 4
             "input": {
                 "source_artifact": 5,
             },
-            "distribution": "bullseye",
+            "environment": "debian/match:codename=bullseye",
             "host_architecture": host_architecture,
             "build_components": build_components,
-            "backend": "schroot",
         }
 
     def create_deb_udeb_files(
@@ -2624,6 +2454,18 @@ broken-packages: 4
         )
         self.assertEqual(self.task.get_source_artifacts_ids(), [2, 1, 3, 4])
 
-    def test_label(self) -> None:
-        """Test get_label."""
+    def test_label_subject_is_none(self) -> None:
+        """Test get_label if dynamic_data.subject is None."""
         self.assertEqual(self.task.get_label(), "build a package")
+
+    def test_label_dynamic_data_is_none(self) -> None:
+        """Test get_label if dynamic_data is None."""
+        self.task.dynamic_data = None
+        self.assertEqual(self.task.get_label(), "build a package")
+
+    def test_label_package_subject(self) -> None:
+        """Test get_label if dynamic_data.subject is set."""
+        self.task.dynamic_data = SbuildDynamicData(
+            input_source_artifact_id=1, environment_id=2, subject="hello"
+        )
+        self.assertEqual(self.task.get_label(), "build hello")
