@@ -17,12 +17,16 @@ from unittest.mock import call, patch
 from debusine.artifacts.local_artifact import DebianSystemImageArtifact
 from debusine.tasks import SimpleSystemImageBuild
 from debusine.tasks.models import (
+    BaseDynamicTaskData,
     DiskImageFormat,
     SystemBootstrapRepositoryCheckSignatureWith,
     SystemBootstrapRepositoryType,
     WorkerType,
 )
-from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
+from debusine.tasks.tests.helper_mixin import (
+    ExternalTaskHelperMixin,
+    FakeTaskDatabase,
+)
 from debusine.test import TestCase
 
 
@@ -82,14 +86,14 @@ class SimpleSystemImageBuildTests(
 
         self.configure_task(override={"bootstrap_options": bootstrap_options})
 
+        self.maxDiff = None
         self.assertEqual(
-            self.task.data.dict(),
+            self.task.data.dict(exclude_unset=True),
             {
                 'bootstrap_options': {
                     'architecture': 'amd64',
                     'extra_packages': ['hello', 'bye'],
                     'variant': "minbase",
-                    "use_signed_by": True,
                 },
                 'bootstrap_repositories': [
                     {
@@ -99,9 +103,6 @@ class SimpleSystemImageBuildTests(
                         'components': ['main', 'contrib'],
                         'mirror': 'https://deb.debian.org/debian',
                         'suite': 'stable',
-                        'keyring': None,
-                        'keyring_package': None,
-                        'types': [SystemBootstrapRepositoryType.DEB],
                     },
                     {
                         'check_signature_with': (
@@ -109,29 +110,31 @@ class SimpleSystemImageBuildTests(
                         ),
                         'components': ['main'],
                         'keyring': {
-                            'install': False,
-                            'sha256sum': '',
                             'url': 'https://example.com/keyring.gpg',
                         },
-                        'keyring_package': None,
                         'mirror': 'https://example.com',
                         'suite': 'bullseye',
                         'types': [SystemBootstrapRepositoryType.DEB_SRC],
                     },
                 ],
-                'customization_script': None,
                 'disk_image': {
                     'format': DiskImageFormat.RAW,
-                    'bootloader': None,
-                    'filename': 'image',
-                    'kernel_package': None,
-                    'partitions': [
-                        {'size': 2, 'filesystem': 'ext4', 'mountpoint': 'none'}
-                    ],
+                    'partitions': [{'size': 2, 'filesystem': 'ext4'}],
                 },
             },
         )
 
+    def test_compute_dynamic_data(self) -> None:
+        task_database = FakeTaskDatabase()
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_database),
+            BaseDynamicTaskData(
+                subject="stable",
+                runtime_context="amd64:None:hello",
+                configuration_context="amd64",
+            ),
+        )
+
     def test_analyze_worker(self) -> None:
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"mmdebstrap": True})
