@@ -34,7 +34,7 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from debusine.artifacts import WorkRequestDebugLogs
-from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.models import ArtifactCategory, EmptyArtifactData
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse
 from debusine.tasks import (
@@ -62,13 +62,17 @@ from debusine.tasks.models import (
     TaskTypes,
     WorkerType,
 )
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
     TestTaskMixin,
 )
 from debusine.test import TestCase
-from debusine.test.utils import preserve_task_registry
+from debusine.test.utils import (
+    create_system_tarball_data,
+    preserve_task_registry,
+)
 
 
 class TestBaseTask(
@@ -270,6 +274,18 @@ class BaseTaskTests(TestCase):
             TestWait.prefix_with_task_name("foo"), "wait:testwait:foo"
         )
 
+    def test_get_subject(self) -> None:
+        self.task.dynamic_data = BaseDynamicTaskDataWithExecutor(
+            subject="hello"
+        )
+
+        self.assertEqual(self.task.get_subject(), "hello")
+
+    def test_get_subject_dynamic_task_data_is_none(self) -> None:
+        self.task.dynamic_data = None
+
+        self.assertIsNone(self.task.get_subject())
+
     def test_configure_sets_data_attribute(self) -> None:
         """task.data is set with configure."""
         task_data = {
@@ -658,6 +674,57 @@ class BaseTaskTests(TestCase):
         """BaseTask.is_worker_task(): nonexistent isn't a WORKER task."""
         self.assertFalse(self.task.is_worker_task("nonexistent"))
 
+    def test_ensure_artifact_categories_artifact(self) -> None:
+        BaseTask.ensure_artifact_categories(
+            configuration_key="source_artifact",
+            category=ArtifactCategory.TEST,
+            expected=[ArtifactCategory.TEST, ArtifactCategory.SOURCE_PACKAGE],
+        )
+
+    def test_ensure_artifact_categories_raise(self) -> None:
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^source_artifact: unexpected artifact category: 'debusine:test'. "
+            r"Valid categories: "
+            r"\['debian:binary-package', 'debian:binary-packages'\]$",
+        ):
+            BaseTask.ensure_artifact_categories(
+                configuration_key="source_artifact",
+                category=ArtifactCategory.TEST,
+                expected=[
+                    ArtifactCategory.BINARY_PACKAGE,
+                    ArtifactCategory.BINARY_PACKAGES,
+                ],
+            )
+
+    def test_instantiate_with_new_data(self) -> None:
+        task = self.task2.instantiate_with_new_data({"foo": "baz"})
+        self.assertEqual(self.task2.data.foo, "bar")
+        self.assertEqual(task.data.foo, "baz")
+
+        self.assertIsNone(task.work_request_id)
+        self.assertIsNone(task.workspace_name)
+        self.assertIsNone(task.worker_host_architecture)
+
+    def test_instantiate_with_new_data_copy_attrs(self) -> None:
+        """instantiate_with_new_data copies extra attributes."""
+        self.task2.work_request_id = 42
+        self.task2.workspace_name = "test_workspace"
+        self.task2.worker_host_architecture = "arm64"
+
+        task = self.task2.instantiate_with_new_data({"foo": "bar"})
+
+        self.assertEqual(task.work_request_id, 42)
+        self.assertEqual(task.workspace_name, "test_workspace")
+        self.assertEqual(task.worker_host_architecture, "arm64")
+
+    def test_instantiate_with_new_data_ignores_old_data(self) -> None:
+        """instantiate_with_new_data ignores existing task data."""
+        self.task2.data.bar = 1
+        task = self.task2.instantiate_with_new_data({"foo": "bar"})
+        self.assertEqual(task.data.foo, "bar")
+        self.assertIsNone(task.data.bar)
+
 
 class TestBaseExternalTask(
     TestTaskMixin, BaseExternalTask[BaseTaskData, BaseDynamicTaskData]
@@ -976,8 +1043,12 @@ class BaseExternalTaskTests(TestCase):
         traceback_str = "".join(traceback.format_exception(exc))
         self.assertIn(traceback_str, log)
 
-    def test_execute_logs_progress(self) -> None:
-        """_execute() logs each of its main steps."""
+    def test_execute_logs_progress_success(self) -> None:
+        """
+        _execute() logs each of its main steps: success.
+
+        For a success, there is no stages.log file.
+        """
         self.task.work_request_id = 1
 
         with (
@@ -1004,6 +1075,67 @@ class BaseExternalTaskTests(TestCase):
                 "INFO:debusine.tasks:Work request 1: Cleaning up",
             ],
         )
+        assert self.task._debug_log_files_directory
+        stages_log = (
+            Path(self.task._debug_log_files_directory.name) / "stages.log"
+        )
+        self.assertFalse(stages_log.exists())
+
+    def test_execute_logs_progress_failure(self) -> None:
+        """
+        _execute() logs each of its main steps: failure.
+
+        For a failure, there is an additional stages.log file.
+        """
+        self.task.work_request_id = 1
+
+        with (
+            mock.patch.object(self.task, "fetch_input", autospec=True),
+            mock.patch.object(
+                self.task, "configure_for_execution", autospec=True
+            ),
+            mock.patch.object(
+                self.task, "run", autospec=True, return_value=False
+            ),
+            mock.patch.object(self.task, "upload_artifacts", autospec=True),
+            mock.patch.object(self.task, "cleanup", autospec=True),
+            self.assertLogs(self.task.logger) as log,
+        ):
+            self.assertFalse(self.task._execute())
+
+        self.assertEqual(
+            log.output,
+            [
+                "INFO:debusine.tasks:Work request 1: Fetching input",
+                "INFO:debusine.tasks:Work request 1: Configuring for execution",
+                "INFO:debusine.tasks:Work request 1: Preparing to run",
+                "INFO:debusine.tasks:Work request 1: Running",
+                "INFO:debusine.tasks:Work request 1: Checking output",
+                "INFO:debusine.tasks:Work request 1: Uploading artifacts",
+                "INFO:debusine.tasks:Work request 1: Cleaning up",
+            ],
+        )
+        assert self.task._debug_log_files_directory
+        stages_log = (
+            Path(self.task._debug_log_files_directory.name) / "stages.log"
+        )
+        stages_log_lines = []
+        for line in stages_log.read_text().splitlines():
+            dt, message = line.split(maxsplit=1)
+            datetime.datetime.fromisoformat(dt)
+            stages_log_lines.append(message)
+        self.assertEqual(
+            stages_log_lines,
+            [
+                "Fetching input",
+                "Configuring for execution",
+                "Preparing to run",
+                "Running",
+                "Checking output",
+                "Uploading artifacts",
+                "Cleaning up",
+            ],
+        )
 
     def setup_upload_work_request_debug_logs(
         self, source_artifacts_ids: list[int] | None = None
@@ -1116,6 +1248,15 @@ class TestsTaskConfigError(TestCase):
         self.assertEqual(str(error), str(original_exc))
         self.assertIs(error.original_exception, original_exc)
 
+    def test_add_parent_message(self) -> None:
+        """Test add_parent_message."""
+        for exc, expected in (
+            (TaskConfigError(None), "test"),
+            (TaskConfigError("msg"), "test: msg"),
+        ):
+            exc.add_parent_message("test")
+            self.assertEqual(exc.args[0], expected)
+
 
 class RunCommandTaskForTesting(
     TestTaskMixin, RunCommandTask[BaseTaskData, BaseDynamicTaskData]
@@ -1854,16 +1995,21 @@ class BaseTaskWithExecutorTests(TestCase):
 
     def test_backend_return_backend(self) -> None:
         """BaseTask.backend return the backend."""
-        backend = BackendType.SCHROOT
+        backend = BackendType.INCUS_LXC
         self.task.data.backend = backend
 
         self.assertEqual(self.task.backend, backend)
 
     def test_get_environment_int(self) -> None:
         """`get_environment` passes through integer lookups."""
-        task_db = FakeTaskDatabase(single_lookups={(1, None): 1})
+        artifact_info = ArtifactInfo(
+            id=1,
+            category=ArtifactCategory.TEST,
+            data=EmptyArtifactData(),
+        )
+        task_db = FakeTaskDatabase(single_lookups={(1, None): artifact_info})
 
-        self.assertEqual(self.task.get_environment(task_db, 1), 1)
+        self.assertEqual(self.task.get_environment(task_db, 1), artifact_info)
 
     def test_get_environment_with_architecture_and_backend(self) -> None:
         """`get_environment` fills in architecture/backend."""
@@ -1876,18 +2022,23 @@ class BaseTaskWithExecutorTests(TestCase):
         )
         assert task.data.environment
         self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.TARBALL)
+        artifact_info = ArtifactInfo(
+            id=2,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
         task_db = FakeTaskDatabase(
             single_lookups={
                 (
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=tarball:backend=unshare",
                     None,
-                ): 2
+                ): artifact_info
             }
         )
 
         self.assertEqual(
-            task.get_environment(task_db, task.data.environment), 2
+            task.get_environment(task_db, task.data.environment), artifact_info
         )
 
     def test_get_environment_with_architecture_from_worker(self) -> None:
@@ -1901,18 +2052,23 @@ class BaseTaskWithExecutorTests(TestCase):
         assert task.data.environment
         task.worker_host_architecture = "amd64"
         self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
+        artifact_info = ArtifactInfo(
+            id=3,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
         task_db = FakeTaskDatabase(
             single_lookups={
                 (
                     "debian/match:codename=bookworm:architecture=amd64:"
                     "format=image:backend=unshare",
                     None,
-                ): 3
+                ): artifact_info
             }
         )
 
         self.assertEqual(
-            task.get_environment(task_db, task.data.environment), 3
+            task.get_environment(task_db, task.data.environment), artifact_info
         )
 
     def test_get_environment_with_architecture_missing(self) -> None:
@@ -1925,18 +2081,23 @@ class BaseTaskWithExecutorTests(TestCase):
         )
         assert task.data.environment
         self.patch_executor(BackendType.UNSHARE, ExecutorImageCategory.IMAGE)
+        artifact_info = ArtifactInfo(
+            id=3,
+            category=ArtifactCategory.SYSTEM_TARBALL,
+            data=create_system_tarball_data(),
+        )
         task_db = FakeTaskDatabase(
             single_lookups={
                 (
                     "debian/match:codename=bookworm:format=image:"
                     "backend=unshare",
                     None,
-                ): 3
+                ): artifact_info
             }
         )
 
         self.assertEqual(
-            task.get_environment(task_db, task.data.environment), 3
+            task.get_environment(task_db, task.data.environment), artifact_info
         )
 
     def test_prepare_executor(self) -> None:
