@@ -467,7 +467,7 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
                 self.fail(buf.getvalue())
 
 
-class PlaygroundMixin(BaseDjangoTestCase):
+class PlaygroundTestCase(BaseDjangoTestCase):
     """
     Playground-specific test case setup.
 
@@ -581,7 +581,10 @@ class PlaygroundMixin(BaseDjangoTestCase):
                 # Ignoring type until we find a way to type self.playground to
                 # accept either a member or a classvar, depending on subclasses
                 self.playground.create_group_role(  # type: ignore[attr-defined]
-                    assignee, role, user, name="assertNoPermissionWhenRole"
+                    assignee,
+                    role,
+                    users=[user],
+                    name="assertNoPermissionWhenRole",
                 )
 
                 if predicate(user):
@@ -646,7 +649,10 @@ class PlaygroundMixin(BaseDjangoTestCase):
                 # Ignoring type until we find a way to type self.playground to
                 # accept either a member or a classvar, depending on subclasses
                 self.playground.create_group_role(  # type: ignore[attr-defined]
-                    assignee, role, user, name="assertPermissionWhenRole"
+                    assignee,
+                    role,
+                    users=[user],
+                    name="assertPermissionWhenRole",
                 )
 
                 if not predicate(user):
@@ -673,8 +679,7 @@ class PlaygroundMixin(BaseDjangoTestCase):
 
 
 class TransactionTestCase(
-    PlaygroundMixin,
-    BaseDjangoTestCase,
+    PlaygroundTestCase,
     django.test.TransactionTestCase,
 ):
     """Debusine-specific extensions to django's TransactionTestCase."""
@@ -700,8 +705,7 @@ class TransactionTestCase(
 
 
 class TestCase(
-    PlaygroundMixin,
-    BaseDjangoTestCase,
+    PlaygroundTestCase,
     django.test.TestCase,
 ):
     """Debusine-specific extensions to django's TestCase."""
