@@ -19,6 +19,7 @@ from debian import deb822
 from debusine.test import TestCase
 from debusine.utils import (
     CALCULATE_HASH_CHUNK_SIZE,
+    DjangoChoicesEnum,
     calculate_hash,
     find_file_suffixes,
     find_files_suffixes,
@@ -261,3 +262,14 @@ class UtilsTests(TestCase):
         """is_command_available returns False if the command does not exist."""
         self.patch_shutil_which(False)
         self.assertFalse(is_command_available("test"))
+
+    def test_choices(self) -> None:
+        """Test that .choices returns the values in Django choices pairs."""
+
+        class TestChoices(DjangoChoicesEnum):
+            FOO = "foo"
+            BAR = "bar"
+
+        self.assertEqual(
+            list(TestChoices.choices), [("foo", "foo"), ("bar", "bar")]
+        )
