@@ -370,8 +370,8 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
                 # Add file to the store
                 file_obj = File.from_local_path(local_file_path)
 
-                file_backend = (
-                    instance.workspace.default_file_store.get_backend_object()
+                file_backend = instance.workspace.scope.upload_file_backend(
+                    file_obj
                 )
                 file_backend.add_file(local_file_path, fileobj=file_obj)
 
@@ -595,3 +595,32 @@ class WorkflowFilterForm(Form):
         </select>
         """
         )
+
+
+class DaysField(IntegerField):
+    """Edit a timedelta as a number of days."""
+
+    def prepare_value(self, value: Any) -> Any:
+        """Convert a timedelta to days."""
+        if isinstance(value, timedelta):
+            return super().prepare_value(value.days)
+        return super().prepare_value(value)
+
+    def to_python(self, value: Any) -> Any:
+        """Convert days to timedelta."""
+        res = super().to_python(value)
+        if res is not None:
+            return timedelta(days=res)
+        return res
+
+
+class WorkspaceForm(BootstrapMixin, ModelFormBase[Workspace]):
+    """Form for configuring a workspace."""
+
+    class Meta:
+        model = Workspace
+        fields = ["public", "expiration_delay", "default_expiration_delay"]
+        field_classes = {
+            "expiration_delay": DaysField,
+            "default_expiration_delay": DaysField,
+        }
