@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -76,12 +76,12 @@ class LogoutTests(ViewTestMixin, TestCase):
         """
         _base.html do not include ?next=logout path.
 
-        URL for user:logout add is_logout_view, _base.html use the variable
+        URL for logout add is_logout_view, _base.html use the variable
         to not add ?next={next_path}
         """
-        response = self.client.post(reverse("user:logout"))
+        response = self.client.post(reverse("logout"))
 
-        next_path = reverse("user:logout")
+        next_path = reverse("logout")
         self.assertNotContains(response, f"?next={next_path}")
 
     def test_view_login_with_next(self) -> None:
