@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -16,6 +16,7 @@ from typing import Any, ClassVar
 from django import forms
 from django.contrib.auth import get_user_model
 from django.core.files.uploadedfile import SimpleUploadedFile
+from django.http import QueryDict
 from django.utils.datastructures import MultiValueDict
 
 try:
@@ -35,10 +36,12 @@ from debusine.test.django import TestCase
 from debusine.web.forms import (
     ArtifactForm,
     BootstrapMixin,
+    DaysField,
     TokenForm,
     WorkRequestForm,
     WorkflowFilterForm,
     WorkspaceChoiceField,
+    WorkspaceForm,
 )
 
 
@@ -243,9 +246,8 @@ class WorkRequestFormTests(TestCase):
         build_components:
         - any
         - all
-        distribution: stable
+        environment: debian/match:codename=bookworm
         host_architecture: amd64
-        backend: schroot
         input:
             source_artifact: 5
         """  # noqa: E501
@@ -352,7 +354,7 @@ class WorkRequestFormTests(TestCase):
         self.assertFalse(form.is_valid())
         self.assertRegex(
             str(form.errors["task_data"][0]),
-            'The backend "auto" requires "environment" to be set',
+            r'environment\s+field required \(type=value_error\.missing\)',
         )
 
     def test_clean_data_yaml_return_dictionary(self) -> None:
@@ -483,15 +485,15 @@ class ArtifactFormTests(TestCase):
         )
         self.assertEqual(artifact.expiration_delay, timedelta(days=1))
 
-        file_backend = self.workspace.default_file_store.get_backend_object()
-
         for file_in_artifact, file_uploaded in zip(
             artifact.fileinartifact_set.order_by("id"), files_to_upload
         ):
             self.assertEqual(file_in_artifact.path, file_uploaded)
             self.assertTrue(file_in_artifact.complete)
+            fileobj = file_in_artifact.file
+            file_backend = self.workspace.scope.upload_file_backend(fileobj)
 
-            with file_backend.get_stream(file_in_artifact.file) as file:
+            with file_backend.get_stream(fileobj) as file:
                 self.assertEqual(file.read(), files_to_upload[file_uploaded])
 
     def test_form_save_no_commit(self) -> None:
@@ -771,38 +773,42 @@ class WorkflowFilterFormTests(TestCase):
             ),
         )
 
-        self.playground.create_workflow(task_name="noop")
-        self.playground.create_workflow(task_name="sbuild", validate=False)
-
         self.form = WorkflowFilterForm()
 
         self.assertEqual(
             self.form.fields["workflow_templates"].widget.choices,
             [
-                ("noop-template", "noop-template"),
-                ("sbuild-template", "sbuild-template"),
+                ("name-1", "name-1"),
             ],
         )
 
     def test_statuses_field(self) -> None:
         """Test statuses field."""
-        self.assertEqual(
-            self.form.fields["statuses"].widget.choices,
-            sorted(WorkRequest.Statuses.choices),
+        choices = sorted(WorkRequest.Statuses.choices)
+        choices.remove(("running", "Running"))
+
+        choices.extend(
+            [
+                (f"running__{value}", f"running__{label}")
+                for value, label in WorkRequest.RuntimeStatuses.choices
+            ]
         )
+        choices.extend([("running__any", "running__Any")])
+
+        self.assertEqual(self.form.fields["statuses"].widget.choices, choices)
 
     def test_runtime_statuses_field(self) -> None:
         """Test runtime_statuses field."""
         self.assertEqual(
             self.form.fields["runtime_statuses"].widget.choices,
             [
-                ("Needs_Input", "Needs Input"),
-                ("Running", "Running"),
-                ("Waiting", "Waiting"),
-                ("Pending", "Pending"),
-                ("Aborted", "Aborted"),
-                ("Completed", "Completed"),
-                ("Blocked", "Blocked"),
+                ("needs_input", "Needs Input"),
+                ("running", "Running"),
+                ("waiting", "Waiting"),
+                ("pending", "Pending"),
+                ("aborted", "Aborted"),
+                ("completed", "Completed"),
+                ("blocked", "Blocked"),
             ],
         )
 
@@ -851,3 +857,92 @@ class WorkflowFilterFormTests(TestCase):
     def test_with_failed_work_requests_failed_field(self) -> None:
         """Test with_failed_work_requests field."""
         self.assertIn("with_failed_work_requests", self.form.fields)
+
+    def test_statuses_runtime_statuses(self) -> None:
+        """Test statuses_runtime_statuses() generated HTML."""
+        self.form.data = QueryDict(
+            "statuses=pending&statuses=running__needs_input"
+        )
+        self.assertHTMLEqual(
+            self.form.statuses_runtime_statuses(),
+            """
+            <select name="statuses" id="id_statuses" multiple>
+                <option value="pending" selected>Pending</option>
+                <optgroup label="Running"><option value="running__needs_input"
+                        selected>Needs Input</option>
+                    <option value="running__running">Running</option>
+                    <option value="running__waiting">Waiting</option>
+                    <option value="running__pending">Pending</option>
+                    <option value="running__aborted">Aborted</option>
+                    <option value="running__completed">Completed</option>
+                    <option value="running__blocked">Blocked</option>
+                    <option value="running__any">Any</option>
+                </optgroup>
+            <option value="completed">Completed</option>
+            <option value="aborted">Aborted</option>
+            <option value="blocked">Blocked</option>
+            </select>
+            """,
+        )
+
+
+class DaysFieldTest(TestCase):
+    """Tests for DaysField."""
+
+    def test_prepare_value(self) -> None:
+        """Test DaysField.prepare_value."""
+        field = DaysField()
+        for orig, prepared in [
+            ("0", "0"),
+            (1, 1),
+            (timedelta(days=4), 4),
+            (None, None),
+        ]:
+            with self.subTest(orig=orig):
+                self.assertEqual(field.prepare_value(orig), prepared)
+
+    def test_to_python(self) -> None:
+        """Test DaysField.to_python."""
+        field = DaysField()
+        for orig, py in [
+            ("0", timedelta(days=0)),
+            (1, timedelta(days=1)),
+            ("", None),
+            (None, None),
+        ]:
+            with self.subTest(orig=orig):
+                self.assertEqual(field.to_python(orig), py)
+
+
+class WorkspaceFormTests(TestCase):
+    """Tests for WorkspaceForm."""
+
+    scenario = scenarios.DefaultContext(set_current=True)
+
+    def test_values_from_instances(self) -> None:
+        """Test form field values."""
+        form = WorkspaceForm(instance=self.scenario.workspace)
+        self.assertTrue(form["public"].initial)
+        self.assertIsNone(form["expiration_delay"].initial)
+        self.assertEqual(
+            form["default_expiration_delay"].initial, timedelta(days=0)
+        )
+
+    def test_values_from_data(self) -> None:
+        """Test setting form field values."""
+        ws = self.scenario.workspace
+        form = WorkspaceForm(
+            data={
+                "public": "false",
+                "expiration_delay": "7",
+                "default_expiration_delay": "14",
+            },
+            instance=ws,
+        )
+        self.assertTrue(form.is_valid())
+        form.save()
+
+        ws.refresh_from_db()
+        self.assertFalse(ws.public)
+        self.assertEqual(ws.expiration_delay, timedelta(days=7))
+        self.assertEqual(ws.default_expiration_delay, timedelta(days=14))
