@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -33,4 +33,11 @@ urlpatterns = [
         "",
         include("debusine.web.urls.workspaces", namespace="workspaces"),
     ),
+    path(
+        "",
+        include(
+            "debusine.web.urls.workflow_templates",
+            namespace="workflow-templates",
+        ),
+    ),
 ]
