@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,10 +9,10 @@
 
 """URLs related to user (e.g. token handling)."""
 
-from django.urls import path
+from django.urls import URLPattern, URLResolver, include, path
 
 from debusine.web.views.auth import (
-    LogoutView,
+    UserDetailView,
     UserTokenCreateView,
     UserTokenDeleteView,
     UserTokenListView,
@@ -21,7 +21,7 @@ from debusine.web.views.auth import (
 
 app_name = "user"
 
-urlpatterns = [
+user_urlpatterns: list[URLPattern | URLResolver] = [
     path(
         "token/",
         UserTokenListView.as_view(),
@@ -43,11 +43,15 @@ urlpatterns = [
         name="token-delete",
     ),
     path(
-        "logout/",
-        LogoutView.as_view(
-            template_name="account/logged_out.html",
-            extra_context={"is_logout_view": True},
-        ),
-        name="logout",
+        "",
+        UserDetailView.as_view(),
+        name="detail",
+    ),
+]
+
+urlpatterns = [
+    path(
+        "<str:username>/",
+        include(user_urlpatterns),
     ),
 ]
