@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -16,7 +16,7 @@ from functools import cached_property
 from typing import Any, assert_never
 
 from django.core.exceptions import PermissionDenied
-from django.db.models import QuerySet
+from django.db.models import F, QuerySet
 from django.db.models.functions import Lower
 from django.http import Http404, HttpRequest, StreamingHttpResponse
 from django.http.response import HttpResponse, HttpResponseBase
@@ -165,8 +165,16 @@ class ArtifactDetailView(
         ctx["artifact_label"] = self.object.get_label()
         ctx["file_list"] = self.file_list
         ctx["artifact_data"] = format_yaml(self.object.data)
-        ctx["reverse_extends"] = self.object.targeted_by.filter(
-            type=ArtifactRelation.Relations.EXTENDS
+        ctx["forward"] = (
+            Artifact.objects.filter(targeted_by__artifact=self.object)
+            .annotate(relation_type=F("targeted_by__type"))
+            .annotate_complete()
+        )
+        ctx["reverse_extends"] = (
+            Artifact.objects.filter(relations__target=self.object)
+            .annotate(relation_type=F("relations__type"))
+            .filter(relation_type=ArtifactRelation.Relations.EXTENDS)
+            .annotate_complete()
         )
 
         if len(self.file_list) == 1:
@@ -206,7 +214,10 @@ class ArtifactDetailFileView(
 
     def get_title(self) -> str:
         """Return the page title."""
-        return self.object.path
+        title = self.object.path
+        if not self.object.complete:
+            title += " (incomplete)"
+        return title
 
     def get_main_ui_shortcuts(self) -> list[ui_shortcuts.UIShortcut]:
         """Return a list of UI shortcuts for this view."""
@@ -301,7 +312,7 @@ class DownloadPathView(PathMixin, FileDownloadMixin, BaseUIView):
         try:
             # Try to return a file
             file_in_artifact = self.artifact.fileinartifact_set.get(
-                path=self.path.rstrip("/")
+                path=self.path.rstrip("/"), complete=True
             )
             return self._get_file(file_in_artifact)
         except FileInArtifact.DoesNotExist:
@@ -310,7 +321,7 @@ class DownloadPathView(PathMixin, FileDownloadMixin, BaseUIView):
 
         # Try to return a .tar.gz / list of files for the directory
         directory_exists = self.artifact.fileinartifact_set.filter(
-            path__startswith=self.path
+            path__startswith=self.path, complete=True
         ).exists()
         if directory_exists:
             return self._get_directory()
@@ -327,8 +338,11 @@ class DownloadPathView(PathMixin, FileDownloadMixin, BaseUIView):
     def _get_artifact(self) -> HttpResponseBase:
         match self.download_format:
             case DownloadFormat.AUTO:
-                if self.artifact.fileinartifact_set.count() == 1:
-                    file_in_artifact = self.artifact.fileinartifact_set.first()
+                files_in_artifact = self.artifact.fileinartifact_set.filter(
+                    complete=True
+                )
+                if files_in_artifact.count() == 1:
+                    file_in_artifact = files_in_artifact.first()
                     assert file_in_artifact is not None
                     return self._get_file(file_in_artifact)
                 else:
