@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -30,6 +30,7 @@ from django.views.generic.base import ContextMixin, View
 from django.views.generic.detail import SingleObjectMixin
 from django.views.generic.edit import FormMixin
 
+from debusine.artifacts.models import CollectionCategory
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import Scope, Workspace
 from debusine.db.models.permissions import (
@@ -156,7 +157,14 @@ class BaseUIView(ContextMixin, View):
                 .order_by("name")
             )
             ctx["workspace_collections"] = (
-                context.workspace.collections.can_display(context.user)
+                context.workspace.collections.exclude(
+                    category=CollectionCategory.WORKFLOW_INTERNAL
+                ).can_display(context.user)
+            )
+            ctx["workflow_templates"] = list(
+                context.workspace.workflowtemplate_set.can_display(
+                    context.user
+                ).order_by("name")
             )
         try:
             ctx["debusine_version"] = version("debusine")
