@@ -161,8 +161,8 @@ class FileWidget(Widget):
     @contextlib.contextmanager
     def _open_file(self) -> Generator["SupportsRead[bytes]"]:
         assert self.file_in_artifact is not None
-        workspace = self.file_in_artifact.artifact.workspace
-        file_backend = workspace.default_file_store.get_backend_object()
+        scope = self.file_in_artifact.artifact.workspace.scope
+        file_backend = scope.download_file_backend(self.file_in_artifact.file)
         with file_backend.get_stream(self.file_in_artifact.file) as stream:
             yield stream
 
@@ -264,10 +264,11 @@ class FileDownloadMixin(View):
             # by end users directly
             return ProblemResponse(str(exc))
 
-        workspace = file_in_artifact.artifact.workspace
-        file_backend = workspace.default_file_store.get_backend_object()
+        scope = file_in_artifact.artifact.workspace.scope
+        file_backend = scope.download_file_backend(file_in_artifact.file)
+        entry = file_backend.get_entry(file_in_artifact.file)
 
-        url = file_backend.get_url(file_in_artifact.file)
+        url = entry.get_url(content_type=ui_info.content_type)
         if url is not None:
             # The client can download the file from the backend
             # TODO: this does not allow to set content-disposition, that is, to
@@ -275,7 +276,7 @@ class FileDownloadMixin(View):
             # be solved.
             return redirect(url)
 
-        with file_backend.get_stream(file_in_artifact.file) as file:
+        with entry.get_stream() as file:
             file_size = file_in_artifact.file.size
             status_code: int
             if content_range is None:
