@@ -50,7 +50,8 @@ class LintianView(WorkRequestPlugin):
     def task(self) -> Lintian:
         """Return the task to display."""
         return Lintian(
-            self.work_request.task_data, self.work_request.dynamic_task_data
+            self.work_request.used_task_data,
+            self.work_request.dynamic_task_data,
         )
 
     @classmethod
@@ -158,10 +159,11 @@ class LintianView(WorkRequestPlugin):
             except FileInArtifact.DoesNotExist:
                 continue
 
-            workspace = artifact.workspace
-            file_backend = workspace.default_file_store.get_backend_object()
+            scope = artifact.workspace.scope
+            fileobj = analysis_in_artifact.file
+            file_backend = scope.download_file_backend(fileobj)
 
-            with file_backend.get_stream(analysis_in_artifact.file) as file:
+            with file_backend.get_stream(fileobj) as file:
                 analysis = json.loads(file.read())
 
             analyses.append(analysis)
