@@ -40,7 +40,6 @@ from debusine.db.models import (
     Workspace,
     default_workspace,
 )
-from debusine.server.file_backend.local import LocalFileBackend
 from debusine.test.django import (
     AllowAll,
     DenyAll,
@@ -135,7 +134,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             create_files=True,
         )
         cls.playground.create_group_role(
-            cls.artifact.workspace, Workspace.Roles.OWNER, cls.user
+            cls.artifact.workspace, Workspace.Roles.OWNER, users=[cls.user]
         )
 
     def permission_tests_get(
@@ -571,7 +570,7 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
         )
         cls.file = cls.artifact.fileinartifact_set.get(path="file.md")
         cls.playground.create_group_role(
-            cls.artifact.workspace, Workspace.Roles.OWNER, cls.user
+            cls.artifact.workspace, Workspace.Roles.OWNER, users=[cls.user]
         )
 
     def permission_tests_get(
@@ -741,7 +740,7 @@ class ArtifactDetailFileViewRawTests(ViewTestMixin, PermissionTests):
         )
         cls.file = cls.artifact.fileinartifact_set.get(path="file.md")
         cls.playground.create_group_role(
-            cls.artifact.workspace, Workspace.Roles.OWNER, cls.user
+            cls.artifact.workspace, Workspace.Roles.OWNER, users=[cls.user]
         )
 
     def permission_tests_get(
@@ -887,7 +886,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
             cls.tree_paths, create_files=True
         )
         cls.playground.create_group_role(
-            cls.artifact.workspace, Workspace.Roles.OWNER, cls.user
+            cls.artifact.workspace, Workspace.Roles.OWNER, users=[cls.user]
         )
 
     def permission_tests_get(
@@ -1262,7 +1261,7 @@ class DownloadPathViewAuthTests(TestCase):
     def make_owner(self) -> None:
         """Make the user owner of the workspace."""
         self.playground.create_group_role(
-            self.artifact.workspace, Workspace.Roles.OWNER, self.user
+            self.artifact.workspace, Workspace.Roles.OWNER, users=[self.user]
         )
 
     def get(
@@ -1449,12 +1448,13 @@ class CreateArtifactViewTests(TestCase):
         # Verify uploaded files
         self.assertEqual(artifact.fileinartifact_set.count(), len(files))
 
-        local_file_backend = LocalFileBackend(workspace.default_file_store)
-
         for file_in_artifact, file_to_upload in zip(
             artifact.fileinartifact_set.all().order_by("id"), files
         ):
-            with local_file_backend.get_stream(file_in_artifact.file) as file:
+            file_backend = workspace.scope.download_file_backend(
+                file_in_artifact.file
+            )
+            with file_backend.get_stream(file_in_artifact.file) as file:
                 assert file_to_upload.file is not None
                 file_to_upload.file.seek(0)
                 content = file_to_upload.file.read()
