@@ -1,4 +1,4 @@
-# Copyright 2022-2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,13 +15,15 @@ from typing import ClassVar
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.template.response import SimpleTemplateResponse
+from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
 from debusine.db.models import Token, User
+from debusine.db.models.auth import Identity
 from debusine.db.playground import scenarios
+from debusine.server.signon.providers import Provider
 from debusine.test.django import TestCase
-from debusine.web.views.auth import UserTokenDeleteView
 from debusine.web.views.tests.utils import (
     ViewTestMixin,
     date_format,
@@ -32,6 +34,26 @@ from debusine.web.views.tests.utils import (
 class LoginViewTests(TestCase):
     """Tests for the LoginView class."""
 
+    def test_get(self) -> None:
+        """Test plain get, no next= argument."""
+        response = self.client.get(reverse("login"))
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(self.client.session["sso_callback_next_url"], "/")
+
+    def test_get_valid_next(self) -> None:
+        """Test plain get, valid next= argument."""
+        response = self.client.get(reverse("login") + "?next=/foo")
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(self.client.session["sso_callback_next_url"], "/foo")
+
+    def test_get_invalid_next(self) -> None:
+        """Test plain get, invalid next= argument."""
+        response = self.client.get(
+            reverse("login") + "?next=https://example.com"
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(self.client.session["sso_callback_next_url"], "/")
+
     def test_successful_login(self) -> None:
         """Successful login redirect to the homepage with authenticated user."""
         username = "testuser"
@@ -49,9 +71,17 @@ class LoginViewTests(TestCase):
         self.assertRedirects(response, reverse("homepage:homepage"))
         self.assertTrue(response.wsgi_request.user.is_authenticated)
 
+    def test_nexturl_in_session(self) -> None:
+        """Login leaves the next url in the session."""
+        response = self.client.get(
+            reverse("login") + "?next=foo",
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(self.client.session["sso_callback_next_url"], "foo")
+
 
 class LogoutViewTests(TestCase):
-    """Tests for the user:logout view."""
+    """Tests for the logout view."""
 
     def test_successful_render(self) -> None:
         """
@@ -61,11 +91,143 @@ class LogoutViewTests(TestCase):
         is implemented by Debusine, and it's rendered here to assert that
         no missing template variables and contains the expected strings.
         """
-        response = self.client.post(reverse("user:logout"))
+        response = self.client.post(reverse("logout"))
 
         self.assertEqual(response.status_code, status.HTTP_200_OK)
 
 
+class UserDetailViewTests(ViewTestMixin, TestCase):
+    """Test for UserDetailView class."""
+
+    scenario = scenarios.DefaultScopeUser()
+
+    def test_anonymous(self) -> None:
+        """Test accessing as anonymous user."""
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+        self.assertEqual(
+            response.context["exception"], "No User matches the given query."
+        )
+
+    def test_detail(self) -> None:
+        """Test viewing the default user."""
+        self.client.force_login(self.scenario.user)
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+
+        dl = self.assertHasElement(tree, "//dl[@id='person-info']")
+        self.assertTextContentEqual(
+            dl.dd[0], self.scenario.user.get_full_name()
+        )
+        self.assertTextContentEqual(dl.dd[1], self.scenario.user.username)
+        self.assertTextContentEqual(
+            dl.dd[2], str(self.scenario.user.token_set.count())
+        )
+
+        self.assertFalse(tree.xpath("//table[@id='person-groups']"))
+        self.assertFalse(tree.xpath("//table[@id='person-identities']"))
+
+    def test_detail_groups(self) -> None:
+        """Test viewing a user with groups."""
+        self.client.force_login(self.scenario.user)
+        debusine_groups = ("foo", "bar", "baz")
+        another_groups = ("wobble", "wibble", "wabble")
+
+        for name in debusine_groups:
+            self.playground.create_group(
+                name=name, scope=self.scenario.scope, users=[self.scenario.user]
+            )
+
+        scope = self.playground.get_or_create_scope("another")
+        for name in another_groups:
+            self.playground.create_group(
+                name=name, scope=scope, users=[self.scenario.user]
+            )
+
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+
+        table = self.assertHasElement(tree, "//table[@id='person-groups']")
+        tbody = self.assertHasElement(table, "tbody")
+        self.assertTextContentEqual(tbody.tr[0].td[0], "another")
+        for li, name in zip(tbody.tr[0].td[1].ul.li, sorted(another_groups)):
+            self.assertTextContentEqual(li, name)
+        self.assertTextContentEqual(tbody.tr[1].td[0], "debusine")
+        for li, name in zip(tbody.tr[1].td[1].ul.li, sorted(debusine_groups)):
+            self.assertTextContentEqual(li, name)
+
+    @override_settings(SIGNON_PROVIDERS=[Provider(name="salsa", label="Salsa")])
+    def test_detail_identities(self) -> None:
+        """Test viewing a user with SSO identities."""
+        self.client.force_login(self.scenario.user)
+
+        identity = Identity.objects.create(
+            user=self.scenario.user,
+            issuer="salsa",
+            subject="playground@debian.org",
+        )
+
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+
+        table = self.assertHasElement(tree, "//table[@id='person-identities']")
+        tr = self.assertHasElement(table, "tbody/tr")
+        self.assertTextContentEqual(tr.td[0], "Salsa")
+        self.assertTextContentEqual(tr.td[1], "playground@debian.org")
+        self.assertTextContentEqual(
+            tr.td[2], identity.last_used.strftime("%Y-%m-%d %H:%M")
+        )
+
+    @override_settings(SIGNON_PROVIDERS=[Provider(name="salsa", label="Salsa")])
+    def test_detail_identities_cannot_manage(self) -> None:
+        """Test viewing another user with SSO identities."""
+        self.client.force_login(self.scenario.user)
+
+        user = self.playground.create_user("test")
+        Identity.objects.create(
+            user=user,
+            issuer="salsa",
+            subject="playground@debian.org",
+        )
+
+        response = self.client.get(user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        self.assertFalse(tree.xpath("//table[@id='person-identities']"))
+
+    def test_detail_identities_without_provider(self) -> None:
+        """Test showing a user with identities without a SSO provider."""
+        self.client.force_login(self.scenario.user)
+
+        Identity.objects.create(
+            user=self.scenario.user,
+            issuer="salsa",
+            subject="playground@debian.org",
+        )
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        self.assertFalse(tree.xpath("//table[@id='person-identities']"))
+
+    @override_settings(SIGNON_PROVIDERS=[Provider(name="salsa", label="Salsa")])
+    def test_detail_identities_with_wrong_provider(self) -> None:
+        """Test showing a user with identities of nonexisting SSO provider."""
+        self.client.force_login(self.scenario.user)
+
+        Identity.objects.create(
+            user=self.scenario.user,
+            issuer="nachos",
+            subject="playground@debian.org",
+        )
+        response = self.client.get(self.scenario.user.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        self.assertFalse(tree.xpath("//table[@id='person-identities']"))
+
+
 class UserTokenListViewTests(ViewTestMixin, TestCase):
     """Tests for UserTokenListView class."""
 
@@ -74,7 +236,12 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
     def test_only_one_scope(self) -> None:
         """Test hiding the scopes list if there is only one."""
         self.client.force_login(self.scenario.user)
-        response = self.client.get(reverse("user:token-list"))
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
         tree = self.assertHTMLValid(response)
         sample_cfg = self.assertHasElement(tree, "//pre[@id='example-config']")
         fqdn = settings.DEBUSINE_FQDN
@@ -96,7 +263,12 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         """Test when there is more than one scope."""
         self.playground.get_or_create_scope("test")
         self.client.force_login(self.scenario.user)
-        response = self.client.get(reverse("user:token-list"))
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
         tree = self.assertHTMLValid(response)
         sample_cfg = self.assertHasElement(tree, "//pre[@id='example-config']")
         fqdn = settings.DEBUSINE_FQDN
@@ -124,7 +296,12 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         another_token = Token.objects.create(user=another_user)
 
         self.client.force_login(self.scenario.user)
-        response = self.client.get(reverse("user:token-list"))
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
         tree = self.assertHTMLValid(response)
         self.assertNavHasUser(tree, self.scenario.user)
 
@@ -133,7 +310,10 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
             "You do not currently have any tokens",
         )
 
-        create_token_url = reverse("user:token-create")
+        create_token_url = reverse(
+            "user:token-create",
+            kwargs={"username": self.scenario.user.username},
+        )
         self.assertContains(
             response,
             f'<a href="{create_token_url}">create a new token</a>',
@@ -143,11 +323,16 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         # Ensure that a token for another user is not displayed
         self.assertNotContains(response, another_token.key)
 
-    @staticmethod
-    def row_for_token(token: Token) -> str:
+    def row_for_token(self, token: Token) -> str:
         """Return HTML for the row of the table for token."""
-        edit_url = reverse("user:token-edit", kwargs={"pk": token.pk})
-        delete_url = reverse("user:token-delete", kwargs={"pk": token.pk})
+        edit_url = reverse(
+            "user:token-edit",
+            kwargs={"pk": token.pk, "username": self.scenario.user.username},
+        )
+        delete_url = reverse(
+            "user:token-delete",
+            kwargs={"pk": token.pk, "username": self.scenario.user.username},
+        )
 
         return (
             "<tr>"
@@ -173,7 +358,12 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         )
 
         self.client.force_login(self.scenario.user)
-        response = self.client.get(reverse("user:token-list"))
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
 
         self.assertContains(response, self.row_for_token(token1), html=True)
         self.assertContains(response, self.row_for_token(token2), html=True)
@@ -190,7 +380,12 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         token1.created_at = token2.created_at + timedelta(minutes=1)
         token1.save()
 
-        response = self.client.get(reverse("user:token-list"))
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
 
         # Tokens appear in the opposite order than before
         self.assertGreater(
@@ -200,13 +395,15 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
 
     def test_no_logged(self) -> None:
         """Request with a non-logged user: does not return any token."""
-        response = self.client.get(reverse("user:token-list"))
-
-        self.assertContains(
-            response,
-            "You need to be authenticated to list tokens",
-            status_code=status.HTTP_403_FORBIDDEN,
-            html=True,
+        response = self.client.get(
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            )
+        )
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+        self.assertEqual(
+            response.context["exception"], "No User matches the given query."
         )
 
 
@@ -228,7 +425,11 @@ class UserTokenCreateViewTests(TestCase):
         self.client.force_login(self.user)
         comment = "Test 1"
         response = self.client.post(
-            reverse("user:token-create"), {"comment": comment}
+            reverse(
+                "user:token-create",
+                kwargs={"username": self.user.username},
+            ),
+            {"comment": comment},
         )
 
         # The token got created...
@@ -256,23 +457,30 @@ class UserTokenCreateViewTests(TestCase):
 
     def test_no_logged(self) -> None:
         """Request with a non-logged user: cannot create a token."""
-        response = self.client.get(reverse("user:token-create"))
-
-        self.assertContains(
-            response,
-            "You need to be authenticated to create a token",
-            status_code=status.HTTP_403_FORBIDDEN,
-            html=True,
+        response = self.client.get(
+            reverse(
+                "user:token-create",
+                kwargs={"username": self.user.username},
+            )
+        )
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+        self.assertEqual(
+            response.context["exception"], "No User matches the given query."
         )
 
     def test_initial_form(self) -> None:
         """Get request to ensure the form is displayed."""
         self.client.force_login(self.user)
 
-        response = self.client.get(reverse("user:token-create"))
+        response = self.client.get(
+            reverse(
+                "user:token-create",
+                kwargs={"username": self.user.username},
+            )
+        )
 
         self.assertContains(
-            response, "<title>Debusine - Create a token</title>", html=True
+            response, "<title>Debusine - Create token</title>", html=True
         )
         self.assertContains(response, "<h1>Create token</h1>", html=True)
         self.assertContains(
@@ -297,11 +505,23 @@ class UserTokenUpdateViewTests(TestCase):
         comment = "This is a token"
 
         response = self.client.post(
-            reverse("user:token-edit", kwargs={"pk": token.pk}),
+            reverse(
+                "user:token-edit",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            ),
             {"comment": comment, "enabled": enabled},
         )
 
-        self.assertRedirects(response, reverse("user:token-list"))
+        self.assertRedirects(
+            response,
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            ),
+        )
 
         token.refresh_from_db()
 
@@ -319,11 +539,21 @@ class UserTokenUpdateViewTests(TestCase):
 
         self.client.force_login(user2)
         response = self.client.get(
-            reverse("user:token-edit", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-edit",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
+        )
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.context["exception"],
+            "testuser2 cannot manage user playground",
         )
-
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
+            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
         )
 
     def test_put_404_not_found(self) -> None:
@@ -339,12 +569,22 @@ class UserTokenUpdateViewTests(TestCase):
         comment = "This is the new comment"
         enabled = False
         response = self.client.put(
-            reverse("user:token-edit", kwargs={"pk": token.pk}),
+            reverse(
+                "user:token-edit",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            ),
             {"comment": comment, "enabled": enabled},
         )
-
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.context["exception"],
+            "testuser2 cannot manage user playground",
+        )
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
+            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
         )
 
     def test_initial_form(self) -> None:
@@ -353,11 +593,17 @@ class UserTokenUpdateViewTests(TestCase):
         self.client.force_login(self.scenario.user)
 
         response = self.client.get(
-            reverse("user:token-edit", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-edit",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
         )
 
         self.assertContains(
-            response, "<title>Debusine - Edit a token</title>", html=True
+            response, "<title>Debusine - Edit token</title>", html=True
         )
         self.assertContains(response, "<h1>Edit token</h1>", html=True)
         self.assertContains(
@@ -383,7 +629,13 @@ class UserTokenDeleteViewTests(TestCase):
         self.client.force_login(self.scenario.user)
 
         response = self.client.get(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
         )
 
         expected_contents = (
@@ -402,10 +654,22 @@ class UserTokenDeleteViewTests(TestCase):
         self.client.force_login(self.scenario.user)
 
         response = self.client.post(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
         )
 
-        self.assertRedirects(response, reverse("user:token-list"))
+        self.assertRedirects(
+            response,
+            reverse(
+                "user:token-list",
+                kwargs={"username": self.scenario.user.username},
+            ),
+        )
 
         # The token was deleted
         self.assertEqual(Token.objects.filter(pk=token.pk).count(), 0)
@@ -418,29 +682,37 @@ class UserTokenDeleteViewTests(TestCase):
         """
         token = self.scenario.user_token
         response = self.client.get(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
         )
-
-        self.assertContains(
-            response,
-            UserTokenDeleteView.permission_denied_message,
-            status_code=status.HTTP_403_FORBIDDEN,
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+        self.assertEqual(
+            response.context["exception"], "No User matches the given query."
         )
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
+            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
         )
 
         response = self.client.post(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
         )
-
-        self.assertContains(
-            response,
-            UserTokenDeleteView.permission_denied_message,
-            status_code=status.HTTP_403_FORBIDDEN,
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+        self.assertEqual(
+            response.context["exception"], "No User matches the given query."
         )
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
+            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
         )
 
     def test_get_post_delete_token_another_user(self) -> None:
@@ -459,21 +731,39 @@ class UserTokenDeleteViewTests(TestCase):
         self.client.force_login(user2)
 
         response = self.client.get(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
+        )
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.context["exception"],
+            "testuser2 cannot manage user playground",
         )
-
-        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
+            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
         )
 
-        response = self.client.get(
-            reverse("user:token-delete", kwargs={"pk": token.pk})
+        response = self.client.post(
+            reverse(
+                "user:token-delete",
+                kwargs={
+                    "pk": token.pk,
+                    "username": self.scenario.user.username,
+                },
+            )
+        )
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.context["exception"],
+            "testuser2 cannot manage user playground",
         )
-
-        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
         self.assertNotContains(
-            response, token.hash, status_code=status.HTTP_404_NOT_FOUND
+            response, token.hash, status_code=status.HTTP_403_FORBIDDEN
         )
 
         self.assertEqual(Token.objects.filter(pk=token.pk).count(), 1)
