@@ -94,7 +94,7 @@ class CollectionViewsTests(ViewTestMixin, TestCase):
         self.playground.create_group_role(
             self.workspace_private,
             Workspace.Roles.OWNER,
-            self.playground.get_default_user(),
+            users=[self.playground.get_default_user()],
         )
         response = self.client.get(
             reverse("workspaces:collections:list", kwargs={"wname": "Private"})
@@ -189,7 +189,7 @@ class CollectionViewsTests(ViewTestMixin, TestCase):
         self.playground.create_group_role(
             self.workspace_private,
             Workspace.Roles.OWNER,
-            self.playground.get_default_user(),
+            users=[self.playground.get_default_user()],
         )
         response = self.client.get(
             reverse(
