@@ -23,8 +23,7 @@ from django.utils.datastructures import CaseInsensitiveMapping
 from rest_framework import status
 
 from debusine.db.context import context
-from debusine.db.models import File, FileInArtifact, FileStore
-from debusine.server.file_backend.interface import FileBackendInterface
+from debusine.db.models import File, FileInArtifact
 from debusine.server.views import ProblemResponse
 from debusine.test.django import TestCase
 from debusine.web.views import files
@@ -358,19 +357,19 @@ class FileDownloadMixinTests(ViewTestMixin, TestCase):
         """
         destination_url = "https://some-backend.net/file?token=asdf"
 
-        file_backend_mocked = mock.create_autospec(spec=FileBackendInterface)
-        file_backend_mocked.get_url.return_value = destination_url
-
-        with mock.patch.object(
-            FileStore,
-            "get_backend_object",
+        with mock.patch(
+            "debusine.server.file_backend.local.LocalFileBackendEntry.get_url",
             autospec=True,
-            return_value=file_backend_mocked,
-        ):
+            return_value=destination_url,
+        ) as mock_get_url:
             response = self.get_stream_response(self.file)
             self.assertEqual(response.status_code, status.HTTP_302_FOUND)
             assert isinstance(response, HttpResponseRedirectBase)
             self.assertEqual(response.url, destination_url)
+            ui_info = FileUI.from_file_in_artifact(self.file)
+            mock_get_url.assert_called_once_with(
+                mock.ANY, content_type=ui_info.content_type
+            )
 
 
 class FileWidgetTests(ViewTestMixin, TestCase):
