@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -113,7 +113,7 @@ class ScopeDetailViewTests(ViewTestMixin, TestCase):
         self.playground.create_group_role(
             self.private_workspace,
             Workspace.Roles.CONTRIBUTOR,
-            self.scenario.user,
+            users=[self.scenario.user],
         )
         self.client.force_login(self.scenario.user)
         response = self.client.get(reverse("scopes:detail"))
