@@ -12,7 +12,7 @@
 import textwrap
 from datetime import timedelta
 from types import GeneratorType
-from typing import ClassVar
+from typing import Any, ClassVar
 
 import lxml
 import yaml
@@ -91,11 +91,10 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         )
         cls.work_request = cls.playground.create_sbuild_work_request(
             source=cls.source,
-            architecture="all",
+            architecture="amd64",
             environment=environment,
             workspace=cls.scenario.workspace,
         )
-        cls.playground.compute_dynamic_data(cls.work_request)
         cls.work_request.mark_running()
         cls.work_request.assign_worker(cls.playground.create_worker())
         cls.work_request.mark_completed(WorkRequest.Results.SUCCESS)
@@ -142,6 +141,30 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
                     else:
                         self.assertTextContentEqual(tr.td[1], label)
 
+    def assertHasMetadata(self, tree: lxml.objectify.ObjectifiedElement) -> Any:
+        """Ensure that metadata is shown in the page, and return it parsed."""
+        metadata = self.assertHasElement(tree, "//div[@id='metadata']")
+        if metadata.xpath("div/ul"):
+            # Orig/configured
+            return (
+                yaml.safe_load(
+                    "".join(
+                        metadata.xpath("//div[@id='task_data_configured']")[
+                            0
+                        ].itertext()
+                    )
+                ),
+                yaml.safe_load(
+                    "".join(
+                        metadata.xpath("//div[@id='task_data_original']")[
+                            0
+                        ].itertext()
+                    )
+                ),
+            )
+        else:
+            return yaml.safe_load("".join(metadata.div[1].itertext()))
+
     def test_get_work_request(self) -> None:
         """View detail return work request information."""
         with context.disable_permission_checks():
@@ -155,7 +178,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         tree = self.assertHTMLValid(response)
 
         sidebar = response.context["sidebar_items"]
-        self.assertEqual(sidebar[0].value, "build a package")
+        self.assertEqual(sidebar[0].value, "build hello")
         self.assertEqual(
             sidebar[0].detail,
             f'{self.work_request.task_type} {self.work_request.task_name} task',
@@ -182,6 +205,29 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
             response, "<h2>Workflow information</h2>", html=True
         )
 
+        metadata = self.assertHasMetadata(tree)
+        self.assertEqual(metadata["host_architecture"], "amd64")
+        self.assertNotIn("task_data_original", response.context)
+        self.assertFalse(response.context["task_data_configured"])
+
+    def test_get_configured_work_request(self) -> None:
+        """View detail return work request information."""
+        # Simulate a configuration step
+        self.work_request.configured_task_data = (
+            self.work_request.task_data.copy()
+        )
+        self.work_request.configured_task_data["host_architecture"] = "arm64"
+        self.work_request.save()
+
+        response = self.client.get(self.work_request.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertIn("task_data_original", response.context)
+        self.assertTrue(response.context["task_data_configured"])
+        tree = self.assertHTMLValid(response)
+        configured, orig = self.assertHasMetadata(tree)
+        self.assertEqual(configured["host_architecture"], "arm64")
+        self.assertEqual(orig["host_architecture"], "amd64")
+
     def test_get_work_request_incomplete(self) -> None:
         """Artifacts with incomplete files are marked as incomplete."""
         artifact, _ = self.playground.create_artifact(
@@ -215,7 +261,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         """Work requests in private workspaces 200 to the authenticated."""
         private_workspace = self.playground.create_workspace(name="Private")
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
         self.work_request.workspace = private_workspace
         self.work_request.save()
@@ -804,7 +850,7 @@ class WorkRequestListViewTests(ViewTestMixin, TestCase):
         )
 
         self.playground.create_group_role(
-            private_workspace, Workspace.Roles.OWNER, self.scenario.user
+            private_workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
         )
 
         self.client.force_login(self.scenario.user)
@@ -1130,7 +1176,8 @@ class WorkRequestCreateViewTests(ViewTestMixin, TestCase):
 
     def test_create_work_request(self) -> None:
         """Post to "work_requests:create" to create a work request."""
-        source_artifact, _ = self.playground.create_artifact()
+        self.playground.create_debian_environment(codename="bookworm")
+        source_artifact = self.playground.create_source_artifact()
         self.client.force_login(self.scenario.user)
         workspace = self.scenario.workspace
         name = "sbuild"
@@ -1140,8 +1187,7 @@ class WorkRequestCreateViewTests(ViewTestMixin, TestCase):
         build_components:
         - any
         - all
-        backend: schroot
-        distribution: stable
+        environment: debian/match:codename=bookworm
         host_architecture: amd64
         input:
           source_artifact: {source_artifact.id}
