@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -244,29 +244,52 @@ class WorkersListViewTests(ViewTestMixin, TestCase):
         # Status is "-"
         self.assertHTMLContentsEquivalent(table.tbody.tr[0].td[2], "<td>-</td>")
 
+    def test_get_ordering(self) -> None:
+        """The workers list can be ordered."""
+        for field in (
+            "name",
+            "worker_type",
+            "registered_at",
+            "last_seen_at",
+            "enabled",
+        ):
+            for asc in ["0", "1"]:
+                response = self.client.get(
+                    f"/-/status/workers/?order={field}&asc={asc}"
+                )
+
+            self.assertEqual(response.context["order"], field)
+            self.assertEqual(response.context["asc"], asc)
+
     def test_get_ordering_ascending(self) -> None:
         """WorkersListView.get_ordering return correct ordering: "name"."""
-        request = self.factory.get("/workers", {"order": "name", "asc": "1"})
+        request = self.factory.get(
+            "/-/status/workers/", {"order": "name", "asc": "1"}
+        )
 
         view = WorkersListView()
         view.request = request
 
-        self.assertEqual(view.get_ordering(), "name")
+        self.assertEqual(view.get_ordering(), ["name"])
 
     def test_get_ordering_descending(self) -> None:
         """WorkersListView.get_ordering return correct ordering: "-name"."""
-        request = self.factory.get("/workers", {"order": "name", "asc": "0"})
+        request = self.factory.get(
+            "/-/status/workers/", {"order": "name", "asc": "0"}
+        )
 
         view = WorkersListView()
         view.request = request
 
-        self.assertEqual(view.get_ordering(), "-name")
+        self.assertEqual(view.get_ordering(), ["-name"])
 
     def test_get_ordering_not_in_list(self) -> None:
         """WorkersListView.get_ordering return correct: "-name"."""
-        request = self.factory.get("/workers", {"order": "a_field", "asc": "1"})
+        request = self.factory.get(
+            "/-/status/workers/", {"order": "a_field", "asc": "1"}
+        )
 
         view = WorkersListView()
         view.request = request
 
-        self.assertEqual(view.get_ordering(), "name")
+        self.assertEqual(view.get_ordering(), ["name"])
