@@ -9,8 +9,10 @@
 
 """Tests for the workspace views."""
 
+from datetime import timedelta
 from typing import ClassVar
 
+from django.template.response import SimpleTemplateResponse
 from rest_framework import status
 
 from debusine.artifacts.models import CollectionCategory
@@ -26,8 +28,8 @@ from debusine.test.django import (
 from debusine.web.views.tests.utils import ViewTestMixin
 
 
-class WorkspaceViewsTests(ViewTestMixin, TestCase):
-    """Tests for Workspace views."""
+class WorkspaceDetailTests(ViewTestMixin, TestCase):
+    """Tests for WorkspaceDetail view."""
 
     scenario = scenarios.DefaultScopeUser()
 
@@ -59,15 +61,58 @@ class WorkspaceViewsTests(ViewTestMixin, TestCase):
         """Test workspace detail view."""
         response = self.client.get(self.workspace_public.get_absolute_url())
         self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertHTMLValid(response)
+        tree = self.assertHTMLValid(response)
+        details = self.assertHasElement(
+            tree, "//table[@id='workspace-details']"
+        )
+        self.assertHasElement(details, "//tr[@id='workspace-details-public']")
+        self.assertHasElement(
+            details, "//tr[@id='workspace-details-default-expiration']"
+        )
 
     def test_detail_permissions(self) -> None:
-        """Test permissions on workspace list view."""
+        """Test permissions."""
         self.assertSetsCurrentWorkspace(
             self.workspace_public,
             self.workspace_public.get_absolute_url(),
         )
 
+    def test_detail_no_expiration(self) -> None:
+        """No expiration is shown on non-expiring workspaces."""
+        self.client.force_login(self.scenario.user)
+        response = self.client.get(self.workspace_public.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        details = self.assertHasElement(
+            tree, "//table[@id='workspace-details']"
+        )
+        self.assertFalse(
+            details.xpath("//tr[@id='workspace-details-expiration']")
+        )
+
+    def test_detail_expiration(self) -> None:
+        """Expiration is shown on expiring workspaces."""
+        self.workspace_public.expiration_delay = timedelta(days=7)
+        self.workspace_public.save()
+
+        self.client.force_login(self.scenario.user)
+        response = self.client.get(self.workspace_public.get_absolute_url())
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertHTMLValid(response)
+        details = self.assertHasElement(
+            tree, "//table[@id='workspace-details']"
+        )
+        tr = self.assertHasElement(
+            details, "//tr[@id='workspace-details-expiration']"
+        )
+        self.assertTextContentEqual(tr.th, "Expires")
+        self.assertTextContentEqual(
+            tr.td,
+            (self.workspace_public.created_at + timedelta(days=7)).strftime(
+                "%Y-%m-%d"
+            ),
+        )
+
     def test_detail_can_create_artifact(self) -> None:
         """User is logged in and can create artifacts."""
         self.client.force_login(self.scenario.user)
@@ -108,7 +153,7 @@ class WorkspaceViewsTests(ViewTestMixin, TestCase):
         self.playground.create_group_role(
             self.workspace_private,
             Workspace.Roles.OWNER,
-            self.playground.get_default_user(),
+            users=[self.playground.get_default_user()],
         )
         response = self.client.get(self.workspace_private.get_absolute_url())
         self.assertEqual(response.status_code, status.HTTP_200_OK)
@@ -190,3 +235,66 @@ class WorkspaceViewsTests(ViewTestMixin, TestCase):
             template.get_absolute_url(),
         )
         self.assertEqual(workflow_templates.tbody.tr[0].td[1], "noop")
+
+
+class WorkspaceUpdateTests(ViewTestMixin, TestCase):
+    """Tests for WorkspaceUpdate view."""
+
+    scenario = scenarios.DefaultContext()
+
+    def test_permissions(self) -> None:
+        """Test permissions."""
+        self.assertSetsCurrentWorkspace(
+            self.scenario.workspace,
+            self.scenario.workspace.get_absolute_url_configure(),
+        )
+        self.assertEnforcesPermission(
+            self.scenario.workspace.can_configure,
+            self.scenario.workspace.get_absolute_url_configure(),
+            "get_context_data",
+        )
+
+    @override_permission(Workspace, "can_configure", AllowAll)
+    def test_configure(self) -> None:
+        """Test changing workspace configuration."""
+        ws = self.scenario.workspace
+        self.playground.create_group_role(
+            ws, Workspace.Roles.OWNER, users=[self.scenario.user]
+        )
+        self.client.force_login(self.scenario.user)
+
+        response = self.client.post(
+            ws.get_absolute_url_configure(),
+            {
+                "public": False,
+                "expiration_delay": 7,
+                "default_expiration_delay": 14,
+            },
+        )
+        self.assertRedirects(response, ws.get_absolute_url())
+
+        ws.refresh_from_db()
+
+        self.assertFalse(ws.public)
+        self.assertEqual(ws.expiration_delay, timedelta(days=7))
+        self.assertEqual(ws.default_expiration_delay, timedelta(days=14))
+
+    @override_permission(Workspace, "can_configure", AllowAll)
+    def test_initial_form(self) -> None:
+        """Get request to ensure the form is displayed."""
+        ws = self.scenario.workspace
+        self.client.force_login(self.scenario.user)
+
+        title_text = f"Configure workspace {ws.name}"
+        response = self.client.get(ws.get_absolute_url_configure())
+        tree = self.assertHTMLValid(response)
+        title = self.assertHasElement(tree, "//head//title")
+        self.assertTextContentEqual(title, f"Debusine - {title_text}")
+        h1 = self.assertHasElement(tree, "//body//h1")
+        self.assertTextContentEqual(h1, title_text)
+
+        assert isinstance(response, SimpleTemplateResponse)
+        form = response.context_data["form"]
+        self.assertTrue(form["public"].initial)
+        self.assertIsNone(form["expiration_delay"].initial)
+        self.assertEqual(form["default_expiration_delay"].initial, timedelta(0))
