@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -27,7 +27,7 @@ from django.views.generic.base import View
 from rest_framework import status
 
 from debusine.db.context import ContextConsistencyError
-from debusine.db.models import User, WorkRequest, Workspace
+from debusine.db.models import User, WorkRequest, WorkflowTemplate, Workspace
 from debusine.db.models.permissions import PermissionUser
 from debusine.test.django import (
     AllowAll,
@@ -359,6 +359,44 @@ class ViewTestMixin(BaseDjangoTestCase):
             self.assertEqual(el.a.get("href"), current.get_absolute_url())
             self.assertTextContentEqual(el.a, current.name)
 
+    def assertNavNoWorkflows(
+        self, tree: lxml.objectify.ObjectifiedElement
+    ) -> None:
+        """Check that the page has no collections in the navbar."""
+        self.assertFalse(tree.xpath("//li[@class='nav-workflows']"))
+
+    def assertNavHasWorkflows(
+        self,
+        tree: lxml.objectify.ObjectifiedElement,
+        workspace: Workspace,
+        workflow_templates: list[WorkflowTemplate],
+    ) -> None:
+        """Check that the page has the given collections in the navbar."""
+        el = self.assertHasElement(tree, "//li[@id='nav-workflows']")
+        if not workflow_templates:
+            workflows_a = self.assertHasElement(el, "a")
+            self.assertFalse(el.xpath("div"))
+        else:
+            workflows_a = self.assertHasElement(el, "div/a[@href!='#']")
+
+        list_url = reverse(
+            "workspaces:workflows:list", kwargs={"wname": workspace.name}
+        )
+        self.assertEqual(workflows_a.get("href"), list_url)
+        self.assertTextContentEqual(workflows_a, "Workflows")
+
+        if not workflow_templates:
+            return
+
+        expected: list[tuple[str, str]] = [
+            (list_url + f"?workflow_templates={wt.name}", wt.name)
+            for wt in workflow_templates
+        ]
+        actual: list[tuple[str, str]] = []
+        for li in el.div.ul.li:
+            actual.append((li.a.get("href") or "", li.a.text or ""))
+        self.assertEqual(expected, actual)
+
     def assertNavNoCollections(
         self, tree: lxml.objectify.ObjectifiedElement
     ) -> None:
@@ -406,8 +444,15 @@ class ViewTestMixin(BaseDjangoTestCase):
         self.assertTextContentEqual(ul.li.a, user.username)
 
         ul = self.assertHasElement(tree, "//ul[@id='navbar-user-actions']")
-        self.assertEqual(ul.li[0].a.get("href"), reverse("user:token-list"))
-        self.assertEqual(ul.li[-1].form.get("action"), reverse("user:logout"))
+        self.assertEqual(
+            ul.li[0].a.get("href"),
+            reverse("user:detail", kwargs={"username": user.username}),
+        )
+        self.assertEqual(
+            ul.li[1].a.get("href"),
+            reverse("user:token-list", kwargs={"username": user.username}),
+        )
+        self.assertEqual(ul.li[-1].form.get("action"), reverse("logout"))
 
 
 def html_check_icon(value: bool) -> str:
