@@ -219,7 +219,13 @@ class WorkRequestDetailView(
             )
 
         context["built_artifacts"] = built_artifacts
-        context["task_data"] = format_yaml(self.object.task_data)
+        context["task_data"] = format_yaml(self.object.used_task_data)
+        if self.object.task_data != self.object.used_task_data:
+            context["task_data_original"] = format_yaml(self.object.task_data)
+            context["task_data_configured"] = True
+        else:
+            context["task_data_configured"] = False
+
         return context
 
     def get_template_names(self) -> list[str]:
