@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -26,20 +26,30 @@ class WorkersListView(BaseUIView, PaginationMixin, ListViewBase[Worker]):
     ordering = "name"
     paginate_by = 50
 
-    def get_ordering(self) -> str:
-        """Return field used for sorting."""
-        order = self.request.GET.get("order")
-        if order in (
-            "name",
-            "worker_type",
-            "registered_at",
-            "last_seen_at",
-            "token__enabled",
-        ):
-            if self.request.GET["asc"] == "0":
-                return "-" + order
+    # Maps URL query parameter names to model field names for sorting
+    ORDERING_FIELDS = {
+        "name": ["name"],
+        "worker_type": ["worker_type"],
+        "registered_at": ["registered_at"],
+        "last_seen_at": ["token__last_seen_at"],
+        "enabled": ["token__enabled"],
+    }
+
+    def get_ordering(self) -> list[str]:
+        """Return fields used for sorting."""
+        order = self.request.GET.get("order", "")
+        if order_fields := self.ORDERING_FIELDS.get(order):
+            if self.request.GET.get("asc", "0") == "0":
+                return [f"-{field}" for field in order_fields]
             else:
-                return order
+                return order_fields
+
+        return ["name"]
+
+    def get_ordering_query_parameter(self) -> str:
+        """Return sorting's GET parameter. Default is "name"."""
+        if (param := self.request.GET.get("order", "")) in self.ORDERING_FIELDS:
+            return param
 
         return "name"
 
@@ -47,7 +57,7 @@ class WorkersListView(BaseUIView, PaginationMixin, ListViewBase[Worker]):
         """Add context to the default ListView data."""
         context = super().get_context_data(**kwargs)
 
-        context["order"] = self.get_ordering().removeprefix("-")
+        context["order"] = self.get_ordering_query_parameter()
 
         context["asc"] = self.request.GET.get("asc", "0")
 
