@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,7 +10,7 @@
 """debusine workflows views."""
 from typing import Any, cast
 
-from django.db.models import QuerySet
+from django.db.models import Q, QuerySet
 
 from debusine.db.context import context
 from debusine.db.models import WorkRequest
@@ -68,7 +68,19 @@ class WorkflowsListView(
                 )
 
             if statuses := self.form.cleaned_data.get("statuses"):
-                queryset = queryset.filter(status__in=statuses)
+                q_objects = Q()
+
+                for status in statuses:
+                    if status == "running__any":
+                        q_objects |= Q(status=WorkRequest.Statuses.RUNNING)
+                    elif status.startswith("running__"):
+                        q_objects |= Q(
+                            workflow_runtime_status=status[len("running__") :]
+                        )
+                    else:
+                        q_objects |= Q(status=status)
+
+                queryset = queryset.filter(q_objects)
 
             if results := self.form.cleaned_data.get("results"):
                 queryset = queryset.filter(result__in=results)
@@ -88,13 +100,6 @@ class WorkflowsListView(
 
                 queryset = queryset.filter(id__in=work_requests_ids_relevant)
 
-            if runtime_statuses := self.form.cleaned_data.get(
-                "runtime_statuses"
-            ):
-                queryset = queryset.filter(
-                    workflow_runtime_status__in=runtime_statuses
-                )
-
         return queryset
 
     def get_ordering(self) -> list[str]:
