@@ -47,6 +47,7 @@ import logging
 import secrets
 import signal
 import socket
+import time
 from concurrent.futures import Future, ThreadPoolExecutor
 from pathlib import Path
 from threading import Lock
@@ -62,10 +63,16 @@ except ImportError:
 
 import tenacity
 
+from debusine.artifacts.models import RuntimeStatistics
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import WorkRequestResponse
-from debusine.tasks import BaseExternalTask, BaseTask, TaskConfigError
+from debusine.tasks import (
+    BaseExternalTask,
+    BaseTask,
+    BaseTaskWithExecutor,
+    TaskConfigError,
+)
 from debusine.tasks.executors import analyze_worker_all_executors
 from debusine.tasks.models import (
     OutputData,
@@ -76,6 +83,7 @@ from debusine.tasks.models import (
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
 from debusine.worker.system_information import (
+    cpu_time,
     host_architecture,
     system_metadata,
 )
@@ -137,6 +145,17 @@ class Worker:
         # safest if done from only one thread.
         self._task_executor = ThreadPoolExecutor(max_workers=1)
 
+        # Statistics collected from the worker and task executors.
+        self._task_statistics: RuntimeStatistics | None = None
+        self._task_statistics_poller: asyncio.Task[None] | None = None
+
+        # The monotonic clock time when the task started executing.
+        self._task_start_time: float | None = None
+
+        # The total CPU time used so far when the task started executing.
+        # (We assume that the worker only runs one task at a time.)
+        self._task_start_cpu_time: float | None = None
+
         # BaseTask.execute (e.g. Sbuild.exec) future is stored here (avoids
         # garbage collection deletion)
         self._task_exec_future: Future[bool] | None = None
@@ -481,6 +500,74 @@ class Worker:
             metadata,
         )
 
+    def _gather_statistics(self) -> None:
+        if self._task_statistics is None:
+            self._task_statistics = RuntimeStatistics()
+
+        # We can only start collecting statistics once an executor is
+        # instantiated.
+        if (
+            not isinstance(self._task_running, BaseTaskWithExecutor)
+            or self._task_running.executor is None
+        ):
+            return
+
+        statistics = self._task_running.executor.get_statistics(
+            self._task_running.executor_instance
+        )
+
+        # We're interested in the high-water mark of these statistics.
+        if (
+            self._task_statistics.disk_space is not None
+            or statistics.disk_space is not None
+        ):
+            self._task_statistics.disk_space = max(
+                self._task_statistics.disk_space or 0,
+                statistics.disk_space or 0,
+            )
+        if (
+            self._task_statistics.memory is not None
+            or statistics.memory is not None
+        ):
+            self._task_statistics.memory = max(
+                self._task_statistics.memory or 0, statistics.memory or 0
+            )
+
+        # We record the initial values of these statistics, but do not
+        # expect them to change afterwards.
+        if self._task_statistics.available_disk_space is None:
+            self._task_statistics.available_disk_space = (
+                statistics.available_disk_space
+            )
+        if self._task_statistics.available_memory is None:
+            self._task_statistics.available_memory = statistics.available_memory
+        if self._task_statistics.cpu_count is None:
+            self._task_statistics.cpu_count = statistics.cpu_count
+
+    async def _poll_statistics(self) -> None:
+        assert self._task_running is not None
+        assert self._task_statistics is not None
+
+        if not isinstance(self._task_running, BaseTaskWithExecutor):
+            return
+
+        while True:
+            self._gather_statistics()
+
+            # Arbitrary polling interval.  We want to poll often enough to
+            # get useful information about relatively brief spikes in disk
+            # or RAM use, but not so often as to spend excessive amounts of
+            # time gathering statistics.
+            await asyncio.sleep(10)
+
+    def _initialize_statistics(self) -> None:
+        self._gather_statistics()
+        self._task_statistics_poller = asyncio.create_task(
+            self._poll_statistics()
+        )
+        self._task_start_time = time.monotonic()
+        self._task_start_cpu_time = cpu_time()
+
     async def _execute_work_request_and_submit_result(
         self, work_request: WorkRequestResponse
     ) -> None:
@@ -497,7 +584,8 @@ class Worker:
             error_message = "Task: %s is of type %s, not %s"
             error_args = (work_request.task_name, task_type, expected_task_type)
             logging.error(error_message, *error_args)
-            self._debusine.work_request_completed_update(
+            await asyncio.to_thread(
+                self._debusine.work_request_completed_update,
                 work_request.id,
                 "error",
                 OutputData(
@@ -520,7 +608,8 @@ class Worker:
             error_message = "Task: %s Error setup: %s"
             error_args = (work_request.task_name, exc)
             logging.error(error_message, *error_args)
-            self._debusine.work_request_completed_update(
+            await asyncio.to_thread(
+                self._debusine.work_request_completed_update,
                 work_request.id,
                 "error",
                 OutputData(
@@ -543,7 +632,8 @@ class Worker:
             error_message = "Task: %s Error configure: %s"
             error_args = (work_request.task_name, exc)
             logging.error(error_message, *error_args)
-            self._debusine.work_request_completed_update(
+            await asyncio.to_thread(
+                self._debusine.work_request_completed_update,
                 work_request.id,
                 "error",
                 OutputData(
@@ -569,6 +659,7 @@ class Worker:
         self._task_running.work_request_id = work_request.id
         self._task_running.workspace_name = work_request.workspace
         self._task_running.worker_host_architecture = host_architecture()
+        self._initialize_statistics()
 
         self._task_name = self._task_running.name
 
@@ -576,15 +667,25 @@ class Worker:
             self._task_running.execute_logging_exceptions
         )
 
-        self._task_exec_future.add_done_callback(
-            functools.partial(
-                self._send_task_result,
-                work_request.id,
+        def send_task_result(task_exec_future: Future[bool]) -> None:
+            assert self._main_event_loop is not None
+            asyncio.run_coroutine_threadsafe(
+                self._send_task_result(work_request.id, task_exec_future),
+                self._main_event_loop,
             )
-        )
+
+        self._task_exec_future.add_done_callback(send_task_result)
 
     def _reset_state(self) -> None:
         """Reset worker to a clean state, ready for the next work request."""
+        self._task_statistics = None
+        # This should already have been done by _send_task_result, but
+        # there's no harm in making sure.
+        if self._task_statistics_poller is not None:
+            self._task_statistics_poller.cancel()
+        self._task_statistics_poller = None
+        self._task_start_time = None
+        self._task_start_cpu_time = None
         self._task_exec_future = None
         self._task_running = None
         if self._task_lock.locked():
@@ -592,12 +693,7 @@ class Worker:
             # work request
             self._task_lock.release()
 
-    @staticmethod
-    async def _exit_thread() -> NoReturn:  # pragma: no cover
-        """Exit current thread, mocked in tests."""
-        raise SystemExit(1)
-
-    def _send_task_result(
+    async def _send_task_result(
         self, work_request_id: int, task_exec_future: Future[bool]
     ) -> None:
         """
@@ -609,12 +705,23 @@ class Worker:
           object
         """
         # This is called unconditionally after Task completion /
-        # failure, on a separate thread (normally the Task thread),
-        # i.e. *not* in the main thread with the asyncio loop
-        assert self._task_running is not None
-
+        # failure, in the main thread with the asyncio loop.
         try:
-            output_data = OutputData()
+            assert self._task_running is not None
+            assert self._task_statistics is not None
+            assert self._task_statistics_poller is not None
+            assert self._task_start_time is not None
+            assert self._task_start_cpu_time is not None
+
+            self._task_statistics_poller.cancel()
+
+            self._task_statistics.duration = int(
+                time.monotonic() - self._task_start_time
+            )
+            self._task_statistics.cpu_time = int(
+                cpu_time() - self._task_start_cpu_time
+            )
+            output_data = OutputData(runtime_statistics=self._task_statistics)
 
             if self._task_running.aborted:
                 logging.info("Task: %s has been aborted", self._task_name)
@@ -635,20 +742,18 @@ class Worker:
                 result = 'success' if task_exec_future.result() else 'failure'
 
             try:
-                self._debusine.work_request_completed_update(
-                    work_request_id, result, output_data
+                await asyncio.to_thread(
+                    self._debusine.work_request_completed_update,
+                    work_request_id,
+                    result,
+                    output_data,
                 )
             except Exception:
-                # Don't leave the worker stuck in "Running" state on the server
+                # Log this, but leave the work request running.  The server
+                # will retry it when this worker next manages to connect and
+                # request a new work request to run.
                 logging.exception(
-                    "Cannot reach server to report work request completed,"
-                    " exiting."
-                )
-                # _main_event_loop initialized in main and never unset
-                assert self._main_event_loop is not None
-                # Exit main asyncio thread from this secondary thread
-                asyncio.run_coroutine_threadsafe(
-                    self._exit_thread(), self._main_event_loop
+                    "Cannot reach server to report work request completed."
                 )
         finally:
             self._reset_state()
