@@ -172,8 +172,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
                 "input": {
                     "source_artifact": 5,
                 },
-                "distribution": "bullseye",
-                "backend": "schroot",
+                "environment": "debian/match:codename=bullseye",
                 "host_architecture": "amd64",
                 "build_components": [
                     "any",
