@@ -16,6 +16,7 @@ import logging
 import signal
 import socket
 import threading
+import time
 from collections.abc import AsyncGenerator, Awaitable
 from contextlib import asynccontextmanager
 from datetime import datetime
@@ -34,6 +35,7 @@ from multidict import CIMultiDict, CIMultiDictProxy
 from yarl import URL
 
 import debusine.tasks.models as task_models
+from debusine.artifacts.models import RuntimeStatistics
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import (
@@ -43,21 +45,28 @@ from debusine.client.models import (
 from debusine.tasks import (
     BaseExternalTask,
     BaseTask,
+    BaseTaskWithExecutor,
     Noop,
     Sbuild,
     TaskConfigError,
 )
+from debusine.tasks.executors import ExecutorStatistics, UnshareExecutor
 from debusine.tasks.models import (
+    BackendType,
     OutputData,
     OutputDataError,
+    SbuildData,
+    SbuildInput,
     TaskTypes,
     WorkerType,
 )
 from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test import TestCase
+from debusine.test.utils import create_artifact_response
 from debusine.worker import Worker
 from debusine.worker.config import ConfigHandler
 from debusine.worker.system_information import (
+    cpu_time,
     host_architecture,
     system_metadata,
 )
@@ -500,11 +509,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
     async def test_process_message_dynamic_metadata_request(self) -> None:
         """Worker receives a request to send the dynamic metadata."""
         executors_metadata = {"executor:unshare:available": True}
-        tasks_metadata = {
-            "sbuild:version": 1,
-            "sbuild:available": True,
-            "sbuild:chroots": 'chroot:unstable-amd64-sbuild',
-        }
+        tasks_metadata = {"sbuild:version": 1, "sbuild:available": True}
 
         analyze_executors_patcher = mock.patch(
             "debusine.worker._worker.analyze_worker_all_executors",
@@ -601,18 +606,120 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(count, times)
 
+    async def test_poll_statistics(self) -> None:
+        """`_poll_statistics` accumulates statistics."""
+        self.setup_worker()
+        assert self.worker is not None
+        self.worker._task_running = Sbuild(
+            SbuildData(
+                input=SbuildInput(source_artifact=1),
+                host_architecture="amd64",
+                backend=BackendType.UNSHARE,
+                environment="debian/match:codename=sid",
+            ).dict()
+        )
+        with mock.patch(
+            "debusine.tasks.executors.images.ImageCache.image_artifact",
+            return_value=create_artifact_response(id=1),
+        ):
+            self.worker._task_running.executor = UnshareExecutor(
+                mock.create_autospec(Debusine), 1
+            )
+        self.worker._task_statistics = RuntimeStatistics()
+        GiB = 1024 * 1024 * 1024
+        got_statistics = [
+            ExecutorStatistics(),
+            ExecutorStatistics(
+                disk_space=GiB,
+                memory=GiB,
+                available_disk_space=10 * GiB,
+                available_memory=4 * GiB,
+                cpu_count=2,
+            ),
+            ExecutorStatistics(
+                disk_space=3 * GiB,
+                memory=2 * GiB,
+                available_disk_space=10 * GiB,
+                available_memory=4 * GiB,
+                cpu_count=2,
+            ),
+            ExecutorStatistics(
+                disk_space=4 * GiB,
+                memory=3 * GiB,
+                available_disk_space=10 * GiB,
+                available_memory=4 * GiB,
+                cpu_count=2,
+            ),
+            ExecutorStatistics(
+                disk_space=2 * GiB,
+                memory=GiB,
+                available_disk_space=10 * GiB,
+                available_memory=4 * GiB,
+                cpu_count=2,
+            ),
+        ]
+        sleep_results = [None] * (len(got_statistics) - 1) + [
+            asyncio.CancelledError
+        ]
+
+        with (
+            self.assertRaises(asyncio.CancelledError),
+            mock.patch(
+                "debusine.tasks.executors.unshare.UnshareExecutor"
+                ".get_statistics",
+                side_effect=got_statistics,
+            ),
+            mock.patch("asyncio.sleep", side_effect=sleep_results),
+        ):
+            await self.worker._poll_statistics()
+
+        self.assertEqual(
+            self.worker._task_statistics,
+            RuntimeStatistics(
+                disk_space=4 * GiB,
+                memory=3 * GiB,
+                available_disk_space=10 * GiB,
+                available_memory=4 * GiB,
+                cpu_count=2,
+            ),
+        )
+
+    async def test_poll_statistics_no_executor(self) -> None:
+        """`_poll_statistics` ignores tasks without an executor."""
+        self.setup_worker()
+        assert self.worker is not None
+        self.worker._task_running = Noop({"result": True})
+        self.worker._task_statistics = RuntimeStatistics()
+
+        with mock.patch("asyncio.sleep") as mock_sleep:
+            await self.worker._poll_statistics()
+
+        mock_sleep.assert_not_called()
+        self.assertEqual(self.worker._task_statistics, RuntimeStatistics())
+
     @asynccontextmanager
-    async def completed_task(self) -> AsyncGenerator[None]:
-        """Context manager that waits for a task to complete."""
+    async def completed_task(
+        self, *, keep_state: bool = False
+    ) -> AsyncGenerator[None]:
+        """
+        Context manager that waits for a task to complete.
+
+        :param keep_state: If True, don't reset state at the end of the
+          task, to allow tests to inspect it.
+        """
         assert self.worker is not None
         assert self.worker._main_event_loop is not None
 
         completed = asyncio.Event()
+        original_reset_state = self.worker._reset_state
 
         async def report_completed(worker: Worker) -> None:
-            # Worker._reset_state would do this; we should clean up too.
-            if worker._task_lock.locked():  # pragma: no cover
-                worker._task_lock.release()
+            if keep_state:
+                # Worker._reset_state would do this; we should clean up too.
+                if worker._task_lock.locked():  # pragma: no cover
+                    worker._task_lock.release()
+            else:
+                original_reset_state()
 
             completed.set()
 
@@ -670,8 +777,34 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             self.patch_debusine_work_request_completed_update()
         )
 
-        async with self.completed_task():
-            await self.worker.connect()
+        with mock.patch(
+            "debusine.tasks.executors.images.ImageCache.image_artifact",
+            return_value=create_artifact_response(id=1),
+        ):
+            executor = UnshareExecutor(mock.create_autospec(Debusine), 1)
+        GiB = 1024 * 1024 * 1024
+
+        with (
+            mock.patch(
+                "debusine.tasks._task.BaseTaskWithExecutor.executor",
+                create=True,
+                new_callable=mock.PropertyMock,
+                return_value=executor,
+            ),
+            mock.patch.object(
+                executor,
+                "get_statistics",
+                return_value=ExecutorStatistics(
+                    disk_space=2 * GiB,
+                    memory=GiB,
+                    available_disk_space=20 * GiB,
+                    available_memory=8 * GiB,
+                    cpu_count=4,
+                ),
+            ),
+        ):
+            async with self.completed_task(keep_state=True):
+                await self.worker.connect()
 
         # The WorkRequest is executed...
         self.assertEqual(mocked_execute.call_count, 1)
@@ -694,6 +827,30 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         output_data = work_request_completed_mocked.call_args[0][2]
         self.assertIsInstance(output_data, OutputData)
+        runtime_statistics = output_data.runtime_statistics
+        assert runtime_statistics is not None
+        assert self.worker._task_start_time is not None
+        assert self.worker._task_start_cpu_time is not None
+        self.assertLessEqual(
+            runtime_statistics.duration,
+            time.monotonic() - self.worker._task_start_time,
+        )
+        self.assertLessEqual(
+            runtime_statistics.cpu_time,
+            cpu_time() - self.worker._task_start_cpu_time,
+        )
+        if isinstance(task, BaseTaskWithExecutor):
+            self.assertEqual(runtime_statistics.disk_space, 2 * GiB)
+            self.assertEqual(runtime_statistics.memory, GiB)
+            self.assertEqual(runtime_statistics.available_disk_space, 20 * GiB)
+            self.assertEqual(runtime_statistics.available_memory, 8 * GiB)
+            self.assertEqual(runtime_statistics.cpu_count, 4)
+        else:
+            self.assertIsNone(runtime_statistics.disk_space)
+            self.assertIsNone(runtime_statistics.memory)
+            self.assertIsNone(runtime_statistics.available_disk_space)
+            self.assertIsNone(runtime_statistics.available_memory)
+            self.assertIsNone(runtime_statistics.cpu_count)
 
     async def test_task_is_received_executed_submit_success(self) -> None:
         """Receive a task, execute it and submit success completion."""
@@ -786,15 +943,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             # Allow the execute call to finish
             lock.release()
 
-    async def test_log_work_request_cannot_report_exit(self) -> None:
-        """Worker can't report to the server, exits."""
-        # Can't catch SystemExit from main thread in async tests
-        patcher = mock.patch(
-            'debusine.worker.Worker._exit_thread', autospec=True
-        )
-        patched_exit_thread = patcher.start()
-        self.addCleanup(patcher.stop)
-
+    async def test_log_work_request_cannot_report_logs(self) -> None:
+        """Worker can't report to the server, so logs this."""
         # Mock Sbuild.execute()
         patcher_execute = mock.patch.object(Sbuild, 'execute', autospec=True)
         mocked_execute = patcher_execute.start()
@@ -823,8 +973,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         with self.assertLogsContains(
             "Cannot reach server", level=logging.ERROR
         ):
-            await self.worker.connect()
-            patched_exit_thread.assert_called()
+            async with self.completed_task():
+                await self.worker.connect()
 
     async def assert_invalid_get_next_for_worker_logging(
         self, get_next_for_worker: ServerConfig.GetNextForWorkerResponse
@@ -1131,11 +1281,11 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         """Sbuild.execute raises TaskConfigError, Worker submits error."""
         patcher_execute = mock.patch.object(Sbuild, 'execute', autospec=True)
         mocked_execute = patcher_execute.start()
-        mocked_execute.side_effect = TaskConfigError('No suitable schroot')
+        mocked_execute.side_effect = TaskConfigError('Broken')
         self.addCleanup(patcher_execute.stop)
 
         await self.assert_execute_work_request_send_and_log_error(
-            'Task: sbuild Error execute: No suitable schroot', "execute-failed"
+            'Task: sbuild Error execute: Broken', "execute-failed"
         )
 
     async def test_process_message_invalid_msg(self) -> None:
@@ -1535,12 +1685,13 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         self.worker._task_name = "Noop"
         self.worker._task_running = Noop({"result": False})
+        self.worker._initialize_statistics()
         self.worker._task_running.abort()
 
         with self.assertLogsContains(
             "Task: Noop has been aborted", level=logging.INFO
         ):
-            self.worker._send_task_result(11, concurrent.futures.Future())
+            await self.worker._send_task_result(11, concurrent.futures.Future())
 
         work_request_completed_update.assert_not_called()
 
@@ -1617,3 +1768,20 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(retry_state.attempt_number, 0)
         self.assertFalse(self.worker._reset_retry_attempt_number)
+
+    async def test_reset_state_cancels_task_statistics_poller(self) -> None:
+        """Worker._reset_state makes sure the statistics poller is cancelled."""
+        self.setup_worker()
+        assert self.worker is not None
+
+        self.worker._task_name = "Noop"
+        self.worker._task_running = Noop({"result": False})
+        self.worker._initialize_statistics()
+        task_statistics_poller = self.worker._task_statistics_poller
+        assert task_statistics_poller is not None
+        self.assertFalse(task_statistics_poller.cancelling())
+
+        self.worker._reset_state()
+
+        self.assertTrue(task_statistics_poller.cancelling())
+        self.assertIsNone(self.worker._task_statistics_poller)
